@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -33,6 +33,9 @@ public:
     ObjectCollection(const string& _collection_name)
         :collection_name(_collection_name){};
 
+    ObjectCollection(const string& cname, const set<int>& cset)
+        :collection_name(cname), collection_set(cset){};
+
     ~ObjectCollection(){};
 
     /**
@@ -41,7 +44,7 @@ public:
      *
      *    @return 0 on success, -1 if the ID was already in the set
      */
-    int add_collection_id(int id);
+    int add(int id);
 
     /**
      *  Deletes an ID from the set.
@@ -49,24 +52,33 @@ public:
      *
      *    @return 0 on success, -1 if the ID was not in the set
      */
-    int del_collection_id(int id);
+    int del(int id);
+
+    /**
+     *  Deletes all IDs from the set.
+     */
+    void clear()
+    {
+        collection_set.clear();
+    }
 
     /**
      *  Returns how many IDs are there in the set.
      *    @return how many IDs are there in the set.
      */
-    int get_collection_size() const
+    int size() const
     {
         return collection_set.size();
     };
 
     /**
-     *  Rebuilds the object from an xml node
-     *    @param node The xml node pointer
+     * Rebuilds the object from an xml object
+     * @param xml xml object
+     * @param xpath_prefix Parent nodes, e.g. "/DATASTORE/"
      *
-     *    @return 0 on success, -1 otherwise
+     * @return 0 on success, -1 otherwise
      */
-    int from_xml_node(const xmlNodePtr node);
+    int from_xml(const ObjectXML* xml, const string& xpath_prefix);
 
     /**
      * Function to print the Collection object into a string in
@@ -76,13 +88,20 @@ public:
      */
     string& to_xml(string& xml) const;
 
-
     /**
      *  Returns a copy of the IDs set
      */
-    set<int> get_collection_copy()
+    set<int> clone() const
     {
-        return set<int> (collection_set);
+        return set<int>(collection_set);
+    };
+
+    /**
+     *  Returns a reference to the IDs set
+     */
+    const set<int>& get_collection() const
+    {
+        return collection_set;
     };
 
     /**
@@ -90,11 +109,24 @@ public:
      * @param id ID to search
      * @return true if the collection contains the given id
      */
-    bool collection_contains(int id)
+    bool contains(int id)
     {
         return collection_set.count(id) > 0;
     }
 
+    /**
+     *  Returns and deletes the first element from the set
+     *    @param the element
+     *    @return 0 on success -1 if the set was empty
+     */
+    int pop(int& elem);
+
+    /**
+     * Adds to the collection the contents of other collection
+     *
+     */
+    ObjectCollection& operator<<(const ObjectCollection& r);
+
 private:
 
     /**
@@ -107,6 +139,13 @@ private:
      */
     set<int> collection_set;
 
+    /**
+     *  Rebuilds the object from an xml node
+     *    @param node The xml node pointer
+     *
+     *    @return 0 on success, -1 otherwise
+     */
+    int from_xml_node(const xmlNodePtr node);
 };
 
 #endif /*OBJECT_COLLECTION_H_*/
