@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -248,8 +248,8 @@ protected:
      *    @param remotes_location, path to remotes in the front-end where the
      *    hooks are installed
      */
-    void register_hooks(vector<const Attribute *> hook_mads,
-                        const string&             remotes_location);
+    void register_hooks(vector<const VectorAttribute *> hook_mads,
+                        const string&                   remotes_location);
 
     /**
      *  Gets an object from the pool (if needed the object is loaded from the
