@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -28,18 +28,23 @@ using namespace std;
 class RequestManagerProxy: public Request
 {
 public:
-    RequestManagerProxy(string _method);
+    RequestManagerProxy(string _method): Request("RequestManagerProxy", "?",
+        "Forwards the request to another OpenNebula"), method(_method)
+    {
+        method_name = ("RequestManagerProxy." + method);
+    };
 
-    ~RequestManagerProxy();
+    ~RequestManagerProxy(){};
 
     void request_execute(xmlrpc_c::paramList const& _paramList,
                          RequestAttributes& att);
 
-    void hide_argument(int arg);
+    void hide_argument(int arg)
+    {
+        hidden_params.insert(arg);
+    };
 
 private:
-    Client *  client;
-
     string    method;
 };
 
