@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -22,7 +22,9 @@
 #include <sstream>
 
 #include "Mad.h"
+#include "ActionSet.h"
 #include "VirtualMachinePool.h"
+#include "History.h"
 
 using namespace std;
 
@@ -83,6 +85,16 @@ public:
      */
     static void process_poll(VirtualMachine* vm, const string &monitor_str);
 
+    /**
+     *  Check if action is supported for imported VMs
+     *    @param action
+     *    @return True if it is supported
+     */
+    bool is_imported_action_supported(History::VMAction action) const
+    {
+        return imported_actions.is_set(action);
+    }
+
 protected:
     /**
      *  Gets a configuration attr from driver configuration file (single
@@ -90,13 +102,10 @@ protected:
      *    @param name of config attribute
      *    @param value of the attribute
      */
-    void get_default(
-        const char *  name,
-        string&       value) const
+    template<typename T>
+    void get_default(const string& name, T& value) const
     {
-        string sn = name;
-
-        driver_conf.get(sn,value);
+        driver_conf.get(name, value);
     }
 
     /**
@@ -106,38 +115,41 @@ protected:
      *    @param vname of the attribute
      *    @param value of the attribute
      */
-    void get_default(
-        const char *  name,
-        const char *  vname,
-        string&       value) const;
+    template<typename T>
+    int get_default(const char* name, const char* vname, T& value) const
+    {
+        const VectorAttribute * vattr = driver_conf.get(name);
 
-    /**
-     *  Gets a configuration attr from driver configuration file (vector
-     *  version)
-     *    @param name of config vector attribute for the domain
-     *    @param vname of the attribute
-     *    @param value of the attribute
-     *
-     *    @return true if the attribute was found
-     */
-    bool get_default(
-        const char *  name,
-        const char *  vname,
-        bool&         value) const;
+        if (vattr == 0)
+        {
+            return -1;
+        }
+
+        return vattr->vector_value(vname, value);
+    }
 
 private:
+    friend class VirtualMachineManager;
+
+    static const string imported_actions_default;
+    static const string imported_actions_default_public;
+
     /**
      *  Configuration file for the driver
      */
     Template    driver_conf;
 
+    /**
+     *  List of available actions for imported VMs. Each bit is an action
+     *  as defined in History.h, 1=supported and 0=not supported
+     */
+    ActionSet<History::VMAction> imported_actions;
+
     /**
      *  Pointer to the Virtual Machine Pool, to access VMs
      */
     VirtualMachinePool * vmpool;
 
-    friend class VirtualMachineManager;
-
     /**
      *  Sends a deploy request to the MAD: "DEPLOY ID XML_DRV_MSG"
      *    @param oid the virtual machine id.
@@ -270,7 +282,6 @@ private:
         const string& drv_msg) const
     {
         write_drv("POLL", oid, drv_msg);
-
     }
 
     /**
@@ -374,16 +385,16 @@ private:
     }
 
     /**
-     *  Sends a disk snapshot revert request to the MAD:
-     *  "DISKSNAPSHOTREVERT ID XML_DRV_MSG"
+     *  Sends a request to update the VM security groups:
+     *  "UPDATESG ID XML_DRV_MSG"
      *    @param oid the virtual machine id.
      *    @param drv_msg xml data for the mad operation
      */
-    void disk_snapshot_revert (
+    void updatesg (
         const int     oid,
         const string& drv_msg) const
     {
-        write_drv("DISKSNAPSHOTREVERT", oid, drv_msg);
+        write_drv("UPDATESG", oid, drv_msg);
     }
 
     /**
