@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -22,6 +22,7 @@
 #include "VirtualNetworkTemplate.h"
 #include "Clusterable.h"
 #include "AddressRangePool.h"
+#include "ObjectCollection.h"
 
 #include <vector>
 #include <string>
@@ -44,6 +45,68 @@ class VirtualNetwork : public PoolObjectSQL, public Clusterable
 {
 public:
 
+    /**
+     *  Defines the Virtual Network type based on its associated driver
+     */
+    enum VirtualNetworkDriver
+    {
+        NONE     = 0,
+        DUMMY    = 1,
+        VLAN     = 2,
+        EBTABLES = 3,
+        FW       = 4,
+        OVSWITCH = 5,
+        VXLAN    = 6
+    };
+
+    static string driver_to_str(VirtualNetworkDriver ob)
+    {
+        switch (ob)
+        {
+            case NONE:     return "";
+            case DUMMY:    return "dummy";
+            case VLAN:     return "802.1Q";
+            case EBTABLES: return "ebtables";
+            case FW:       return "fw";
+            case OVSWITCH: return "ovswitch";
+            case VXLAN:    return "vxlan";
+        }
+    };
+
+    static VirtualNetworkDriver str_to_driver(string& ob)
+    {
+        one_util::tolower(ob);
+
+        if ( ob == "dummy" )
+        {
+            return DUMMY;
+        }
+        else if ( ob == "802.1q" )
+        {
+            return VLAN;
+        }
+        else if ( ob == "ebtables" )
+        {
+            return EBTABLES;
+        }
+        else if ( ob == "fw" )
+        {
+            return FW;
+        }
+        else if ( ob == "ovswitch" )
+        {
+            return OVSWITCH;
+        }
+        else if ( ob == "vxlan" )
+        {
+            return VXLAN;
+        }
+        else
+        {
+            return NONE;
+        }
+    };
+
     // *************************************************************************
     // Virtual Network Public Methods
     // *************************************************************************
@@ -82,7 +145,7 @@ public:
      *  @param error_msg If the action fails, this message contains the reason.
      *  @return 0 on success
      */
-    int add_var(vector<Attribute *> &var, string& error_msg);
+    int add_var(vector<VectorAttribute *> &var, string& error_msg);
 
     /**
      * Removes an address range from the VNET
@@ -156,64 +219,80 @@ public:
 
     /**
      *    Gets a new address lease for a specific VM
-     *    @param vid VM identifier
+     *    @param ot the type of the object requesting the address
+     *    @param oid the id of the object requesting the address
      *    @param nic the VM NIC attribute to be filled with the lease info.
      *    @param inherit attributes from the address range to include in the NIC
      *    @return 0 if success
      */
-    int allocate_addr(int vid, VectorAttribute * nic,
-        const vector<string>& inherit)
+    int allocate_addr(PoolObjectSQL::ObjectType ot, int oid,
+            VectorAttribute * nic, const vector<string>& inherit)
     {
-        return ar_pool.allocate_addr(PoolObjectSQL::VM, vid, nic, inherit);
+        return ar_pool.allocate_addr(ot, oid, nic, inherit);
     }
 
     /**
      *    Gets a new address lease for a specific VM by MAC
-     *    @param vid VM identifier
+     *    @param ot the type of the object requesting the address
+     *    @param oid the id of the object requesting the address
      *    @param mac the MAC address requested
      *    @param nic the VM NIC attribute to be filled with the lease info.
      *    @param inherit attributes from the address range to include in the NIC
      *    @return 0 if success
      */
-    int allocate_by_mac(int vid, const string& mac, VectorAttribute * nic,
-        const vector<string>& inherit)
+    int allocate_by_mac(PoolObjectSQL::ObjectType ot, int oid, const string& mac,
+            VectorAttribute * nic, const vector<string>& inherit)
     {
-        return ar_pool.allocate_by_mac(mac, PoolObjectSQL::VM, vid, nic, inherit);
+        return ar_pool.allocate_by_mac(mac, ot, oid, nic, inherit);
     }
 
     /**
      *    Gets a new address lease for a specific VM by IP
-     *    @param vid VM identifier
+     *    @param ot the type of the object requesting the address
+     *    @param oid the id of the object requesting the address
      *    @param ip the IP address requested
      *    @param nic the VM NIC attribute to be filled with the lease info.
      *    @param inherit attributes from the address range to include in the NIC
      *    @return 0 if success
      */
-    int allocate_by_ip(int vid, const string& ip, VectorAttribute * nic,
-        const vector<string>& inherit)
+    int allocate_by_ip(PoolObjectSQL::ObjectType ot, int oid, const string& ip,
+            VectorAttribute * nic, const vector<string>& inherit)
     {
-        return ar_pool.allocate_by_ip(ip, PoolObjectSQL::VM, vid, nic, inherit);
+        return ar_pool.allocate_by_ip(ip, ot, oid, nic, inherit);
     }
 
     /**
      *  Release previously given address lease
      *    @param arid of the address range where the address was leased from
-     *    @param vid the ID of the VM
+     *    @param ot the type of the object requesting the address
+     *    @param oid the id of the object requesting the address
      *    @param mac MAC address identifying the lease
      */
-    void free_addr(unsigned int arid, int vid, const string& mac)
+    void free_addr(unsigned int arid, PoolObjectSQL::ObjectType ot, int oid,
+                    const string& mac)
     {
-        ar_pool.free_addr(arid, PoolObjectSQL::VM, vid, mac);
+        ar_pool.free_addr(arid, ot, oid, mac);
+
+        if (ot == PoolObjectSQL::VROUTER)
+        {
+            vrouters.del(oid);
+        }
     }
 
     /**
      *  Release previously given address lease
-     *    @param vid the ID of the VM
+     *    @param ot the type of the object requesting the address
+     *    @param oid the id of the object requesting the address
      *    @param mac MAC address identifying the lease
      */
-    void free_addr(int vid, const string& mac)
+    void free_addr(PoolObjectSQL::ObjectType ot, int oid, const string& mac)
     {
-        ar_pool.free_addr(PoolObjectSQL::VM, vid, mac);
+        ar_pool.free_addr(ot, oid, mac);
+
+        if (ot == PoolObjectSQL::VROUTER)
+        {
+            vrouters.del(oid);
+        }
     }
 
     /**
@@ -255,6 +334,21 @@ public:
             int                     vid,
             const vector<string>&   inherit_attrs);
 
+    /**
+     * Modifies the given nic attribute adding the following attributes:
+     *  * IP:  leased from network
+     *  * MAC: leased from network
+     *  @param nic attribute for the VRouter template
+     *  @param vrid of the VRouter getting the lease
+     *  @param inherit_attrs Attributes to be inherited from the vnet template
+     *      into the nic
+     *  @return 0 on success
+     */
+    int vrouter_nic_attribute(
+            VectorAttribute *       nic,
+            int                     vrid,
+            const vector<string>&   inherit_attrs);
+
     /**
      * From a Security Group rule that uses this vnet, creates a new rule
      * copy for each AR.
@@ -377,10 +471,12 @@ public:
      *  A vector containing just -1 means all VMs.
      *  @param vnet_ids list of VNET the user can access reservation info from.
      *  A vector containing just -1 means all VNETs.
+     *  @param vrs list of VRouter the user can access reservation info from.
+     *  A vector containing just -1 means all VRouters.
      *  @return a reference to the generated string
      */
     string& to_xml_extended(string& xml, const vector<int>& vms,
-        const vector<int>& vnets) const;
+        const vector<int>& vnets, const vector<int>& vrs) const;
 
     /**
      *  Gets a string based attribute (single) from an address range. If the
@@ -423,6 +519,11 @@ private:
     // Binded physical attributes
     // -------------------------------------------------------------------------
 
+    /**
+     * Name of the vn mad
+     */
+    string vn_mad;
+
     /**
      *  Name of the bridge this VNW binds to
      */
@@ -439,9 +540,9 @@ private:
     string  vlan_id;
 
     /**
-     *  Whether or not to isolate this network with the vnm driver
+     *  If the VLAN has been set automatically
      */
-    int     vlan;
+    bool  vlan_id_automatic;
 
     /**
      *  Parent VNET ID if any
@@ -458,6 +559,11 @@ private:
      */
     AddressRangePool ar_pool;
 
+    /**
+     *  Set of Virtual Router IDs
+     */
+    ObjectCollection vrouters;
+
     // *************************************************************************
     // DataBase implementation (Private)
     // *************************************************************************
@@ -490,7 +596,8 @@ private:
      *  @return a reference to the generated string
      */
     string& to_xml_extended(string& xml, bool extended,
-        const vector<int>& vm_ids, const vector<int>& vnet_oids) const;
+        const vector<int>& vm_ids, const vector<int>& vnet_oids,
+        const vector<int>& vr_ids) const;
 
     /**
      *  Rebuilds the object from an xml formatted string
@@ -501,7 +608,7 @@ private:
     int from_xml(const string &xml_str);
 
     /**
-     * Updates the BRIDGE, PHY_DEV, VLAN_ID and VLAN attributes.
+     * Updates the BRIDGE, PHY_DEV, and VLAN_ID attributes.
      *    @param error string describing the error if any
      *    @return 0 on success
      */
@@ -517,8 +624,7 @@ private:
                    const string&            _gname,
                    int                      _umask,
                    int                      _parent_vid,
-                   int                      _cluster_id,
-                   const string&            _cluster_name,
+                   const set<int>           &_cluster_ids,
                    VirtualNetworkTemplate * _vn_template = 0);
 
     ~VirtualNetwork();
