@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -28,7 +28,7 @@ include OpenNebula
 module OpenNebulaHelper
     ONE_VERSION=<<-EOT
 OpenNebula #{OpenNebula::VERSION}
-Copyright 2002-2015, OpenNebula Project, OpenNebula Systems
+Copyright 2002-2016, OpenNebula Project, OpenNebula Systems
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -267,6 +267,12 @@ EOT
             :description => 'VNC IP where to listen for connections. '<<
                 'By default is 0.0.0.0 (all interfaces).'
         },
+        {
+            :name   => 'vnc_keymap',
+            :large  => '--vnc-keymap keymap',
+            :format => String,
+            :description => 'VNC keyboard layout'
+        },
         {
             :name   => 'spice',
             :large  => '--spice',
@@ -285,12 +291,18 @@ EOT
             :description => 'spice IP where to listen for connections. '<<
                 'By default is 0.0.0.0 (all interfaces).'
         },
+        {
+            :name   => 'spice_keymap',
+            :large  => '--spice-keymap keymap',
+            :format => String,
+            :description => 'spice keyboard layout'
+        },
         {
             :name   => 'ssh',
             :large  => '--ssh [file]',
             :description => "Add an ssh public key to the context. If the \n"<<
-                (' '*31)<<"file is omited then the user variable \n"<<
-                (' '*31)<<"SSH_PUBLIC_KEY will be used.",
+                (' '*31) << "file is omited then the user variable \n"<<
+                (' '*31) << "SSH_PUBLIC_KEY will be used.",
             :format => String,
             :proc => lambda do |o, options|
                 if !o
@@ -313,8 +325,8 @@ EOT
         },
         {
             :name   => 'boot',
-            :large  => '--boot device',
-            :description => 'Select boot device (hd|fd|cdrom|network)',
+            :large  => '--boot device_list',
+            :description => 'Set boot device list e.g. disk0,disk2,nic0',
             :format => String
         },
         {
@@ -329,12 +341,28 @@ EOT
             :large  => '--init script1,script2',
             :format => Array,
             :description => 'Script or scripts to start in context'
+        },
+        {
+            :name   => 'startscript',
+            :large  => '--startscript [file]',
+            :format => String,
+            :description => 'Start script to be executed'
         }
     ]
 
-    TEMPLATE_OPTIONS_VM=[TEMPLATE_NAME_VM]+TEMPLATE_OPTIONS+[DRY]
+    FORCE={
+        :name  => 'force',
+        :large  => '--force',
+        :description => 'Overwrite the file'
+    }
 
-    CAPACITY_OPTIONS_VM=[TEMPLATE_OPTIONS[0],TEMPLATE_OPTIONS[1],TEMPLATE_OPTIONS[3]]
+    TEMPLATE_OPTIONS_VM   = [TEMPLATE_NAME_VM] + TEMPLATE_OPTIONS + [DRY]
+
+    CAPACITY_OPTIONS_VM   = [TEMPLATE_OPTIONS[0], TEMPLATE_OPTIONS[1],
+      TEMPLATE_OPTIONS[3]]
+
+    UPDATECONF_OPTIONS_VM = TEMPLATE_OPTIONS[6..15] + [TEMPLATE_OPTIONS[2],
+      TEMPLATE_OPTIONS[17], TEMPLATE_OPTIONS[18]]
 
     OPTIONS = XML, NUMERIC, KILOBYTES
 
@@ -665,12 +693,12 @@ EOT
             end
         end
 
-        private
-
         def retrieve_resource(id)
             factory(id)
         end
 
+        private
+
         def pool_to_array(pool)
             if !pool.instance_of?(Hash)
                 phash = pool.to_hash
@@ -714,16 +742,17 @@ EOT
         client=OneHelper.client
 
         pool = case poolname
-        when "HOST"      then OpenNebula::HostPool.new(client)
-        when "GROUP"     then OpenNebula::GroupPool.new(client)
-        when "USER"      then OpenNebula::UserPool.new(client)
-        when "DATASTORE" then OpenNebula::DatastorePool.new(client)
-        when "CLUSTER"   then OpenNebula::ClusterPool.new(client)
-        when "VNET"      then OpenNebula::VirtualNetworkPool.new(client)
-        when "IMAGE"     then OpenNebula::ImagePool.new(client)
-        when "VMTEMPLATE" then OpenNebula::TemplatePool.new(client)
-        when "VM"        then OpenNebula::VirtualMachinePool.new(client)
-        when "ZONE"      then OpenNebula::ZonePool.new(client)
+        when "HOST"        then OpenNebula::HostPool.new(client)
+        when "GROUP"       then OpenNebula::GroupPool.new(client)
+        when "USER"        then OpenNebula::UserPool.new(client)
+        when "DATASTORE"   then OpenNebula::DatastorePool.new(client)
+        when "CLUSTER"     then OpenNebula::ClusterPool.new(client)
+        when "VNET"        then OpenNebula::VirtualNetworkPool.new(client)
+        when "IMAGE"       then OpenNebula::ImagePool.new(client)
+        when "VMTEMPLATE"  then OpenNebula::TemplatePool.new(client)
+        when "VM"          then OpenNebula::VirtualMachinePool.new(client)
+        when "ZONE"        then OpenNebula::ZonePool.new(client)
+        when "MARKETPLACE" then OpenNebula::MarketPlacePool.new(client)
         end
 
         rc = pool.info
@@ -819,6 +848,15 @@ EOT
         end
     end
 
+    def OpenNebulaHelper.clusters_str(clusters)
+        if clusters.nil?
+            "-"
+        else
+            [clusters].flatten.join(',')
+        end
+
+    end
+
     def OpenNebulaHelper.update_template(id, resource, path=nil, xpath='TEMPLATE')
         return update_template_helper(false, id, resource, path, xpath)
     end
@@ -915,7 +953,7 @@ EOT
     end
 
     def self.create_context(options)
-        context_options = [:ssh, :net_context, :context, :init, :files_ds]
+        context_options = [:ssh, :net_context, :context, :init, :files_ds, :startscript]
         if !(options.keys & context_options).empty?
             lines=[]
 
@@ -953,8 +991,20 @@ EOT
                 lines << %Q<INIT_SCRIPTS="#{options[:init].join(' ')}">
             end
 
+            if options[:startscript]
+                script = nil
+                begin
+                    script = File.read(options[:startscript]).strip
+                rescue Exception => e
+                    STDERR.puts e.message
+                    exit(-1)
+                end
+                script = Base64::strict_encode64(script)
+                lines<<"START_SCRIPT_BASE64=\"#{script}\""
+            end
+
             if !lines.empty?
-                "CONTEXT=[\n"<<lines.map{|l| "  "<<l }.join(",\n")<<"\n]\n"
+                "CONTEXT=[\n" << lines.map{|l| "  " << l }.join(",\n") << "\n]\n"
             else
                 nil
             end
@@ -963,7 +1013,7 @@ EOT
         end
     end
 
-    def self.create_template(options)
+    def self.create_template(options, template_obj=nil)
         template=''
 
         template<<"NAME=\"#{options[:name]}\"\n" if options[:name]
@@ -1005,7 +1055,10 @@ EOT
             if options[:vnc_password]
                 template << ", PASSWD=\"#{options[:vnc_password]}\""
             end
-            template<<' ]'<<"\n"
+            if options[:vnc_keymap]
+                template << ", KEYMAP=\"#{options[:vnc_keymap]}\""
+            end
+            template<<' ]' << "\n"
         end
 
         if options[:spice]
@@ -1014,12 +1067,26 @@ EOT
             if options[:spice_password]
                 template << ", PASSWD=\"#{options[:spice_password]}\""
             end
-            template<<' ]'<<"\n"
+            if options[:spice_keymap]
+                template << ", KEYMAP=\"#{options[:spice_keymap]}\""
+            end
+            template<<' ]' << "\n"
         end
 
         context=create_context(options)
         template<<context if context
 
+        if options[:userdata] && !template_obj.nil?
+            if template_obj.has_elements?('TEMPLATE/EC2')
+                template_obj.add_element(
+                    'TEMPLATE/EC2',
+                    'USERDATA' => options[:userdata])
+
+                template << template_obj.template_like_str(
+                    'TEMPLATE', false, 'EC2')
+            end
+        end
+
         [0, template]
     end
 
@@ -1034,4 +1101,105 @@ EOT
         # in options hash
         (template_options-options.keys)!=template_options
     end
+
+    def self.sunstone_url
+        if (one_sunstone = ENV['ONE_SUNSTONE'])
+           one_sunstone
+        elsif (one_xmlrpc = ENV['ONE_XMLRPC'])
+            uri = URI(one_xmlrpc)
+            "#{uri.scheme}://#{uri.host}:9869"
+        else
+            "http://localhost:9869"
+        end
+    end
+
+    def self.download_resource_sunstone(kind, id, path, force)
+        client = OneHelper.client
+        user, password = client.one_auth.split(":", 2)
+
+        # Step 1: Build Session to get Cookie
+        uri = URI(File.join(sunstone_url,"login"))
+
+        req = Net::HTTP::Post.new(uri)
+        req.basic_auth user, password
+
+        begin
+            res = Net::HTTP.start(uri.hostname, uri.port) do |http|
+                http.request(req)
+            end
+        rescue
+            return OpenNebula::Error.new("Error connecting to '#{uri}'.")
+        end
+
+        cookie = res.response['set-cookie'].split('; ')[0]
+
+        if cookie.nil?
+           return OpenNebula::Error.new("Unable to get Cookie. Is OpenNebula running?")
+        end
+
+        # Step 2: Open '/' to get the csrftoken
+        uri = URI(sunstone_url)
+
+        req = Net::HTTP::Get.new(uri)
+        req['Cookie'] = cookie
+
+        begin
+            res = Net::HTTP.start(uri.hostname, uri.port) do |http|
+                http.request(req)
+            end
+        rescue
+            return OpenNebula::Error.new("Error connecting to '#{uri}'.")
+        end
+
+        m = res.body.match(/var csrftoken = '(.*)';/)
+        csrftoken = m[1] rescue nil
+
+        if csrftoken.nil?
+           return OpenNebula::Error.new("Unable to get csrftoken.")
+        end
+
+        # Step 3: Download resource
+        uri = URI(File.join(sunstone_url,
+                            kind.to_s,
+                            id.to_s,
+                            "download?csrftoken=#{csrftoken}"))
+
+        req = Net::HTTP::Get.new(uri)
+
+        req['Cookie'] = cookie
+        req['User-Agent'] = "OpenNebula CLI"
+
+        begin
+            File.open(path, 'wb') do |f|
+                Net::HTTP.start(uri.hostname, uri.port) do |http|
+                    http.request(req) do |res|
+                        res.read_body do |chunk|
+                            f.write(chunk)
+                        end
+                    end
+                end
+            end
+        rescue Errno::EACCES
+            return OpenNebula::Error.new("Target file not writable.")
+        end
+
+        error_message = nil
+
+        File.open(path, 'rb') do |f|
+            begin
+                f.seek(-1024, IO::SEEK_END)
+            rescue Errno::EINVAL
+            end
+
+            tail = f.read
+
+            m = tail.match(/@\^_\^@ (.*) @\^_\^@/m)
+            error_message = m[1] if m
+        end
+
+        if error_message
+            File.unlink(path)
+            return OpenNebula::Error.new("Remote server error: #{error_message}")
+        end
+    end
 end
