@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -119,8 +119,7 @@ module ElasticIP
 
         user_id = retrieve_uid
         eip     = params["PublicIp"]
-        vmid    = params['InstanceId']
-        vmid    = vmid.split('-')[1] if vmid[0]==?i
+        vmid    = params['InstanceId'].sub(/^i\-0*/, '')
 
         unless vnet["TEMPLATE/EC2_ADDRESSES[IP=\"#{eip}\" and UID=\"#{retrieve_uid}\"]/IP"]
             rc = OpenNebula::Error.new("address:#{eip} does not exist")
