@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems              */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems              */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -22,13 +22,12 @@
 const char * Datastore::table = "datastore_pool";
 
 const char * Datastore::db_names =
-        "oid, name, body, uid, gid, owner_u, group_u, other_u, cid";
+        "oid, name, body, uid, gid, owner_u, group_u, other_u";
 
 const char * Datastore::db_bootstrap =
     "CREATE TABLE IF NOT EXISTS datastore_pool ("
     "oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, "
-    "gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, "
-    "cid INTEGER)";
+    "gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER)";
 
 /* ************************************************************************ */
 /* Datastore :: Constructor/Destructor                                      */
@@ -41,19 +40,19 @@ Datastore::Datastore(
         const string&       gname,
         int                 umask,
         DatastoreTemplate*  ds_template,
-        int                 cluster_id,
-        const string&       cluster_name):
+        const set<int>      &cluster_ids):
             PoolObjectSQL(-1,DATASTORE,"",uid,gid,uname,gname,table),
-            ObjectCollection("IMAGES"),
-            Clusterable(cluster_id, cluster_name),
+            Clusterable(cluster_ids),
             ds_mad(""),
             tm_mad(""),
             base_path(""),
             type(IMAGE_DS),
+            disk_type(Image::FILE),
             total_mb(0),
             free_mb(0),
             used_mb(0),
-            state(READY)
+            state(READY),
+            images("IMAGES")
 {
     if (ds_template != 0)
     {
@@ -108,6 +107,7 @@ void Datastore::disk_attribute(
     ostringstream oss;
     string st;
     string inherit_val;
+    string current_val;
 
     vector<string>::const_iterator it;
 
@@ -117,13 +117,9 @@ void Datastore::disk_attribute(
     disk->replace("DATASTORE_ID", oss.str());
     disk->replace("TM_MAD",       get_tm_mad());
 
-    if ( get_cluster_id() != ClusterPool::NONE_CLUSTER_ID )
-    {
-        oss.str("");
-        oss << get_cluster_id();
+    set<int> cluster_ids = get_cluster_ids();
 
-        disk->replace("CLUSTER_ID", oss.str());
-    }
+    disk->replace("CLUSTER_ID", one_util::join(cluster_ids, ','));
 
     get_template_attribute("CLONE_TARGET", st);
 
@@ -141,13 +137,19 @@ void Datastore::disk_attribute(
 
     for (it = inherit_attrs.begin(); it != inherit_attrs.end(); it++)
     {
+        current_val = disk->vector_value((*it).c_str());
         get_template_attribute((*it).c_str(), inherit_val);
 
-        if (!inherit_val.empty())
+        if ( current_val.empty() && !inherit_val.empty() )
         {
             disk->replace(*it, inherit_val);
         }
     }
+
+    if (VirtualMachine::is_volatile(disk))
+    {
+        disk->replace("DISK_TYPE", Image::disk_type_to_str(get_disk_type()));
+    }
 }
 
 /* ************************************************************************ */
@@ -203,29 +205,87 @@ static int check_tm_target_type(string& tm_tt)
 
 /* -------------------------------------------------------------------------- */
 
+int Datastore::set_ds_mad(std::string &mad, std::string &error_str)
+{
+    const VectorAttribute* vatt;
+    std::vector <std::string> vrequired_attrs;
+
+    int    rc;
+    std::string required_attrs, required_attr, value;
+
+    std::ostringstream oss;
+
+    if ( type == SYSTEM_DS ) //No ds_mad for SYSTEM_DS
+    {
+        return 0;
+    }
+
+    rc = Nebula::instance().get_ds_conf_attribute(mad, vatt);
+
+    if ( rc != 0 )
+    {
+        goto error_conf;
+    }
+
+    rc = vatt->vector_value("REQUIRED_ATTRS", required_attrs);
+
+    if ( rc == -1 ) //No required attributes
+    {
+        return 0;
+    }
+
+    vrequired_attrs = one_util::split(required_attrs, ',');
+
+    for ( std::vector<std::string>::const_iterator it = vrequired_attrs.begin();
+         it != vrequired_attrs.end(); it++ )
+    {
+        required_attr = *it;
+
+        required_attr = one_util::trim(required_attr);
+        one_util::toupper(required_attr);
+
+        get_template_attribute(required_attr.c_str(), value);
+
+        if ( value.empty() )
+        {
+            goto error_required;
+        }
+    }
+
+    return 0;
+
+error_conf:
+    oss << "DS_MAD named \"" << mad << "\" is not defined in oned.conf";
+    goto error_common;
+
+error_required:
+    oss << "Datastore template is missing the \"" << required_attr
+        << "\" attribute or it's empty.";
+
+error_common:
+    error_str = oss.str();
+    return -1;
+}
+
+/* -------------------------------------------------------------------------- */
+
 int Datastore::set_tm_mad(string &tm_mad, string &error_str)
 {
     const VectorAttribute* vatt;
 
-    int    rc;
     string st;
 
     ostringstream oss;
 
-    rc = Nebula::instance().get_tm_conf_attribute(tm_mad, vatt);
-
-    if (rc != 0)
+    if ( Nebula::instance().get_tm_conf_attribute(tm_mad, vatt) != 0 )
     {
-        oss << "TM_MAD named \"" << tm_mad << "\" is not defined in oned.conf";
-
-        error_str = oss.str();
-
-        return -1;
+        goto error_conf;
     }
 
     if (type == SYSTEM_DS)
     {
         bool shared_type;
+        bool ds_migrate;
 
         if (vatt->vector_value("SHARED", shared_type) == -1)
         {
@@ -241,6 +301,20 @@ int Datastore::set_tm_mad(string &tm_mad, string &error_str)
             replace_template_attribute("SHARED", "NO");
         }
 
+        if (vatt->vector_value("DS_MIGRATE", ds_migrate) == -1)
+        {
+            ds_migrate = true;
+        }
+
+        if (ds_migrate)
+        {
+            replace_template_attribute("DS_MIGRATE", "YES");
+        }
+        else
+        {
+            replace_template_attribute("DS_MIGRATE", "NO");
+        }
+
         remove_template_attribute("LN_TARGET");
         remove_template_attribute("CLONE_TARGET");
     }
@@ -269,22 +343,112 @@ int Datastore::set_tm_mad(string &tm_mad, string &error_str)
 
     return 0;
 
+error_conf:
+    oss << "TM_MAD named \"" << tm_mad << "\" is not defined in oned.conf";
+    goto error_common;
+
 error:
     oss << "Attribute shared, ln_target or clone_target in TM_MAD_CONF for "
         << tm_mad << " is missing or has wrong value in oned.conf";
 
+error_common:
     error_str = oss.str();
-
     return -1;
 }
 
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int Datastore::set_ds_disk_type(string& s_dt, string& error)
+{
+    if (s_dt.empty())
+    {
+       disk_type = Image::FILE;
+    }
+    else
+    {
+       disk_type = Image::str_to_disk_type(s_dt);
+    }
+
+    switch(type)
+    {
+        case IMAGE_DS:
+            switch(disk_type)
+            {
+                //Valid disk types for Image DS
+                case Image::FILE:
+                case Image::BLOCK:
+                case Image::ISCSI:
+                case Image::RBD:
+                case Image::GLUSTER:
+                case Image::SHEEPDOG:
+                    break;
+
+                case Image::CD_ROM:
+                case Image::RBD_CDROM:
+                case Image::SHEEPDOG_CDROM:
+                case Image::GLUSTER_CDROM:
+                    error = "Invalid DISK_TYPE for an Image Datastore.";
+                    return -1;
+
+                case Image::NONE:
+                    error = "Unknown DISK_TYPE in template.";
+                    return -1;
+            }
+            break;
+
+        case SYSTEM_DS:
+            switch(disk_type)
+            {
+                //Valid disk types for System DS
+                case Image::FILE:
+                case Image::RBD:
+                    break;
+
+                case Image::GLUSTER:
+                case Image::SHEEPDOG:
+                case Image::BLOCK:
+                case Image::ISCSI:
+                case Image::CD_ROM:
+                case Image::RBD_CDROM:
+                case Image::SHEEPDOG_CDROM:
+                case Image::GLUSTER_CDROM:
+                    error = "Invalid DISK_TYPE for a System Datastore.";
+                    return -1;
+
+                case Image::NONE:
+                    error = "Unknown DISK_TYPE in template.";
+                    return -1;
+            }
+            break;
+
+        case FILE_DS:
+            disk_type = Image::FILE;
+            break;
+    }
+
+    switch(type)
+    {
+        case IMAGE_DS:
+        case SYSTEM_DS:
+            add_template_attribute("DISK_TYPE", Image::disk_type_to_str(disk_type));
+            break;
+        case FILE_DS:
+            break;
+    }
+
+    return 0;
+}
+
+/* -------------------------------------------------------------------------- */
+
 int Datastore::insert(SqlDB *db, string& error_str)
 {
-    int           rc;
+    string s_disk_type;
+    string s_ds_type;
+    string datastore_location;
+
     ostringstream oss;
-    string        s_disk_type;
-    string        s_ds_type;
-    string        datastore_location;
 
     // -------------------------------------------------------------------------
     // Check default datastore attributes
@@ -311,6 +475,11 @@ int Datastore::insert(SqlDB *db, string& error_str)
         goto error_ds;
     }
 
+    if (set_ds_mad(ds_mad, error_str) != 0)
+    {
+        goto error_common;
+    }
+
     get_template_attribute("TM_MAD", tm_mad);
 
     if ( tm_mad.empty() == true )
@@ -323,64 +492,35 @@ int Datastore::insert(SqlDB *db, string& error_str)
         goto error_common;
     }
 
-    erase_template_attribute("BASE_PATH", base_path);
+    remove_template_attribute("BASE_PATH");
 
-    if (base_path.empty() == true )
-    {
-        Nebula& nd = Nebula::instance();
-        nd.get_configuration_attribute("DATASTORE_BASE_PATH", base_path);
-    }
+    Nebula::instance().get_ds_location(base_path);
 
     if ( base_path.at(base_path.size()-1) != '/' )
     {
         base_path += "/";
     }
 
-    add_template_attribute("BASE_PATH", base_path);
-
     oss << base_path << oid;
 
     base_path = oss.str();
 
     erase_template_attribute("DISK_TYPE", s_disk_type);
 
-    disk_type = Image::FILE;
-
-    if ( type == IMAGE_DS )
+    if (set_ds_disk_type(s_disk_type, error_str) == -1)
     {
-        if (!s_disk_type.empty())
-        {
-            disk_type = Image::str_to_disk_type(s_disk_type);
-
-            switch(disk_type)
-            {
-                case Image::NONE:
-                    goto error_disk_type;
-                    break;
-                case Image::RBD_CDROM:
-                case Image::SHEEPDOG_CDROM:
-                case Image::GLUSTER_CDROM:
-                    goto error_invalid_disk_type;
-                    break;
-                default:
-                    break;
-            }
-        }
-
-        add_template_attribute("DISK_TYPE", Image::disk_type_to_str(disk_type));
+        goto error_common;
     }
 
     if ( tm_mad.empty() == true )
     {
         goto error_empty_tm;
     }
+
     //--------------------------------------------------------------------------
     // Insert the Datastore
     //--------------------------------------------------------------------------
-
-    rc = insert_replace(db, false, error_str);
-
-    return rc;
+    return insert_replace(db, false, error_str);
 
 error_exclusive:
     error_str = "SYSTEM datastores cannot have DS_MAD defined.";
@@ -394,14 +534,6 @@ error_empty_tm:
     error_str = "No TM_MAD in template.";
     goto error_common;
 
-error_disk_type:
-    error_str = "Unknown DISK_TYPE in template.";
-    goto error_common;
-
-error_invalid_disk_type:
-    error_str = "Invalid DISK_TYPE in template.";
-    goto error_common;
-
 error_common:
     NebulaLog::log("DATASTORE", Log::ERROR, error_str);
     return -1;
@@ -460,9 +592,7 @@ int Datastore::insert_replace(SqlDB *db, bool replace, string& error_str)
         <<          gid                 << ","
         <<          owner_u             << ","
         <<          group_u             << ","
-        <<          other_u             << ","
-        <<          cluster_id          << ")";
-
+        <<          other_u             << ")";
 
     rc = db->exec(oss);
 
@@ -498,34 +628,32 @@ error_common:
 string& Datastore::to_xml(string& xml) const
 {
     ostringstream   oss;
-    string          collection_xml;
+    string          clusters_xml;
+    string          images_xml;
     string          template_xml;
     string          perms_xml;
 
-    ObjectCollection::to_xml(collection_xml);
-
     oss <<
     "<DATASTORE>"               <<
-        "<ID>"                  << oid          << "</ID>"          <<
-        "<UID>"                 << uid          << "</UID>"         <<
-        "<GID>"                 << gid          << "</GID>"         <<
-        "<UNAME>"               << uname        << "</UNAME>"       <<
-        "<GNAME>"               << gname        << "</GNAME>"       <<
-        "<NAME>"                << name         << "</NAME>"        <<
+        "<ID>"                  << oid          << "</ID>"        <<
+        "<UID>"                 << uid          << "</UID>"       <<
+        "<GID>"                 << gid          << "</GID>"       <<
+        "<UNAME>"               << uname        << "</UNAME>"     <<
+        "<GNAME>"               << gname        << "</GNAME>"     <<
+        "<NAME>"                << name         << "</NAME>"      <<
         perms_to_xml(perms_xml) <<
-        "<DS_MAD><![CDATA["     << ds_mad       << "]]></DS_MAD>"   <<
-        "<TM_MAD><![CDATA["     << tm_mad       << "]]></TM_MAD>"   <<
-        "<BASE_PATH><![CDATA["  << base_path    << "]]></BASE_PATH>"<<
-        "<TYPE>"                << type         << "</TYPE>"        <<
-        "<DISK_TYPE>"           << disk_type    << "</DISK_TYPE>"   <<
-        "<STATE>"               << state        << "</STATE>"       <<
-        "<CLUSTER_ID>"          << cluster_id   << "</CLUSTER_ID>"  <<
-        "<CLUSTER>"             << cluster      << "</CLUSTER>"     <<
-        "<TOTAL_MB>"            << total_mb     << "</TOTAL_MB>"    <<
-        "<FREE_MB>"             << free_mb      << "</FREE_MB>"     <<
-        "<USED_MB>"             << used_mb      << "</USED_MB>"     <<
-        collection_xml          <<
-        obj_template->to_xml(template_xml)                  <<
+        "<DS_MAD>"    << one_util::escape_xml(ds_mad)    << "</DS_MAD>"    <<
+        "<TM_MAD>"    << one_util::escape_xml(tm_mad)    << "</TM_MAD>"    <<
+        "<BASE_PATH>" << one_util::escape_xml(base_path) << "</BASE_PATH>" <<
+        "<TYPE>"                << type         << "</TYPE>"      <<
+        "<DISK_TYPE>"           << disk_type    << "</DISK_TYPE>" <<
+        "<STATE>"               << state        << "</STATE>"     <<
+        Clusterable::to_xml(clusters_xml)       <<
+        "<TOTAL_MB>"            << total_mb     << "</TOTAL_MB>"  <<
+        "<FREE_MB>"             << free_mb      << "</FREE_MB>"   <<
+        "<USED_MB>"             << used_mb      << "</USED_MB>"   <<
+        images.to_xml(images_xml)               <<
+        obj_template->to_xml(template_xml)      <<
     "</DATASTORE>";
 
     xml = oss.str();
@@ -544,6 +672,8 @@ int Datastore::from_xml(const string& xml)
     int int_state;
     vector<xmlNodePtr> content;
 
+    ostringstream oss;
+
     // Initialize the internal XML object
     update_from_str(xml);
 
@@ -556,17 +686,13 @@ int Datastore::from_xml(const string& xml)
     rc += xpath(name,         "/DATASTORE/NAME",      "not_found");
     rc += xpath(ds_mad,       "/DATASTORE/DS_MAD",    "not_found");
     rc += xpath(tm_mad,       "/DATASTORE/TM_MAD",    "not_found");
-    rc += xpath(base_path,    "/DATASTORE/BASE_PATH", "not_found");
     rc += xpath(int_ds_type,  "/DATASTORE/TYPE",      -1);
     rc += xpath(int_disk_type,"/DATASTORE/DISK_TYPE", -1);
     rc += xpath(int_state,    "/DATASTORE/STATE",     0);
 
-    rc += xpath(cluster_id, "/DATASTORE/CLUSTER_ID", -1);
-    rc += xpath(cluster,    "/DATASTORE/CLUSTER",    "not_found");
-
-    rc += xpath(total_mb, "/DATASTORE/TOTAL_MB", 0);
-    rc += xpath(free_mb,  "/DATASTORE/FREE_MB",  0);
-    rc += xpath(used_mb,  "/DATASTORE/USED_MB",  0);
+    rc += xpath<long long>(total_mb,"/DATASTORE/TOTAL_MB",0);
+    rc += xpath<long long>(free_mb, "/DATASTORE/FREE_MB", 0);
+    rc += xpath<long long>(used_mb, "/DATASTORE/USED_MB", 0);
 
     // Permissions
     rc += perms_from_xml();
@@ -575,19 +701,11 @@ int Datastore::from_xml(const string& xml)
     type      = static_cast<Datastore::DatastoreType>(int_ds_type);
     state     = static_cast<DatastoreState>(int_state);
 
-    // Get associated classes
-    ObjectXML::get_nodes("/DATASTORE/IMAGES", content);
+    // Set of Image IDs
+    rc += images.from_xml(this, "/DATASTORE/");
 
-    if (content.empty())
-    {
-        return -1;
-    }
-
-    // Set of IDs
-    rc += ObjectCollection::from_xml_node(content[0]);
-
-    ObjectXML::free_nodes(content);
-    content.clear();
+    // Set of cluster IDs
+    rc += Clusterable::from_xml(this, "/DATASTORE/");
 
     // Get associated classes
     ObjectXML::get_nodes("/DATASTORE/TEMPLATE", content);
@@ -601,6 +719,17 @@ int Datastore::from_xml(const string& xml)
 
     ObjectXML::free_nodes(content);
 
+    Nebula::instance().get_ds_location(base_path);
+
+    if ( base_path.at(base_path.size()-1) != '/' )
+    {
+        base_path += "/";
+    }
+
+    oss << base_path << oid;
+
+    base_path = oss.str();
+
     if (rc != 0)
     {
         return -1;
@@ -617,20 +746,20 @@ int Datastore::post_update_template(string& error_str)
     string new_ds_mad;
     string new_tm_mad;
     string s_ds_type;
-    string new_disk_type_st;
+    string new_disk_type;
     string new_base_path;
 
-    Image::DiskType new_disk_type;
-
-    DatastoreType old_ds_type;
     DatastoreType new_ds_type;
 
+    DatastoreType   old_ds_type   = type;
+    Image::DiskType old_disk_type = disk_type;
+    string          old_tm_mad    = tm_mad;
+    string          old_ds_mad    = ds_mad;
+
     /* ---------------------------------------------------------------------- */
     /* Set the TYPE of the Datastore (class & template)                       */
     /* ---------------------------------------------------------------------- */
 
-    old_ds_type = type;
-
     get_template_attribute("TYPE", s_ds_type);
 
     if (!s_ds_type.empty())
@@ -657,6 +786,24 @@ int Datastore::post_update_template(string& error_str)
 
     replace_template_attribute("TYPE", type_to_str(type));
 
+    /* ---------------------------------------------------------------------- */
+    /* Set the DISK_TYPE (class & template)                                   */
+    /* ---------------------------------------------------------------------- */
+
+    erase_template_attribute("DISK_TYPE", new_disk_type);
+
+    if (!new_disk_type.empty())
+    {
+        if ( set_ds_disk_type(new_disk_type, error_str) == -1 )
+        {
+            //Rollback variable changes
+            type      = old_ds_type;
+            disk_type = old_disk_type;
+
+            return -1;
+        }
+    }
+
     /* ---------------------------------------------------------------------- */
     /* Set the TM_MAD of the Datastore (class & template)                     */
     /* ---------------------------------------------------------------------- */
@@ -673,7 +820,8 @@ int Datastore::post_update_template(string& error_str)
 
         if (set_tm_mad(new_tm_mad, error_str) != 0)
         {
-            type = old_ds_type;
+            type      = old_ds_type;
+            disk_type = old_disk_type;
 
             return -1;
         }
@@ -685,31 +833,6 @@ int Datastore::post_update_template(string& error_str)
         replace_template_attribute("TM_MAD", tm_mad);
     }
 
-    /* ---------------------------------------------------------------------- */
-    /* Set the DISK_TYPE (class & template)                                   */
-    /* ---------------------------------------------------------------------- */
-
-    erase_template_attribute("DISK_TYPE", new_disk_type_st);
-
-    if ( type == IMAGE_DS )
-    {
-        if (!new_disk_type_st.empty())
-        {
-            new_disk_type = Image::str_to_disk_type(new_disk_type_st);
-
-            if (new_disk_type != Image::NONE)
-            {
-                disk_type = new_disk_type;
-            }
-        }
-
-        add_template_attribute("DISK_TYPE", Image::disk_type_to_str(disk_type));
-    }
-    else
-    {
-        disk_type = Image::FILE;
-    }
-
     /* ---------------------------------------------------------------------- */
     /* Set the DS_MAD of the Datastore (class & template)                     */
     /* ---------------------------------------------------------------------- */
@@ -738,25 +861,17 @@ int Datastore::post_update_template(string& error_str)
     }
 
     /* ---------------------------------------------------------------------- */
-    /* Set the BASE_PATH of the Datastore (class & template)                  */
+    /* Verify that the template has the required attributees                  */
     /* ---------------------------------------------------------------------- */
 
-    erase_template_attribute("BASE_PATH", new_base_path);
-
-    if ( !new_base_path.empty())
+    if ( set_ds_mad(ds_mad, error_str) !=  0 )
     {
-        ostringstream oss;
+        type      = old_ds_type;
+        disk_type = old_disk_type;
+        tm_mad    = old_tm_mad;
+        ds_mad    = old_ds_mad;
 
-        if ( new_base_path.at(new_base_path.size()-1) != '/' )
-        {
-            new_base_path += "/";
-        }
-
-        add_template_attribute("BASE_PATH", new_base_path);
-
-        oss << new_base_path << oid;
-
-        base_path = oss.str();
+        return -1;
     }
 
     return 0;
@@ -797,3 +912,27 @@ bool Datastore::get_avail_mb(long long &avail)
 
     return check;
 }
+
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+bool Datastore::is_persistent_only()
+{
+    int rc;
+    bool persistent_only = false;
+
+    const VectorAttribute* vatt;
+
+    rc = Nebula::instance().get_ds_conf_attribute(ds_mad, vatt);
+
+    if ( rc != 0 )
+    {
+        // No DS_MAD_CONF is available for this DS_MAD.
+        // Assuming this DS is not PERSISTENT_ONLY
+        return false;
+    }
+
+    vatt->vector_value("PERSISTENT_ONLY", persistent_only);
+
+    return persistent_only;
+};
