@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -21,6 +21,7 @@
 #include "TransferManager.h"
 #include "ImageManager.h"
 #include "Quotas.h"
+#include "Request.h"
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
@@ -64,7 +65,8 @@ int DispatchManager::deploy (
 error:
 
     oss.str("");
-    oss << "Could not deploy VM " << vid << ", wrong state.";
+    oss << "Could not deploy VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     vm->unlock();
@@ -78,6 +80,7 @@ int DispatchManager::import (
     VirtualMachine *    vm)
 {
     ostringstream oss;
+    string import_state;
 
     if ( vm == 0 )
     {
@@ -92,15 +95,24 @@ int DispatchManager::import (
 
     time_t the_time = time(0);
     int    cpu, mem, disk;
-    vector<Attribute *> pci;
+    vector<VectorAttribute *> pci;
 
     vm->get_requirements(cpu, mem, disk, pci);
 
     hpool->add_capacity(vm->get_hid(), vm->get_oid(), cpu, mem, disk, pci);
 
-    vm->set_state(VirtualMachine::ACTIVE);
+    import_state = vm->get_import_state();
 
-    vm->set_state(VirtualMachine::RUNNING);
+    if(import_state == "POWEROFF")
+    {
+        vm->set_state(VirtualMachine::POWEROFF);
+        vm->set_state(VirtualMachine::LCM_INIT);
+    }
+    else
+    {
+        vm->set_state(VirtualMachine::ACTIVE);
+        vm->set_state(VirtualMachine::RUNNING);
+    }
 
     vmpool->update(vm);
 
@@ -158,7 +170,8 @@ int DispatchManager::migrate(
 
 error:
     oss.str("");
-    oss << "Could not migrate VM " << vid << ", wrong state.";
+    oss << "Could not migrate VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     vm->unlock();
@@ -200,7 +213,8 @@ int DispatchManager::live_migrate(
 
 error:
     oss.str("");
-    oss << "Could not live-migrate VM " << vid << ", wrong state.";
+    oss << "Could not live-migrate VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     vm->unlock();
@@ -210,11 +224,42 @@ error:
 /* ************************************************************************** */
 /* ************************************************************************** */
 
-int DispatchManager::shutdown (
+void DispatchManager::free_vm_resources(VirtualMachine * vm)
+{
+    Template *    tmpl;
+
+    int uid;
+    int gid;
+
+    vm->release_network_leases();
+    vm->release_disk_images();
+
+    vm->set_exit_time(time(0));
+
+    vm->set_state(VirtualMachine::LCM_INIT);
+    vm->set_state(VirtualMachine::DONE);
+    vmpool->update(vm);
+
+    uid  = vm->get_uid();
+    gid  = vm->get_gid();
+    tmpl = vm->clone_template();
+
+    vm->unlock();
+
+    Quotas::vm_del(uid, gid, tmpl);
+
+    delete tmpl;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int DispatchManager::terminate(
         int     vid,
         bool    hard,
         string& error_str)
 {
+    int rc = 0;
     ostringstream oss;
 
     VirtualMachine * vm = vmpool->get(vid,true);
@@ -224,44 +269,63 @@ int DispatchManager::shutdown (
         return -1;
     }
 
-    oss << "Shutting down VM " << vid;
+    oss << "Terminating VM " << vid;
     NebulaLog::log("DiM",Log::DEBUG,oss);
 
-    if (vm->get_state()     == VirtualMachine::POWEROFF ||
-        vm->get_state()     == VirtualMachine::SUSPENDED ||
-       (vm->get_state()     == VirtualMachine::ACTIVE &&
-        (vm->get_lcm_state() == VirtualMachine::RUNNING ||
-         vm->get_lcm_state() == VirtualMachine::UNKNOWN)))
+    switch (vm->get_state())
     {
-        if (hard)
-        {
-            lcm->trigger(LifeCycleManager::CANCEL,vid);
-        }
-        else
-        {
-            lcm->trigger(LifeCycleManager::SHUTDOWN,vid);
-        }
-    }
-    else
-    {
-        goto error;
+        //Hard has no effect, VM is not RUNNING
+        case VirtualMachine::SUSPENDED:
+        case VirtualMachine::POWEROFF:
+        case VirtualMachine::STOPPED:
+        case VirtualMachine::UNDEPLOYED:
+            lcm->trigger(LifeCycleManager::SHUTDOWN, vid);
+            vm->unlock();
+            break;
+
+        case VirtualMachine::INIT:
+        case VirtualMachine::PENDING:
+        case VirtualMachine::HOLD:
+        case VirtualMachine::CLONING:
+        case VirtualMachine::CLONING_FAILURE:
+            free_vm_resources(vm);
+            break;
+
+        case VirtualMachine::DONE:
+            vm->unlock();
+            break;
+
+        case VirtualMachine::ACTIVE:
+            switch (vm->get_lcm_state())
+            {
+                case VirtualMachine::RUNNING:
+                case VirtualMachine::UNKNOWN:
+                    if (hard)
+                    {
+                        lcm->trigger(LifeCycleManager::CANCEL,vid);
+                    }
+                    else
+                    {
+                        lcm->trigger(LifeCycleManager::SHUTDOWN,vid);
+                    }
+                    break;
+
+                default:
+                    oss.str("");
+                    oss << "Could not terminate VM " << vid
+                        << ", wrong state " << vm->state_str() << ".";
+
+                    NebulaLog::log("DiM",Log::ERROR,oss);
+                    error_str = oss.str();
+
+                    rc = -2;
+                    break;
+            }
+            vm->unlock();
+            break;
     }
 
-    vm->unlock();
-
-    return 0;
-
-error:
-    oss.str("");
-    oss << "Could not shutdown VM " << vid << ", wrong state.";
-    NebulaLog::log("DiM",Log::ERROR,oss);
-
-    oss.str("");
-    oss << "This action is not available for state " << vm->state_str();
-    error_str = oss.str();
-
-    vm->unlock();
-    return -2;
+    return rc;
 }
 
 /* -------------------------------------------------------------------------- */
@@ -309,7 +373,8 @@ int DispatchManager::undeploy(
 
 error:
     oss.str("");
-    oss << "Could not undeploy VM " << vid << ", wrong state.";
+    oss << "Could not undeploy VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -365,7 +430,8 @@ int DispatchManager::poweroff (
 error:
 
     oss.str("");
-    oss << "Could not power off VM " << vid << ", wrong state.";
+    oss << "Could not power off VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -413,7 +479,8 @@ int DispatchManager::hold(
 error:
 
     oss.str("");
-    oss << "Could not hold VM " << vid << ", wrong state.";
+    oss << "Could not hold VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -460,7 +527,8 @@ int DispatchManager::release(
 
 error:
     oss.str("");
-    oss << "Could not release VM " << vid << ", wrong state.";
+    oss << "Could not release VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -507,7 +575,8 @@ int DispatchManager::stop(
 
 error:
     oss.str("");
-    oss << "Could not stop VM " << vid << ", wrong state.";
+    oss << "Could not stop VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -553,7 +622,8 @@ int DispatchManager::suspend(
 
 error:
     oss.str("");
-    oss << "Could not suspend VM " << vid << ", wrong state.";
+    oss << "Could not suspend VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -611,7 +681,8 @@ int DispatchManager::resume(
 
 error:
     oss.str("");
-    oss << "Could not resume VM " << vid << ", wrong state.";
+    oss << "Could not resume VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -669,7 +740,8 @@ int DispatchManager::reboot(
 
 error:
     oss.str("");
-    oss << "Could not reboot VM " << vid << ", wrong state.";
+    oss << "Could not reboot VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -719,7 +791,8 @@ int DispatchManager::resched(
 
 error:
     oss.str("");
-    oss << "Could not set rescheduling flag for VM " << vid << ", wrong state.";
+    oss << "Could not set rescheduling flag for VM " << vid
+        << ", wrong state " << vm->state_str() << ".";
     NebulaLog::log("DiM",Log::ERROR,oss);
 
     oss.str("");
@@ -734,72 +807,102 @@ error:
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void DispatchManager::finalize_cleanup(VirtualMachine * vm)
+int DispatchManager::recover(VirtualMachine * vm, bool success, string& error)
 {
-    Template *    tmpl;
+    int rc = 0;
 
-    int uid;
-    int gid;
+    switch (vm->get_state())
+    {
+        case VirtualMachine::CLONING_FAILURE:
+            if (success)
+            {
+                lcm->trigger(LifeCycleManager::DISK_LOCK_SUCCESS,vm->get_oid());
+            }
+            else
+            {
+                lcm->trigger(LifeCycleManager::DISK_LOCK_FAILURE,vm->get_oid());
+            }
+            break;
 
-    vm->release_network_leases();
-    vm->release_disk_images();
+        case VirtualMachine::ACTIVE:
+            lcm->recover(vm, success);
+            break;
 
-    vm->set_exit_time(time(0));
+        default:
+            rc    = -1;
 
-    vm->set_state(VirtualMachine::LCM_INIT);
-    vm->set_state(VirtualMachine::DONE);
-    vmpool->update(vm);
+            ostringstream oss;
+            oss << "Could not perform a recover operation on VM " << vm->get_oid()
+                << ", wrong state " << vm->state_str() << ".";
+            error = oss.str();
 
-    uid  = vm->get_uid();
-    gid  = vm->get_gid();
-    tmpl = vm->clone_template();
+            break;
+    }
 
     vm->unlock();
 
-    Quotas::vm_del(uid, gid, tmpl);
-
-    delete tmpl;
+    return rc;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int DispatchManager::finalize(
-    int     vid,
-    string& error_str)
+int DispatchManager::retry(VirtualMachine * vm, string& error)
+{
+    int rc = 0;
+
+    switch (vm->get_state())
+    {
+        case VirtualMachine::ACTIVE:
+            lcm->retry(vm);
+            break;
+
+        default:
+            rc    = -1;
+
+            ostringstream oss;
+            oss << "Could not perform a retry on VM " << vm->get_oid()
+                << ", wrong state " << vm->state_str() << ".";
+            error = oss.str();
+
+            break;
+    }
+
+    vm->unlock();
+
+    return rc;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int DispatchManager::delete_vm(VirtualMachine * vm, string& error)
 {
-    VirtualMachine * vm;
-    Host * host;
     ostringstream oss;
 
-    vector<Attribute *> pci;
+    int cpu, mem, disk;
+    vector<VectorAttribute *> pci;
 
-    VirtualMachine::VmState state;
     bool is_public_host = false;
     int  host_id = -1;
 
-    vm = vmpool->get(vid,true);
-
-    if ( vm == 0 )
-    {
-        return -1;
-    }
-
     if(vm->hasHistory())
     {
         host_id = vm->get_hid();
     }
 
-    vm->unlock();
+    int vid = vm->get_oid();
 
     if(host_id != -1)
     {
-        host = hpool->get(host_id,true);
+        Host * host = hpool->get(host_id,true);
 
         if ( host == 0 )
         {
             oss << "Error getting host " << host_id;
-            error_str = oss.str();
+            error = oss.str();
+
+            vm->unlock();
 
             return -1;
         }
@@ -809,56 +912,53 @@ int DispatchManager::finalize(
         host->unlock();
     }
 
-    vm = vmpool->get(vid,true);
-
-    state = vm->get_state();
-
-    oss << "Finalizing VM " << vid;
+    oss << "Deleting VM " << vm->get_oid();
     NebulaLog::log("DiM",Log::DEBUG,oss);
 
-    switch (state)
+    switch (vm->get_state())
     {
         case VirtualMachine::SUSPENDED:
         case VirtualMachine::POWEROFF:
-            int cpu, mem, disk;
-
             vm->get_requirements(cpu, mem, disk, pci);
-            hpool->del_capacity(vm->get_hid(),vm->get_oid(),cpu,mem,disk,pci);
+
+            hpool->del_capacity(vm->get_hid(), vid, cpu, mem, disk, pci);
 
             if (is_public_host)
             {
-                vmm->trigger(VirtualMachineManager::CLEANUP,vid);
+                vmm->trigger(VirtualMachineManager::CLEANUP, vid);
             }
             else
             {
-                tm->trigger(TransferManager::EPILOG_DELETE,vid);
+                tm->trigger(TransferManager::EPILOG_DELETE, vid);
             }
 
-            finalize_cleanup(vm);
+            free_vm_resources(vm);
         break;
 
         case VirtualMachine::STOPPED:
         case VirtualMachine::UNDEPLOYED:
             if (is_public_host)
             {
-                vmm->trigger(VirtualMachineManager::CLEANUP,vid);
+                vmm->trigger(VirtualMachineManager::CLEANUP, vid);
             }
             else
             {
-                tm->trigger(TransferManager::EPILOG_DELETE,vid);
+                tm->trigger(TransferManager::EPILOG_DELETE, vid);
             }
 
-            finalize_cleanup(vm);
+            free_vm_resources(vm);
         break;
 
         case VirtualMachine::INIT:
         case VirtualMachine::PENDING:
         case VirtualMachine::HOLD:
-            finalize_cleanup(vm);
+        case VirtualMachine::CLONING:
+        case VirtualMachine::CLONING_FAILURE:
+            free_vm_resources(vm);
         break;
 
         case VirtualMachine::ACTIVE:
-            lcm->trigger(LifeCycleManager::DELETE,vid);
+            lcm->trigger(LifeCycleManager::DELETE, vid);
             vm->unlock();
         break;
 
@@ -873,42 +973,35 @@ int DispatchManager::finalize(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int DispatchManager::resubmit(
-    int     vid,
-    string& error_str)
+int DispatchManager::delete_recreate(VirtualMachine * vm, string& error)
 {
-    VirtualMachine * vm;
-    ostringstream    oss;
-    int              rc = 0;
+    ostringstream oss;
+
+    int rc = 0;
 
     Template *           vm_quotas = 0;
     map<int, Template *> ds_quotas;
 
     int vm_uid, vm_gid;
 
-    vm = vmpool->get(vid,true);
-
-    if ( vm == 0 )
-    {
-        return -1;
-    }
-
     switch (vm->get_state())
     {
         case VirtualMachine::POWEROFF:
-            error_str = "Cannot delete-recreate a powered off VM. Resume it first";
-            NebulaLog::log("DiM",Log::ERROR,error_str);
-            rc = -2;
+            error = "Cannot delete-recreate a powered off VM. Resume it first";
+            NebulaLog::log("DiM", Log::ERROR, error);
+            rc = -1;
         break;
 
         case VirtualMachine::SUSPENDED:
-            error_str = "Cannot delete-recreate a suspended VM. Resume it first";
-            NebulaLog::log("DiM",Log::ERROR,error_str);
-            rc = -2;
+            error = "Cannot delete-recreate a suspended VM. Resume it first";
+            NebulaLog::log("DiM", Log::ERROR, error);
+            rc = -1;
         break;
 
         case VirtualMachine::INIT:
         case VirtualMachine::PENDING:
+        case VirtualMachine::CLONING:
+        case VirtualMachine::CLONING_FAILURE:
         break;
 
         case VirtualMachine::STOPPED:
@@ -932,13 +1025,13 @@ int DispatchManager::resubmit(
         break;
 
         case VirtualMachine::ACTIVE: //Cleanup VM resources before PENDING
-            lcm->trigger(LifeCycleManager::CLEAN, vid);
+            lcm->trigger(LifeCycleManager::DELETE_RECREATE, vm->get_oid());
         break;
 
         case VirtualMachine::DONE:
-            error_str = "Cannot delete-recreate a VM already in DONE state";
-            NebulaLog::log("DiM",Log::ERROR,error_str);
-            rc = -2;
+            error = "Cannot delete-recreate a VM already in DONE state";
+            NebulaLog::log("DiM", Log::ERROR, error);
+            rc = -1;
         break;
     }
 
@@ -962,31 +1055,19 @@ int DispatchManager::resubmit(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int DispatchManager::attach(int vid,
-                            VirtualMachineTemplate * tmpl,
-                            string &                 error_str)
+int DispatchManager::attach(int vid, VirtualMachineTemplate * tmpl, string & err)
 {
     ostringstream oss;
 
-    int max_disk_id;
-    int uid;
-    int oid;
-    int image_id;
-    int image_cluster_id;
-
-    set<string>       used_targets;
-    VectorAttribute * disk;
-    Snapshots *       snap;
-
     VirtualMachine * vm = vmpool->get(vid, true);
 
     if ( vm == 0 )
     {
         oss << "Could not attach a new disk to VM " << vid
             << ", VM does not exist" ;
-        error_str = oss.str();
+        err = oss.str();
 
-        NebulaLog::log("DiM", Log::ERROR, error_str);
+        NebulaLog::log("DiM", Log::ERROR, err);
 
         return -1;
     }
@@ -1005,52 +1086,17 @@ int DispatchManager::attach(int vid,
     {
         oss << "Could not attach a new disk to VM " << vid << ", wrong state "
             << vm->state_str() << ".";
-        error_str = oss.str();
+        err = oss.str();
 
-        NebulaLog::log("DiM", Log::ERROR, error_str);
+        NebulaLog::log("DiM", Log::ERROR, err);
 
         vm->unlock();
         return -1;
     }
 
-    vm->get_disk_info(max_disk_id, used_targets);
-
     vm->set_resched(false);
 
-    uid = vm->get_uid();
-    oid = vm->get_oid();
-
-    vmpool->update(vm);
-
-    vm->unlock();
-
-    disk = VirtualMachine::set_up_attach_disk(oid,
-                                              tmpl,
-                                              used_targets,
-                                              max_disk_id,
-                                              uid,
-                                              image_id,
-                                              &snap,
-                                              error_str);
-    vm = vmpool->get(vid, true);
-
-    if ( vm == 0 )
-    {
-		if ( disk != 0 )
-		{
-			imagem->release_image(oid, image_id, false);
-
-			delete snap;
-			delete disk;
-		}
-
-        error_str = "VM does not exist after setting its state to HOTPLUG.";
-
-        NebulaLog::log("DiM", Log::ERROR, error_str);
-        return -1;
-    }
-
-    if ( disk == 0 )
+    if ( vm->set_up_attach_disk(tmpl, err) != 0 )
     {
         if ( vm->get_lcm_state() == VirtualMachine::HOTPLUG )
         {
@@ -1066,47 +1112,10 @@ int DispatchManager::attach(int vid,
 
         vm->unlock();
 
-        NebulaLog::log("DiM", Log::ERROR, error_str);
+        NebulaLog::log("DiM", Log::ERROR, err);
         return -1;
     }
 
-    // Check that we don't have a cluster incompatibility.
-    if (disk->vector_value("CLUSTER_ID", image_cluster_id) == 0)
-    {
-      if (vm->get_cid() != image_cluster_id)
-      {
-          imagem->release_image(oid, image_id, false);
-
-          delete snap;
-          delete disk;
-
-          if ( vm->get_lcm_state() == VirtualMachine::HOTPLUG )
-          {
-              vm->set_state(VirtualMachine::RUNNING);
-          }
-          else
-          {
-              vm->set_state(VirtualMachine::LCM_INIT);
-              vm->set_state(VirtualMachine::POWEROFF);
-          }
-
-          vmpool->update(vm);
-
-          vm->unlock();
-
-          error_str = "Could not attach disk because of cluster incompatibility.";
-
-          NebulaLog::log("DiM", Log::ERROR, error_str);
-          return -1;
-      }
-    }
-
-    // Set the VM info in the history before the disk is attached to the
-    // VM template
-    vm->set_vm_info();
-
-    vm->set_attach_disk(disk, snap);
-
     if ( vm->get_lcm_state() == VirtualMachine::HOTPLUG )
     {
         time_t the_time = time(0);
@@ -1439,16 +1448,6 @@ int DispatchManager::attach_nic(
 {
     ostringstream oss;
 
-    int max_nic_id;
-    int uid;
-    int oid;
-    int rc;
-
-    set<int> vm_sgs;
-
-    VectorAttribute *        nic;
-    vector<VectorAttribute*> sg_rules;
-
     VirtualMachine * vm = vmpool->get(vid, true);
 
     if ( vm == 0 )
@@ -1476,18 +1475,6 @@ int DispatchManager::attach_nic(
         return -1;
     }
 
-    nic = vm->get_attach_nic_info(tmpl, max_nic_id, error_str);
-
-    if ( nic == 0 )
-    {
-        vm->unlock();
-
-        NebulaLog::log("DiM", Log::ERROR, error_str);
-        return -1;
-    }
-
-    vm->get_security_groups(vm_sgs);
-
     if (vm->get_state()     == VirtualMachine::ACTIVE &&
         vm->get_lcm_state() == VirtualMachine::RUNNING )
     {
@@ -1496,56 +1483,8 @@ int DispatchManager::attach_nic(
 
     vm->set_resched(false);
 
-    uid = vm->get_uid();
-    oid = vm->get_oid();
-
-    vmpool->update(vm);
-
-    vm->unlock();
-
-    rc = VirtualMachine::set_up_attach_nic(oid,
-                                    vm_sgs,
-                                    nic,
-                                    sg_rules,
-                                    max_nic_id,
-                                    uid,
-                                    error_str);
-    vm = vmpool->get(vid, true);
-
-    if ( vm == 0 )
+    if ( vm->set_up_attach_nic(tmpl, error_str) != 0 )
     {
-        delete nic;
-
-        if ( rc == 0 )
-        {
-            VirtualMachine::release_network_leases(nic, vid);
-
-            vector<VectorAttribute*>::iterator it;
-            for(it = sg_rules.begin(); it != sg_rules.end(); it++)
-            {
-                delete *it;
-            }
-        }
-
-        oss << "Could not attach a new NIC to VM " << vid
-            << ", VM does not exist after setting its state to HOTPLUG." ;
-        error_str = oss.str();
-
-        NebulaLog::log("DiM", Log::ERROR, error_str);
-
-        return -1;
-    }
-
-    if ( rc != 0 )
-    {
-        delete nic;
-
-        vector<VectorAttribute*>::iterator it;
-        for(it = sg_rules.begin(); it != sg_rules.end(); it++)
-        {
-            delete *it;
-        }
-
         if (vm->get_lcm_state() == VirtualMachine::HOTPLUG_NIC)
         {
             vm->set_state(VirtualMachine::RUNNING);
@@ -1560,12 +1499,6 @@ int DispatchManager::attach_nic(
         return -1;
     }
 
-    // Set the VM info in the history before the nic is attached to the
-    // VM template
-    vm->set_vm_info();
-
-    vm->set_attach_nic(nic, sg_rules);
-
     if (vm->get_lcm_state() == VirtualMachine::HOTPLUG_NIC)
     {
         time_t the_time = time(0);
@@ -1599,7 +1532,7 @@ int DispatchManager::attach_nic(
     {
         vm->log("DiM", Log::INFO, "VM NIC Successfully attached.");
 
-        vm->clear_attach_nic();
+        vm->attach_nic_success();
     }
 
     vmpool->update(vm);
@@ -1618,6 +1551,7 @@ int DispatchManager::detach_nic(
     string&  error_str)
 {
     ostringstream oss;
+    string        tmp_error;
 
     VirtualMachine * vm = vmpool->get(vid, true);
 
@@ -1644,7 +1578,7 @@ int DispatchManager::detach_nic(
         return -1;
     }
 
-    if ( vm->set_attach_nic(nic_id) == -1 )
+    if ( vm->set_detach_nic(nic_id) == -1 )
     {
         oss << "Could not detach NIC with NIC_ID " << nic_id
             << ", it does not exist.";
@@ -1698,9 +1632,11 @@ int DispatchManager::detach_nic(
     }
     else
     {
+        vmpool->update(vm);
+
         vm->unlock();
 
-        vmpool->delete_attach_nic(vid);
+        vmpool->detach_nic_success(vid);
 
         vm->log("DiM", Log::INFO, "VM NIC Successfully detached.");
     }
@@ -1855,8 +1791,7 @@ int DispatchManager::disk_snapshot_revert(
     VirtualMachine::LcmState lstate = vm->get_lcm_state();
 
     if ((state !=VirtualMachine::POWEROFF || lstate !=VirtualMachine::LCM_INIT)&&
-        (state !=VirtualMachine::SUSPENDED|| lstate !=VirtualMachine::LCM_INIT)&&
-        (state !=VirtualMachine::ACTIVE   || lstate !=VirtualMachine::RUNNING))
+        (state !=VirtualMachine::SUSPENDED|| lstate !=VirtualMachine::LCM_INIT))
     {
         oss << "Could not revert to disk snapshot for VM " << vid
             << ", wrong state " << vm->state_str() << ".";
@@ -1894,11 +1829,6 @@ int DispatchManager::disk_snapshot_revert(
             vm->set_state(VirtualMachine::DISK_SNAPSHOT_REVERT_SUSPENDED);
             break;
 
-        case VirtualMachine::ACTIVE:
-            vm->set_state(VirtualMachine::ACTIVE);
-            vm->set_state(VirtualMachine::DISK_SNAPSHOT_REVERT);
-            break;
-
         default: break;
     }
 
@@ -1906,19 +1836,7 @@ int DispatchManager::disk_snapshot_revert(
 
     vm->unlock();
 
-    switch(state)
-    {
-        case VirtualMachine::POWEROFF:
-        case VirtualMachine::SUSPENDED:
-            tm->trigger(TransferManager::SNAPSHOT_REVERT, vid);
-            break;
-
-        case VirtualMachine::ACTIVE:
-            vmm->trigger(VirtualMachineManager::DISK_SNAPSHOT_REVERT, vid);
-            break;
-
-        default: break;
-    }
+    tm->trigger(TransferManager::SNAPSHOT_REVERT, vid);
 
     return 0;
 }
