@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------------ */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems              */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems              */
 /*                                                                          */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may  */
 /* not use this file except in compliance with the License. You may obtain  */
@@ -32,29 +32,23 @@ Host::Host(
     const string& _hostname,
     const string& _im_mad_name,
     const string& _vmm_mad_name,
-    const string& _vnm_mad_name,
     int           _cluster_id,
     const string& _cluster_name):
         PoolObjectSQL(id,HOST,_hostname,-1,-1,"","",table),
-        Clusterable(_cluster_id, _cluster_name),
+        ClusterableSingle(_cluster_id, _cluster_name),
         state(INIT),
         im_mad_name(_im_mad_name),
         vmm_mad_name(_vmm_mad_name),
-        vnm_mad_name(_vnm_mad_name),
         last_monitored(0),
         vm_collection("VMS")
 {
-    string default_cpu; //TODO - Get these two from oned.conf
-    string default_mem;
-
     obj_template = new HostTemplate;
 
-    add_template_attribute("RESERVED_CPU", default_cpu);
-    add_template_attribute("RESERVED_MEM", default_cpu);
+    add_template_attribute("RESERVED_CPU", "");
+    add_template_attribute("RESERVED_MEM", "");
 
     replace_template_attribute("IM_MAD", im_mad_name);
     replace_template_attribute("VM_MAD", vmm_mad_name);
-    replace_template_attribute("VN_MAD", vnm_mad_name);
 }
 
 Host::~Host()
@@ -182,13 +176,11 @@ int Host::extract_ds_info(
             Template        &tmpl,
             map<int, const VectorAttribute*> &ds)
 {
-    char *    error_msg;
-    int       rc;
+    char * error_msg;
+    int    rc;
 
-    const VectorAttribute *  vatt;
-    vector<const Attribute*> ds_att;
-
-    vector<const Attribute*>::const_iterator it;
+    vector<const VectorAttribute*> ds_att;
+    vector<const VectorAttribute*>::const_iterator it;
 
     // -------------------------------------------------------------------------
     // Parse Template
@@ -223,18 +215,11 @@ int Host::extract_ds_info(
     {
         int dsid;
 
-        vatt = dynamic_cast<const VectorAttribute*>(*it);
-
-        if (vatt == 0)
-        {
-            continue;
-        }
-
-        rc = vatt->vector_value("ID", dsid);
+        rc = (*it)->vector_value("ID", dsid);
 
         if (rc == 0 && dsid != -1)
         {
-            ds.insert(make_pair(dsid, vatt));
+            ds.insert(make_pair(dsid, *it));
         }
     }
 
@@ -256,15 +241,13 @@ int Host::update_info(Template        &tmpl,
     vector<Attribute*>::iterator it;
     vector<Attribute*>           vm_att;
     vector<Attribute*>           ds_att;
-    vector<Attribute*>           pci_att;
-    vector<Attribute*>           local_ds_att;
+    vector<VectorAttribute*>     pci_att;
+    vector<VectorAttribute*>     local_ds_att;
 
     int   rc;
     int   vmid;
     float val;
 
-    string error_st;
-
     ostringstream zombie;
     ostringstream wild;
 
@@ -276,6 +259,11 @@ int Host::update_info(Template        &tmpl,
     int num_zombies = 0;
     int num_wilds   = 0;
 
+    if ( state == OFFLINE )
+    {
+        return -1;
+    }
+
     // -------------------------------------------------------------------------
     // Remove expired information from current template
     // -------------------------------------------------------------------------
@@ -296,35 +284,32 @@ int Host::update_info(Template        &tmpl,
     // Copy monitor, extract share info & update last_monitored and state
     // -------------------------------------------------------------------------
 
-    obj_template->merge(&tmpl, error_st);
+    obj_template->merge(&tmpl);
 
     touch(true);
 
-    if (isEnabled())
-    {
-        get_reserved_capacity(reserved_cpu, reserved_mem);
+    get_reserved_capacity(reserved_cpu, reserved_mem);
 
-        erase_template_attribute("TOTALCPU", val);
-        host_share.max_cpu = val - reserved_cpu;
-        erase_template_attribute("TOTALMEMORY", val);
-        host_share.max_mem = val - reserved_mem;
-        erase_template_attribute("DS_LOCATION_TOTAL_MB", val);
-        host_share.max_disk = val;
+    erase_template_attribute("TOTALCPU", val);
+    host_share.max_cpu = val - reserved_cpu;
+    erase_template_attribute("TOTALMEMORY", val);
+    host_share.max_mem = val - reserved_mem;
+    erase_template_attribute("DS_LOCATION_TOTAL_MB", val);
+    host_share.max_disk = val;
 
-        erase_template_attribute("FREECPU", val);
-        host_share.free_cpu = val;
-        erase_template_attribute("FREEMEMORY", val);
-        host_share.free_mem = val;
-        erase_template_attribute("DS_LOCATION_FREE_MB", val);
-        host_share.free_disk = val;
+    erase_template_attribute("FREECPU", val);
+    host_share.free_cpu = val;
+    erase_template_attribute("FREEMEMORY", val);
+    host_share.free_mem = val;
+    erase_template_attribute("DS_LOCATION_FREE_MB", val);
+    host_share.free_disk = val;
 
-        erase_template_attribute("USEDCPU", val);
-        host_share.used_cpu = val;
-        erase_template_attribute("USEDMEMORY", val);
-        host_share.used_mem = val;
-        erase_template_attribute("DS_LOCATION_USED_MB", val);
-        host_share.used_disk = val;
-    }
+    erase_template_attribute("USEDCPU", val);
+    host_share.used_cpu = val;
+    erase_template_attribute("USEDMEMORY", val);
+    host_share.used_mem = val;
+    erase_template_attribute("DS_LOCATION_USED_MB", val);
+    host_share.used_disk = val;
 
     // -------------------------------------------------------------------------
     // Correlate VM information with the list of running VMs
@@ -334,7 +319,7 @@ int Host::update_info(Template        &tmpl,
 
     obj_template->remove("VM", vm_att);
 
-    tmp_lost_vms = vm_collection.get_collection_copy();
+    tmp_lost_vms = vm_collection.clone();
 
     tmp_zombie_vms.clear();
 
@@ -360,10 +345,18 @@ int Host::update_info(Template        &tmpl,
 
         if (rc == 0 && vmid != -1)
         {
-            if (tmp_lost_vms.erase(vmid) == 1) //Good, known
+            map<int, string>::iterator it_vm;
+
+            it_vm = found.find(vmid);
+
+            if ( tmp_lost_vms.erase(vmid) == 1 ) //Good, known
             {
                 found.insert(make_pair(vmid, vatt->vector_value("POLL")));
             }
+            else if ( it_vm != found.end() )
+            {
+                it_vm->second += " " + vatt->vector_value("POLL");
+            }
             else //Bad, known but should not be here
             {
                 tmp_zombie_vms.insert(vmid);
@@ -476,9 +469,35 @@ int Host::update_info(Template        &tmpl,
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+void Host::enable()
+{
+    if (state == OFFLINE)
+    {
+        Nebula::instance().get_im()->start_monitor(this, true);
+    }
+
+    state = INIT;
+};
+
+/* -------------------------------------------------------------------------- */
+
 void Host::disable()
 {
+    if (state == OFFLINE)
+    {
+        Nebula::instance().get_im()->start_monitor(this, true);
+    }
+
     state = DISABLED;
+};
+
+/* -------------------------------------------------------------------------- */
+
+void Host::offline()
+{
+    Nebula::instance().get_im()->stop_monitor(get_oid(),get_name(),get_im_mad());
+
+    state = OFFLINE;
 
     host_share.max_cpu = 0;
     host_share.max_mem = 0;
@@ -506,7 +525,7 @@ void Host::error_info(const string& message, set<int> &vm_ids)
 {
     ostringstream oss;
 
-    vm_ids = vm_collection.get_collection_copy();
+    vm_ids = vm_collection.clone();
 
     oss << "Error monitoring Host " << get_name() << " (" << get_oid() << ")"
         << ": " << message;
@@ -598,15 +617,14 @@ string& Host::to_xml(string& xml) const
 
     oss <<
     "<HOST>"
-       "<ID>"               << oid              << "</ID>"              <<
-       "<NAME>"             << name             << "</NAME>"            <<
-       "<STATE>"            << state            << "</STATE>"           <<
-       "<IM_MAD><![CDATA["  << im_mad_name      << "]]></IM_MAD>"       <<
-       "<VM_MAD><![CDATA["  << vmm_mad_name     << "]]></VM_MAD>"       <<
-       "<VN_MAD><![CDATA["  << vnm_mad_name     << "]]></VN_MAD>"       <<
-       "<LAST_MON_TIME>"    << last_monitored   << "</LAST_MON_TIME>"   <<
-       "<CLUSTER_ID>"       << cluster_id       << "</CLUSTER_ID>"      <<
-       "<CLUSTER>"          << cluster          << "</CLUSTER>"         <<
+       "<ID>"            << oid              << "</ID>"              <<
+       "<NAME>"          << name             << "</NAME>"            <<
+       "<STATE>"         << state            << "</STATE>"           <<
+       "<IM_MAD>"        << one_util::escape_xml(im_mad_name)  << "</IM_MAD>" <<
+       "<VM_MAD>"        << one_util::escape_xml(vmm_mad_name) << "</VM_MAD>" <<
+       "<LAST_MON_TIME>" << last_monitored   << "</LAST_MON_TIME>"   <<
+       "<CLUSTER_ID>"    << cluster_id       << "</CLUSTER_ID>"      <<
+       "<CLUSTER>"       << cluster          << "</CLUSTER>"         <<
        host_share.to_xml(share_xml)  <<
        vm_collection.to_xml(vm_collection_xml) <<
        obj_template->to_xml(template_xml) <<
@@ -637,9 +655,8 @@ int Host::from_xml(const string& xml)
 
     rc += xpath(im_mad_name, "/HOST/IM_MAD", "not_found");
     rc += xpath(vmm_mad_name, "/HOST/VM_MAD", "not_found");
-    rc += xpath(vnm_mad_name, "/HOST/VN_MAD", "not_found");
 
-    rc += xpath(last_monitored, "/HOST/LAST_MON_TIME", 0);
+    rc += xpath<time_t>(last_monitored, "/HOST/LAST_MON_TIME", 0);
 
     rc += xpath(cluster_id, "/HOST/CLUSTER_ID", -1);
     rc += xpath(cluster,    "/HOST/CLUSTER",    "not_found");
@@ -681,17 +698,7 @@ int Host::from_xml(const string& xml)
     content.clear();
 
     // ------------ VMS collection ---------------
-
-    ObjectXML::get_nodes("/HOST/VMS", content);
-
-    if (content.empty())
-    {
-        return -1;
-    }
-
-    rc += vm_collection.from_xml_node(content[0]);
-
-    ObjectXML::free_nodes(content);
+    rc += vm_collection.from_xml(this, "/HOST/");
 
     if (rc != 0)
     {
@@ -707,7 +714,6 @@ int Host::post_update_template(string& error)
     string vcenter_password;
     string new_im_mad;
     string new_vm_mad;
-    string new_vn_mad;
 
     get_template_attribute("VCENTER_PASSWORD", vcenter_password);
 
@@ -737,7 +743,6 @@ int Host::post_update_template(string& error)
 
     get_template_attribute("IM_MAD", new_im_mad);
     get_template_attribute("VM_MAD", new_vm_mad);
-    get_template_attribute("VN_MAD", new_vn_mad);
 
     if (new_im_mad != ""){
         im_mad_name = new_im_mad;
@@ -747,13 +752,8 @@ int Host::post_update_template(string& error)
         vmm_mad_name = new_vm_mad;
     }
 
-    if (new_im_mad != ""){
-        vnm_mad_name = new_vn_mad;
-    }
-
     replace_template_attribute("IM_MAD", im_mad_name);
     replace_template_attribute("VM_MAD", vmm_mad_name);
-    replace_template_attribute("VN_MAD", vnm_mad_name);
 
     return 0;
 };
