@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -125,7 +125,23 @@ int ImageManager::acquire_image(int vm_id, Image *img, string& error)
             ipool->update(img);
         break;
 
+        case Image::LOCKED:
+            img->inc_running(vm_id);
+
+            if ( img->is_persistent() )
+            {
+                img->set_state(Image::LOCKED_USED_PERS);
+            }
+            else
+            {
+                img->set_state(Image::LOCKED_USED);
+            }
+
+            ipool->update(img);
+        break;
+
         case Image::USED_PERS:
+        case Image::LOCKED_USED_PERS:
             oss << "Cannot acquire image " << img->get_oid()
                 << ", it is persistent and already in use";
 
@@ -134,13 +150,13 @@ int ImageManager::acquire_image(int vm_id, Image *img, string& error)
         break;
 
         case Image::USED:
+        case Image::LOCKED_USED:
             img->inc_running(vm_id);
             ipool->update(img);
         break;
 
         case Image::INIT:
         case Image::DISABLED:
-        case Image::LOCKED:
         case Image::ERROR:
         case Image::DELETE:
         case Image::CLONE:
@@ -204,6 +220,23 @@ void ImageManager::release_image(int vm_id, int iid, bool failed)
             img->unlock();
         break;
 
+        case Image::LOCKED_USED_PERS:
+            img->dec_running(vm_id);
+
+            if (failed == true)
+            {
+                img->set_state(Image::ERROR);
+            }
+            else
+            {
+                img->set_state(Image::LOCKED);
+            }
+
+            ipool->update(img);
+
+            img->unlock();
+        break;
+
         case Image::USED:
             if ( img->dec_running(vm_id) == 0  && img->get_cloning() == 0 )
             {
@@ -215,6 +248,17 @@ void ImageManager::release_image(int vm_id, int iid, bool failed)
             img->unlock();
         break;
 
+        case Image::LOCKED_USED:
+            if ( img->dec_running(vm_id) == 0  && img->get_cloning() == 0 )
+            {
+                img->set_state(Image::LOCKED);
+            }
+
+            ipool->update(img);
+
+            img->unlock();
+        break;
+
         case Image::LOCKED:
             oss << "Releasing image in wrong state: "
                 << Image::state_to_str(img->get_state());
@@ -243,7 +287,8 @@ void ImageManager::release_image(int vm_id, int iid, bool failed)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void ImageManager::release_cloning_image(int iid, int clone_img_id)
+void ImageManager::release_cloning_resource(
+        int iid, PoolObjectSQL::ObjectType ot, int clone_oid)
 {
     Image * img = ipool->get(iid,true);
 
@@ -272,7 +317,7 @@ void ImageManager::release_cloning_image(int iid, int clone_img_id)
     {
         case Image::USED:
         case Image::CLONE:
-            if (img->dec_cloning(clone_img_id) == 0  && img->get_running() == 0)
+            if (img->dec_cloning(ot, clone_oid) == 0  && img->get_running() == 0)
             {
                 img->set_state(Image::READY);
             }
@@ -287,12 +332,10 @@ void ImageManager::release_cloning_image(int iid, int clone_img_id)
         case Image::ERROR:
         case Image::USED_PERS:
         case Image::LOCKED:
-            ostringstream oss;
-
-            oss << "Releasing image in wrong state: "
-                << Image::state_to_str(img->get_state());
-
-            NebulaLog::log("ImM", Log::ERROR, oss.str());
+        case Image::LOCKED_USED:
+        case Image::LOCKED_USED_PERS:
+            NebulaLog::log("ImM", Log::ERROR, "Release cloning image"
+                " in wrong state");
             break;
     }
 
@@ -321,9 +364,13 @@ int ImageManager::enable_image(int iid, bool to_enable, string& error_str)
         switch (img->get_state())
         {
             case Image::DISABLED:
-            case Image::ERROR:
                 img->set_state(Image::READY);
                 ipool->update(img);
+            break;
+            case Image::ERROR:
+                img->set_state_unlock();
+                ipool->update(img);
+            break;
             case Image::READY:
             break;
             default:
@@ -343,6 +390,7 @@ int ImageManager::enable_image(int iid, bool to_enable, string& error_str)
             case Image::ERROR:
                 img->set_state(Image::DISABLED);
                 ipool->update(img);
+            break;
             case Image::DISABLED:
             break;
             default:
@@ -438,6 +486,8 @@ int ImageManager::delete_image(int iid, string& error_str)
 
         case Image::USED:
         case Image::USED_PERS:
+        case Image::LOCKED_USED:
+        case Image::LOCKED_USED_PERS:
             oss << "There are " << img->get_running() << " VMs using it.";
             error_str = oss.str();
 
@@ -474,7 +524,7 @@ int ImageManager::delete_image(int iid, string& error_str)
         return -1;
     }
 
-    drv_msg = format_message(img->to_xml(img_tmpl), ds_data);
+    drv_msg = format_message(img->to_xml(img_tmpl), ds_data, "");
     source  = img->get_source();
     size    = img->get_size();
     ds_id   = img->get_ds_id();
@@ -564,8 +614,7 @@ int ImageManager::delete_image(int iid, string& error_str)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int ImageManager::can_clone_image(  int             cloning_id,
-                                    ostringstream&  oss_error)
+int ImageManager::can_clone_image(int cloning_id, ostringstream&  oss_error)
 {
     Image *       img;
 
@@ -589,6 +638,8 @@ int ImageManager::can_clone_image(  int             cloning_id,
         case Image::ERROR:
         case Image::DELETE:
         case Image::LOCKED:
+        case Image::LOCKED_USED:
+        case Image::LOCKED_USED_PERS:
             oss_error << "Cannot clone image in state: "
                 << Image::state_to_str(state);
 
@@ -607,9 +658,66 @@ int ImageManager::can_clone_image(  int             cloning_id,
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+int ImageManager::set_clone_state(
+        PoolObjectSQL::ObjectType ot, int new_id, int cloning_id, string& error)
+{
+    int     rc  = 0;
+    Image * img = ipool->get(cloning_id, true);
+
+    if (img == 0)
+    {
+        error = "Cannot clone image, it does not exist";
+        return -1;
+    }
+
+    switch(img->get_state())
+    {
+        case Image::READY:
+            img->inc_cloning(ot, new_id);
+
+            if (img->is_persistent())
+            {
+                img->set_state(Image::CLONE);
+            }
+            else
+            {
+                img->set_state(Image::USED);
+            }
+
+            ipool->update(img);
+            break;
+
+        case Image::USED:
+        case Image::CLONE:
+            img->inc_cloning(ot, new_id);
+            ipool->update(img);
+            break;
+
+        case Image::USED_PERS:
+        case Image::INIT:
+        case Image::DISABLED:
+        case Image::ERROR:
+        case Image::DELETE:
+        case Image::LOCKED:
+        case Image::LOCKED_USED:
+        case Image::LOCKED_USED_PERS:
+            error = "Cannot clone image in current state";
+            rc    = -1;
+            break;
+    }
+
+    img->unlock();
+
+    return rc;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 int ImageManager::clone_image(int   new_id,
                               int   cloning_id,
                               const string& ds_data,
+                              const string& extra_data,
                               string& error)
 {
     const ImageManagerDriver* imd = get();
@@ -629,66 +737,22 @@ int ImageManager::clone_image(int   new_id,
         return -1;
     }
 
-    img = ipool->get(cloning_id, true);
-
-    if (img == 0)
+    if ( set_img_clone_state(new_id, cloning_id, error) == -1 )
     {
-        error = "Cannot clone image, it does not exist";
         return -1;
     }
 
-    switch(img->get_state())
-    {
-        case Image::READY:
-            img->inc_cloning(new_id);
-
-            if (img->is_persistent())
-            {
-                img->set_state(Image::CLONE);
-            }
-            else
-            {
-                img->set_state(Image::USED);
-            }
-
-            ipool->update(img);
-
-            img->unlock();
-        break;
-
-        case Image::USED:
-        case Image::CLONE:
-            img->inc_cloning(new_id);
-
-            ipool->update(img);
-
-            img->unlock();
-        break;
-
-        case Image::USED_PERS:
-        case Image::INIT:
-        case Image::DISABLED:
-        case Image::ERROR:
-        case Image::DELETE:
-        case Image::LOCKED:
-            oss << "Cannot clone image in state: "
-                << Image::state_to_str(img->get_state());
-
-            error = oss.str();
-            img->unlock();
-            return -1;
-        break;
-    }
-
     img = ipool->get(new_id,true);
 
-    if (img == 0) //TODO: Rollback cloning counter
+    if (img == 0)
     {
+        release_cloning_image(cloning_id, new_id);
+
         error = "Target image deleted during cloning operation";
         return -1;
     }
 
-    drv_msg = format_message(img->to_xml(img_tmpl), ds_data);
+    drv_msg = format_message(img->to_xml(img_tmpl), ds_data, extra_data);
 
     imd->clone(img->get_oid(), *drv_msg);
 
@@ -707,7 +771,10 @@ int ImageManager::clone_image(int   new_id,
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int ImageManager::register_image(int iid, const string& ds_data, string& error)
+int ImageManager::register_image(int iid,
+                                 const string& ds_data,
+                                 const string& extra_data,
+                                 string& error)
 {
     const ImageManagerDriver* imd = get();
 
@@ -734,7 +801,7 @@ int ImageManager::register_image(int iid, const string& ds_data, string& error)
         return -1;
     }
 
-    drv_msg = format_message(img->to_xml(img_tmpl), ds_data);
+    drv_msg = format_message(img->to_xml(img_tmpl), ds_data, extra_data);
     path    = img->get_path();
 
     if ( path.empty() == true ) //NO PATH
@@ -750,7 +817,7 @@ int ImageManager::register_image(int iid, const string& ds_data, string& error)
         }
         else if ( !source.empty() ) //Source in Template
         {
-            img->set_state(Image::READY);
+            img->set_state_unlock();
             ipool->update(img);
 
             oss << "Using source " << source
@@ -859,7 +926,7 @@ int ImageManager::stat_image(Template*     img_tmpl,
 
     add_request(&sr);
 
-    drv_msg = format_message(img_data.str(), ds_data);
+    drv_msg = format_message(img_data.str(), ds_data, "");
 
     imd->stat(sr.id, *drv_msg);
 
@@ -882,13 +949,15 @@ int ImageManager::stat_image(Template*     img_tmpl,
 
 string * ImageManager::format_message(
     const string& img_data,
-    const string& ds_data)
+    const string& ds_data,
+    const string& extra_data)
 {
     ostringstream oss;
 
     oss << "<DS_DRIVER_ACTION_DATA>"
         << img_data
         << ds_data
+        << extra_data
         << "</DS_DRIVER_ACTION_DATA>";
 
     return one_util::base64_encode(oss.str());
@@ -1011,7 +1080,7 @@ int ImageManager::delete_snapshot(int iid, int sid, string& error)
     img->set_target_snapshot(sid);
 
     string img_tmpl;
-    string * drv_msg = format_message(img->to_xml(img_tmpl), ds_data);
+    string * drv_msg = format_message(img->to_xml(img_tmpl), ds_data, "");
 
     imd->snapshot_delete(iid, *drv_msg);
 
@@ -1115,7 +1184,7 @@ int ImageManager::revert_snapshot(int iid, int sid, string& error)
     img->set_target_snapshot(sid);
 
     string   img_tmpl;
-    string * drv_msg = format_message(img->to_xml(img_tmpl), ds_data);
+    string * drv_msg = format_message(img->to_xml(img_tmpl), ds_data, "");
 
     imd->snapshot_revert(iid, *drv_msg);
 
@@ -1212,7 +1281,7 @@ int ImageManager::flatten_snapshot(int iid, int sid, string& error)
     img->set_target_snapshot(sid);
 
     string   img_tmpl;
-    string * drv_msg = format_message(img->to_xml(img_tmpl), ds_data);
+    string * drv_msg = format_message(img->to_xml(img_tmpl), ds_data, "");
 
     imd->snapshot_flatten(iid, *drv_msg);
 
