@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -223,7 +223,7 @@ static int cp_action(istringstream& is,
 
     image->set_source(source);
 
-    image->set_state(Image::READY);
+    image->set_state_unlock();
 
     ipool->update(image);
 
@@ -320,7 +320,7 @@ static int clone_action(istringstream& is,
 
     image->set_source(source);
 
-    image->set_state(Image::READY);
+    image->set_state_unlock();
 
     image->clear_cloning_id();
 
@@ -444,7 +444,7 @@ static int mkfs_action(istringstream& is,
 
     if (!is_saving)
     {
-        image->set_state(Image::READY);
+        image->set_state_unlock();
     }
 
     ipool->update(image);
@@ -684,6 +684,25 @@ static void monitor_action(istringstream& is,
 
     ds->unlock();
 
+    vector<VectorAttribute *> vm_disk_info;
+    vector<VectorAttribute *>::iterator it;
+
+    monitor_data.get("VM", vm_disk_info);
+
+    for ( it = vm_disk_info.begin(); it != vm_disk_info.end(); ++it )
+    {
+        int    vm_id;
+        string poll_info;
+
+        if ( (*it)->vector_value("ID", vm_id) == -1 ||
+                (*it)->vector_value("POLL", poll_info) == -1 )
+        {
+            continue;
+        }
+
+        VirtualMachineManagerDriver::process_poll(vm_id, poll_info);
+    }
+
     oss << "Datastore " << ds_name << " (" << id << ") successfully monitored.";
 
     NebulaLog::log("ImM", Log::DEBUG, oss);
@@ -711,7 +730,7 @@ static void snap_delete_action(istringstream& is,
         return;
     }
 
-    image->set_state(Image::READY);
+    image->set_state_unlock();
 
     int snap_id = image->get_target_snapshot();
 
@@ -787,7 +806,7 @@ static void snap_revert_action(istringstream& is,
 
     int snap_id = image->get_target_snapshot();
 
-    image->set_state(Image::READY);
+    image->set_state_unlock();
 
     if (snap_id == -1)
     {
@@ -871,7 +890,7 @@ static void snap_flatten_action(istringstream& is,
         NebulaLog::log("ImM", Log::ERROR, oss);
     }
 
-    image->set_state(Image::READY);
+    image->set_state_unlock();
 
     image->clear_target_snapshot();
 
