@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -35,7 +35,7 @@ void  LifeCycleManager::deploy_action(int vid)
     {
         time_t thetime = time(0);
         int    cpu,mem,disk;
-        vector<Attribute *> pci;
+        vector<VectorAttribute *> pci;
 
         VirtualMachine::LcmState vm_state;
         TransferManager::Actions tm_action;
@@ -212,7 +212,7 @@ void  LifeCycleManager::migrate_action(int vid)
     VirtualMachine *    vm;
 
     int    cpu, mem, disk;
-    vector<Attribute *> pci;
+    vector<VectorAttribute *> pci;
 
     time_t the_time = time(0);
 
@@ -251,19 +251,25 @@ void  LifeCycleManager::migrate_action(int vid)
         vmm->trigger(VirtualMachineManager::SAVE,vid);
     }
     else if (vm->get_state() == VirtualMachine::POWEROFF ||
-        vm->get_state() == VirtualMachine::SUSPENDED )
+             vm->get_state() == VirtualMachine::SUSPENDED ||
+             (vm->get_state() == VirtualMachine::ACTIVE &&
+              vm->get_lcm_state() == VirtualMachine::UNKNOWN ))
     {
-        //------------------------------------------------------
-        //   Bypass SAVE_MIGRATE & go to PROLOG_MIGRATE_POWEROFF
-        //------------------------------------------------------
+        //----------------------------------------------------------------------
+        // Bypass SAVE_MIGRATE & go to PROLOG_MIGRATE_POWEROFF/SUSPENDED/UNKNOWN
+        //----------------------------------------------------------------------
         if (vm->get_state() == VirtualMachine::POWEROFF)
         {
             vm->set_state(VirtualMachine::PROLOG_MIGRATE_POWEROFF);
         }
-        else // VirtualMachine::SUSPENDED
+        else if (vm->get_state() == VirtualMachine::SUSPENDED)
         {
             vm->set_state(VirtualMachine::PROLOG_MIGRATE_SUSPEND);
         }
+        else //VirtualMachine::UNKNOWN
+        {
+            vm->set_state(VirtualMachine::PROLOG_MIGRATE_UNKNOWN);
+        }
 
         vm->set_state(VirtualMachine::ACTIVE);
 
@@ -292,50 +298,6 @@ void  LifeCycleManager::migrate_action(int vid)
 
         tm->trigger(TransferManager::PROLOG_MIGR,vid);
     }
-    else if (vm->get_state()     == VirtualMachine::ACTIVE &&
-        vm->get_lcm_state() == VirtualMachine::UNKNOWN)
-    {
-        //----------------------------------------------------
-        //   Bypass SAVE_MIGRATE & PROLOG_MIGRATE goto BOOT
-        //----------------------------------------------------
-
-        vm->set_resched(false);
-
-        vm->set_state(VirtualMachine::BOOT);
-
-        vm->delete_snapshots();
-
-        vm->reset_info();
-
-        vmpool->update(vm);
-
-        vm->set_stime(the_time);
-
-        vm->set_previous_action(History::MIGRATE_ACTION);
-
-        vm->set_previous_etime(the_time);
-
-        vm->set_previous_vm_info();
-
-        vm->set_previous_running_etime(the_time);
-
-        vm->set_previous_reason(History::USER);
-
-        vmpool->update_previous_history(vm);
-
-        vmpool->update_history(vm);
-
-        vm->get_requirements(cpu, mem, disk, pci);
-
-        hpool->add_capacity(vm->get_hid(), vm->get_oid(), cpu, mem, disk, pci);
-
-        hpool->del_capacity(vm->get_previous_hid(), vm->get_oid(), cpu, mem,
-            disk, pci);
-
-        //----------------------------------------------------
-
-        vmm->trigger(VirtualMachineManager::DEPLOY, vid);
-    }
     else
     {
         vm->log("LCM", Log::ERROR, "migrate_action, VM in a wrong state.");
@@ -365,7 +327,7 @@ void  LifeCycleManager::live_migrate_action(int vid)
         vm->get_lcm_state() == VirtualMachine::RUNNING)
     {
         int cpu, mem, disk;
-        vector<Attribute *> pci;
+        vector<VectorAttribute *> pci;
 
         //----------------------------------------------------
         //                   MIGRATE STATE
@@ -425,7 +387,7 @@ void  LifeCycleManager::shutdown_action(int vid, bool hard)
 
         if (hard)
         {
-            vm->set_action(History::SHUTDOWN_HARD_ACTION);
+            vm->set_action(History::TERMINATE_HARD_ACTION);
 
             //----------------------------------------------------
 
@@ -433,7 +395,7 @@ void  LifeCycleManager::shutdown_action(int vid, bool hard)
         }
         else
         {
-            vm->set_action(History::SHUTDOWN_ACTION);
+            vm->set_action(History::TERMINATE_ACTION);
 
             //----------------------------------------------------
 
@@ -448,16 +410,12 @@ void  LifeCycleManager::shutdown_action(int vid, bool hard)
     else if (vm->get_state() == VirtualMachine::SUSPENDED ||
              vm->get_state() == VirtualMachine::POWEROFF)
     {
-        //----------------------------------------------------
-        //   Bypass SHUTDOWN
-        //----------------------------------------------------
-
         vm->set_state(VirtualMachine::ACTIVE);
         vm->set_state(VirtualMachine::EPILOG);
 
         vmpool->update(vm);
 
-        vm->set_action(History::SHUTDOWN_ACTION);
+        vm->set_action(History::TERMINATE_ACTION);
 
         vm->set_epilog_stime(time(0));
 
@@ -465,7 +423,25 @@ void  LifeCycleManager::shutdown_action(int vid, bool hard)
 
         //----------------------------------------------------
 
-        tm->trigger(TransferManager::EPILOG,vid);
+        tm->trigger(TransferManager::EPILOG, vid);
+    }
+    else if (vm->get_state() == VirtualMachine::STOPPED ||
+             vm->get_state() == VirtualMachine::UNDEPLOYED)
+    {
+        vm->set_state(VirtualMachine::ACTIVE);
+        vm->set_state(VirtualMachine::EPILOG);
+
+        vmpool->update(vm);
+
+        vm->set_action(History::TERMINATE_ACTION);
+
+        vm->set_epilog_stime(time(0));
+
+        vmpool->update_history(vm);
+
+        //----------------------------------------------------
+
+        tm->trigger(TransferManager::EPILOG_LOCAL, vid);
     }
     else
     {
@@ -776,7 +752,7 @@ void LifeCycleManager::delete_action(int vid)
 
     if ( image_id != -1 )
     {
-        Image*     image = ipool->get(image_id, true);
+        Image * image = ipool->get(image_id, true);
 
         if ( image != 0 )
         {
@@ -792,7 +768,7 @@ void LifeCycleManager::delete_action(int vid)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void  LifeCycleManager::clean_action(int vid)
+void LifeCycleManager::delete_recreate_action(int vid)
 {
     Template *           vm_quotas = 0;
     map<int, Template *> ds_quotas;
@@ -870,14 +846,17 @@ void  LifeCycleManager::clean_action(int vid)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void  LifeCycleManager::clean_up_vm(VirtualMachine * vm, bool dispose, int& image_id)
+void LifeCycleManager::clean_up_vm(VirtualMachine * vm, bool dispose,
+        int& image_id)
 {
-    int    cpu, mem, disk;
-    vector<Attribute *> pci;
+    int cpu, mem, disk;
+    unsigned int port;
+
+    vector<VectorAttribute *> pci;
     time_t the_time = time(0);
 
     VirtualMachine::LcmState state = vm->get_lcm_state();
-    int                      vid   = vm->get_oid();
+    int vid   = vm->get_oid();
 
     if (dispose)
     {
@@ -903,8 +882,16 @@ void  LifeCycleManager::clean_up_vm(VirtualMachine * vm, bool dispose, int& imag
     vm->set_reason(History::USER);
 
     vm->get_requirements(cpu, mem, disk, pci);
+
     hpool->del_capacity(vm->get_hid(), vm->get_oid(), cpu, mem, disk, pci);
 
+    const VectorAttribute * graphics = vm->get_template_attribute("GRAPHICS");
+
+    if ( graphics != 0 && (graphics->vector_value("PORT", port) == 0))
+    {
+        clpool->release_vnc_port(vm->get_cid(), port);
+    }
+
     switch (state)
     {
         case VirtualMachine::PROLOG:
@@ -956,7 +943,7 @@ void  LifeCycleManager::clean_up_vm(VirtualMachine * vm, bool dispose, int& imag
         break;
 
         case VirtualMachine::HOTPLUG_NIC:
-            vm->clear_attach_nic();
+            vm->attach_nic_success();
             vmpool->update(vm);
 
             vm->set_running_etime(the_time);
@@ -1012,7 +999,6 @@ void  LifeCycleManager::clean_up_vm(VirtualMachine * vm, bool dispose, int& imag
         break;
 
         case VirtualMachine::DISK_SNAPSHOT:
-        case VirtualMachine::DISK_SNAPSHOT_REVERT:
             vm->clear_snapshot_disk();
             vmpool->update(vm);
 
@@ -1072,6 +1058,8 @@ void  LifeCycleManager::clean_up_vm(VirtualMachine * vm, bool dispose, int& imag
         case VirtualMachine::PROLOG_MIGRATE_POWEROFF_FAILURE:
         case VirtualMachine::PROLOG_MIGRATE_SUSPEND:
         case VirtualMachine::PROLOG_MIGRATE_SUSPEND_FAILURE:
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN:
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE:
             vm->set_prolog_etime(the_time);
             vmpool->update_history(vm);
 
@@ -1102,7 +1090,7 @@ void  LifeCycleManager::clean_up_vm(VirtualMachine * vm, bool dispose, int& imag
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void  LifeCycleManager::recover(VirtualMachine * vm, bool success)
+void LifeCycleManager::recover(VirtualMachine * vm, bool success)
 {
     LifeCycleManager::Actions lcm_action = LifeCycleManager::FINALIZE;
 
@@ -1143,6 +1131,8 @@ void  LifeCycleManager::recover(VirtualMachine * vm, bool success)
         case VirtualMachine::PROLOG_MIGRATE_POWEROFF_FAILURE:
         case VirtualMachine::PROLOG_MIGRATE_SUSPEND:
         case VirtualMachine::PROLOG_MIGRATE_SUSPEND_FAILURE:
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN:
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE:
             if (success)
             {
                 lcm_action = LifeCycleManager::PROLOG_SUCCESS;
@@ -1296,7 +1286,6 @@ void  LifeCycleManager::recover(VirtualMachine * vm, bool success)
         case VirtualMachine::DISK_SNAPSHOT_REVERT_SUSPENDED:
         case VirtualMachine::DISK_SNAPSHOT_DELETE_SUSPENDED:
         case VirtualMachine::DISK_SNAPSHOT:
-        case VirtualMachine::DISK_SNAPSHOT_REVERT:
         case VirtualMachine::DISK_SNAPSHOT_DELETE:
             if (success)
             {
@@ -1388,6 +1377,14 @@ void LifeCycleManager::retry(VirtualMachine * vm)
             tm->trigger(TransferManager::PROLOG_MIGR, vid);
             break;
 
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE:
+            vm->set_state(VirtualMachine::PROLOG_MIGRATE_UNKNOWN);
+
+            vmpool->update(vm);
+
+            tm->trigger(TransferManager::PROLOG_MIGR, vid);
+            break;
+
         case VirtualMachine::PROLOG_RESUME_FAILURE:
             vm->set_state(VirtualMachine::PROLOG_RESUME);
 
@@ -1452,13 +1449,13 @@ void LifeCycleManager::retry(VirtualMachine * vm)
         case VirtualMachine::SHUTDOWN:
         case VirtualMachine::SHUTDOWN_POWEROFF:
         case VirtualMachine::SHUTDOWN_UNDEPLOY:
-            if (vm->get_action() == History::SHUTDOWN_HARD_ACTION)
+            if (vm->get_action() == History::TERMINATE_ACTION)
             {
-                vmm->trigger(VirtualMachineManager::CANCEL,vid);
+                vmm->trigger(VirtualMachineManager::SHUTDOWN,vid);
             }
             else
             {
-                vmm->trigger(VirtualMachineManager::SHUTDOWN,vid);
+                vmm->trigger(VirtualMachineManager::CANCEL,vid);
             }
             break;
 
@@ -1479,6 +1476,7 @@ void LifeCycleManager::retry(VirtualMachine * vm)
         case VirtualMachine::PROLOG_MIGRATE:
         case VirtualMachine::PROLOG_MIGRATE_POWEROFF:
         case VirtualMachine::PROLOG_MIGRATE_SUSPEND:
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN:
             tm->trigger(TransferManager::PROLOG_MIGR,vid);
             break;
 
@@ -1514,7 +1512,6 @@ void LifeCycleManager::retry(VirtualMachine * vm)
         case VirtualMachine::DISK_SNAPSHOT_REVERT_SUSPENDED:
         case VirtualMachine::DISK_SNAPSHOT_DELETE_SUSPENDED:
         case VirtualMachine::DISK_SNAPSHOT:
-        case VirtualMachine::DISK_SNAPSHOT_REVERT:
         case VirtualMachine::DISK_SNAPSHOT_DELETE:
         case VirtualMachine::RUNNING:
         case VirtualMachine::UNKNOWN:
@@ -1523,3 +1520,186 @@ void LifeCycleManager::retry(VirtualMachine * vm)
 
     return;
 }
+
+/*  -------------------------------------------------------------------------- */
+/*  -------------------------------------------------------------------------- */
+
+void  LifeCycleManager::updatesg_action(int sgid)
+{
+    VirtualMachine * vm;
+
+    int  vmid;
+
+    SecurityGroup  * sg = sgpool->get(sgid, true);
+
+    if ( sg == 0 )
+    {
+        return;
+    }
+
+    // -------------------------------------------------------------------------
+    // Iterate over SG VMs, rules at hypervisor are updated one by one
+    // -------------------------------------------------------------------------
+    do
+    {
+        bool is_error = false;
+        bool is_tmpl  = false;
+        bool is_update= false;
+
+        if (sg->get_outdated(vmid) != 0)
+        {
+            sgpool->update(sg);
+            sg->unlock();
+
+            return;
+        }
+
+        sg->unlock();
+
+        vm = vmpool->get(vmid, true);
+
+        if ( vm == 0 )
+        {
+            continue;
+        }
+
+        VirtualMachine::LcmState lstate = vm->get_lcm_state();
+        VirtualMachine::VmState  state  = vm->get_state();
+
+        if ( state != VirtualMachine::ACTIVE ) //Update just VM information
+        {
+            is_tmpl = true;
+        }
+        else
+        {
+            switch (lstate)
+            {
+                //Cannnot update these VMs, SG rules being updated/created
+                case VirtualMachine::BOOT:
+                case VirtualMachine::BOOT_MIGRATE:
+                case VirtualMachine::BOOT_SUSPENDED:
+                case VirtualMachine::BOOT_STOPPED:
+                case VirtualMachine::BOOT_UNDEPLOY:
+                case VirtualMachine::BOOT_POWEROFF:
+                case VirtualMachine::BOOT_UNKNOWN:
+                case VirtualMachine::BOOT_FAILURE:
+                case VirtualMachine::BOOT_MIGRATE_FAILURE:
+                case VirtualMachine::BOOT_UNDEPLOY_FAILURE:
+                case VirtualMachine::BOOT_STOPPED_FAILURE:
+                case VirtualMachine::MIGRATE:
+                case VirtualMachine::HOTPLUG_NIC:
+                case VirtualMachine::UNKNOWN:
+                    is_error = true;
+                    break;
+
+                //Update just VM information
+                case VirtualMachine::LCM_INIT:
+                case VirtualMachine::PROLOG:
+                case VirtualMachine::PROLOG_MIGRATE_FAILURE:
+                case VirtualMachine::PROLOG_MIGRATE_POWEROFF_FAILURE:
+                case VirtualMachine::PROLOG_MIGRATE_SUSPEND_FAILURE:
+                case VirtualMachine::PROLOG_RESUME_FAILURE:
+                case VirtualMachine::PROLOG_UNDEPLOY_FAILURE:
+                case VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE:
+                case VirtualMachine::PROLOG_FAILURE:
+                case VirtualMachine::PROLOG_MIGRATE:
+                case VirtualMachine::PROLOG_MIGRATE_POWEROFF:
+                case VirtualMachine::PROLOG_MIGRATE_SUSPEND:
+                case VirtualMachine::PROLOG_MIGRATE_UNKNOWN:
+                case VirtualMachine::PROLOG_RESUME:
+                case VirtualMachine::PROLOG_UNDEPLOY:
+                case VirtualMachine::EPILOG:
+                case VirtualMachine::EPILOG_STOP:
+                case VirtualMachine::EPILOG_UNDEPLOY:
+                case VirtualMachine::EPILOG_FAILURE:
+                case VirtualMachine::EPILOG_STOP_FAILURE:
+                case VirtualMachine::EPILOG_UNDEPLOY_FAILURE:
+                case VirtualMachine::SHUTDOWN:
+                case VirtualMachine::SHUTDOWN_POWEROFF:
+                case VirtualMachine::SHUTDOWN_UNDEPLOY:
+                case VirtualMachine::SAVE_STOP:
+                case VirtualMachine::SAVE_SUSPEND:
+                case VirtualMachine::SAVE_MIGRATE:
+                case VirtualMachine::CLEANUP_RESUBMIT:
+                case VirtualMachine::CLEANUP_DELETE:
+                case VirtualMachine::DISK_SNAPSHOT_POWEROFF:
+                case VirtualMachine::DISK_SNAPSHOT_REVERT_POWEROFF:
+                case VirtualMachine::DISK_SNAPSHOT_DELETE_POWEROFF:
+                case VirtualMachine::DISK_SNAPSHOT_SUSPENDED:
+                case VirtualMachine::DISK_SNAPSHOT_REVERT_SUSPENDED:
+                case VirtualMachine::DISK_SNAPSHOT_DELETE_SUSPENDED:
+                case VirtualMachine::HOTPLUG_SAVEAS_POWEROFF:
+                case VirtualMachine::HOTPLUG_SAVEAS_SUSPENDED:
+                case VirtualMachine::HOTPLUG_PROLOG_POWEROFF:
+                case VirtualMachine::HOTPLUG_EPILOG_POWEROFF:
+                    is_tmpl = true;
+                    break;
+
+                //Update VM information + SG rules at host
+                case VirtualMachine::RUNNING:
+                case VirtualMachine::HOTPLUG:
+                case VirtualMachine::HOTPLUG_SNAPSHOT:
+                case VirtualMachine::HOTPLUG_SAVEAS:
+                case VirtualMachine::DISK_SNAPSHOT:
+                case VirtualMachine::DISK_SNAPSHOT_DELETE:
+                    is_update = true;
+                    break;
+            }
+        }
+
+        // ---------------------------------------------------------------------
+        // Update VM template with the new security group rules & trigger update
+        // ---------------------------------------------------------------------
+        if ( is_tmpl || is_update )
+        {
+            vector<VectorAttribute *> sg_rules;
+
+            sgpool->get_security_group_rules(-1, sgid, sg_rules);
+
+            vm->remove_security_group(sgid);
+
+            vm->add_template_attribute(sg_rules);
+
+            vmpool->update(vm);
+        }
+
+        if ( is_update )
+        {
+            vmm->updatesg(vm, sgid);
+        }
+
+        vm->unlock();
+
+        sg = sgpool->get(sgid, true);
+
+        if ( sg == 0 )
+        {
+            return;
+        }
+
+        // ---------------------------------------------------------------------
+        // Update VM status in the security group and exit
+        // ---------------------------------------------------------------------
+        if ( is_error )
+        {
+            sg->add_error(vmid);
+        }
+        else if ( is_tmpl )
+        {
+            sg->add_vm(vmid);
+        }
+        else if ( is_update )
+        {
+            sg->add_updating(vmid);
+        }
+
+        sgpool->update(sg);
+
+        if (is_update)
+        {
+            sg->unlock();
+            return;
+        }
+    } while (true);
+}
+
