@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -38,7 +38,7 @@ void  LifeCycleManager::save_success_action(int vid)
     if ( vm->get_lcm_state() == VirtualMachine::SAVE_MIGRATE )
     {
         int    cpu, mem, disk;
-        vector<Attribute *> pci;
+        vector<VectorAttribute *> pci;
 
         time_t the_time = time(0);
 
@@ -157,7 +157,7 @@ void  LifeCycleManager::save_failure_action(int vid)
     if ( vm->get_lcm_state() == VirtualMachine::SAVE_MIGRATE )
     {
         int    cpu, mem, disk;
-        vector<Attribute *> pci;
+        vector<VectorAttribute *> pci;
 
         time_t the_time = time(0);
 
@@ -261,7 +261,7 @@ void  LifeCycleManager::deploy_success_action(int vid)
     if ( vm->get_lcm_state() == VirtualMachine::MIGRATE )
     {
         int    cpu,mem,disk;
-        vector<Attribute *> pci;
+        vector<VectorAttribute *> pci;
 
         time_t the_time = time(0);
 
@@ -335,7 +335,7 @@ void  LifeCycleManager::deploy_failure_action(int vid)
     if ( vm->get_lcm_state() == VirtualMachine::MIGRATE )
     {
         int    cpu, mem, disk;
-        vector<Attribute *> pci;
+        vector<VectorAttribute *> pci;
 
         time_t the_time = time(0);
 
@@ -639,6 +639,8 @@ void LifeCycleManager::prolog_success_action(int vid)
         case VirtualMachine::PROLOG_MIGRATE_FAILURE: //recover success
         case VirtualMachine::PROLOG:
         case VirtualMachine::PROLOG_FAILURE: //recover success
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN:
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE: //recover success
             switch (lcm_state)
             {
                 case VirtualMachine::PROLOG_RESUME:
@@ -661,6 +663,8 @@ void LifeCycleManager::prolog_success_action(int vid)
 
                 case VirtualMachine::PROLOG:
                 case VirtualMachine::PROLOG_FAILURE: //recover success
+                case VirtualMachine::PROLOG_MIGRATE_UNKNOWN:
+                case VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE: //recover success
                     action = VirtualMachineManager::DEPLOY;
                     vm->set_state(VirtualMachine::BOOT);
                     break;
@@ -763,6 +767,11 @@ void  LifeCycleManager::prolog_failure_action(int vid)
             vmpool->update(vm);
             break;
 
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN:
+            vm->set_state(VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE);
+            vmpool->update(vm);
+            break;
+
         case VirtualMachine::PROLOG_RESUME:
             vm->set_state(VirtualMachine::PROLOG_RESUME_FAILURE);
             vmpool->update(vm);
@@ -776,6 +785,7 @@ void  LifeCycleManager::prolog_failure_action(int vid)
         case VirtualMachine::PROLOG_MIGRATE_FAILURE: //recover failure from failure state
         case VirtualMachine::PROLOG_MIGRATE_POWEROFF_FAILURE:
         case VirtualMachine::PROLOG_MIGRATE_SUSPEND_FAILURE:
+        case VirtualMachine::PROLOG_MIGRATE_UNKNOWN_FAILURE:
         case VirtualMachine::PROLOG_RESUME_FAILURE:
         case VirtualMachine::PROLOG_UNDEPLOY_FAILURE:
         case VirtualMachine::PROLOG_FAILURE:
@@ -797,10 +807,11 @@ void  LifeCycleManager::prolog_failure_action(int vid)
 void  LifeCycleManager::epilog_success_action(int vid)
 {
     VirtualMachine *    vm;
-    vector<Attribute *> pci;
+    vector<VectorAttribute *> pci;
 
     time_t the_time = time(0);
     int    cpu,mem,disk;
+    unsigned int port;
 
     VirtualMachine::LcmState state;
     DispatchManager::Actions action;
@@ -873,6 +884,13 @@ void  LifeCycleManager::epilog_success_action(int vid)
 
     hpool->del_capacity(vm->get_hid(), vm->get_oid(), cpu, mem, disk, pci);
 
+    const VectorAttribute * graphics = vm->get_template_attribute("GRAPHICS");
+
+    if ( graphics != 0 && (graphics->vector_value("PORT", port) == 0))
+    {
+        clpool->release_vnc_port(vm->get_cid(), port);
+    }
+
     //----------------------------------------------------
 
     dm->trigger(action,vid);
@@ -1542,7 +1560,7 @@ void LifeCycleManager::attach_nic_success_action(int vid)
 
     if ( vm->get_lcm_state() == VirtualMachine::HOTPLUG_NIC )
     {
-        vm->clear_attach_nic();
+        vm->attach_nic_success();
 
         vm->set_state(VirtualMachine::RUNNING);
 
@@ -1574,7 +1592,7 @@ void LifeCycleManager::attach_nic_failure_action(int vid)
     {
         vm->unlock();
 
-        vmpool->delete_attach_nic(vid);
+        vmpool->attach_nic_failure(vid);
 
         vm = vmpool->get(vid,true);
 
@@ -1601,7 +1619,39 @@ void LifeCycleManager::attach_nic_failure_action(int vid)
 
 void LifeCycleManager::detach_nic_success_action(int vid)
 {
-    attach_nic_failure_action(vid);
+    VirtualMachine *  vm;
+
+    vm = vmpool->get(vid,true);
+
+    if ( vm == 0 )
+    {
+        return;
+    }
+
+    if ( vm->get_lcm_state() == VirtualMachine::HOTPLUG_NIC )
+    {
+        vm->unlock();
+
+        vmpool->detach_nic_success(vid);
+
+        vm = vmpool->get(vid,true);
+
+        if ( vm == 0 )
+        {
+            return;
+        }
+
+        vm->set_state(VirtualMachine::RUNNING);
+
+        vmpool->update(vm);
+
+        vm->unlock();
+    }
+    else
+    {
+        vm->log("LCM",Log::ERROR,"detach_nic_success_action, VM in a wrong state");
+        vm->unlock();
+    }
 }
 
 /* -------------------------------------------------------------------------- */
@@ -1609,7 +1659,29 @@ void LifeCycleManager::detach_nic_success_action(int vid)
 
 void LifeCycleManager::detach_nic_failure_action(int vid)
 {
-    attach_nic_success_action(vid);
+    VirtualMachine * vm;
+
+    vm = vmpool->get(vid,true);
+
+    if ( vm == 0 )
+    {
+        return;
+    }
+
+    if ( vm->get_lcm_state() == VirtualMachine::HOTPLUG_NIC )
+    {
+        vm->detach_nic_failure();
+
+        vm->set_state(VirtualMachine::RUNNING);
+
+        vmpool->update(vm);
+    }
+    else
+    {
+        vm->log("LCM",Log::ERROR,"detach_nic_failure_action, VM in a wrong state");
+    }
+
+    vm->unlock();
 }
 
 /* -------------------------------------------------------------------------- */
@@ -1662,7 +1734,7 @@ void LifeCycleManager::saveas_success_action(int vid)
         return;
     }
 
-    image->set_state(Image::READY);
+    image->set_state_unlock();
 
     ipool->update(image);
 
@@ -1768,7 +1840,6 @@ void LifeCycleManager::disk_snapshot_success(int vid)
     switch (state)
     {
         case VirtualMachine::DISK_SNAPSHOT:
-        case VirtualMachine::DISK_SNAPSHOT_REVERT:
             vm->set_state(VirtualMachine::RUNNING);
         case VirtualMachine::DISK_SNAPSHOT_POWEROFF:
         case VirtualMachine::DISK_SNAPSHOT_SUSPENDED:
@@ -1914,7 +1985,6 @@ void LifeCycleManager::disk_snapshot_failure(int vid)
             vm->delete_disk_snapshot(disk_id, snap_id, &ds_quotas, &vm_quotas);
             break;
 
-        case VirtualMachine::DISK_SNAPSHOT_REVERT:
         case VirtualMachine::DISK_SNAPSHOT_DELETE:
             vm->set_state(VirtualMachine::RUNNING);
         case VirtualMachine::DISK_SNAPSHOT_DELETE_POWEROFF:
@@ -2001,3 +2071,113 @@ void LifeCycleManager::disk_snapshot_failure(int vid)
     return;
 }
 
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void LifeCycleManager::disk_lock_success(int vid)
+{
+    VirtualMachine * vm = vmpool->get(vid,true);
+    Image *          image;
+
+    if ( vm == 0 )
+    {
+        return;
+    }
+
+    if ( vm->get_state() != VirtualMachine::CLONING &&
+         vm->get_state() != VirtualMachine::CLONING_FAILURE )
+    {
+        vm->unlock();
+        return;
+    }
+
+    set<int> ids;
+
+    vm->get_cloning_image_ids(ids);
+
+    vm->unlock();
+
+    vector< pair<int,string> > ready;
+    vector< pair<int,string> >::iterator rit;
+
+    set<int> error;
+
+    for (set<int>::iterator id = ids.begin(); id != ids.end(); id++)
+    {
+        image = ipool->get(*id, true);
+
+        if (image != 0)
+        {
+            switch (image->get_state()) {
+                case Image::USED:
+                case Image::USED_PERS:
+                    ready.push_back( make_pair(*id, image->get_source()) );
+                    break;
+
+                case Image::ERROR:
+                    error.insert(*id);
+                    break;
+
+                case Image::INIT:
+                case Image::READY:
+                case Image::DISABLED:
+                case Image::LOCKED:
+                case Image::CLONE:
+                case Image::DELETE:
+                case Image::LOCKED_USED:
+                case Image::LOCKED_USED_PERS:
+                    break;
+            }
+
+            image->unlock();
+        }
+    }
+
+    vm = vmpool->get(vid,true);
+
+    if (vm == 0)
+    {
+        return;
+    }
+
+    for (rit = ready.begin(); rit != ready.end(); rit++)
+    {
+        vm->clear_cloning_image_id(rit->first, rit->second);
+    }
+
+    if (ids.size() == ready.size())
+    {
+        bool on_hold = false;
+
+        vm->get_template_attribute("SUBMIT_ON_HOLD", on_hold);
+
+        if(on_hold)
+        {
+            vm->set_state(VirtualMachine::HOLD);
+        }
+        else
+        {
+            vm->set_state(VirtualMachine::PENDING);
+        }
+    }
+    else if (error.size() > 0)
+    {
+        vm->set_state(VirtualMachine::CLONING_FAILURE);
+    }
+    else
+    {
+        vm->set_state(VirtualMachine::CLONING);
+    }
+
+    vmpool->update(vm);
+
+    vm->unlock();
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void LifeCycleManager::disk_lock_failure(int vid)
+{
+    disk_lock_success(vid);
+}
