@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,18 +25,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#ifdef SYSLOG_LOG
-#   include <syslog.h>
-
-#   include "log4cpp/Category.hh"
-#   include "log4cpp/CategoryStream.hh"
-#   include "log4cpp/Appender.hh"
-#   include "log4cpp/SyslogAppender.hh"
-#   include "log4cpp/Layout.hh"
-#   include "log4cpp/PatternLayout.hh"
-#   include "log4cpp/Priority.hh"
-#endif /* SYSLOG_LOG */
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
@@ -119,7 +107,18 @@ void FileLog::log(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void CerrLog::log(
+StdLog::StdLog(const MessageType level, int oid,
+       const PoolObjectSQL::ObjectType obj_type):Log(level)
+{
+    ostringstream oss;
+
+    oss << "[" << PoolObjectSQL::type_to_str(obj_type) << " " << oid << "]";
+    resource_label = oss.str();
+}
+
+/* -------------------------------------------------------------------------- */
+
+void StdLog::log(
     const char *            module,
     const MessageType       type,
     const char *            message)
@@ -133,61 +132,51 @@ void CerrLog::log(
         the_time = time(NULL);
 
 #ifdef SOLARIS
-        ctime_r(&(the_time),str,sizeof(char)*26);
+        ctime_r(&(the_time), str, sizeof(char)*26);
 #else
-        ctime_r(&(the_time),str);
+        ctime_r(&(the_time), str);
 #endif
         // Get rid of final enter character
         str[24] = '\0';
 
-        cerr << str << " ";
-        cerr << "[Z"<< zone_id<< "]";
-        cerr << "[" << module << "]";
-        cerr << "[" << error_names[type] << "]: ";
-        cerr << message;
-        cerr << endl;
+        std::clog << str << " "
+				  << resource_label
+                  << "[Z"<< zone_id<< "]"
+                  << "[" << module << "]"
+                  << "[" << error_names[type] << "]: "
+                  << message << endl;
 
-        cerr.flush();
+        std::clog.flush();
     }
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-#ifdef SYSLOG_LOG
-
-const char * SysLog::CATEGORY = "ROOT";
-string       SysLog::LABEL;
-
-/* -------------------------------------------------------------------------- */
-
 SysLog::SysLog(const MessageType level,
-               const string&     label):Log(level)
+               const string&     label):Log(level), resource_label("")
 {
     static bool initialized = false;
 
     if (!initialized) //Initialize just once for all SysLog instances
     {
-        ostringstream     oss;
-        log4cpp::Appender *appender;
-
-        oss << label << "[" << getpid() << "]";
-
-        LABEL = oss.str();
-
-        appender = new log4cpp::SyslogAppender(CATEGORY, LABEL, LOG_DAEMON);
-        appender->setLayout(new log4cpp::PatternLayout());
-
-        log4cpp::Category& root = log4cpp::Category::getRoot();
-
-        root.setPriority(SysLog::get_priority_level(level));
-        root.addAppender(appender);
+        openlog(label.c_str(), LOG_PID, LOG_DAEMON);
 
         initialized = true;
     }
 };
 
 /* -------------------------------------------------------------------------- */
+
+SysLog::SysLog(const MessageType level, int oid,
+    const PoolObjectSQL::ObjectType obj_type):Log(level)
+{
+    ostringstream oss;
+
+    oss << "[" << PoolObjectSQL::type_to_str(obj_type) << " " << oid << "]";
+    resource_label = oss.str();
+}
+
 /* -------------------------------------------------------------------------- */
 
 void SysLog::log(
@@ -195,123 +184,28 @@ void SysLog::log(
     const MessageType       type,
     const char *            message)
 {
-    log4cpp::Category&               root  = log4cpp::Category::getRoot();
-    log4cpp::Priority::PriorityLevel level = get_priority_level(type);
-
-    istringstream smessage;
-    string        line;
-
-    smessage.str(message);
-
-    while ( getline(smessage, line) )
+    if( type <= log_level)
     {
-        root << level << "[Z"<< zone_id<< "]"
-                      << "[" << module << "]"
-                      << "[" << error_names[type] << "]: "
-                      << line;
+        istringstream smessage(message);
+        string        line;
+
+        int slevel = level(type);
+
+        while ( getline(smessage, line) )
+        {
+            ostringstream oss;
+
+            oss << resource_label
+				<< "[Z"<< zone_id<< "]"
+                << "[" << module << "]"
+                << "[" << error_names[type] << "]: "
+                << line;
+
+            syslog(slevel, oss.str().c_str());
+        }
     }
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-log4cpp::Priority::PriorityLevel SysLog::get_priority_level(
-    const MessageType level)
-{
-    log4cpp::Priority::PriorityLevel priority_level;
-
-    switch (level)
-    {
-    case Log::ERROR:
-        priority_level = log4cpp::Priority::ERROR;
-        break;
-
-    case Log::WARNING:
-        priority_level = log4cpp::Priority::WARN;
-        break;
-
-    case Log::INFO:
-        priority_level = log4cpp::Priority::INFO;
-        break;
-
-    case Log::DEBUG:
-    case Log::DDEBUG:
-    case Log::DDDEBUG:
-        priority_level = log4cpp::Priority::DEBUG;
-        break;
-
-    default:
-        priority_level = log4cpp::Priority::NOTSET;
-        break;
-    }
-
-    return priority_level;
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-const char * SysLogResource::CATEGORY = "RESOURCE";
-
-/* -------------------------------------------------------------------------- */
-
-SysLogResource::SysLogResource(
-        int                             oid,
-        const PoolObjectSQL::ObjectType obj_type,
-        const MessageType               clevel):SysLog(clevel)
-{
-    static bool   initialized = false;
-    ostringstream oss_label;
-    string        obj_type_str;
-
-    if (!initialized)
-    {
-        log4cpp::Appender *appender;
-
-        appender = new log4cpp::SyslogAppender(CATEGORY,
-                                               SysLog::LABEL,
-                                               LOG_DAEMON);
-
-        appender->setLayout(new log4cpp::PatternLayout());
-
-        log4cpp::Category& res = log4cpp::Category::getInstance(CATEGORY);
-
-        res.addAppender(appender);
-        res.setPriority(SysLog::get_priority_level(clevel));
-
-        initialized = true;
-    }
-
-    obj_type_str = PoolObjectSQL::type_to_str(obj_type);
-
-    oss_label << "[" << obj_type_str << " " << oid << "]";
-    obj_label = oss_label.str();
-}
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-void SysLogResource::log(
-    const char *            module,
-    const MessageType       type,
-    const char *            message)
-{
-    log4cpp::Category& res = log4cpp::Category::getInstance(CATEGORY);
-    log4cpp::Priority::PriorityLevel level = get_priority_level(type);
-
-    istringstream   smessage;
-    string          line;
-
-    smessage.str(message);
-
-    while ( getline(smessage, line) )
-    {
-        res << level << obj_label
-                     << "[Z" << zone_id << "]"
-                     << "[" << module << "]"
-                     << "[" << error_names[type] << "]: "
-                     << line;
-    }
-}
-
-#endif  /* SYSLOG_LOG */
