@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -30,7 +30,8 @@ GREP=${GREP:-grep}
 ISCSIADM=${ISCSIADM:-iscsiadm}
 LVCREATE=${LVCREATE:-lvcreate}
 LVREMOVE=${LVREMOVE:-lvremove}
-LVRENAME=${LVRENAME:-lvrename}
+LVCHANGE=${LVCHANGE:-lvchange}
+LVSCAN=${LVSCAN:-lvscan}
 LVS=${LVS:-lvs}
 LN=${LN:-ln}
 MD5SUM=${MD5SUM:-md5sum}
@@ -42,6 +43,7 @@ RADOS=${RADOS:-rados}
 RBD=${RBD:-rbd}
 READLINK=${READLINK:-readlink}
 RM=${RM:-rm}
+CP=${CP:-cp}
 SCP=${SCP:-scp}
 SED=${SED:-sed}
 SSH=${SSH:-ssh}
@@ -154,12 +156,13 @@ function exec_and_log
 # the command fails
 function monitor_and_log
 {
-    EXEC_OUT=`bash -s 2>/dev/null <<EOF
+    EXEC_OUT="$(bash -s 2>/dev/null <<EOF
 export LANG=C
 export LC_ALL=C
 set -xv
 $1
-EOF`
+EOF
+)"
     EXEC_RC=$?
 
     if [ $EXEC_RC -ne 0 ]; then
@@ -173,7 +176,7 @@ EOF`
         exit $EXEC_RC
     fi
 
-    echo $EXEC_OUT
+    echo "$EXEC_OUT"
 }
 
 # Executes a command, if it fails returns error message and exits. Similar to
@@ -311,57 +314,20 @@ function force_shutdown {
 # filesystem with its proper parameters based on the filesystem type
 function mkfs_command {
     DST=$1
-    FSTYPE=${2:-ext3}
+    FSTYPE=$2
     SIZE=${3:-0}
 
-    # Specific options for different FS
-    case "$FSTYPE" in
-        "ext2"|"ext3"|"ext4"|"ntfs")
-            OPTS="-F"
-            ;;
+    if [ "$FSTYPE" = "qcow2" ]; then
+        QEMU_FORMAT="qcow2"
+    else
+        QEMU_FORMAT="raw"
+    fi
 
-        "reiserfs"|"xfs")
-            OPTS="-f -q"
-            ;;
+    echo "$QEMU_IMG create -f ${QEMU_FORMAT} ${DST} ${SIZE}M"
 
-        "jfs")
-            OPTS="-q"
-            ;;
-        "raw"|"")
-            echo ""
-            return 0
-            ;;
-        "swap")
-            echo "$MKSWAP -L swap $DST"
-            return 0
-            ;;
-        "qcow2")
-            echo "$QEMU_IMG create -f qcow2 $DST ${SIZE}M"
-            return 0
-            ;;
-        "vmdk_"*)
-            VMWARE_DISK_TYPE=`echo $FSTYPE|cut -d'_' -f 2`
-
-            echo "$VMWARE_DISK_TYPE" | \
-            grep '\<thin\>\|\<zeroedthic\>\|\<eagerzeroedthick\>' 2>&1 /dev/null
-
-            if [ $? -eq 1 ] ; then
-                VMWARE_DISK_TYPE="thin"
-            fi
-
-            echo "$VMKFSTOOLS -U $DST/disk.vmdk; \
-                  rm -f $DST/*; \
-                  $VMKFSTOOLS -c ${SIZE}M -d ${VMWARE_DISK_TYPE} $DST/disk.vmdk"
-            return 0
-            ;;
-        *)
-            echo ""
-            echo "Filesystem '$FSTYPE' not valid." 1>&2
-            return 1
-            ;;
-    esac
-
-    echo "$MKFS -t $FSTYPE $OPTS $DST"
+    if [ "$FSTYPE" = "swap" ]; then
+        echo "$MKSWAP -L swap $DST"
+    fi
 }
 
 #This function executes $2 at $1 host and report error $3
@@ -422,11 +388,12 @@ EOF`
 # the command fails
 function ssh_monitor_and_log
 {
-    SSH_EXEC_OUT=`$SSH $1 sh -s 2>/dev/null <<EOF
+    SSH_EXEC_OUT="$($SSH $1 sh -s 2>/dev/null <<EOF
 export LANG=C
 export LC_ALL=C
 $2
-EOF`
+EOF
+)"
     SSH_EXEC_RC=$?
 
     if [ $SSH_EXEC_RC -ne 0 ]; then
@@ -440,20 +407,26 @@ EOF`
         exit $SSH_EXEC_RC
     fi
 
-    echo $SSH_EXEC_OUT
+    echo "$SSH_EXEC_OUT"
 }
 
-#Creates path ($2) at $1
+# Creates path ($2) at $1. If third parameter is "monitor" creates the
+# file ".monitor" in the directory. Used for ssh disk monitoring
 function ssh_make_path
 {
     SSH_EXEC_ERR=`$SSH $1 sh -s 2>&1 1>/dev/null <<EOF
+set -e
 if [ ! -d $2 ]; then
    mkdir -p $2
+
+   if [ "monitor" = "$3" ]; then
+       touch "\$(dirname $2)/.monitor"
+   fi
 fi
 EOF`
     SSH_EXEC_RC=$?
 
-    if [ $? -ne 0 ]; then
+    if [ $SSH_EXEC_RC -ne 0 ]; then
         error_message "Error creating directory $2 at $1: $SSH_EXEC_ERR"
 
         exit $SSH_EXEC_RC
@@ -593,3 +566,36 @@ function iqn_get_host {
     VG_NAME=$(iqn_get_vg_name "$IQN")
     echo ${TARGET%%.$VG_NAME.$LV_NAME}
 }
+
+# ------------------------------------------------------------------------------
+# VMM helpers
+# ------------------------------------------------------------------------------
+
+# This function builds the XML necessary for attach-disk operations
+# that require declaration of host sources
+#   @param $1 - Space separated list of hosts
+#   @return The XML via STDOUT
+function get_source_xml {
+    for host in $1 ; do
+        BCK_IFS=$IFS
+        IFS=':'
+
+        unset k HOST_PARTS SOURCE_HOST
+
+        for part in $host ; do
+            HOST_PARTS[k++]="$part"
+        done
+
+        SOURCE_HOST="$SOURCE_HOST<host name='${HOST_PARTS[0]}'"
+
+        if [ -n "${HOST_PARTS[1]}" ]; then
+            SOURCE_HOST="$SOURCE_HOST port='${HOST_PARTS[1]}'"
+        fi
+
+        SOURCE_HOST="$SOURCE_HOST/>"
+
+        IFS=$BCK_IFS
+    done
+
+    echo "$SOURCE_HOST"
+}
