@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2015, OpenNebula Project, OpenNebula Systems
+ * Copyright 2002-2016, OpenNebula Project, OpenNebula Systems
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,17 +31,33 @@ public class Host extends PoolElement{
     private static final String ALLOCATE        = METHOD_PREFIX + "allocate";
     private static final String INFO            = METHOD_PREFIX + "info";
     private static final String DELETE          = METHOD_PREFIX + "delete";
-    private static final String ENABLE          = METHOD_PREFIX + "enable";
+    private static final String STATUS          = METHOD_PREFIX + "status";
     private static final String UPDATE          = METHOD_PREFIX + "update";
     private static final String MONITORING      = METHOD_PREFIX + "monitoring";
     private static final String RENAME          = METHOD_PREFIX + "rename";
 
     private static final String[] HOST_STATES =
         {"INIT", "MONITORING_MONITORED", "MONITORED", "ERROR", "DISABLED",
-         "MONITORING_ERROR",  "MONITORING_INIT", "MONITORING_DISABLED"};
+         "MONITORING_ERROR",  "MONITORING_INIT", "MONITORING_DISABLED", "OFFLINE"};
 
     private static final String[] SHORT_HOST_STATES =
-        {"init", "update", "on", "err", "off", "retry", "init", "off"};
+        {"init", "update", "on", "err", "dsbl", "retry", "init", "dsbl", "off"};
+
+    public enum Status {
+        ENABLED,
+        DISABLED,
+        OFFLINE;
+
+        public int code() {
+            switch(this) {
+                case ENABLED:  return 0;
+                case DISABLED: return 1;
+                case OFFLINE:  return 2;
+            }
+
+            return -1;
+        }
+    }
 
     /**
      * Creates a new Host representation.
@@ -77,9 +93,6 @@ public class Host extends PoolElement{
      * @param vmm The name of the virtual machine manager mad name
      * (vmm_mad_name), this values are taken from the oned.conf with the
      * tag name VM_MAD (name)
-     * @param vnm The name of the virtual network manager mad name
-     * (vnm_mad_name), this values are taken from the oned.conf with the
-     * tag name VN_MAD (name)
      * @param clusterId The cluster ID. If it is -1, this host won't be
      * added to any cluster.
      *
@@ -90,10 +103,9 @@ public class Host extends PoolElement{
                                        String hostname,
                                        String im,
                                        String vmm,
-                                       String vnm,
                                        int    clusterId)
     {
-        return client.call(ALLOCATE, hostname, im, vmm, vnm, clusterId);
+        return client.call(ALLOCATE, hostname, im, vmm, clusterId);
     }
 
     /**
@@ -106,9 +118,6 @@ public class Host extends PoolElement{
      * @param vmm The name of the virtual machine manager mad name
      * (vmm_mad_name), this values are taken from the oned.conf with the
      * tag name VM_MAD (name)
-     * @param vnm The name of the virtual network manager mad name
-     * (vnm_mad_name), this values are taken from the oned.conf with the
-     * tag name VN_MAD (name)
      *
      * @return If successful the message contains the associated
      * id generated for this host
@@ -117,10 +126,9 @@ public class Host extends PoolElement{
             Client client,
             String hostname,
             String im,
-            String vmm,
-            String vnm)
+            String vmm)
     {
-        return allocate(client, hostname, im, vmm, vnm, -1);
+        return allocate(client, hostname, im, vmm, -1);
     }
 
     /**
@@ -157,9 +165,9 @@ public class Host extends PoolElement{
      * target host, if set false it will disable it.
      * @return A encapsulated response.
      */
-    public static OneResponse enable(Client client, int id, boolean enable)
+    public static OneResponse status(Client client, int id, Status status)
     {
-        return client.call(ENABLE, id, enable);
+        return client.call(STATUS, id, status.code());
     }
 
     /**
@@ -230,16 +238,6 @@ public class Host extends PoolElement{
         return delete(client, id);
     }
 
-    /**
-     * Enables or disables the host.
-     *
-     * @see Host#enable(Client, int, boolean)
-     */
-    public OneResponse enable(boolean enable)
-    {
-        return enable(client, id, enable);
-    }
-
     /**
      * Enables the host.
      *
@@ -247,7 +245,7 @@ public class Host extends PoolElement{
      */
     public OneResponse enable()
     {
-        return enable(true);
+        return status(client, id, Status.ENABLED);
     }
 
     /**
@@ -257,7 +255,17 @@ public class Host extends PoolElement{
      */
     public OneResponse disable()
     {
-        return enable(false);
+        return status(client, id, Status.DISABLED);
+    }
+
+    /**
+     * Sets the host offline
+     *
+     * @return A encapsulated response.
+     */
+    public OneResponse offline()
+    {
+        return status(client, id, Status.OFFLINE);
     }
 
     /**
