@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2002-2015, OpenNebula Project, OpenNebula Systems
+ * Copyright 2002-2016, OpenNebula Project, OpenNebula Systems
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -63,7 +63,8 @@ public class VirtualMachine extends PoolElement{
         "DONE",
         "FAILED",
         "POWEROFF",
-        "UNDEPLOYED" };
+        "UNDEPLOYED",
+        "CLONING" };
 
     private static final String[] SHORT_VM_STATES =
     {
@@ -76,7 +77,8 @@ public class VirtualMachine extends PoolElement{
         "done",
         "fail",
         "poff",
-        "unde" };
+        "unde",
+        "clon" };
 
     private static final String[] LCM_STATE =
     {
@@ -139,7 +141,9 @@ public class VirtualMachine extends PoolElement{
         "DISK_SNAPSHOT_DELETE_SUSPENDED",
         "DISK_SNAPSHOT",
         "DISK_SNAPSHOT_REVERT",
-        "DISK_SNAPSHOT_DELETE"
+        "DISK_SNAPSHOT_DELETE",
+        "PROLOG_MIGRATE_UNKNOWN",
+        "PROLOG_MIGRATE_UNKNOWN_FAILURE"
     };
 
     private static final String[] SHORT_LCM_STATES =
@@ -203,7 +207,9 @@ public class VirtualMachine extends PoolElement{
         "snap",     // DISK_SNAPSHOT_DELETE_SUSPENDED
         "snap",     // DISK_SNAPSHOT
         "snap",     // DISK_SNAPSHOT_REVERT
-        "snap"      // DISK_SNAPSHOT_DELETE
+        "snap",     // DISK_SNAPSHOT_DELETE
+        "migr",     // PROLOG_MIGRATE_UNKNOWN
+        "fail"      // PROLOG_MIGRATE_UNKNOWN_FAILURE
     };
 
     /**
