@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -46,11 +46,12 @@ module OpenNebula
             :disksaveas     => "vm.disksaveas",
             :disksnapshotcreate => "vm.disksnapshotcreate",
             :disksnapshotrevert => "vm.disksnapshotrevert",
-            :disksnapshotdelete => "vm.disksnapshotdelete"
+            :disksnapshotdelete => "vm.disksnapshotdelete",
+            :updateconf     => "vm.updateconf"
         }
 
         VM_STATE=%w{INIT PENDING HOLD ACTIVE STOPPED SUSPENDED DONE FAILED
-            POWEROFF UNDEPLOYED}
+            POWEROFF UNDEPLOYED CLONING CLONING_FAILURE}
 
         LCM_STATE=%w{
             LCM_INIT
@@ -111,21 +112,24 @@ module OpenNebula
             DISK_SNAPSHOT_REVERT_SUSPENDED
             DISK_SNAPSHOT_DELETE_SUSPENDED
             DISK_SNAPSHOT
-            DISK_SNAPSHOT_REVERT
             DISK_SNAPSHOT_DELETE
+            PROLOG_MIGRATE_UNKNOWN
+            PROLOG_MIGRATE_UNKNOWN_FAILURE
         }
 
         SHORT_VM_STATES={
-            "INIT"      => "init",
-            "PENDING"   => "pend",
-            "HOLD"      => "hold",
-            "ACTIVE"    => "actv",
-            "STOPPED"   => "stop",
-            "SUSPENDED" => "susp",
-            "DONE"      => "done",
-            "FAILED"    => "fail",
-            "POWEROFF"  => "poff",
-            "UNDEPLOYED"=> "unde"
+            "INIT"              => "init",
+            "PENDING"           => "pend",
+            "HOLD"              => "hold",
+            "ACTIVE"            => "actv",
+            "STOPPED"           => "stop",
+            "SUSPENDED"         => "susp",
+            "DONE"              => "done",
+            "FAILED"            => "fail",
+            "POWEROFF"          => "poff",
+            "UNDEPLOYED"        => "unde",
+            "CLONING"           => "clon",
+            "CLONING_FAILURE"   => "fail"
         }
 
         SHORT_LCM_STATES={
@@ -186,8 +190,9 @@ module OpenNebula
             "DISK_SNAPSHOT_REVERT_SUSPENDED"=> "snap",
             "DISK_SNAPSHOT_DELETE_SUSPENDED"=> "snap",
             "DISK_SNAPSHOT"        => "snap",
-            "DISK_SNAPSHOT_REVERT" => "snap",
-            "DISK_SNAPSHOT_DELETE" => "snap"
+            "DISK_SNAPSHOT_DELETE" => "snap",
+            "PROLOG_MIGRATE_UNKNOWN" => "migr",
+            "PROLOG_MIGRATE_UNKNOWN_FAILURE" => "fail"
         }
 
         MIGRATE_REASON=%w{NONE ERROR USER}
@@ -202,7 +207,7 @@ module OpenNebula
             undeploy undeploy-hard hold release stop suspend resume boot delete
             delete-recreate reboot reboot-hard resched unresched poweroff
             poweroff-hard disk-attach disk-detach nic-attach nic-detach
-            snap-create snap-delete}
+            snap-create snap-delete terminate terminate-hard}
 
         EXTERNAL_IP_ATTRS = [
             'GUEST_IP',
@@ -316,6 +321,19 @@ module OpenNebula
         def deploy(host_id, enforce=false, ds_id=-1)
             enforce ||= false
             ds_id ||= -1
+
+            self.info
+
+            # Add dsid as VM template parameter for vcenter
+            if ds_id!=-1 &&
+               !self["/VM/USER_TEMPLATE/PUBLIC_CLOUD/TYPE"].nil? &&
+               self["/VM/USER_TEMPLATE/PUBLIC_CLOUD/TYPE"].downcase == "vcenter"
+                ds = OpenNebula::Datastore.new_with_id(ds_id, @client)
+                rc = ds.info
+                return rc if OpenNebula.is_error?(rc)
+               self.update("VCENTER_DATASTORE=#{ds['/DATASTORE/NAME']}", true)
+            end
+
             return call(VM_METHODS[:deploy],
                         @pe_id,
                         host_id.to_i,
@@ -324,10 +342,12 @@ module OpenNebula
         end
 
         # Shutdowns an already deployed VM
-        def shutdown(hard=false)
-            action(hard ? 'shutdown-hard' : 'shutdown')
+        def terminate(hard=false)
+            action(hard ? 'terminate-hard' : 'terminate')
         end
 
+        alias_method :shutdown, :terminate
+
         # Shuts down an already deployed VM, saving its state in the system DS
         def undeploy(hard=false)
             action(hard ? 'undeploy-hard' : 'undeploy')
@@ -343,16 +363,6 @@ module OpenNebula
             action(hard ? 'reboot-hard' : 'reboot')
         end
 
-        # @deprecated use {#reboot}
-        def reset
-            reboot(true)
-        end
-
-        # @deprecated use {#shutdown}
-        def cancel
-            shutdown(true)
-        end
-
         # Sets a VM to hold state, scheduler will not deploy it
         def hold
             action('hold')
@@ -418,25 +428,6 @@ module OpenNebula
             return call(VM_METHODS[:detachnic], @pe_id, nic_id)
         end
 
-        # Deletes a VM from the pool
-        def delete(recreate=false)
-            if recreate
-                action('delete-recreate')
-            else
-                action('delete')
-            end
-        end
-
-        # @deprecated use {#delete} instead
-        def finalize(recreate=false)
-            delete(recreate)
-        end
-
-        # @deprecated use {#delete} instead
-        def resubmit
-            action('delete-recreate')
-        end
-
         # Sets the re-scheduling flag for the VM
         def resched
             action('resched')
@@ -647,8 +638,8 @@ module OpenNebula
 
         # Recovers an ACTIVE VM
         #
-        # @param result [Integer] Recover with failure (0), success (1) or
-        # retry (2)
+        # @param result [Integer] Recover with failure (0), success (1),
+        # retry (2), delete (3), delete-recreate (4)
         # @param result [info] Additional information needed to recover the VM
         # @return [nil, OpenNebula::Error] nil in case of success, Error
         #   otherwise
@@ -656,6 +647,29 @@ module OpenNebula
             return call(VM_METHODS[:recover], @pe_id, result)
         end
 
+        # Deletes a VM from the pool
+        def delete(recreate=false)
+            if recreate
+                recover(4)
+            else
+                recover(3)
+            end
+        end
+
+		#  Changes the attributes of a VM in power off, failure and undeploy
+		#  states
+		#  @param new_conf, string describing the new attributes. Each attribute
+        #  will replace the existing ones or delete it if empty. Attributes that
+        #  can be updated are: INPUT/{TYPE, BUS}; RAW/{TYPE, DATA, DATA_VMX},
+        #  OS/{BOOT, BOOTLOADER, ARCH, MACHINE, KERNEL, INITRD},
+        #  FEATURES/{ACPI, APIC, PAE, LOCALTIME, HYPERV, GUEST_AGENT},
+        #  and GRAPHICS/{TYPE, LISTEN, PASSWD, KEYMAP}
+        # @return [nil, OpenNebula::Error] nil in case of success, Error
+        #   otherwise
+		def updateconf(new_conf)
+            return call(VM_METHODS[:updateconf], @pe_id, new_conf)
+		end
+
         ########################################################################
         # Helpers to get VirtualMachine information
         ########################################################################
@@ -708,6 +722,7 @@ module OpenNebula
                 self['USER_TEMPLATE/KEEP_DISKS_ON_DONE'].downcase=="yes"
         end
 
+
         # Clones the VM's source Template, replacing the disks with live snapshots
         # of the current disks. The VM capacity and NICs are also preserved
         #
