@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -30,8 +30,8 @@ require 'set'
 require 'nokogiri'
 
 module OneDBFsck
-    VERSION = "4.11.80"
-    LOCAL_VERSION = "4.13.85"
+    VERSION = "4.90.0"
+    LOCAL_VERSION = "4.90.0"
 
     def check_db_version()
         db_version = read_db_version()
@@ -59,9 +59,10 @@ EOT
 
     IMAGE_STATES=%w{INIT READY USED DISABLED LOCKED ERROR CLONE DELETE USED_PERS}
 
-    VM_BIN  = 0x0000001000000000
-    NET_BIN = 0x0000004000000000
-    HOLD    = 0xFFFFFFFF
+    VM_BIN      = 0x0000001000000000
+    NET_BIN     = 0x0000004000000000
+    VROUTER_BIN = 0x0004000000000000
+    HOLD        = 0xFFFFFFFF
 
     def fsck
 
@@ -150,9 +151,10 @@ EOT
         tables = ["group_pool", "user_pool", "acl", "image_pool", "host_pool",
             "network_pool", "template_pool", "vm_pool", "cluster_pool",
             "datastore_pool", "document_pool", "zone_pool", "secgroup_pool",
-            "vdc_pool"]
+            "vdc_pool", "vrouter_pool", "marketplace_pool", "marketplaceapp_pool"]
 
-        federated_tables = ["group_pool", "user_pool", "acl", "zone_pool", "vdc_pool"]
+        federated_tables = ["group_pool", "user_pool", "acl", "zone_pool",
+            "vdc_pool", "marketplace_pool", "marketplaceapp_pool"]
 
         tables.each do |table|
             max_oid = -1
@@ -383,8 +385,8 @@ EOT
             cluster[row[:oid]][:name]       = row[:name]
 
             cluster[row[:oid]][:hosts]      = []
-            cluster[row[:oid]][:datastores] = []
-            cluster[row[:oid]][:vnets]      = []
+            cluster[row[:oid]][:datastores] = Set.new
+            cluster[row[:oid]][:vnets]      = Set.new
         end
 
         hosts_fix       = {}
@@ -440,98 +442,54 @@ EOT
 
             log_time()
 
-            @db.fetch("SELECT oid,body,cid FROM datastore_pool") do |row|
-                doc = Document.new(row[:body])
+            @db.fetch("SELECT oid,body FROM datastore_pool") do |row|
+                doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
-                cluster_id = doc.root.get_text('CLUSTER_ID').to_s.to_i
-                cluster_name = doc.root.get_text('CLUSTER')
+                doc.root.xpath("CLUSTERS/ID").each do |e|
+                    cluster_id = e.text.to_i
 
-                if cluster_id != row[:cid]
-                    log_error("Datastore #{row[:oid]} is in cluster #{cluster_id}, but cid column has cluster #{row[:cid]}")
-                    hosts_fix[row[:oid]] = {:body => row[:body], :cid => cluster_id}
-                end
-
-                if cluster_id != -1
                     cluster_entry = cluster[cluster_id]
 
                     if cluster_entry.nil?
                         log_error("Datastore #{row[:oid]} is in cluster #{cluster_id}, but it does not exist")
 
-                        doc.root.each_element('CLUSTER_ID') do |e|
-                            e.text = "-1"
-                        end
+                        e.remove
 
-                        doc.root.each_element('CLUSTER') do |e|
-                            e.text = ""
-                        end
-
-                        datastores_fix[row[:oid]] = {:body => doc.root.to_s, :cid => -1}
+                        datastores_fix[row[:oid]] = {:body => doc.root.to_s}
                     else
                         cluster_entry[:datastores] << row[:oid]
-
-                        if cluster_name != cluster_entry[:name]
-                            log_error("Datastore #{row[:oid]} has a wrong name for cluster #{cluster_id}, #{cluster_name}. It will be changed to #{cluster_entry[:name]}")
-
-                            doc.root.each_element('CLUSTER') do |e|
-                                e.text = cluster_entry[:name]
-                            end
-
-                            datastores_fix[row[:oid]] = {:body => doc.root.to_s, :cid => cluster_id}
-                        end
                     end
                 end
             end
 
             datastores_fix.each do |id, entry|
-                @db[:datastore_pool].where(:oid => id).update(:body => entry[:body], :cid => entry[:cid])
+                @db[:datastore_pool].where(:oid => id).update(:body => entry[:body])
             end
 
             log_time()
 
-            @db.fetch("SELECT oid,body,cid FROM network_pool") do |row|
-                doc = Document.new(row[:body])
+            @db.fetch("SELECT oid,body FROM network_pool") do |row|
+                doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
 
-                cluster_id = doc.root.get_text('CLUSTER_ID').to_s.to_i
-                cluster_name = doc.root.get_text('CLUSTER')
+                doc.root.xpath("CLUSTERS/ID").each do |e|
+                    cluster_id = e.text.to_i
 
-                if cluster_id != row[:cid]
-                    log_error("VNet #{row[:oid]} is in cluster #{cluster_id}, but cid column has cluster #{row[:cid]}")
-                    hosts_fix[row[:oid]] = {:body => row[:body], :cid => cluster_id}
-                end
-
-                if cluster_id != -1
                     cluster_entry = cluster[cluster_id]
 
                     if cluster_entry.nil?
                         log_error("VNet #{row[:oid]} is in cluster #{cluster_id}, but it does not exist")
 
-                        doc.root.each_element('CLUSTER_ID') do |e|
-                            e.text = "-1"
-                        end
+                        e.remove
 
-                        doc.root.each_element('CLUSTER') do |e|
-                            e.text = ""
-                        end
-
-                        vnets_fix[row[:oid]] = {:body => doc.root.to_s, :cid => -1}
+                        vnets_fix[row[:oid]] = {:body => doc.root.to_s}
                     else
-                        if cluster_name != cluster_entry[:name]
-                            log_error("VNet #{row[:oid]} has a wrong name for cluster #{cluster_id}, #{cluster_name}. It will be changed to #{cluster_entry[:name]}")
-
-                            doc.root.each_element('CLUSTER') do |e|
-                                e.text = cluster_entry[:name]
-                            end
-
-                            vnets_fix[row[:oid]] = {:body => doc.root.to_s, :cid => -1}
-                        end
-
                         cluster_entry[:vnets] << row[:oid]
                     end
                 end
             end
 
             vnets_fix.each do |id, entry|
-                @db[:network_pool].where(:oid => id).update(:body => entry[:body], :cid => entry[:cid])
+                @db[:network_pool].where(:oid => id).update(:body => entry[:body])
             end
         end
 
@@ -577,6 +535,11 @@ EOT
                     end
 
                     ds_new_elem.add_element("ID").text = id.to_s
+
+                    if @db.fetch("SELECT * FROM cluster_datastore_relation WHERE cid=#{cluster_id} AND oid=#{id}").empty?
+                        log_error("Table cluster_datastore_relation is missing relation cluster #{cluster_id}, datastore #{id}")
+                        @db[:cluster_datastore_relation].insert(:cid => cluster_id, :oid => id)
+                    end
                 end
 
                 ds_elem.each_element("ID") do |id_elem|
@@ -597,6 +560,11 @@ EOT
                     end
 
                     vnets_new_elem.add_element("ID").text = id.to_s
+
+                    if @db.fetch("SELECT * FROM cluster_network_relation WHERE cid=#{cluster_id} AND oid=#{id}").empty?
+                        log_error("Table cluster_network_relation is missing relation cluster #{cluster_id}, vnet #{id}")
+                        @db[:cluster_network_relation].insert(:cid => cluster_id, :oid => id)
+                    end
                 end
 
                 vnets_elem.each_element("ID") do |id_elem|
@@ -611,12 +579,45 @@ EOT
             end
         end
 
-        log_time()
-
         # Rename table
         @db.run("DROP TABLE cluster_pool")
         @db.run("ALTER TABLE cluster_pool_new RENAME TO cluster_pool")
 
+        log_time()
+
+        @db.transaction do
+            @db.run "CREATE TABLE cluster_datastore_relation_new (cid INTEGER, oid INTEGER, PRIMARY KEY(cid, oid));"
+
+            @db.fetch("SELECT * from cluster_datastore_relation") do |row|
+                if (cluster[row[:cid]][:datastores].count(row[:oid]) != 1)
+                    log_error("Table cluster_datastore_relation contains relation cluster #{row[:cid]}, datastore #{row[:oid]}, but it should not")
+                else
+                    @db[:cluster_datastore_relation_new].insert(row)
+                end
+            end
+
+            @db.run("DROP TABLE cluster_datastore_relation")
+            @db.run("ALTER TABLE cluster_datastore_relation_new RENAME TO cluster_datastore_relation")
+        end
+
+        log_time()
+
+        @db.transaction do
+            @db.run "CREATE TABLE cluster_network_relation_new (cid INTEGER, oid INTEGER, PRIMARY KEY(cid, oid));"
+
+            @db.fetch("SELECT * from cluster_network_relation") do |row|
+                if (cluster[row[:cid]][:vnets].count(row[:oid]) != 1)
+                    log_error("Table cluster_network_relation contains relation cluster #{row[:cid]}, vnet #{row[:oid]}, but it should not")
+                else
+                    @db[:cluster_network_relation_new].insert(row)
+                end
+            end
+
+            @db.run("DROP TABLE cluster_network_relation")
+            @db.run("ALTER TABLE cluster_network_relation_new RENAME TO cluster_network_relation")
+        end
+
+        log_time()
 
         ########################################################################
         # Datastore
@@ -677,7 +678,7 @@ EOT
 
         log_time()
 
-        @db.run "CREATE TABLE datastore_pool_new (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, cid INTEGER);"
+        @db.run "CREATE TABLE datastore_pool_new (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER);"
 
         @db.transaction do
             @db.fetch("SELECT * from datastore_pool") do |row|
@@ -702,7 +703,7 @@ EOT
 
                 images_elem.each_element("ID") do |id_elem|
                     log_error(
-                        "Image #{id_elem.text} is in Cluster #{ds_id} "<<
+                        "Image #{id_elem.text} is in Datastore #{ds_id} "<<
                         "image id list, but it should not")
                 end
 
@@ -728,6 +729,7 @@ EOT
         counters[:host]  = {}
         counters[:image] = {}
         counters[:vnet]  = {}
+        counters[:vrouter]  = {}
 
         # Initialize all the host counters to 0
         @db.fetch("SELECT oid, name FROM host_pool") do |row|
@@ -745,8 +747,9 @@ EOT
         @db.fetch("SELECT oid,body FROM image_pool") do |row|
             if counters[:image][row[:oid]].nil?
                 counters[:image][row[:oid]] = {
-                    :vms    => Set.new,
-                    :clones => Set.new
+                    :vms        => Set.new,
+                    :clones     => Set.new,
+                    :app_clones => Set.new
                 }
             end
 
@@ -757,8 +760,9 @@ EOT
 
                 if counters[:image][img_id].nil?
                     counters[:image][img_id] = {
-                        :vms    => Set.new,
-                        :clones => Set.new
+                        :vms        => Set.new,
+                        :clones     => Set.new,
+                        :app_clones => Set.new
                     }
                 end
 
@@ -786,7 +790,17 @@ EOT
 
         log_time()
 
+        # Initialize all the vrouter counters to 0
+        @db.fetch("SELECT oid FROM vrouter_pool") do |row|
+            counters[:vrouter][row[:oid]] = {
+                :vms   => Set.new
+            }
+        end
+
+        log_time()
+
         vms_fix = {}
+        cluster_vnc = {}
 
         # Aggregate information of the RUNNING vms
         @db.fetch("SELECT oid,body FROM vm_pool WHERE state<>6") do |row|
@@ -795,6 +809,15 @@ EOT
             state     = vm_doc.root.at_xpath('STATE').text.to_i
             lcm_state = vm_doc.root.at_xpath('LCM_STATE').text.to_i
 
+            # VNC ports per cluster
+            cid = doc.root.at_xpath("HISTORY_RECORDS/HISTORY[last()]/CID").text.to_i rescue nil
+            port = doc.root.at_xpath("TEMPLATE/GRAPHICS[TYPE='vnc']/PORT").text.to_i rescue nil
+
+            if cid && port
+                cluster_vnc[cid] ||= Set.new
+                cluster_vnc[cid] << port
+            end
+
             # Images used by this VM
             vm_doc.root.xpath("TEMPLATE/DISK/IMAGE_ID").each do |e|
                 img_id = e.text.to_i
@@ -825,7 +848,7 @@ EOT
 
                         if ar_id_e.nil?
                             if !counters[:vnet][net_id][:no_ar_leases][mac_s_to_i(mac)].nil?
-                                log_error("VNet has more than one VM with the same MAC address (#{mac}). "<<
+                                log_error("VNet #{net_id} has more than one lease with the same MAC address (#{mac}). "<<
                                     "FSCK can't handle this, and consistency is not guaranteed", false)
                             end
 
@@ -836,7 +859,8 @@ EOT
                                 :ip6_ula    => nic.at_xpath("IP6_ULA").nil? ? nil : nic.at_xpath("IP6_ULA").text,
                                 :mac        => mac,
                                 :vm         => row[:oid],
-                                :vnet       => nil
+                                :vnet       => nil,
+                                :vrouter    => nil
                             }
                         else
                             ar_id = ar_id_e.text.to_i
@@ -852,7 +876,8 @@ EOT
                                     :ip6_ula    => nic.at_xpath("IP6_ULA").nil? ? nil : nic.at_xpath("IP6_ULA").text,
                                     :mac        => mac,
                                     :vm         => row[:oid],
-                                    :vnet       => nil
+                                    :vnet       => nil,
+                                    :vrouter    => nil
                                 }
                             end
                         end
@@ -860,6 +885,21 @@ EOT
                 end
             end
 
+            # See if it's part of a Virtual Router
+            vrouter_e = vm_doc.root.at_xpath("TEMPLATE/VROUTER_ID")
+
+            if !vrouter_e.nil?
+                vr_id = vrouter_e.text.to_i
+                counters_vrouter = counters[:vrouter][vr_id]
+
+                if counters_vrouter.nil?
+                    log_error("VM #{row[:oid]} is part of VRouter #{vr_id}, but "<<
+                        "it does not exist", false)
+                else
+                    counters_vrouter[:vms].add(row[:oid])
+                end
+            end
+
             # Host resources
 
             # Only states that add to Host resources consumption are
@@ -916,6 +956,29 @@ EOT
 
         log_time()
 
+        # VNC Bitmap
+
+        @db.run "DROP TABLE cluster_vnc_bitmap;"
+        @db.run "CREATE TABLE cluster_vnc_bitmap (id INTEGER, map LONGTEXT, PRIMARY KEY(id));"
+
+        size = 65536
+
+        cluster_vnc.keys.sort.each do |cluster_id|
+          map = ""
+          size.times.each do |i|
+              map << (cluster_vnc[cluster_id].include?(size - 1 - i) ? "1" : "0")
+          end
+
+          map_encoded = Base64::strict_encode64(Zlib::Deflate.deflate(map))
+
+          @db[:cluster_vnc_bitmap].insert(
+            :id      => cluster_id,
+            :map     => map_encoded
+          )
+        end
+
+        log_time()
+
         # Bug #4000 may cause history records with etime=0 when they should
         # be closed. The last history can be fixed with the VM etime, but
         # previous history entries need to be fixed manually
@@ -969,6 +1032,125 @@ EOT
 
         log_time()
 
+        ########################################################################
+        # Virtual Routers
+        #
+        # VROUTER/VMS/ID
+        ########################################################################
+
+        vrouters_fix = {}
+
+        # Aggregate information of the RUNNING vms
+        @db.fetch("SELECT oid,body FROM vrouter_pool") do |row|
+            vrouter_doc = Nokogiri::XML(row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
+
+            # VNets used by this Virtual Router
+            vrouter_doc.root.xpath("TEMPLATE/NIC").each do |nic|
+                net_id = nil
+                nic.xpath("NETWORK_ID").each do |nid|
+                    net_id = nid.text.to_i
+                end
+
+                floating = false
+
+                nic.xpath("FLOATING_IP").each do |floating_e|
+                    floating = (floating_e.text.upcase == "YES")
+                end
+
+                if !net_id.nil? && floating
+                    if counters[:vnet][net_id].nil?
+                        log_error("VRouter #{row[:oid]} is using VNet #{net_id}, "<<
+                            "but it does not exist", false)
+                    else
+                        mac = nic.at_xpath("MAC").nil? ? nil : nic.at_xpath("MAC").text
+
+                        ar_id_e = nic.at_xpath('AR_ID')
+
+                        if ar_id_e.nil?
+                            if !counters[:vnet][net_id][:no_ar_leases][mac_s_to_i(mac)].nil?
+                                log_error("VNet #{net_id} has more than one lease with the same MAC address (#{mac}). "<<
+                                    "FSCK can't handle this, and consistency is not guaranteed", false)
+                            end
+
+                            counters[:vnet][net_id][:no_ar_leases][mac_s_to_i(mac)] = {
+                                :ip         => nic.at_xpath("IP").nil? ? nil : nic.at_xpath("IP").text,
+                                :ip6_global => nic.at_xpath("IP6_GLOBAL").nil? ? nil : nic.at_xpath("IP6_GLOBAL").text,
+                                :ip6_link   => nic.at_xpath("IP6_LINK").nil? ? nil : nic.at_xpath("IP6_LINK").text,
+                                :ip6_ula    => nic.at_xpath("IP6_ULA").nil? ? nil : nic.at_xpath("IP6_ULA").text,
+                                :mac        => mac,
+                                :vm         => nil,
+                                :vnet       => nil,
+                                :vrouter    => row[:oid],
+                            }
+                        else
+                            ar_id = ar_id_e.text.to_i
+
+                            if counters[:vnet][net_id][:ar_leases][ar_id].nil?
+                                log_error("VRouter #{row[:oid]} is using VNet #{net_id}, AR #{ar_id}, "<<
+                                    "but the AR does not exist", false)
+                            else
+                                counters[:vnet][net_id][:ar_leases][ar_id][mac_s_to_i(mac)] = {
+                                    :ip         => nic.at_xpath("IP").nil? ? nil : nic.at_xpath("IP").text,
+                                    :ip6_global => nic.at_xpath("IP6_GLOBAL").nil? ? nil : nic.at_xpath("IP6_GLOBAL").text,
+                                    :ip6_link   => nic.at_xpath("IP6_LINK").nil? ? nil : nic.at_xpath("IP6_LINK").text,
+                                    :ip6_ula    => nic.at_xpath("IP6_ULA").nil? ? nil : nic.at_xpath("IP6_ULA").text,
+                                    :mac        => mac,
+                                    :vm         => nil,
+                                    :vnet       => nil,
+                                    :vrouter    => row[:oid],
+                                }
+                            end
+                        end
+                    end
+                end
+            end
+
+            # re-do list of VM IDs
+            error = false
+
+            counters_vrouter = counters[:vrouter][row[:oid]]
+
+            vms_elem = vrouter_doc.root.at_xpath("VMS").remove
+
+            vms_new_elem = vrouter_doc.create_element("VMS")
+            vrouter_doc.root.add_child(vms_new_elem)
+
+            counters_vrouter[:vms].each do |id|
+                id_elem = vms_elem.at_xpath("ID[.=#{id}]")
+
+                if id_elem.nil?
+                    log_error(
+                        "VM #{id} is missing from VRouter #{row[:oid]} VM id list")
+
+                    error = true
+                else
+                    id_elem.remove
+                end
+
+                vms_new_elem.add_child(vrouter_doc.create_element("ID")).content = id.to_s
+            end
+
+            vms_elem.xpath("ID").each do |id_elem|
+                log_error(
+                    "VM #{id_elem.text} is in VRouter #{row[:oid]} VM id list, "<<
+                    "but it should not")
+
+                error = true
+            end
+
+            if (error)
+                vrouters_fix[row[:oid]] = vrouter_doc.root.to_s
+            end
+        end
+
+        @db.transaction do
+            vrouters_fix.each do |id, body|
+                @db[:vrouter_pool].where(:oid => id).update(:body => body)
+            end
+        end
+
+        log_time()
+
         ########################################################################
         # Hosts
         #
@@ -1076,6 +1258,131 @@ EOT
 
         log_time()
 
+        ########################################################################
+        # Marketplace
+        #
+        # MARKETPLACE/MARKETPLACEAPPS/ID
+        ########################################################################
+        # App
+        #
+        # MARKETPLACEAPP/MARKETPLACE_ID
+        # MARKETPLACEAPP/MARKETPLACE
+        # MARKETPLACEAPP/ORIGIN_ID
+        ########################################################################
+
+        marketplace = {}
+
+        @db.fetch("SELECT oid, name FROM marketplace_pool") do |row|
+            marketplace[row[:oid]] = {:name => row[:name], :apps => []}
+        end
+
+        apps_fix = {}
+
+        @db.fetch("SELECT oid,body FROM marketplaceapp_pool") do |row|
+            doc = Document.new(row[:body])
+
+            market_id   = doc.root.get_text('MARKETPLACE_ID').to_s.to_i
+            market_name = doc.root.get_text('MARKETPLACE')
+
+            ####################################################################
+            # TODO, BUG: this code will only work for a standalone oned.
+            # In a federation, the image ID will refer to a different image
+            # in each zone
+            ####################################################################
+
+            origin_id = doc.root.get_text('ORIGIN_ID').to_s.to_i
+            if origin_id >= 0 && doc.root.get_text('STATE').to_s.to_i == 2 # LOCKED
+                counters[:image][origin_id][:app_clones].add(row[:oid])
+            end
+
+            ####################################################################
+            #####################################################################
+
+            if market_id != -1
+                market_entry = marketplace[market_id]
+
+                if market_entry.nil?
+                    log_error("Marketplace App #{row[:oid]} has marketplace #{market_id}, but it does not exist. The app is probably unusable, and needs to be deleted manually:\n"<<
+                        "  * The DB entry can be deleted with the command:\n"<<
+                        "    DELETE FROM marketplaceapp_pool WHERE oid=#{row[:oid]};\n"<<
+                        "  * Run fsck again.\n", false)
+                else
+                    if market_name != market_entry[:name]
+                        log_error("Marketplace App #{row[:oid]} has a wrong name for marketplace #{market_id}, #{market_name}. It will be changed to #{market_entry[:name]}")
+
+                        doc.root.each_element('MARKETPLACE') do |e|
+                            e.text = market_entry[:name]
+                        end
+
+                        apps_fix[row[:oid]] = doc.root.to_s
+                    end
+
+                    market_entry[:apps] << row[:oid]
+                end
+            end
+        end
+
+        if !db_version[:is_slave]
+            @db.transaction do
+                apps_fix.each do |id, body|
+                    @db[:marketplaceapp_pool].where(:oid => id).update(:body => body)
+                end
+            end
+        elsif !apps_fix.empty?
+            log_msg("^ Marketplace App errors need to be fixed in the master OpenNebula")
+        end
+
+        log_time()
+
+        markets_fix = {}
+
+        @db.fetch("SELECT oid,body FROM marketplace_pool") do |row|
+            market_id = row[:oid]
+            doc = Document.new(row[:body])
+
+            apps_elem = doc.root.elements.delete("MARKETPLACEAPPS")
+
+            apps_new_elem = doc.root.add_element("MARKETPLACEAPPS")
+
+            error = false
+
+            marketplace[market_id][:apps].each do |id|
+                id_elem = apps_elem.elements.delete("ID[.=#{id}]")
+
+                if id_elem.nil?
+                    error = true
+
+                    log_error(
+                        "Marketplace App #{id} is missing from Marketplace #{market_id} "<<
+                        "app id list")
+                end
+
+                apps_new_elem.add_element("ID").text = id.to_s
+            end
+
+            apps_elem.each_element("ID") do |id_elem|
+                error = true
+
+                log_error(
+                    "Marketplace App #{id_elem.text} is in Marketplace #{market_id} "<<
+                    "app id list, but it should not")
+            end
+
+            markets_fix[row[:oid]] = doc.root.to_s
+        end
+
+        if !db_version[:is_slave]
+            @db.transaction do
+                markets_fix.each do |id, body|
+                    @db[:marketplace_pool].where(:oid => id).update(:body => body)
+                end
+            end
+        elsif !markets_fix.empty?
+            log_msg("^ Marketplace errors need to be fixed in the master OpenNebula")
+        end
+
+        log_time()
+
         ########################################################################
         # Image
         #
@@ -1084,6 +1391,7 @@ EOT
         #
         # IMAGE/CLONING_OPS
         # IMAGE/CLONES/ID
+        # IMAGE/APP_CLONES/ID
         #
         # IMAGE/CLONING_ID
         #
@@ -1102,8 +1410,10 @@ EOT
                 persistent = ( doc.root.get_text('PERSISTENT').to_s == "1" )
                 current_state = doc.root.get_text('STATE').to_s.to_i
 
-                rvms            = counters[:image][oid][:vms].size
-                n_cloning_ops   = counters[:image][oid][:clones].size
+                counters_img = counters[:image][oid]
+
+                rvms          = counters_img[:vms].size
+                n_cloning_ops = counters_img[:clones].size + counters_img[:app_clones].size
 
                 # rewrite running_vms
                 doc.root.each_element("RUNNING_VMS") {|e|
@@ -1118,7 +1428,7 @@ EOT
 
                 vms_new_elem = doc.root.add_element("VMS")
 
-                counters[:image][oid][:vms].each do |id|
+                counters_img[:vms].each do |id|
                     id_elem = vms_elem.elements.delete("ID[.=#{id}]")
 
                     if id_elem.nil?
@@ -1135,7 +1445,8 @@ EOT
 
                 if ( persistent && rvms > 0 )
                     n_cloning_ops = 0
-                    counters[:image][oid][:clones] = Set.new
+                    counters_img[:clones]     = Set.new
+                    counters_img[:app_clones] = Set.new
                 end
 
                 # Check number of clones
@@ -1151,7 +1462,7 @@ EOT
 
                 clones_new_elem = doc.root.add_element("CLONES")
 
-                counters[:image][oid][:clones].each do |id|
+                counters_img[:clones].each do |id|
                     id_elem = clones_elem.elements.delete("ID[.=#{id}]")
 
                     if id_elem.nil?
@@ -1165,6 +1476,25 @@ EOT
                     log_error("Image #{id_elem.text} is in Image #{oid} CLONES id list, but it should not")
                 end
 
+                # re-do list of Apps cloning this one
+                clones_elem = doc.root.elements.delete("APP_CLONES")
+
+                clones_new_elem = doc.root.add_element("APP_CLONES")
+
+                counters_img[:app_clones].each do |id|
+                    id_elem = clones_elem.elements.delete("ID[.=#{id}]")
+
+                    if id_elem.nil?
+                        log_error("Marketplace App #{id} is missing from Image #{oid} APP_CLONES id list")
+                    end
+
+                    clones_new_elem.add_element("ID").text = id.to_s
+                end
+
+                clones_elem.each_element("ID") do |id_elem|
+                    log_error("Marketplace App #{id_elem.text} is in Image #{oid} APP_CLONES id list, but it should not")
+                end
+
 
                 # Check state
 
@@ -1278,7 +1608,8 @@ EOT
                             :ip6_ula    => nil,
                             :mac        => nil,
                             :vm         => nil,
-                            :vnet       => row[:oid]
+                            :vnet       => row[:oid],
+                            :vrouter    => nil
                         }
 
                         #MAC
@@ -1322,7 +1653,7 @@ EOT
 
 
         # Create a new empty table where we will store the new calculated values
-        @db.run "CREATE TABLE network_pool_new (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, cid INTEGER, pid INTEGER, UNIQUE(name,uid));"
+        @db.run "CREATE TABLE network_pool_new (oid INTEGER PRIMARY KEY, name VARCHAR(128), body MEDIUMTEXT, uid INTEGER, gid INTEGER, owner_u INTEGER, group_u INTEGER, other_u INTEGER, pid INTEGER, UNIQUE(name,uid));"
 
         @db.transaction do
         @db[:network_pool].each do |row|
@@ -1389,7 +1720,8 @@ EOT
                         :ip6_ula    => nil,
                         :mac        => nil,
                         :vm         => nil,
-                        :vnet       => nil
+                        :vnet       => nil,
+                        :vrouter    => nil
                     }
 
                     # MAC
@@ -1432,9 +1764,12 @@ EOT
                     if (binary_magic & VM_BIN != 0)
                         lease[:vm] = lease_oid
                         lease_obj = "VM"
-                    else # binary_magic & NET_BIN != 0
+                    elsif (binary_magic & NET_BIN != 0)
                         lease[:vnet] = lease_oid
                         lease_obj = "VNet"
+                    else #(binary_magic & VROUTER_BIN != 0)
+                        lease[:vrouter] = lease_oid
+                        lease_obj = "VRouter"
                     end
 
                     counter_lease = counter_ar[mac]
@@ -1457,8 +1792,9 @@ EOT
                         if counter_lease != lease
 
                             # Things that can be fixed
-                            if (counter_lease[:vm] != lease[:vm] ||
-                                counter_lease[:vnet] != lease[:vnet])
+                            if (counter_lease[:vm]      != lease[:vm] ||
+                                counter_lease[:vnet]    != lease[:vnet] ||
+                                counter_lease[:vrouter] != lease[:vrouter])
 
                                 new_lease_obj = ""
                                 new_lease_oid = 0
@@ -1470,12 +1806,18 @@ EOT
 
                                     new_binary_magic = (VM_BIN |
                                         (new_lease_oid & 0xFFFFFFFF))
-                                else
+                                elsif !counter_lease[:vnet].nil?
                                     new_lease_obj = "VNet"
                                     new_lease_oid = counter_lease[:vnet].to_i
 
                                     new_binary_magic = (NET_BIN |
                                         (new_lease_oid & 0xFFFFFFFF))
+                                else #if !counter_lease[:vrouter].nil?
+                                    new_lease_obj = "VRouter"
+                                    new_lease_oid = counter_lease[:vrouter].to_i
+
+                                    new_binary_magic = (VROUTER_BIN |
+                                        (new_lease_oid & 0xFFFFFFFF))
                                 end
 
                                 if (lease[:vm] == HOLD)
@@ -1527,12 +1869,18 @@ EOT
 
                         new_binary_magic = (VM_BIN |
                             (new_lease_oid & 0xFFFFFFFF))
-                    else
+                    elsif !counter_lease[:vnet].nil?
                         new_lease_obj = "VNet"
                         new_lease_oid = counter_lease[:vnet].to_i
 
                         new_binary_magic = (NET_BIN |
                             (new_lease_oid & 0xFFFFFFFF))
+                    else #if !counter_lease[:vrouter].nil?
+                        new_lease_obj = "VRouter"
+                        new_lease_oid = counter_lease[:vrouter].to_i
+
+                        new_binary_magic = (VROUTER_BIN |
+                            (new_lease_oid & 0xFFFFFFFF))
                     end
 
                     log_error("VNet #{oid} AR #{ar_id} does not have a lease "<<
@@ -1799,6 +2147,31 @@ EOT
         end
 
 
+        @db.fetch("SELECT body FROM vrouter_pool WHERE #{where_filter}") do |vrouter_row|
+            vrouter_doc = Nokogiri::XML(vrouter_row[:body],nil,NOKOGIRI_ENCODING){|c| c.default_xml.noblanks}
+
+            # VNet quotas
+            vrouter_doc.root.xpath("TEMPLATE/NIC").each { |nic|
+                net_id = nil
+                nic.xpath("NETWORK_ID").each do |nid|
+                    net_id = nid.text
+                end
+
+                floating = false
+
+                nic.xpath("FLOATING_IP").each do |floating_e|
+                    floating = (floating_e.text.upcase == "YES")
+                end
+
+                if !net_id.nil? && floating
+                    vnet_usage[net_id] = 0 if vnet_usage[net_id].nil?
+
+                    vnet_usage[net_id] += 1
+                end
+            }
+
+        end
+
         # VM quotas
 
         vm_elem = nil
@@ -2004,6 +2377,7 @@ EOT
     end
 
     def mac_s_to_i(mac)
+        return nil if mac.empty?
         return mac.split(":").map {|e|
             e.to_i(16).to_s(16).rjust(2,"0")}.join("").to_i(16)
     end
@@ -2015,6 +2389,7 @@ EOT
     end
 
     def ip6_prefix_s_to_i(prefix)
+        return nil if prefix.empty?
         return prefix.split(":", 4).map {|e|
             e.to_i(16).to_s(16).rjust(4, "0")}.join("").to_i(16)
     end
