@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -200,8 +200,7 @@ int PoolObjectSQL::replace_template(
         {
             obj_template->remove_all_except_restricted();
 
-            string aux_error;
-            new_tmpl->merge(obj_template, aux_error);
+            new_tmpl->merge(obj_template);
         }
     }
 
@@ -260,7 +259,8 @@ int PoolObjectSQL::append_template(
     {
         old_tmpl = new Template(*obj_template);
 
-        obj_template->merge(new_tmpl, error);
+        obj_template->merge(new_tmpl);
+
         delete new_tmpl;
     }
     else
@@ -362,7 +362,16 @@ void PoolObjectSQL::get_permissions(PoolObjectAuth& auth)
 
     if(cl != 0)
     {
-        auth.cid = cl->get_cluster_id();
+        auth.cids = cl->get_cluster_ids();
+    }
+    else
+    {
+        ClusterableSingle* cls = dynamic_cast<ClusterableSingle*>(this);
+
+        if(cls != 0)
+        {
+            auth.cids.insert(cls->get_cluster_id());
+        }
     }
 }
 
@@ -526,9 +535,9 @@ string& PoolObjectSQL::lock_db_to_xml(string& xml) const
     int locked_int = locked ? 1 : 0;
 
     oss << "<LOCK>"
-            << "<LOCKED>"           << locked_int   << "</LOCKED>"
-            << "<OWNER><![CDATA["   << lock_owner   << "]]></OWNER>"
-            << "<EXPIRES>"          << lock_expires << "</EXPIRES>"
+            << "<LOCKED>"  << locked_int   << "</LOCKED>"
+            << "<OWNER>"   << one_util::escape_xml(lock_owner) << "</OWNER>"
+            << "<EXPIRES>" << lock_expires << "</EXPIRES>"
         << "</LOCK>";
 
     xml = oss.str();
@@ -545,7 +554,7 @@ int PoolObjectSQL::lock_db_from_xml()
 
     rc += xpath(locked_int,   "/*/LOCK/LOCKED", 0);
     rc += xpath(lock_owner,   "/*/LOCK/OWNER", "");
-    rc += xpath(lock_expires, "/*/LOCK/EXPIRES", 0);
+    rc += xpath<time_t>(lock_expires, "/*/LOCK/EXPIRES", 0);
 
     locked = locked_int;
 
