@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -17,6 +17,7 @@
 #include "RequestManagerVMTemplate.h"
 #include "PoolObjectAuth.h"
 #include "Nebula.h"
+#include "RequestManagerClone.h"
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
@@ -26,11 +27,98 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
 {
     int    id   = xmlrpc_c::value_int(paramList.getInt(1));
     string name = xmlrpc_c::value_string(paramList.getString(2));
-    bool   on_hold = false; //Optional XML-RPC argument
-    string str_uattrs;      //Optional XML-RPC argument
+    bool   on_hold = false;        //Optional XML-RPC argument
+    string str_uattrs;             //Optional XML-RPC argument
+    bool   clone_template = false; //Optional XML-RPC argument
 
-    int  rc;
-    int  vid;
+    if ( paramList.size() > 3 )
+    {
+        on_hold    = xmlrpc_c::value_boolean(paramList.getBoolean(3));
+        str_uattrs = xmlrpc_c::value_string(paramList.getString(4));
+    }
+
+    if ( paramList.size() > 5 )
+    {
+        clone_template = xmlrpc_c::value_boolean(paramList.getBoolean(5));
+    }
+
+    VMTemplate * tmpl = static_cast<VMTemplatePool* > (pool)->get(id,true);
+
+    if ( tmpl == 0 )
+    {
+        att.resp_id = id;
+        failure_response(NO_EXISTS, att);
+        return;
+    }
+
+    bool is_vrouter = tmpl->is_vrouter();
+
+    string original_tmpl_name = tmpl->get_name();
+
+    tmpl->unlock();
+
+    if (is_vrouter)
+    {
+        att.resp_msg = "Virtual router templates cannot be instantiated";
+        failure_response(ACTION, att);
+        return;
+    }
+
+    int instantiate_id = id;
+
+    if (clone_template)
+    {
+        int new_id;
+
+        VMTemplateClone tmpl_clone;
+        string          tmpl_name = name;
+
+        ostringstream   oss;
+
+        if (tmpl_name.empty())
+        {
+            tmpl_name = original_tmpl_name + "-copy";
+        }
+
+        ErrorCode ec = tmpl_clone.request_execute(id, tmpl_name, new_id, true,
+			str_uattrs, att);
+
+        if (ec != SUCCESS)
+        {
+            failure_response(ec, att);
+            return;
+        }
+
+        instantiate_id = new_id;
+
+        oss << "CLONING_TEMPLATE_ID=" << id << "\n";
+
+        str_uattrs = oss.str();
+    }
+
+    int       vid;
+    ErrorCode ec;
+
+    ec = request_execute(instantiate_id, name, on_hold, str_uattrs, 0, vid, att);
+
+    if ( ec == SUCCESS )
+    {
+        success_response(vid, att);
+    }
+    else
+    {
+        failure_response(ec, att);
+    }
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+Request::ErrorCode VMTemplateInstantiate::request_execute(int id, string name,
+        bool on_hold, const string &str_uattrs, Template* extra_attrs, int& vid,
+        RequestAttributes& att)
+{
+    int rc;
 
     ostringstream sid;
 
@@ -39,38 +127,25 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
     Nebula& nd = Nebula::instance();
 
     VirtualMachinePool* vmpool  = nd.get_vmpool();
-    VMTemplatePool *    tpool   = static_cast<VMTemplatePool *>(pool);
+    VMTemplatePool *    tpool   = nd.get_tpool();
 
     VirtualMachineTemplate * tmpl;
     VirtualMachineTemplate * extended_tmpl = 0;
     VirtualMachineTemplate   uattrs;
     VMTemplate *             rtmpl;
 
-    string error_str;
     string aname;
-
     string tmpl_name;
 
-    if ( paramList.size() > 3 )
-    {
-        on_hold = xmlrpc_c::value_boolean(paramList.getBoolean(3));
-
-        str_uattrs = xmlrpc_c::value_string(paramList.getString(4));
-    }
-
     /* ---------------------------------------------------------------------- */
     /* Get, check and clone the template                                      */
     /* ---------------------------------------------------------------------- */
-
     rtmpl = tpool->get(id,true);
 
     if ( rtmpl == 0 )
     {
-        failure_response(NO_EXISTS,
-                get_error(object_name(auth_object),id),
-                att);
-
-        return;
+        att.resp_id = id;
+        return NO_EXISTS;
     }
 
     tmpl_name = rtmpl->get_name();
@@ -80,43 +155,12 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
 
     rtmpl->unlock();
 
-    // Parse & merge user attributes (check if the request user is not oneadmin)
-    if (!str_uattrs.empty())
+    ErrorCode ec = merge(tmpl, str_uattrs, att);
+
+    if (ec != SUCCESS)
     {
-        rc = uattrs.parse_str_or_xml(str_uattrs, error_str);
-
-        if ( rc != 0 )
-        {
-            failure_response(INTERNAL, error_str, att);
-            delete tmpl;
-            return;
-        }
-
-        if (att.uid!=UserPool::ONEADMIN_ID && att.gid!=GroupPool::ONEADMIN_ID)
-        {
-            if (uattrs.check(aname))
-            {
-                ostringstream oss;
-
-                oss << "User Template includes a restricted attribute "<< aname;
-
-                failure_response(AUTHORIZATION,
-                        authorization_error(oss.str(), att),
-                        att);
-
-                delete tmpl;
-                return;
-            }
-        }
-
-        rc = tmpl->merge(&uattrs, error_str);
-
-        if ( rc != 0 )
-        {
-            failure_response(INTERNAL, error_str, att);
-            delete tmpl;
-            return;
-        }
+        delete tmpl;
+        return ec;
     }
 
     /* ---------------------------------------------------------------------- */
@@ -136,11 +180,13 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
         tmpl->set(new SingleAttribute("NAME",name));
     }
 
+    //--------------------------------------------------------------------------
+
     if ( att.uid != 0 )
     {
         AuthRequest ar(att.uid, att.group_ids);
 
-        ar.add_auth(auth_op, perms); //USE TEMPLATE
+        ar.add_auth(AuthRequest::USE, perms); //USE TEMPLATE
 
         if (!str_uattrs.empty())
         {
@@ -149,42 +195,37 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
             tmpl->to_xml(tmpl_str);
 
             // CREATE TEMPLATE
-            ar.add_create_auth(att.uid, att.gid, auth_object, tmpl_str);
+            ar.add_create_auth(att.uid, att.gid, PoolObjectSQL::TEMPLATE, tmpl_str);
         }
 
         VirtualMachine::set_auth_request(att.uid, ar, tmpl);
 
         if (UserPool::authorize(ar) == -1)
         {
-            failure_response(AUTHORIZATION,
-                    authorization_error(ar.message, att),
-                    att);
+            att.resp_msg = ar.message;
 
             delete tmpl;
-            return;
+            return AUTHORIZATION;
         }
 
         extended_tmpl = new VirtualMachineTemplate(*tmpl);
 
         VirtualMachine::disk_extended_info(att.uid, extended_tmpl);
 
-        if ( quota_authorization(extended_tmpl, Quotas::VIRTUALMACHINE, att) == false )
+        if (quota_authorization(extended_tmpl, Quotas::VIRTUALMACHINE, att,
+                    att.resp_msg) == false)
         {
             delete tmpl;
             delete extended_tmpl;
-            return;
+            return AUTHORIZATION;
         }
     }
 
     rc = vmpool->allocate(att.uid, att.gid, att.uname, att.gname, att.umask,
-            tmpl, &vid, error_str, on_hold);
+            tmpl, &vid, att.resp_msg, on_hold);
 
     if ( rc < 0 )
     {
-        failure_response(INTERNAL,
-                allocate_error(PoolObjectSQL::VM,error_str),
-                att);
-
         if (extended_tmpl != 0)
         {
             quota_rollback(extended_tmpl, Quotas::VIRTUALMACHINE, att);
@@ -192,14 +233,50 @@ void VMTemplateInstantiate::request_execute(xmlrpc_c::paramList const& paramList
 
         delete extended_tmpl;
 
-        return;
+        return ALLOCATE;
     }
 
     delete extended_tmpl;
 
-    success_response(vid, att);
+    return SUCCESS;
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+Request::ErrorCode VMTemplateInstantiate::merge(
+                Template *      tmpl,
+                const string    &str_uattrs,
+                RequestAttributes& att)
+{
+    if (str_uattrs.empty())
+    {
+		return SUCCESS;
+	}
+
+	int rc;
+
+	VirtualMachineTemplate  uattrs;
+	string                  aname;
+
+	rc = uattrs.parse_str_or_xml(str_uattrs, att.resp_msg);
+
+	if ( rc != 0 )
+	{
+		return INTERNAL;
+	}
+
+	if (att.uid!=UserPool::ONEADMIN_ID && att.gid!=GroupPool::ONEADMIN_ID)
+	{
+		if (uattrs.check(aname))
+		{
+			att.resp_msg ="User Template includes a restricted attribute " + aname;
+
+			return AUTHORIZATION;
+		}
+	}
+
+	tmpl->merge(&uattrs);
+
+    return SUCCESS;
+}
