@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -70,9 +70,9 @@ public:
         return oid;
     };
 
-    int get_cid() const
+    bool is_in_cluster(int cid) const
     {
-        return cluster_id;
+        return cluster_ids.count(cid) != 0;
     };
 
     /**
@@ -103,7 +103,7 @@ public:
 private:
 
     int oid;
-    int cluster_id;
+    set<int> cluster_ids;
 
     int uid;
     int gid;
