@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -40,12 +40,8 @@ int DatastorePoolXML::load_info(xmlrpc_c::value &result)
 {
     try
     {
-        client->call( client->get_endpoint(),           // serverUrl
-                      "one.datastorepool.info",         // methodName
-                      "s",                              // arguments format
-                      &result,                          // resultP
-                      client->get_oneauth().c_str()     // argument
-                    );
+        client->call("one.datastorepool.info", "", &result);
+
         return 0;
     }
     catch (exception const& e)
