@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,6 +19,7 @@
 #include "DatastoreXML.h"
 #include "NebulaUtil.h"
 #include "NebulaLog.h"
+#include "ObjectCollection.h"
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
@@ -35,8 +36,12 @@ const char * DatastoreXML::ds_paths[] = {
 
 void DatastoreXML::init_attributes()
 {
-    xpath(oid,        "/DATASTORE/ID",          -1);
-    xpath(cluster_id, "/DATASTORE/CLUSTER_ID",  -1);
+    xpath(oid,  "/DATASTORE/ID",          -1);
+
+    ObjectCollection cluster_collection("CLUSTERS");
+    cluster_collection.from_xml(this, "/DATASTORE/");
+
+    cluster_ids = cluster_collection.clone();
 
     xpath(uid,      "/DATASTORE/UID",  -1);
     xpath(gid,      "/DATASTORE/GID",  -1);
@@ -53,16 +58,16 @@ void DatastoreXML::init_attributes()
     xpath(other_m, "/DATASTORE/PERMISSIONS/OTHER_M", 0);
     xpath(other_a, "/DATASTORE/PERMISSIONS/OTHER_A", 0);
 
-    xpath(free_mb,    "/DATASTORE/FREE_MB",     0);
+    xpath<long long>(free_mb, "/DATASTORE/FREE_MB", 0);
 
     long long total_mb, used_mb, limit_mb;
 
-    xpath(total_mb, "/DATASTORE/TOTAL_MB", 0);
-    xpath(used_mb,  "/DATASTORE/USED_MB",  0);
+    xpath<long long>(total_mb, "/DATASTORE/TOTAL_MB", 0);
+    xpath<long long>(used_mb,  "/DATASTORE/USED_MB",  0);
 
     monitored = (free_mb != 0 || total_mb != 0 || used_mb != 0);
 
-    int rc = xpath(limit_mb, "/DATASTORE/TEMPLATE/LIMIT_MB", 0);
+    int rc = xpath<long long>(limit_mb, "/DATASTORE/TEMPLATE/LIMIT_MB", 0);
 
     if (rc == 0)
     {
@@ -125,7 +130,7 @@ void DatastoreXML::get_permissions(PoolObjectAuth& auth)
     auth.oid = oid;
     auth.uid = uid;
     auth.gid = gid;
-    auth.cid = cluster_id;
+    auth.cids = cluster_ids;
 
     auth.owner_u = owner_u;
     auth.owner_m = owner_m;
