@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -74,12 +74,8 @@ int HostPoolXML::load_info(xmlrpc_c::value &result)
 {
     try
     {
-        client->call( client->get_endpoint(),           // serverUrl
-                      "one.hostpool.info",              // methodName
-                      "s",                              // arguments format
-                      &result,                          // resultP
-                      client->get_oneauth().c_str()     // argument
-                    );
+        client->call("one.hostpool.info", "", &result);
+
         return 0;
     }
     catch (exception const& e)
@@ -112,23 +108,20 @@ void HostPoolXML::merge_clusters(ClusterPoolXML * clpool)
 
         cluster_id = host->get_cid();
 
-        if(cluster_id != -1) //ClusterPool::NONE_CLUSTER_ID
+        cluster = clpool->get(cluster_id);
+
+        if(cluster != 0)
         {
-            cluster = clpool->get(cluster_id);
+            nodes.clear();
 
-            if(cluster != 0)
+            cluster->get_nodes("/CLUSTER/TEMPLATE", nodes);
+
+            if (!nodes.empty())
             {
-                nodes.clear();
-
-                cluster->get_nodes("/CLUSTER/TEMPLATE", nodes);
-
-                if (!nodes.empty())
-                {
-                    host->add_node("/HOST", nodes[0], "CLUSTER_TEMPLATE");
-                }
-
-                cluster->free_nodes(nodes);
+                host->add_node("/HOST", nodes[0], "CLUSTER_TEMPLATE");
             }
+
+            cluster->free_nodes(nodes);
         }
     }
 }
