@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -32,20 +32,24 @@ define(function(require) {
     "DONE",
     "FAILED",
     "POWEROFF",
-    "UNDEPLOYED"
+    "UNDEPLOYED",
+    "CLONING",
+    "CLONING_FAILURE"
   ];
 
   var STATES = {
-    INIT       : 0,
-    PENDING    : 1,
-    HOLD       : 2,
-    ACTIVE     : 3,
-    STOPPED    : 4,
-    SUSPENDED  : 5,
-    DONE       : 6,
-    //FAILED   : 7,
-    POWEROFF   : 8,
-    UNDEPLOYED : 9
+    INIT            : 0,
+    PENDING         : 1,
+    HOLD            : 2,
+    ACTIVE          : 3,
+    STOPPED         : 4,
+    SUSPENDED       : 5,
+    DONE            : 6,
+    //FAILED        : 7,
+    POWEROFF        : 8,
+    UNDEPLOYED      : 9,
+    CLONING         : 10,
+    CLONING_FAILURE : 11
   };
 
   var LCM_STATES_STR = [
@@ -107,8 +111,9 @@ define(function(require) {
     "DISK_SNAPSHOT_REVERT_SUSPENDED",
     "DISK_SNAPSHOT_DELETE_SUSPENDED",
     "DISK_SNAPSHOT",
-    "DISK_SNAPSHOT_REVERT",
     "DISK_SNAPSHOT_DELETE",
+    "PROLOG_MIGRATE_UNKNOWN",
+    "PROLOG_MIGRATE_UNKNOWN_FAILURE",
   ];
 
   var LCM_STATES = {
@@ -170,8 +175,10 @@ define(function(require) {
     DISK_SNAPSHOT_REVERT_SUSPENDED  : 55,
     DISK_SNAPSHOT_DELETE_SUSPENDED  : 56,
     DISK_SNAPSHOT                   : 57,
-    DISK_SNAPSHOT_REVERT            : 58,
-    DISK_SNAPSHOT_DELETE            : 59
+    //DISK_SNAPSHOT_REVERT          : 58,
+    DISK_SNAPSHOT_DELETE            : 59,
+    PROLOG_MIGRATE_UNKNOWN          : 60,
+    PROLOG_MIGRATE_UNKNOWN_FAILURE  : 61
   };
 
   var SHORT_LCM_STATES_STR = [
@@ -235,6 +242,8 @@ define(function(require) {
     Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT
     Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_REVERT
     Locale.tr("SNAPSHOT"),  // DISK_SNAPSHOT_DELETE
+    Locale.tr("MIGRATE"),   // PROLOG_MIGRATE_UNKNOWN
+    Locale.tr("FAILURE"),   // PROLOG_MIGRATE_UNKNOWN_FAILURE
   ];
 
   var VNC_STATES = [
@@ -261,6 +270,15 @@ define(function(require) {
     'SL_PRIMARYIPADDRESS'
   ];
 
+  var NIC_IP_ATTRS = [
+    "IP",
+    "IP6_GLOBAL",
+    "IP6_ULA",
+    "VROUTER_IP",
+    "VROUTER_IP6_GLOBAL",
+    "VROUTER_IP6_ULA"
+  ];
+
   var EXTERNAL_NETWORK_ATTRIBUTES = [
     'GUEST_IP',
     'GUEST_IP_ADDRESSES',
@@ -280,33 +298,35 @@ define(function(require) {
   ];
 
   var MIGRATE_ACTION_STR = [
-    "none",
-    "migrate",
-    "live-migrate",
-    "shutdown",
-    "shutdown-hard",
-    "undeploy",
-    "undeploy-hard",
-    "hold",
-    "release",
-    "stop",
-    "suspend",
-    "resume",
-    "boot",
-    "delete",
-    "delete-recreate",
-    "reboot",
-    "reboot-hard",
-    "resched",
-    "unresched",
-    "poweroff",
-    "poweroff-hard",
-    "disk-attach",
-    "disk-detach",
-    "nic-attach",
-    "nic-detach",
-    "snap-create",
-    "snap-delete"
+    "none",                // NONE_ACTION            = 0
+    "migrate",             // MIGRATE_ACTION         = 1
+    "live-migrate",        // LIVE_MIGRATE_ACTION    = 2
+    "shutdown",            // //SHUTDOWN_ACTION        = 3
+    "shutdown-hard",       // //SHUTDOWN_HARD_ACTION   = 4
+    "undeploy",            // UNDEPLOY_ACTION        = 5
+    "undeploy-hard",       // UNDEPLOY_HARD_ACTION   = 6
+    "hold",                // HOLD_ACTION            = 7
+    "release",             // RELEASE_ACTION         = 8
+    "stop",                // STOP_ACTION            = 9
+    "suspend",             // SUSPEND_ACTION         = 10
+    "resume",              // RESUME_ACTION          = 11
+    "boot",                // //BOOT_ACTION            = 12
+    "delete",              // DELETE_ACTION          = 13
+    "delete-recreate",     // DELETE_RECREATE_ACTION = 14
+    "reboot",              // REBOOT_ACTION          = 15
+    "reboot-hard",         // REBOOT_HARD_ACTION     = 16
+    "resched",             // RESCHED_ACTION         = 17
+    "unresched",           // UNRESCHED_ACTION       = 18
+    "poweroff",            // POWEROFF_ACTION        = 19
+    "poweroff-hard",       // POWEROFF_HARD_ACTION   = 20
+    "disk-attach",         // DISK_ATTACH_ACTION     = 21
+    "disk-detach",         // DISK_DETACH_ACTION     = 22
+    "nic-attach",          // NIC_ATTACH_ACTION      = 23
+    "nic-detach",          // NIC_DETACH_ACTION      = 24
+    "snap-create",         // DISK_SNAPSHOT_CREATE_ACTION = 25
+    "snap-delete",         // DISK_SNAPSHOT_DELETE_ACTION = 26
+    "terminate",           // TERMINATE_ACTION       = 27
+    "terminate-hard"       // TERMINATE_HARD_ACTION  = 28
   ];
 
   var VM = {
@@ -314,9 +334,6 @@ define(function(require) {
     "create": function(params) {
       OpenNebulaAction.create(params, RESOURCE);
     },
-    "del": function(params) {
-      OpenNebulaAction.del(params, RESOURCE);
-    },
     "list": function(params) {
       OpenNebulaAction.list(params, RESOURCE);
     },
@@ -333,13 +350,13 @@ define(function(require) {
       var action_obj = params.data.extra_param;
       OpenNebulaAction.simple_action(params, RESOURCE, "chmod", action_obj);
     },
-    "shutdown": function(params) {
+    "terminate": function(params) {
       var action_obj = {"hard": false};
-      OpenNebulaAction.simple_action(params, RESOURCE, "shutdown", action_obj);
+      OpenNebulaAction.simple_action(params, RESOURCE, "terminate", action_obj);
     },
-    "shutdown_hard" : function(params) {
+    "terminate_hard" : function(params) {
       var action_obj = {"hard": true};
-      OpenNebulaAction.simple_action(params, RESOURCE, "shutdown", action_obj);
+      OpenNebulaAction.simple_action(params, RESOURCE, "terminate", action_obj);
     },
     "hold": function(params) {
       OpenNebulaAction.simple_action(params, RESOURCE, "hold");
@@ -359,9 +376,6 @@ define(function(require) {
     "resume": function(params) {
       OpenNebulaAction.simple_action(params, RESOURCE, "resume");
     },
-    "resubmit": function(params) {
-      OpenNebulaAction.simple_action(params, RESOURCE, "resubmit");
-    },
     "poweroff" : function(params) {
       var action_obj = {"hard": false};
       OpenNebulaAction.simple_action(params, RESOURCE, "poweroff", action_obj);
@@ -378,11 +392,13 @@ define(function(require) {
       var action_obj = {"hard": true};
       OpenNebulaAction.simple_action(params, RESOURCE, "undeploy", action_obj);
     },
-    "reboot" : function(params) {
-      OpenNebulaAction.simple_action(params, RESOURCE, "reboot");
+    "reboot": function(params) {
+      var action_obj = {"hard": false};
+      OpenNebulaAction.simple_action(params, RESOURCE, "reboot", action_obj);
     },
-    "reset" : function(params) {
-      OpenNebulaAction.simple_action(params, RESOURCE, "reset");
+    "reboot_hard" : function(params) {
+      var action_obj = {"hard": true};
+      OpenNebulaAction.simple_action(params, RESOURCE, "reboot", action_obj);
     },
 
     "log": function(params) {
@@ -450,10 +466,18 @@ define(function(require) {
         }
       });
     },
+    "append": function(params) {
+      var action_obj = {"template_raw" : params.data.extra_param, append : true};
+      OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
+    },
     "update": function(params) {
       var action_obj = {"template_raw" : params.data.extra_param};
       OpenNebulaAction.simple_action(params, RESOURCE, "update", action_obj);
     },
+    "updateconf": function(params) {
+      var action_obj = {"template_raw" : params.data.extra_param};
+      OpenNebulaAction.simple_action(params, RESOURCE, "updateconf", action_obj);
+    },
     "monitor" : function(params) {
       OpenNebulaAction.monitor(params, RESOURCE, false);
     },
@@ -530,6 +554,7 @@ define(function(require) {
         case LCM_STATES.BOOT_STOPPED_FAILURE:
         case LCM_STATES.PROLOG_RESUME_FAILURE:
         case LCM_STATES.PROLOG_UNDEPLOY_FAILURE:
+        case LCM_STATES.PROLOG_MIGRATE_UNKNOWN_FAILURE:
           return true;
 
         default:
@@ -583,7 +608,7 @@ define(function(require) {
   function isNICGraphsSupported(element) {
     var history = retrieveLastHistoryRecord(element)
     if (history) {
-      return $.inArray(history.VMMMAD, ['vcenter', 'az', 'sl']) == -1;
+      return $.inArray(history.VM_MAD, ['vcenter', 'az']) == -1;
     } else {
       return false;
     }
@@ -592,7 +617,7 @@ define(function(require) {
   function isNICAttachSupported(element) {
     var history = retrieveLastHistoryRecord(element)
     if (history) {
-      return $.inArray(history.VMMMAD, ['ec2', 'az', 'sl']) == -1;
+      return $.inArray(history.VM_MAD, ['ec2', 'az']) == -1;
     } else {
       return false;
     }
@@ -662,17 +687,11 @@ define(function(require) {
         }
 
         $.each(nic, function(index, value) {
-          if (value.IP) {
-            ips.push(value.IP);
-          }
-
-          if (value.IP6_GLOBAL) {
-            ips.push(value.IP6_GLOBAL);
-          }
-
-          if (value.IP6_ULA) {
-            ips.push(value.IP6_ULA);
-          }
+          $.each(NIC_IP_ATTRS, function(j, attr){
+            if (value[attr]) {
+              ips.push(value[attr]);
+            }
+          });
         });
       }
     }
