@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -16,17 +16,10 @@
 
 define(function(require) {
   require('jquery');
-  var OpenNebulaSystem = require('opennebula/system');
 
   // Clone the local config object in a private var
   var _config = $.extend(true, {}, config);
 
-  var _defaultCost = {
-    cpuCost    : 0,
-    memoryCost : 0,
-    diskCost   : 0
-  };
-
   var Config = {
     'isTabEnabled': function(tabName) {
       var enabled = _config['view']['enabled_tabs'].indexOf(tabName) != -1;
@@ -62,7 +55,7 @@ define(function(require) {
         if (_config['view']['tabs'][tabName]['provision_tabs']) {
           return _config['view']['tabs'][tabName]['provision_tabs'][panelTabName];
         } else {
-          // if provision_tabs is not defined use panel_tabs. 
+          // if provision_tabs is not defined use panel_tabs.
           // This attribute was used in before 4.14, provision_tabs was include in 4.14.2
           return _config['view']['tabs'][tabName]['panel_tabs'][panelTabName];
         }
@@ -149,31 +142,9 @@ define(function(require) {
     'vncWSS': _config['user_config']['vnc_wss'],
     'requestVNCPassword': _config['system_config']['vnc_request_password'],
     'logo': (_config['view']["small_logo"] || "images/one_small_logo.png"),
+    'vmLogos': (_config['vm_logos']),
     'enabledTabs': _config['view']['enabled_tabs'],
-    "defaultCost" : _defaultCost,
-    "initDefaultCost" : function() {
-      OpenNebulaSystem.onedconf({
-        data : {},
-        timeout: true,
-        success: function (request, onedconf){
-          if (onedconf.DEFAULT_COST != undefined){
-            if (onedconf.DEFAULT_COST.CPU_COST != undefined){
-              _defaultCost.cpuCost = parseInt(onedconf.DEFAULT_COST.CPU_COST);
-            }
-            if (onedconf.DEFAULT_COST.MEMORY_COST != undefined){
-              _defaultCost.memoryCost = parseInt(onedconf.DEFAULT_COST.MEMORY_COST);
-            }
-            if (onedconf.DEFAULT_COST.DISK_COST != undefined){
-              _defaultCost.diskCost = parseInt(onedconf.DEFAULT_COST.DISK_COST);
-            }
-          }
-        },
-        error: function(request, error_json){
-          console.error("There was an error requesting oned.conf: "+
-                        error_json.error.message);
-        }
-      });
-    }
+    'onedConf': _config['oned_conf']
   }
 
   return Config;
