@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -16,9 +16,9 @@
 
 define(function(require) {
   require('jquery');
-  require('foundation.reveal');
-  require('foundation.tab');
-  require('foundation.dropdown');
+  //require('foundation.reveal');
+  //require('foundation.tab');
+  //require('foundation.dropdown');
 
   var Config = require('sunstone-config');
   var Locale = require('utils/locale');
@@ -58,6 +58,11 @@ define(function(require) {
         _addPanelsHooks(_tabId, panelsHooks);
       }
 
+      var initHooks = tabObj.initHooks;
+      if (initHooks) {
+        _addInitHooks(_tabId, initHooks);
+      }
+
       var dialogs = tabObj.dialogs;
       if (dialogs) {
         _addDialogs(dialogs)
@@ -89,6 +94,11 @@ define(function(require) {
     return false;
   }
 
+  var _addInitHooks = function(tabId, hooks) {
+    SunstoneCfg["tabs"][tabId]['initHooks'] = hooks;
+    return false;
+  }
+
   var _addPanels = function(tabId, panels) {
     var indexedPanels = {}
     $.each(panels, function(index, panel) {
@@ -115,6 +125,14 @@ define(function(require) {
       _insertButtonsInTab(tabName);
       _setupDataTable(tabName);
 
+      var hooks = SunstoneCfg['tabs'][tabName].initHooks;
+
+      if (hooks) {
+        $.each(hooks, function(i, hook){
+          hook.init();
+        });
+      }
+
       // TODO Add openenbula actions
       /*if (config['view']['autorefresh']) {
         var tabContext = $("#" + tabName);
@@ -131,8 +149,16 @@ define(function(require) {
   }
 
   var _setupDataTable = function(tabName) {
-    if (SunstoneCfg['tabs'][tabName].dataTable) {
-      SunstoneCfg['tabs'][tabName].dataTable.initialize();
+    var dataTable = SunstoneCfg['tabs'][tabName].dataTable;
+    if (dataTable) {
+      dataTable.initialize();
+      if (dataTable.labelsColumn) {
+        $('#' + tabName + 'labels_buttons').html(
+          '<button type="button" data-toggle="' + tabName + 'LabelsDropdown" class="only-sunstone-info only-sunstone-list top_button secondary button dropdown">' +
+            '<i class="fa fa-tags"/></button>' +
+          '<div id="' + tabName + 'LabelsDropdown" class="only-sunstone-info only-sunstone-list labels-dropdown dropdown-pane menu vertical" data-dropdown data-close-on-click="true"></div>').foundation();
+
+      }
     }
   }
 
@@ -149,26 +175,50 @@ define(function(require) {
     if (condition && !condition()) {return;}
 
     if (tabInfo.no_content === true) {
-      tabClass += " tab_with_no_content"
+      tabClass += " tab_with_no_content is-accordion-submenu-parent"
     } else {
       tabInfo['tabName'] = tabName;
       var TabTemplate = require('hbs!sunstone/tab')
-      $('div.right-content').append(TabTemplate(tabInfo));
+      $('div.sunstone-content').append(TabTemplate(tabInfo));
     }
 
-    var liItem = '<li id="li_' + tabName + '" class="' + tabClass + ' ' + parent + '"><a href="#">' + tabInfo.title + '</a></li>';
+    var liItem;
+    var title = '';
+    if (tabInfo.icon) {
+      title += '<i class="fa fa-lg fa-fw ' + tabInfo.icon + '"></i> ';
+    }
+    title += tabInfo.title;
 
-    $('div#menu ul#navigation').append(liItem);
+    if (parent !== '') {
+      liItem = '<li id="li_' + tabName + '" class="' + tabClass + '">' + 
+              '<a href="#">' + title + '</a>' + 
+            '</li>';
+
+      if ($('#menu ul#navigation #li_' + parent + ' .menu').length > 0) {
+        $('#menu ul#navigation #li_' + parent + ' .menu').append(liItem);
+      } else {
+        $('#menu ul#navigation #li_' + parent).append(
+            '<ul class="menu vertical nested" data-submenu>' +
+              liItem +
+            '</ul>')
+      }
+    } else {
+      liItem = '<li id="li_' + tabName + '" class="' + tabClass + '">' + 
+              '<a href="#">' + title + '</a>' + 
+            '</li>';
+
+      $('div#menu ul#navigation').append(liItem);
+    }
 
     //if this is a submenu...
-    if (parent.length) {
-      var children = $('div#menu ul#navigation #li_' + parent);
-      //if there are other submenus, insert after last of them
-      if (children.length) {
-        $('div#menu li#li_' + tabName).hide();//hide by default
-        $('div#menu li#li_' + parent + ' span').css("display", "inline-block");
-      }
-    };
+    //if (parent.length) {
+    //  var children = $('div#menu ul#navigation #li_' + parent);
+    //  //if there are other submenus, insert after last of them
+    //  if (children.length) {
+    //    $('div#menu li#li_' + tabName).hide();//hide by default
+    //    $('div#menu li#li_' + parent + ' span').css("display", "inline-block");
+    //  }
+    //};
 
     if (tabInfo.forms) {
       $.each(tabInfo.forms, function(key, value) {
@@ -177,7 +227,7 @@ define(function(require) {
     }
 
     if (tabInfo.setup) {
-      var context = $('div#' + tabName, $('div.right-content'));
+      var context = $('div#' + tabName, $('div.sunstone-content'));
 
       tabInfo.setup(context);
     }
@@ -196,7 +246,7 @@ define(function(require) {
       context = customContext;
     } else {
       customId = tabName;
-      context = $('div#' + tabName, $('div.right-content'));
+      context = $('div#' + tabName, $('div.sunstone-content'));
     }
 
     var actionBlock = $('div.action_blocks', context)
@@ -247,14 +297,14 @@ define(function(require) {
           buttonContext = $("#" + customId + "create_buttons", buttonsRow);
           icon = button.icon ? button.icon : '<i class="fa fa-plus"/>';
           text = button.text ? icon + ' ' + button.text : icon;
-          strClass.push("success", "button", "small", "radius");
+          strClass.push("success", "button");
           buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
           break;
         case "refresh":
           buttonContext = $("#" + customId + "refresh_buttons", buttonsRow);
-          icon = button.icon ? button.icon : '<i class="fa fa-refresh fa-lg"/>';
+          icon = button.icon ? button.icon : '<i class="fa fa-refresh"/>';
           text = button.text ? icon + ' ' + button.text : icon;
-          strClass.push("white_button", "refresh", "secondary", "button", "small", "radius");
+          strClass.push("refresh", "button",  "secondary");
           buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
           break;
         case "top":
@@ -263,19 +313,19 @@ define(function(require) {
               '<i class="fa fa-refresh fa-stack-2x" style="color: #dfdfdf"></i>' +
               '<i class="fa fa-play fa-stack-1x"></i>' +
             '</span>';
-          strClass.push("white_button", "toggle_top_button", "only-right-list", "secondary", "button", "small", "radius");
+          strClass.push("toggle_top_button", "only-sunstone-list", "button",  "hollow");
           buttonCode = '<a class="' + strClass.join(' ') + '" style="padding-left:0px; margin-right: 20px">' + text + '</a>';
           break;
         case "main":
           buttonContext = $("#" + customId + "main_buttons", buttonsRow);
           text = button.text;
-          strClass.push("secondary", "button", "small", "radius");
+          strClass.push("button");
           buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
           break;
         case "vmsplay_buttons":
           buttonContext = $("#" + customId + "vmsplay_buttons", buttonsRow);
           text = button.text;
-          strClass.push("secondary", "button", "small", "radius");
+          strClass.push("button");
           buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
           break;
         case "vmspause_buttons":
@@ -316,13 +366,13 @@ define(function(require) {
         case "del":
           buttonContext = $("#" + customId + "delete_buttons", buttonsRow);
           text = '<i class=" fa fa-trash-o"/> ';
-          strClass.push("alert", "button", "small", "radius");
+          strClass.push("alert", "button");
           buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
           break;
         default:
           buttonContext = $("#" + customId + "main_buttons", buttonsRow);
           text = button.text;
-          strClass.push("secondary", "button", "small", "radius");
+          strClass.push("button");
           buttonCode = '<button class="' + strClass.join(' ') + '" href="' + buttonName + '">' + text + '</button>';
         }
 
@@ -332,37 +382,39 @@ define(function(require) {
       //$('.top_button',actionBlock).addClass("secondary small button")
 
       actionBlock.append(buttonsRow);
+      //actionBlock.foundation();
+      Foundation.reflow(actionBlock, 'dropdown');
 
       if ($("#" + customId + "more_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "more_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "more_buttons]", actionBlock).remove()
       }
 
       if ($("#" + customId + "user_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "user_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "user_buttons]", actionBlock).remove()
       }
 
       if ($("#" + customId + "vmsplanification_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "vmsplanification_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "vmsplanification_buttons]", actionBlock).remove()
       }
 
       if ($("#" + customId + "vmsdelete_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "vmsdelete_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "vmsdelete_buttons]", actionBlock).remove()
       }
 
       if ($("#" + customId + "vmsstop_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "vmsstop_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "vmsstop_buttons]", actionBlock).remove()
       }
 
       if ($("#" + customId + "vmspause_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "vmspause_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "vmspause_buttons]", actionBlock).remove()
       }
 
       if ($("#" + customId + "vmsrepeat_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "vmsrepeat_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "vmsrepeat_buttons]", actionBlock).remove()
       }
 
       if ($("#" + customId + "user_buttons li", actionBlock).length == 0) {
-        $("button[data-dropdown=" + customId + "user_buttons]", actionBlock).remove()
+        $("button[data-toggle=" + customId + "user_buttons]", actionBlock).remove()
       }
       //actionBlock.foundationButtons();
       $('.top_button, .list_button', actionBlock).prop('disabled', false);
@@ -375,12 +427,10 @@ define(function(require) {
         return false;
       })
 
-      $('#' + customId + 'submit_button', actionBlock).on("click", function() {
+      $('.submit_button', actionBlock).on("click", function() {
         _submitFormPanel(tabName);
         return false;
       })
-
-      $(document).foundation();
     }//if tab exists
   }
 
@@ -414,12 +464,6 @@ define(function(require) {
         error = _runAction(value);
       }
 
-      if (!error && !$(this).hasClass("refresh")) {
-        //proceed to close confirm dialog in
-        //case it was open
-        _getDialogInstance(CONFIRM_DIALOG_ID).hide();
-      };
-
       return false;
     });
 
@@ -437,30 +481,38 @@ define(function(require) {
     //with a select box before running the action.
     $(document).on("click", '.confirm_with_select_button', function() {
       var dialogInstance = _getDialogInstance(CONFIRM_WITH_SELECT_DIALOG_ID);
-      $('#' + CONFIRM_WITH_SELECT_DIALOG_ID).data('buttonAction', $(this).attr('href'));
-      $('#' + CONFIRM_WITH_SELECT_DIALOG_ID).data('buttonTab', $(this).parents('.tab').attr('id'));
+
+      dialogInstance.setParams({
+        'buttonAction' : $(this).attr('href'),
+        'buttonTab' : $(this).parents('.tab').attr('id')
+      });
+
+      dialogInstance.reset();
       dialogInstance.show();
+
       return false;
     });
 
-    $(document).foundation('dropdown', 'reflow');
+    //$(document).foundation('dropdown', 'reflow');
 
     // Button to return to the list view from the detailed view
-    $(document).on("click", "a[href='back']", function(e) {
+    $(document).on("click", "button[href='back']", function(e) {
       $(".navigation-active-li a", $("#navigation")).click();
       e.preventDefault();
     });
   }
 
   var _setupTabs = function() {
-    var topTabs = $(".left-content ul li.topTab");
-    var subTabs = $(".left-content ul li.subTab");
+    Foundation.reflow($('#menu'), 'accordion-menu');
+    Foundation.reflow($('div.sunstone-content'), 'sticky')
+    var topTabs = $(".sunstone-menu-content ul li.topTab");
+    var subTabs = $(".sunstone-menu-content ul li.subTab > a");
 
     subTabs.on("click", function() {
-      if ($(this).hasClass('topTab')) {
+      if ($(this).closest('li').hasClass('topTab')) {
         return false;
       } else {
-        var tabName = $(this).attr('id').substring(3);
+        var tabName = $(this).closest('li').attr('id').substring(3);
         _showTab(tabName);
         return false;
       }
@@ -471,8 +523,8 @@ define(function(require) {
 
       if ($(this).hasClass("tab_with_no_content")) {
         //Subtabs have a class with the name of  this tab
-        var subtabs = $('div#menu li.' + tabName);
-        subtabs.fadeToggle('fast');
+        //var subtabs = $('div#menu li.' + tabName);
+        //subtabs.fadeToggle('fast');
         return false;
       } else {
         _showTab(tabName);
@@ -488,12 +540,13 @@ define(function(require) {
     $(".tab").hide();
     tab.show();
 
-    $(".right-info", tab).hide();
-    $(".right-form", tab).hide();
-    $(".right-list", tab).show();
-    $(".only-right-info", tab).hide();
-    $(".only-right-form", tab).hide();
-    $(".only-right-list", tab).show();
+    $(".sunstone-info", tab).hide();
+    $(".sunstone-form", tab).hide();
+    $(".sunstone-list", tab).fadeIn();
+    $(".only-sunstone-info", tab).hide();
+    $(".only-sunstone-form", tab).hide();
+    $(".only-sunstone-list", tab).fadeIn();
+    $('.action_blocks', tab).removeClass('large-12').addClass('large-9');
   };
 
   var _showRighInfo = function(tabName) {
@@ -501,15 +554,18 @@ define(function(require) {
     $(".tab").hide();
     tab.show();
 
-    $(".right-list", tab).hide();
-    $(".right-form", tab).hide();
-    $(".right-info", tab).show();
-    $(".only-right-list", tab).hide();
-    $(".only-right-form", tab).hide();
-    $(".only-right-info", tab).show();
+    $(".sunstone-list", tab).hide();
+    $(".sunstone-form", tab).hide();
+    $(".sunstone-info", tab).fadeIn();
+    $(".only-sunstone-list", tab).hide();
+    $(".only-sunstone-form", tab).hide();
+    $(".only-sunstone-info", tab).fadeIn();
+    $('.action_blocks', tab).removeClass('large-9').addClass('large-12');
   }
 
   var _showTab = function(tabName) {
+    $('.labels-tree', '#navigation').remove();
+
     if (!SunstoneCfg['tabs'][tabName]) {
       return false;
     }
@@ -534,6 +590,9 @@ define(function(require) {
 
     var dataTable = SunstoneCfg['tabs'][tabName]['dataTable'];
     if (dataTable) {
+      if (dataTable.clearLabelsFilter) {
+        dataTable.clearLabelsFilter();
+      }
       dataTable.recountCheckboxes();
     }
 
@@ -552,7 +611,7 @@ define(function(require) {
     $(".resource-info-header", context).text("");
 
     var loading = '<div style="margin-top: 20px; text-align: center; width: 100%"><img src="images/pbar.gif" alt="loading..." /></div>';
-    $(".right-info", context).html(loading);
+    $(".sunstone-info", context).html(loading);
     _showRighInfo(tabName);
 
     Sunstone.runAction(infoAction, elementId);
@@ -560,10 +619,19 @@ define(function(require) {
     $('.top_button, .list_button', context).attr('disabled', false);
   }
 
+  // Returns the element that is currently shown in the right info
+  var _getElementRightInfo = function(tabName, context) {
+    var context = context || $(".sunstone-info", $("#" + tabName));
+    return context.data('element');
+  }
+
   var _insertPanels = function(tabName, info, contextTabId, context) {
-    var context = context || $(".right-info", $("#" + tabName));
+    var context = context || $(".sunstone-info", $("#" + tabName));
+
+    context.data('element', info[Object.keys(info)[0]]);
+
     var containerId = tabName + '-panels';
-    var activaTab = $("dd.active a", $("#" + containerId));
+    var activaTab = $("li.is-active a", $("#" + containerId));
     if (activaTab) {
       var activaTabHref = activaTab.attr('href');
     }
@@ -629,24 +697,26 @@ define(function(require) {
     })
 
     context.html(html);
-
     $.each(SunstoneCfg['tabs'][tabName]["panelInstances"], function(panelName, panel) {
       panel.setup(context);
 
       if(isRefresh && prevPanelStates[panelName] && panel.setState){
         panel.setState( prevPanelStates[panelName], context );
       }
+    });
 
-      if (panel.onShow) {
-        context.off('click', '[href="#' + panel.panelId + '"]');
-        context.on('click', '[href="#' + panel.panelId + '"]', function(){
-          panel.onShow(context);
-        });
+    $('#' + containerId + 'Tabs', context).on('change.zf.tabs', function(target) {
+      var tabIdWithHash = $('.is-active > a', this)[0].hash;
+      var panel = SunstoneCfg['tabs'][tabName]["panelInstances"][tabIdWithHash.substring(1)];
+      if (panel && panel.onShow) {
+        panel.onShow(context);
       }
     });
 
-    context.foundation('tab', 'reflow');
-    $('[href=' + activaTabHref + ']', context).trigger("click");
+    Foundation.reflow(context, 'tabs');
+
+    //context.foundation();
+    $('[href="' + activaTabHref + '"]', context).trigger("click");
 
     if (hooks) {
       $.each(hooks, function(i, hook){
@@ -758,7 +828,6 @@ define(function(require) {
   function _showFormPanel(tabId, formPanelId, action, onShow2) {
     var context = $("#" + tabId);
     _popFormPanelLoading(tabId);
-    _enableFormPanelSubmit(tabId);
     _showFormPanelSubmit(tabId);
 
     setTimeout(function() {
@@ -769,9 +838,11 @@ define(function(require) {
 
       if (!formPanelInstance) {
         formContext =
-        $('<div class="tabs-content tabs-contentForm" form-panel-id="'+formPanelId+'">\
-          <div class="wizardForms content active" id="'+tab.tabName+'-wizardForms"></div>\
-          <div class="advancedForms content" id="'+tab.tabName+'-advancedForms"></div>\
+        $('<div class="tabs-content tabs-contentForm" ' +
+                'data-tabs-content="' + tab.tabName + 'FormTabs" ' +
+                'form-panel-id="'+formPanelId+'">\
+          <div class="wizardForms tabs-panel is-active" id="'+tab.tabName+'-wizardForms"></div>\
+          <div class="advancedForms tabs-panel" id="'+tab.tabName+'-advancedForms"></div>\
         </div>').appendTo( $(".contentForm", context) );
 
         // Create panelInstance, insert in the DOM and setup
@@ -781,13 +852,17 @@ define(function(require) {
           return false;
         } // Panel not defined
 
+        Foundation.reflow(context, 'tabs');
+
         formPanelInstance = new formPanel();
-        formPanelInstance.setAction(formContext, action);
         tab["formPanelInstances"][formPanelId] = formPanelInstance;
         formPanelInstance.insert(formContext);
       }
 
-      formPanelInstance.setAction(formContext, action);
+      if (action != undefined){
+        formPanelInstance.setAction(formContext, action);
+      }
+
       tab["activeFormPanel"] = formPanelInstance;
 
       // Hide wizard/advanced selector if advanced not defined
@@ -820,15 +895,17 @@ define(function(require) {
     // Workaround until Foundation.abide support hidden forms
     
     var context = $("#" + tabId);
-    $(".right-form-title", context).text(Locale.tr("Submitting..."));
+    $(".sunstone-form-title", context).text(Locale.tr("Submitting..."));
     $(".submit_button", context).text(Locale.tr("Submitting..."));
 
+    _disableFormPanelSubmit(tabId);
+
     setTimeout(function() {
       var formPanelInstance = SunstoneCfg["tabs"][tabId].activeFormPanel
 
-      if ($(".wizardForms.active", context).length > 0) {
+      if ($(".wizardForms.is-active", context).length > 0) {
         $('#' + formPanelInstance.formPanelId + 'Wizard').submit();
-      } else if ($(".advancedForms.active", context).length > 0) {
+      } else if ($(".advancedForms.is-active", context).length > 0) {
         $('#' + formPanelInstance.formPanelId + 'Advanced').submit();
       }
     }, 13)
@@ -836,7 +913,6 @@ define(function(require) {
 
   var _resetFormPanel = function(tabId, formPanelId) {
     _popFormPanelLoading(tabId);
-    _enableFormPanelSubmit(tabId);
 
     setTimeout(function() {
       var formPanelInstance;
@@ -854,10 +930,8 @@ define(function(require) {
         formPanelInstance.reset(context);
       }
 
-      _hideFormPanelLoading(tabId);
-
-      if (formPanelInstance) {
-        formPanelInstance.onShow(context);
+      if (_formPanelVisible($("#"+tabId))){
+        _showFormPanel(tabId, formPanelId);
       }
     }, 13)
   }
@@ -877,7 +951,7 @@ define(function(require) {
       context = $(".tab:visible");  // current tab
       tabId = context.attr("id");
     }
-    //$(".right-form", context).html(content);
+    //$(".sunstone-form", context).html(content);
 
     $(".loadingForm", context).hide();
     $(".tabs-contentForm", context).hide();
@@ -885,30 +959,33 @@ define(function(require) {
     var formPanelInstance = SunstoneCfg["tabs"][tabId].activeFormPanel;
     if (formPanelInstance) {
       // Set title and button strings
-      $(".right-form-title", context).text(formPanelInstance.title());
+      $(".sunstone-form-title", context).text(formPanelInstance.title());
       $(".submit_button", context).text(formPanelInstance.buttonText());
 
-      $("div[form-panel-id="+formPanelInstance.formPanelId+"]", context).show();
+      $("div[form-panel-id="+formPanelInstance.formPanelId+"]", context).fadeIn();
     }
 
+    _enableFormPanelSubmit(tabId);
   }
 
   function _hideFormPanel(tabId) {
     var context = $("#" + tabId);
-    $("a[href=back]", context).trigger("click");
+    $('[href="back"]', context).trigger("click");
   }
 
   function _popFormPanelLoading(tabId) {
     var context = $("#" + tabId);
-    $(".right-list", context).hide();
-    $(".right-info", context).hide();
-    $(".right-form", context).show();
-    $(".only-right-list", context).hide();
-    $(".only-right-info", context).hide();
-    $(".only-right-form", context).show();
+    $(".sunstone-list", context).hide();
+    $(".sunstone-info", context).hide();
+    $(".sunstone-form", context).show();
+    $(".only-sunstone-list", context).hide();
+    $(".only-sunstone-info", context).hide();
+    $(".only-sunstone-form", context).show();
+    $('.action_blocks', context).removeClass('large-9').addClass('large-12');
 
-    $(".right-form-title", context).text(Locale.tr("Loading..."));
+    $(".sunstone-form-title", context).text(Locale.tr("Loading..."));
     $(".submit_button", context).text(Locale.tr("Loading..."));
+    _disableFormPanelSubmit(tabId);
 
     $(".tabs-contentForm", context).hide();
     $(".loadingForm", context).show();
@@ -950,11 +1027,15 @@ define(function(require) {
   }
 
   var _rightInfoVisible = function(context) {
-    return $(".right-info", context).is(':visible');
+    return $(".sunstone-info", context).is(':visible');
   }
 
   var _rightListVisible = function(context) {
-    return $(".right-list", context).is(':visible');
+    return $(".sunstone-list", context).is(':visible');
+  }
+
+  var _formPanelVisible = function(context) {
+    return $(".sunstone-form", context).is(':visible');
   }
 
   var _rightInfoResourceId = function(context) {
@@ -971,6 +1052,12 @@ define(function(require) {
     }
   }
 
+  var _getResource = function(tabName) {
+    if (SunstoneCfg['tabs'][tabName]) {
+      return SunstoneCfg['tabs'][tabName].resource;
+    }
+  }
+
   var _getDialogInstance = function(dialogId) {
     var dialogInstance = SunstoneCfg['dialogInstances'][dialogId];
     if (dialogInstance == undefined) {
@@ -989,6 +1076,7 @@ define(function(require) {
 
     "insertTabs": _insertTabs,
     "insertPanels": _insertPanels,
+    "getElementRightInfo": _getElementRightInfo,
 
     'showTab': _showTab,
     "showElement" : _showElement,
@@ -1011,6 +1099,7 @@ define(function(require) {
     "getAction": _getAction,
     "getButton": _getButton,
     "getDataTable": _getDataTable,
+    "getResource": _getResource,
     "getDialog": _getDialogInstance,
 
     "insertButtonsInTab": _insertButtonsInTab,
