@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,13 +19,14 @@ define(function(require) {
     DEPENDENCIES
    */
 
-  require('foundation.tab');
+  //require('foundation.tab');
   var BaseFormPanel = require('utils/form-panels/form-panel');
   var Sunstone = require('sunstone');
   var Locale = require('utils/locale');
   var Notifier = require('utils/notifier');
   var Tips = require('utils/tips');
   var ResourceSelect = require('utils/resource-select');
+  var Config = require('sunstone-config');
 
   /*
     TEMPLATES
@@ -89,13 +90,22 @@ define(function(require) {
     $("#name", dialog).focus();
 
     var cluster_id = $("div#cluster_id .resource_list_select", dialog).val();
-    if (!cluster_id) cluster_id = "-1";
+    if (!cluster_id) cluster_id = "0";
 
     var cluster_id_raw = $("div#datastore_cluster_raw .resource_list_select", dialog).val();
-    if (!cluster_id_raw) cluster_id_raw = "-1";
+    if (!cluster_id_raw) cluster_id_raw = "0";
 
-    ResourceSelect.insert('div#cluster_id', dialog, "Cluster", cluster_id, false);
-    ResourceSelect.insert('div#datastore_cluster_raw', dialog, "Cluster", cluster_id_raw, false);
+    ResourceSelect.insert({
+        context: $('#cluster_id', dialog),
+        resourceName: 'Cluster',
+        initValue: cluster_id
+      });
+
+    ResourceSelect.insert({
+        context: $('#datastore_cluster_raw', dialog),
+        resourceName: 'Cluster',
+        initValue: cluster_id_raw
+      });
 
     return false;
   }
@@ -109,10 +119,12 @@ define(function(require) {
       'input[name="ds_tab_custom_tm_mad"]', dialog).parent().hide();
 
     $('select#ds_mad', dialog).change(function() {
-      if ($(this).val() == "custom")
+      if ($(this).val() == "custom") {
           $('input[name="ds_tab_custom_ds_mad"]', dialog).parent().show();
-      else
-          $('input[name="ds_tab_custom_ds_mad"]', dialog).parent().hide();
+      } else {
+        _setRequiredFields(dialog, $(this).val());
+        $('input[name="ds_tab_custom_ds_mad"]', dialog).parent().hide();
+      }
     });
 
     $('select#tm_mad', dialog).change(function() {
@@ -125,17 +137,12 @@ define(function(require) {
     $('#presets', dialog).change(function() {
       _hideAll(dialog);
       var choice_str = $(this).val();
+
       switch (choice_str)
       {
         case 'fs':
           _selectFilesystem(dialog);
           break;
-        case 'vmware_vmfs':
-          _selectVmwareVmfs(dialog);
-          break;
-        case 'block_lvm':
-          _selectBlockLvm(dialog);
-          break;
         case 'fs_lvm':
           _selectFsLvm(dialog);
           break;
@@ -148,12 +155,20 @@ define(function(require) {
         case 'dev':
           _selectDevices(dialog);
           break;
+        case 'iscsi_libvirt':
+          _selectISCSI(dialog);
+          break;
+        case 'vcenter':
+          _selectvCenter(dialog);
+          break;
         case 'custom':
           _selectCustom(dialog);
           break;
       }
     });
 
+    $('#presets', dialog).change();
+
     // Hide disk_type
     $('select#disk_type', dialog).parent().hide();
 
@@ -173,7 +188,6 @@ define(function(require) {
     var type            = $('#disk_type', dialog).val();
 
     var safe_dirs       = $('#safe_dirs', dialog).val();
-    var base_path       = $('#base_path', dialog).val();
     var restricted_dirs = $('#restricted_dirs', dialog).val();
     var limit_transfer_bw = $('#limit_transfer_bw', dialog).val();
     var datastore_capacity_check = $('#datastore_capacity_check', dialog).is(':checked');
@@ -191,6 +205,11 @@ define(function(require) {
     var ceph_user       = $('#ceph_user', dialog).val();
     var rbd_format      = $('#rbd_format', dialog).val();
     var staging_dir     = $('#staging_dir', dialog).val();
+    var ceph_conf       = $('#ceph_conf', dialog).val();
+    var iscsi_host      = $('#iscsi_host', dialog).val();
+    var iscsi_user      = $('#iscsi_user', dialog).val();
+    var iscsi_usage     = $('#iscsi_usage', dialog).val();
+    var vcenter_cluster = $('#vcenter_cluster', dialog).val();
 
     var ds_obj = {
       "datastore" : {
@@ -207,9 +226,6 @@ define(function(require) {
     if (ds_type != "SYSTEM_DS")
         ds_obj.datastore.ds_mad = ds_mad;
 
-    if (base_path)
-        ds_obj.datastore.base_path = base_path;
-
     if (safe_dirs)
         ds_obj.datastore.safe_dirs = safe_dirs;
 
@@ -261,6 +277,21 @@ define(function(require) {
     if (staging_dir)
         ds_obj.datastore.staging_dir = staging_dir;
 
+    if (ceph_conf)
+        ds_obj.datastore.ceph_conf = ceph_conf;
+
+    if (iscsi_host)
+        ds_obj.datastore.iscsi_host = iscsi_host;
+
+    if (iscsi_user)
+        ds_obj.datastore.iscsi_user = iscsi_user;
+
+    if (iscsi_usage)
+        ds_obj.datastore.iscsi_usage = iscsi_usage;
+
+    if (vcenter_cluster)
+        ds_obj.datastore.vcenter_cluster = vcenter_cluster;
+
     Sunstone.runAction("Datastore.create", ds_obj);
     return false;
   }
@@ -303,6 +334,11 @@ define(function(require) {
     $('label[for="ceph_user"],input#ceph_user', dialog).parent().hide();
     $('label[for="rbd_format"],input#rbd_format', dialog).parent().hide();
     $('label[for="staging_dir"],input#staging_dir', dialog).parent().hide();
+    $('label[for="ceph_conf"],input#ceph_conf', dialog).parent().hide();
+    $('label[for="iscsi_host"],input#iscsi_host', dialog).parent().hide();
+    $('label[for="iscsi_user"],input#iscsi_user', dialog).parent().hide();
+    $('label[for="iscsi_usage"],input#iscsi_usage', dialog).parent().hide();
+    $('label[for="vcenter_cluster"],input#vcenter_cluster', dialog).parent().hide();
     $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().hide();
     $('label[for="no_decompress"],input#no_decompress', dialog).parent().hide();
     $('select#ds_mad', dialog).removeAttr('disabled');
@@ -320,7 +356,7 @@ define(function(require) {
   }
 
   function _selectFilesystem(dialog) {
-    $('select#ds_mad', dialog).val('fs');
+    $('select#ds_mad', dialog).val('fs').change();
     $('select#tm_mad', dialog).val('shared');
     $('select#ds_mad', dialog).attr('disabled', 'disabled');
     $('select#tm_mad', dialog).children('option').each(function() {
@@ -339,35 +375,16 @@ define(function(require) {
     $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().fadeIn();
     $('input#safe_dirs', dialog).removeAttr('disabled');
     $('select#disk_type', dialog).removeAttr('disabled');
-    $('input#base_path', dialog).removeAttr('disabled');
     $('input#limit_mb', dialog).removeAttr('disabled');
     $('input#restricted_dirs', dialog).removeAttr('disabled');
     $('label[for="bridge_list"],input#bridge_list', dialog).parent().fadeIn();
     $('label[for="staging_dir"],input#staging_dir', dialog).parent().fadeIn();
   }
 
-  function _selectVmwareVmfs(dialog) {
-    $('label[for="bridge_list"],input#bridge_list', dialog).parent().fadeIn();
-    $('label[for="ds_tmp_dir"],input#ds_tmp_dir', dialog).parent().fadeIn();
-    $('select#ds_mad', dialog).val('vmfs');
-    $('select#ds_mad', dialog).attr('disabled', 'disabled');
-    $('select#tm_mad', dialog).val('vmfs');
-    $('select#tm_mad', dialog).attr('disabled', 'disabled');
-    $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().fadeIn();
-    $('label[for="no_decompress"],input#no_decompress', dialog).parent().fadeIn();
-    $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().fadeIn();
-    $('select#disk_type', dialog).val('file');
-    $('select#disk_type', dialog).attr('disabled', 'disabled');
-    $('input#safe_dirs', dialog).removeAttr('disabled');
-    $('input#base_path', dialog).removeAttr('disabled');
-    $('input#limit_mb', dialog).removeAttr('disabled');
-    $('input#restricted_dirs', dialog).removeAttr('disabled');
-  }
-
   function _selectCeph(dialog) {
     $('input#image_ds_type', dialog).click();
-    $('input[name=ds_type]', dialog).attr('disabled', 'disabled');
-    $('select#ds_mad', dialog).val('ceph');
+    $('input#file_ds_type', dialog).attr('disabled', 'disabled');
+    $('select#ds_mad', dialog).val('ceph').change();
     $('select#ds_mad', dialog).attr('disabled', 'disabled');
     $('select#tm_mad', dialog).val('ceph');
     $('select#tm_mad', dialog).attr('disabled', 'disabled');
@@ -378,39 +395,19 @@ define(function(require) {
     $('label[for="ceph_user"],input#ceph_user', dialog).parent().fadeIn();
     $('label[for="rbd_format"],input#rbd_format', dialog).parent().fadeIn();
     $('label[for="staging_dir"],input#staging_dir', dialog).parent().fadeIn();
+    $('label[for="ceph_conf"],input#ceph_conf', dialog).parent().fadeIn();
     $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().fadeIn();
     $('label[for="no_decompress"],input#no_decompress', dialog).parent().fadeIn();
     $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().fadeIn();
     $('select#disk_type', dialog).val('RBD');
     $('select#disk_type', dialog).attr('disabled', 'disabled');
     $('input#safe_dirs', dialog).removeAttr('disabled');
-    $('input#base_path', dialog).removeAttr('disabled');
-    $('input#limit_mb', dialog).removeAttr('disabled');
-    $('input#restricted_dirs', dialog).removeAttr('disabled');
-  }
-
-  function _selectBlockLvm(dialog) {
-    $('select#ds_mad', dialog).val('lvm');
-    $('select#ds_mad', dialog).attr('disabled', 'disabled');
-    $('select#tm_mad', dialog).val('lvm');
-    $('select#tm_mad', dialog).attr('disabled', 'disabled');
-    $('input#image_ds_type', dialog).click();
-    $('input[name=ds_type]', dialog).attr('disabled', 'disabled');
-    $('label[for="bridge_list"],input#bridge_list', dialog).parent().fadeIn();
-    $('label[for="vg_name"],input#vg_name', dialog).fadeIn();
-    $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().fadeIn();
-    $('label[for="no_decompress"],input#no_decompress', dialog).parent().fadeIn();
-    $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().fadeIn();
-    $('select#disk_type', dialog).val('block');
-    $('select#disk_type', dialog).attr('disabled', 'disabled');
-    $('input#safe_dirs', dialog).removeAttr('disabled');
-    $('input#base_path', dialog).removeAttr('disabled');
     $('input#limit_mb', dialog).removeAttr('disabled');
     $('input#restricted_dirs', dialog).removeAttr('disabled');
   }
 
   function _selectFsLvm(dialog) {
-    $('select#ds_mad', dialog).val('fs');
+    $('select#ds_mad', dialog).val('fs').change();
     $('select#ds_mad', dialog).attr('disabled', 'disabled');
     $('select#tm_mad', dialog).val('fs_lvm');
     $('select#tm_mad', dialog).attr('disabled', 'disabled');
@@ -422,13 +419,12 @@ define(function(require) {
     $('select#disk_type', dialog).val('block');
     $('select#disk_type', dialog).attr('disabled', 'disabled');
     $('input#safe_dirs', dialog).removeAttr('disabled');
-    $('input#base_path', dialog).removeAttr('disabled');
     $('input#limit_mb', dialog).removeAttr('disabled');
     $('input#restricted_dirs', dialog).removeAttr('disabled');
   }
 
   function _selectGluster(dialog) {
-    $('select#ds_mad', dialog).val('fs');
+    $('select#ds_mad', dialog).val('fs').change();
     $('select#ds_mad', dialog).attr('disabled', 'disabled');
     $('select#tm_mad', dialog).val('shared');
     $('select#tm_mad', dialog).children('option').each(function() {
@@ -449,13 +445,12 @@ define(function(require) {
     $('label[for="no_decompress"],input#no_decompress', dialog).parent().fadeIn();
     $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().fadeIn();
     $('input#safe_dirs', dialog).removeAttr('disabled');
-    $('input#base_path', dialog).removeAttr('disabled');
     $('input#limit_mb', dialog).removeAttr('disabled');
     $('input#restricted_dirs', dialog).removeAttr('disabled');
   }
 
   function _selectDevices(dialog) {
-    $('select#ds_mad', dialog).val('dev');
+    $('select#ds_mad', dialog).val('dev').change();
     $('select#ds_mad', dialog).attr('disabled', 'disabled');
     $('select#tm_mad', dialog).val('dev');
     $('select#tm_mad', dialog).attr('disabled', 'disabled');
@@ -467,22 +462,80 @@ define(function(require) {
     $('label[for="no_decompress"],input#no_decompress', dialog).parent().hide();
     $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().hide();
     $('input#safe_dirs', dialog).attr('disabled', 'disabled');
-    $('input#base_path', dialog).attr('disabled', 'disabled');
     $('input#limit_mb', dialog).attr('disabled', 'disabled');
     $('input#restricted_dirs', dialog).attr('disabled', 'disabled');
   }
 
+  function _selectISCSI(dialog) {
+    $('select#ds_mad', dialog).val('iscsi_libvirt').change();
+    $('select#ds_mad', dialog).attr('disabled', 'disabled');
+    $('select#tm_mad', dialog).val('iscsi_libvirt');
+    $('select#tm_mad', dialog).attr('disabled', 'disabled');
+    $('input#image_ds_type', dialog).click();
+    $('input[name=ds_type]', dialog).attr('disabled', 'disabled');
+    $('label[for="iscsi_host"],input#iscsi_host', dialog).parent().fadeIn();
+    $('label[for="iscsi_user"],input#iscsi_user', dialog).parent().fadeIn();
+    $('label[for="iscsi_usage"],input#iscsi_usage', dialog).parent().fadeIn();
+    $('select#disk_type', dialog).val('iscsi');
+    $('select#disk_type', dialog).attr('disabled', 'disabled');
+    $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().hide();
+    $('label[for="no_decompress"],input#no_decompress', dialog).parent().hide();
+    $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().hide();
+    $('input#safe_dirs', dialog).attr('disabled', 'disabled');
+    $('input#limit_mb', dialog).attr('disabled', 'disabled');
+    $('input#restricted_dirs', dialog).attr('disabled', 'disabled');
+  }
+
+  function _selectvCenter(dialog) {
+    $('select#ds_mad', dialog).val('vcenter').change();
+    $('select#ds_mad', dialog).attr('disabled', 'disabled');
+    $('select#tm_mad', dialog).val('vcenter');
+    $('select#tm_mad', dialog).attr('disabled', 'disabled');
+    $('input#image_ds_type', dialog).click();
+    $('input[name=ds_type]', dialog).attr('disabled', 'disabled');
+    $('select#disk_type', dialog).val('block');
+    $('select#disk_type', dialog).attr('disabled', 'disabled');
+    $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().hide();
+    $('label[for="no_decompress"],input#no_decompress', dialog).parent().hide();
+    $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().hide();
+    $('input#safe_dirs', dialog).attr('disabled', 'disabled');
+    $('input#limit_mb', dialog).attr('disabled', 'disabled');
+    $('input#restricted_dirs', dialog).attr('disabled', 'disabled');
+    $('label[for="vcenter_cluster"],input#vcenter_cluster', dialog).parent().fadeIn();
+  }
+
   function _selectCustom(dialog) {
     _hideAll(dialog);
-    $('select#ds_mad', dialog).val('fs');
+    $('select#ds_mad', dialog).val('fs').change();
     $('select#tm_mad', dialog).val('shared');
     $('input#safe_dirs', dialog).removeAttr('disabled');
     $('select#disk_type', dialog).removeAttr('disabled');
-    $('input#base_path', dialog).removeAttr('disabled');
     $('input#limit_mb', dialog).removeAttr('disabled');
     $('input#restricted_dirs', dialog).removeAttr('disabled');
     $('label[for="limit_transfer_bw"],input#limit_transfer_bw', dialog).parent().fadeIn();
     $('label[for="no_decompress"],input#no_decompress', dialog).parent().fadeIn();
     $('label[for="datastore_capacity_check"],input#datastore_capacity_check', dialog).parent().fadeIn();
   }
+
+  function _setRequiredFields(dialog, mad) {
+    // Disable all required attributes except for those that come in the
+    // template.
+    $('[required_active]', dialog).removeAttr('required')
+                                       .removeAttr('required_active');
+
+    if (Config.onedConf.DS_MAD_CONF !== undefined) {
+      $.each(Config.onedConf.DS_MAD_CONF, function(i, e){
+          if (e["NAME"] == mad) {
+            if (!$.isEmptyObject(e["REQUIRED_ATTRS"])) {
+              $.each(e["REQUIRED_ATTRS"].split(","), function(i, e){
+                $('#' + e.toLowerCase(), dialog).attr('required', true).attr('required_active', '');
+              });
+            }
+            return false;
+          }
+        }
+      );
+    }
+  }
 });
+
