@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -29,7 +29,8 @@ define(function(require) {
   var RESOURCE = "Group";
   var XML_ROOT = "GROUP";
 
-  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID, XML_ROOT);
+  var _commonActions = new CommonActions(OpenNebulaResource, RESOURCE, TAB_ID,
+    XML_ROOT, Locale.tr("Group created"));
 
   var _actions = {
     "Group.create" : _commonActions.create(CREATE_DIALOG_ID),
@@ -40,6 +41,7 @@ define(function(require) {
     "Group.delete" : _commonActions.del(),
     "Group.update" : _commonActions.update(),
     "Group.update_template" : _commonActions.updateTemplate(),
+    "Group.append_template" : _commonActions.appendTemplate(),
     "Group.update_dialog" : _commonActions.checkAndShowUpdate(),
     "Group.show_to_update" : _commonActions.showUpdate(CREATE_DIALOG_ID),
 
