@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,7 +19,7 @@ define(function(require) {
     DEPENDENCIES
    */
 
-  require('foundation.tab');
+//  require('foundation.tab');
   var BaseFormPanel = require('utils/form-panels/form-panel');
   var Sunstone = require('sunstone');
   var Locale = require('utils/locale');
@@ -146,7 +146,9 @@ define(function(require) {
   function _setup(context) {
     var that = this;
 
-    context.foundation('tab', 'reflow');
+    //context.foundation('tabs');
+    Foundation.reflow(context, 'tabs');
+
     this.userCreation.setup( $("#admin_user_wrapper",context) );
     Tips.setup(context);
 
@@ -183,8 +185,9 @@ define(function(require) {
     });
 
     if (this.action == "create") {
-      $("input#group_view_cloud").attr('checked','checked').change();
-      $("input#group_admin_view_groupadmin").attr('checked','checked').change();
+      $("input#group_view_cloud").prop('checked', true).change();
+      $("input#group_admin_view_cloud").prop('checked', true).change();
+      $("input#group_admin_view_groupadmin").prop('checked', true).change();
 
       _generateViewsSelect(context, "admin", "groupadmin");
       _generateViewsSelect(context, "user", "cloud");
@@ -255,30 +258,32 @@ define(function(require) {
     } else if (this.action == "update") {
       var template_json = this.element.TEMPLATE;
 
-      delete template_json["SUNSTONE_VIEWS"];
-      delete template_json["DEFAULT_VIEW"];
-      delete template_json["GROUP_ADMIN_VIEWS"];
-      delete template_json["GROUP_ADMIN_DEFAULT_VIEW"];
+      delete template_json['SUNSTONE'];
 
+      var sunstone_template = {};
       if (views.length != 0){
-        template_json["SUNSTONE_VIEWS"] = views.join(",");
+        sunstone_template["VIEWS"] = views.join(",");
       }
 
       if (default_view != undefined){
-        template_json["DEFAULT_VIEW"] = default_view;
+        sunstone_template["DEFAULT_VIEW"] = default_view;
       }
 
       if (admin_views.length != 0){
-        template_json["GROUP_ADMIN_VIEWS"] = admin_views.join(",");
+        sunstone_template["GROUP_ADMIN_VIEWS"] = admin_views.join(",");
       }
 
       if (default_admin_view != undefined){
-        template_json["GROUP_ADMIN_DEFAULT_VIEW"] = default_admin_view;
+        sunstone_template["GROUP_ADMIN_DEFAULT_VIEW"] = default_admin_view;
+      }
+
+      if (!$.isEmptyObject(sunstone_template)) {
+        template_json['SUNSTONE'] = sunstone_template;
       }
 
       var template_str = TemplateUtils.templateToString(template_json);
 
-      Sunstone.runAction("Group.update",this.resourceId, template_str);
+      Sunstone.runAction("Group.update", this.resourceId, template_str);
       return false;
     }
   }
@@ -305,40 +310,41 @@ define(function(require) {
 
     $('input[id^="group_view"]', context).removeAttr('checked');
 
-    if (element.TEMPLATE.SUNSTONE_VIEWS){
-      views_str = element.TEMPLATE.SUNSTONE_VIEWS;
+    var sunstone_template = element.TEMPLATE.SUNSTONE;
+    if (sunstone_template && sunstone_template.VIEWS){
+      views_str = sunstone_template.VIEWS;
 
       var views = views_str.split(",");
       $.each(views, function(){
         $('input[id^="group_view"][value="'+this.trim()+'"]',
-          context).attr('checked','checked').change();
+          context).prop('checked', true).change();
       });
     }
 
     $('input[id^="group_default_view"]', context).removeAttr('checked');
 
-    if (element.TEMPLATE.DEFAULT_VIEW){
-      $('#user_view_default', context).val(element.TEMPLATE.DEFAULT_VIEW.trim()).change();
+    if (sunstone_template && sunstone_template.DEFAULT_VIEW){
+      $('#user_view_default', context).val(sunstone_template.DEFAULT_VIEW.trim()).change();
     } else {
       $('#user_view_default', context).val("").change();
     }
 
     $('input[id^="group_admin_view"]', context).removeAttr('checked');
 
-    if (element.TEMPLATE.GROUP_ADMIN_VIEWS){
-      views_str = element.TEMPLATE.GROUP_ADMIN_VIEWS;
+    if (sunstone_template && sunstone_template.GROUP_ADMIN_VIEWS){
+      views_str = sunstone_template.GROUP_ADMIN_VIEWS;
 
       var views = views_str.split(",");
       $.each(views, function(){
         $('input[id^="group_admin_view"][value="'+this.trim()+'"]',
-          context).attr('checked','checked').change();
+          context).prop('checked', true).change();
       });
     }
 
     $('input[id^="group_default_admin_view"]', context).removeAttr('checked');
 
-    if (element.TEMPLATE.GROUP_ADMIN_DEFAULT_VIEW){
-      $('#admin_view_default', context).val(element.TEMPLATE.GROUP_ADMIN_DEFAULT_VIEW.trim()).change();
+    if (sunstone_template && sunstone_template.GROUP_ADMIN_DEFAULT_VIEW){
+      $('#admin_view_default', context).val(sunstone_template.GROUP_ADMIN_DEFAULT_VIEW.trim()).change();
     } else {
       $('#admin_view_default', context).val("").change();
     }
