@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -15,17 +15,20 @@
 /* -------------------------------------------------------------------------- */
 
 define(function(require) {
-  require('foundation.alert');
+//  require('foundation.alert');
   var OpenNebula = require('opennebula');
   var Locale = require('utils/locale');
   var Config = require('sunstone-config');
   var Notifier = require('utils/notifier');
   var Humanize = require('utils/humanize');
   var ResourceSelect = require('utils/resource-select');
+  var LabelsUtils = require('utils/labels/utils');
+  var TemplateUtils = require('utils/template-utils');
 
   var TemplateTemplatesList = require('hbs!./list');
 
   var _accordionId = 0;
+  var TEMPLATE_LABELS_COLUMN = 3;
 
   return {
     'generate': generate_provision_templates_list,
@@ -43,12 +46,13 @@ define(function(require) {
   function generate_provision_templates_list(context, opts) {
     context.off();
     context.html(html(opts));
+    Foundation.reflow(context, 'accordion');
     setup_provision_templates_list(context, opts);
   }
 
   function html(opts_arg){
     opts = $.extend({
-        title: Locale.tr("Saved Templates"),
+        title: Locale.tr("Templates"),
         refresh: true,
         create: true,
         active: true,
@@ -61,13 +65,13 @@ define(function(require) {
 
   function update_provision_templates_datatable(datatable, timeout) {
     datatable.html('<div class="text-center">'+
-      '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
+      '<span class="fa-stack fa-5x">'+
         '<i class="fa fa-cloud fa-stack-2x"></i>'+
         '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>'+
       '</span>'+
       '<br>'+
       '<br>'+
-      '<span style="font-size: 18px; color: #999">'+
+      '<span>'+
       '</span>'+
       '</div>');
 
@@ -78,18 +82,29 @@ define(function(require) {
           datatable.fnClearTable(true);
           if (item_list.length == 0) {
             datatable.html('<div class="text-center">'+
-              '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
+              '<span class="fa-stack fa-5x">'+
                 '<i class="fa fa-cloud fa-stack-2x"></i>'+
                 '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
               '</span>'+
               '<br>'+
               '<br>'+
-              '<span style="font-size: 18px; color: #999">'+
+              '<span>'+
                 Locale.tr("There are no templates available")+
               '</span>'+
               '</div>');
           } else {
             datatable.fnAddData(item_list);
+
+            // Labels are inserted only in the new VM wizard
+            LabelsUtils.clearLabelsFilter(datatable, TEMPLATE_LABELS_COLUMN);
+            var context = $('.labels-dropdown', datatable.closest('#provisionVMInstantiateTemplatesRow'));
+            LabelsUtils.insertLabelsMenu({
+              'context': context,
+              'dataTable': datatable,
+              'labelsColumn': TEMPLATE_LABELS_COLUMN,
+              'labelsPath': 'VMTEMPLATE.TEMPLATE.LABELS',
+              'placeholder': Locale.tr("No labels defined")
+            });
           }
         },
         error: Notifier.onError
@@ -109,27 +124,26 @@ define(function(require) {
       "aoColumns": [
           { "mDataProp": "VMTEMPLATE.ID" },
           { "mDataProp": "VMTEMPLATE.NAME" },
-          { "mDataProp": "VMTEMPLATE.TEMPLATE.SAVED_TEMPLATE_ID", "sDefaultContent" : "-"  },
           { "mDataProp": "VMTEMPLATE.UID" }
       ],
       "fnPreDrawCallback": function (oSettings) {
         // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
         if (this.$('tr', {"filter": "applied"} ).length == 0) {
           this.html('<div class="text-center">'+
-            '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
+            '<span class="fa-stack fa-5x">'+
               '<i class="fa fa-cloud fa-stack-2x"></i>'+
               '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
             '</span>'+
             '<br>'+
             '<br>'+
-            '<span style="font-size: 18px; color: #999">'+
-              Locale.tr("There are no saved templates available")+
+            '<span>'+
+              Locale.tr("There are no templates available")+
               '<br>'+
               Locale.tr("Create a template by saving a running Virtual Machine")+
             '</span>'+
             '</div>');
         } else {
-          $(".provision_templates_table", context).html('<ul class="provision_templates_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
+          $(".provision_templates_table", context).html('<div class="provision_templates_ul large-up-3 medium-up-3 small-up-1"></div>');
         }
         return true;
       },
@@ -140,56 +154,68 @@ define(function(require) {
           if (data.UID == config['user_id']) {
 
             if (data.PERMISSIONS.GROUP_U == "1") {
-              actions_html += '<a class="provision_confirm_unshare_template_button left" title="'+ Locale.tr("Unshare")+'" style="color:#555" href="#"><i class="fa fa-fw fa-lg fa-ban only-on-hover"/></a>';
-              actions_html += '<span style="font-size:12px; color: #777">' + Locale.tr("SHARED") + '</span>';
+              actions_html += '<a class="provision_confirm_unshare_template_button left" title="'+ Locale.tr("Unshare")+'" href="#"><i class="fa fa-fw fa-lg fa-ban only-on-hover"/></a>';
+              actions_html += '<span>' + Locale.tr("SHARED") + '</span>';
             } else {
-              actions_html += '<a class="provision_confirm_chmod_template_button left" title="'+ Locale.tr("Share")+'" style="color:#555" href="#"><i class="fa fa-fw fa-lg fa-share-alt only-on-hover"/></a>';
+              actions_html += '<a class="provision_confirm_chmod_template_button left" title="'+ Locale.tr("Share")+'" href="#"><i class="fa fa-fw fa-lg fa-share-alt only-on-hover"/></a>';
             }
           }
         }
 
         if (Config.isTabActionEnabled("provision-tab", "Template.delete")) {
-          actions_html += '<a class="provision_confirm_delete_template_button" title="'+ Locale.tr("Delete")+'"  style="color:#555" href="#"><i class="fa fa-fw fa-lg fa-trash-o right only-on-hover"/></a>';
+          actions_html += '<a class="provision_confirm_delete_template_button" title="'+ Locale.tr("Delete")+'"  href="#"><i class="fa fa-fw fa-lg fa-trash-o only-on-hover"/></a>';
         }
 
-        $(".provision_templates_ul", context).append('<li>'+
-            '<ul class="provision-pricing-table" opennebula_id="'+data.ID+'" saved_to_image_id="'+data.TEMPLATE.SAVED_TO_IMAGE_ID+'" datatable_index="'+iDisplayIndexFull+'">'+
-              '<li class="provision-title text-left" title="'+data.NAME+'">'+
-                data.NAME +
+        var cpu_txt = "";
+        var mem_txt = "";
+
+        if(data.TEMPLATE.CPU){
+          cpu_txt = 'x'+TemplateUtils.htmlEncode(data.TEMPLATE.CPU);
+        }
+
+        if(data.TEMPLATE.MEMORY){
+          if (data.TEMPLATE.MEMORY > 1000){
+            mem_txt = Math.floor(data.TEMPLATE.MEMORY/1024)+'GB';
+          } else {
+            mem_txt = TemplateUtils.htmlEncode(data.TEMPLATE.MEMORY)+'MB';
+          }
+        }
+
+        $(".provision_templates_ul", context).append('<div class="column">'+
+            '<ul class="provision-pricing-table menu vertical" opennebula_id="'+data.ID+'" datatable_index="'+iDisplayIndexFull+'">'+
+              '<li class="provision-title" title="'+data.NAME+'">'+
+                '<span class="without-link">' +
+                  TemplateUtils.htmlEncode(data.NAME) +
+                '</span>' +
               '</li>'+
-              '<li class="provision-bullet-item text-left" >'+
+              '<li class="provision-bullet-item">'+
+                (TemplateUtils.htmlEncode(data.TEMPLATE.DESCRIPTION) || '')+
+              '</li>'+
+              '<li class="provision-bullet-item" >'+
                 '<i class="fa fa-fw fa-lg fa-laptop"/> '+
-                'x'+data.TEMPLATE.CPU+' - '+
-                ((data.TEMPLATE.MEMORY > 1000) ?
-                  (Math.floor(data.TEMPLATE.MEMORY/1024)+'GB') :
-                  (data.TEMPLATE.MEMORY+'MB'))+
-                ' - '+
+                cpu_txt+' - '+
+                mem_txt+' - '+
                 get_provision_disk_image(data) +
               '</li>'+
-              '<li class="provision-description text-left" style="padding-top:0px; padding-bottom: 5px">'+
-                (data.TEMPLATE.DESCRIPTION || '')+
-              '</li>'+
-              '<li class="provision-bullet-item" style="padding-top:10px">'+
+              '<li class="provision-bullet-item text-right">'+
                 actions_html+
               '</li>'+
-              '<li class="provision-bullet-item-last text-left" >'+
+              '<li class="provision-bullet-item-last" >'+
                 '<span class="">'+
                   '<i class="fa fa-fw fa-lg fa-user"/> '+
-                  data.UNAME+
+                  TemplateUtils.htmlEncode(data.UNAME)+
                 '</span>'+
                 '<span class="right">'+
                   Humanize.prettyTimeAgo(data.REGTIME)+
                 '</span>'+
               '</li>'+
             '</ul>'+
-          '</li>');
+          '</div>');
 
         return nRow;
       }
     });
 
-    provision_templates_datatable.fnFilter("^(?!\-$)", 2, true, false);
-
     $('.provision_list_templates_search', context).on('input',function(){
       provision_templates_datatable.fnFilter( $(this).val() );
     })
@@ -207,23 +233,20 @@ define(function(require) {
 
     $(".provision_list_templates_filter", context).on("change", ".resource_list_select", function(){
       if ($(this).val() != "-2"){
-        provision_templates_datatable.fnFilter("^" + $(this).val() + "$", 3, true, false);
+        provision_templates_datatable.fnFilter("^" + $(this).val() + "$", 2, true, false);
       } else {
-        provision_templates_datatable.fnFilter("", 3);
+        provision_templates_datatable.fnFilter("", 2);
       }
     })
 
-    ResourceSelect.insert(
-      ".provision_list_templates_filter",
-      context,
-      "User",
-      (opts.filter_expression ? opts.filter_expression : "-2"),
-      false,
-      '<option value="-2">'+Locale.tr("ALL")+'</option>',
-      null,
-      null,
-      true,
-      true);
+    ResourceSelect.insert({
+        context: $('.provision_list_templates_filter', context),
+        resourceName: 'User',
+        initValue: (opts.filter_expression ? opts.filter_expression : "-2"),
+        extraOptions: '<option value="-2">' + Locale.tr("ALL") + '</option>',
+        triggerChange: true,
+        onlyName: true
+      });
 
     context.on("click", ".provision_templates_list_filter_button", function(){
       $(".provision_list_templates_filter", context).fadeIn();
@@ -237,20 +260,22 @@ define(function(require) {
         var template_name = $(".provision-title", ul_context).text();
 
         $(".provision_confirm_delete_template_div", context).html(
-          '<div data-alert class="alert-box secondary radius">'+
+          '<div data-closable class="callout large secondary">'+
             '<div class="row">'+
-            '<div class="large-9 columns">'+
-              '<span style="font-size: 14px; line-height: 20px">'+
-                Locale.tr("Handle with care! This action will inmediately destroy the template")+
+            '<div class="large-12 columns">'+
+              '<p>'+
+                Locale.tr("Handle with care! This action will immediately destroy the template")+
                 ' "' + template_name + '" ' +
                 Locale.tr("and the image associated.") +
-              '</span>'+
+              '</p>'+
             '</div>'+
-            '<div class="large-3 columns">'+
-              '<a href"#" class="provision_delete_template_button alert button large-12 radius right" style="margin-right: 15px" template_id="'+template_id+'">'+Locale.tr("Delete")+'</a>'+
+            '<div class="large-12 columns">'+
+              '<button href"#" class="provision_delete_template_button alert button right" template_id="'+template_id+'">'+Locale.tr("Delete")+'</button>'+
             '</div>'+
             '</div>'+
-            '<a href="#" class="close">&times;</a>'+
+            '<button class="close-button" aria-label="Dismiss alert" type="button" data-close>' +
+              '<span aria-hidden="true">&times;</span>' +
+            '</button>' +
           '</div>');
       });
 
@@ -261,7 +286,7 @@ define(function(require) {
 
         var template_id = $(this).attr("template_id");
 
-        OpenNebula.Template.delete_from_provision({
+        OpenNebula.Template.delete_recursive({
           timeout: true,
           data : {
             id : template_id
@@ -285,17 +310,17 @@ define(function(require) {
         var template_name = $(".provision-title", ul_context).text();
 
         $(".provision_confirm_delete_template_div", context).html(
-          '<div data-alert class="alert-box secondary radius">'+
+          '<div data-alert class="label secondary radius">'+
             '<div class="row">'+
             '<div class="large-8 columns">'+
-              '<span style="font-size: 14px; line-height: 20px">'+
+              '<span>'+
                 Locale.tr("The template")+
                 ' "' + template_name + '" ' +
                 Locale.tr("and the image associated will be shared and all the users will be able to instantiate new VMs using this template.") +
               '</span>'+
             '</div>'+
             '<div class="large-4 columns">'+
-              '<a href"#" class="provision_chmod_template_button success button large-12 radius right" style="margin-right: 15px" template_id="'+template_id+'">'+Locale.tr("Share template")+'</a>'+
+              '<a href"#" class="provision_chmod_template_button success button large-12 radius right" template_id="'+template_id+'">'+Locale.tr("Share template")+'</a>'+
             '</div>'+
             '</div>'+
             '<a href="#" class="close">&times;</a>'+
@@ -309,11 +334,14 @@ define(function(require) {
 
         var template_id = $(this).attr("template_id");
 
-        OpenNebula.Template.chmod_from_provision({
+        OpenNebula.Template.chmod({
           timeout: true,
           data : {
             id : template_id,
-            extra_param: {'group_u': 1}
+            extra_param: {
+              'group_u': 1,
+              'recursive' : true
+            }
           },
           success: function (){
             $(".provision_templates_list_refresh_button", context).trigger("click");
@@ -328,17 +356,17 @@ define(function(require) {
         var template_name = $(".provision-title", ul_context).first().text();
 
         $(".provision_confirm_delete_template_div", context).html(
-          '<div data-alert class="alert-box secondary radius">'+
+          '<div data-alert class="label secondary radius">'+
             '<div class="row">'+
             '<div class="large-8 columns">'+
-              '<span style="font-size: 14px; line-height: 20px">'+
+              '<span>'+
                 Locale.tr("The template")+
                 ' "' + template_name + '" ' +
                 Locale.tr("and the image associated will be unshared and the users will not be able to instantiate new VMs using this template.") +
               '</span>'+
             '</div>'+
             '<div class="large-4 columns">'+
-              '<a href"#" class="provision_unshare_template_button success button large-12 radius right" style="margin-right: 15px" template_id="'+template_id+'">'+Locale.tr("Unshare template")+'</a>'+
+              '<a href"#" class="provision_unshare_template_button success button large-12 radius right" template_id="'+template_id+'">'+Locale.tr("Unshare template")+'</a>'+
             '</div>'+
             '</div>'+
             '<a href="#" class="close">&times;</a>'+
@@ -351,11 +379,14 @@ define(function(require) {
 
         var template_id = $(this).attr("template_id");
 
-        OpenNebula.Template.chmod_from_provision({
+        OpenNebula.Template.chmod({
           timeout: true,
           data : {
             id : template_id,
-            extra_param: {'group_u': 0}
+            extra_param: {
+              'group_u': 0,
+              'recursive' : true
+            }
           },
           success: function (){
             $(".provision_templates_list_refresh_button", context).trigger("click");
@@ -367,7 +398,7 @@ define(function(require) {
 
     OpenNebula.Action.clear_cache("VMTEMPLATE");
     update_provision_templates_datatable(provision_templates_datatable, 0);
-    context.foundation();
+    // context.foundation();
   }
 
   function get_provision_disk_image(data) {
