@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -15,7 +15,7 @@
 /* -------------------------------------------------------------------------- */
 
 define(function(require) {
-  require('foundation.alert');
+//  require('foundation.alert');
   var Sunstone = require('sunstone');
   var OpenNebula = require('opennebula');
   var OpenNebulaVM = require('opennebula/vm');
@@ -25,8 +25,15 @@ define(function(require) {
   var Humanize = require('utils/humanize');
   var ResourceSelect = require('utils/resource-select');
   var Graphs = require('utils/graphs');
+  var TemplateUtils = require('utils/template-utils');
+  var StateActions = require('tabs/vms-tab/utils/state-actions');
 
   var TemplateVmsList = require('hbs!./list');
+  var TemplateConfirmSaveAsTemplate = require('hbs!./confirm_save_as_template');
+  var TemplateConfirmTerminate = require('hbs!./confirm_terminate');
+  var TemplateConfirmPoweroff = require('hbs!./confirm_poweroff');
+  var TemplateConfirmUndeploy = require('hbs!./confirm_undeploy');
+  var TemplateConfirmReboot = require('hbs!./confirm_reboot');
 
   var TAB_ID = require('../tabId');
   var _accordionId = 0;
@@ -49,6 +56,8 @@ define(function(require) {
     context.off();
     context.html(html(opts));
 
+    Foundation.reflow(context, 'accordion');
+    
     if (opts.data) {
       $(".provision_vms_table", context).data("opennebula", opts.data)
     }
@@ -73,13 +82,13 @@ define(function(require) {
     datatable.fnClearTable(true);
     if (item_list.length == 0) {
       datatable.html('<div class="text-center">' +
-        '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
+        '<span class="fa-stack fa-5x">' +
           '<i class="fa fa-cloud fa-stack-2x"></i>' +
           '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>' +
         '</span>' +
         '<br>' +
         '<br>' +
-        '<span style="font-size: 18px; color: #999">' +
+        '<span>' +
           Locale.tr("There are no Virtual Machines") +
         '</span>' +
         '<br>' +
@@ -92,13 +101,13 @@ define(function(require) {
 
   function update_provision_vms_datatable(datatable, timeout) {
     datatable.html('<div class="text-center">' +
-      '<span class="fa-stack fa-5x" style="color: #dfdfdf">' +
+      '<span class="fa-stack fa-5x">' +
         '<i class="fa fa-cloud fa-stack-2x"></i>' +
         '<i class="fa  fa-spinner fa-spin fa-stack-1x fa-inverse"></i>' +
       '</span>' +
       '<br>' +
       '<br>' +
-      '<span style="font-size: 18px; color: #999">' +
+      '<span>' +
       '</span>' +
       '</div>');
 
@@ -121,6 +130,7 @@ define(function(require) {
   function setup_provision_vms_list(context, opts) {
     var provision_vms_datatable = $('.provision_vms_table', context).dataTable({
       "iDisplayLength": 6,
+      "bAutoWidth": false,
       "sDom" : '<"H">t<"F"lp>',
       "aLengthMenu": [[6, 12, 36, 72], [6, 12, 36, 72]],
       "aaSorting"  : [[0, "desc"]],
@@ -136,18 +146,18 @@ define(function(require) {
         // create a thumbs container if it doesn't exist. put it in the dataTables_scrollbody div
         if (this.$('tr', {"filter": "applied"} ).length == 0) {
           this.html('<div class="text-center">'+
-            '<span class="fa-stack fa-5x" style="color: #dfdfdf">'+
+            '<span class="fa-stack fa-5x">'+
               '<i class="fa fa-cloud fa-stack-2x"></i>'+
               '<i class="fa fa-info-circle fa-stack-1x fa-inverse"></i>'+
             '</span>'+
             '<br>'+
             '<br>'+
-            '<span style="font-size: 18px; color: #999">'+
+            '<span>'+
               Locale.tr("There are no Virtual Machines")+
             '</span>'+
             '</div>');
         } else {
-          $(".provision_vms_table", context).html('<ul class="provision_vms_ul large-block-grid-3 medium-block-grid-3 small-block-grid-1 text-center"></ul>');
+          $(".provision_vms_table", context).html('<div class="provision_vms_ul large-up-3 medium-up-3 small-up-1"></div>');
         }
 
         return true;
@@ -156,30 +166,30 @@ define(function(require) {
         var data = aData.VM;
         var state = get_provision_vm_state(data);
 
-        $(".provision_vms_ul", context).append('<li>'+
-            '<ul class="provision-pricing-table" opennebula_id="'+data.ID+'" datatable_index="'+iDisplayIndexFull+'">'+
-              '<li class="provision-title text-left">'+
-                '<a class="provision_info_vm_button" style="color:#555" href="#">'+
-                '<span class="'+ state.color +'-color" title="'+state.str+'">'+
-                  '<i class="fa fa-fw fa-square"/> '+
+        $(".provision_vms_ul", context).append('<div class="column">'+
+            '<ul class="provision-pricing-table menu vertical" opennebula_id="'+data.ID+'" datatable_index="'+iDisplayIndexFull+'">'+
+              '<li class="provision-title">'+
+                '<a class="provision_info_vm_button">'+
+                '<span class="'+ state.color +'-color right" title="'+state.str+'">'+
+                  '<i class="fa fa-square"/>'+
                 '</span>'+
                 data.NAME + '</a>'+
               '</li>'+
-              '<li class="provision-bullet-item text-left" >'+
+              '<li class="provision-bullet-item" >'+
                 '<i class="fa fa-fw fa-lg fa-laptop"/> '+
                 'x'+data.TEMPLATE.CPU+' - '+
                 ((data.TEMPLATE.MEMORY > 1000) ?
                   (Math.floor(data.TEMPLATE.MEMORY/1024)+'GB') :
-                  (data.TEMPLATE.MEMORY+'MB'))+
+                  (TemplateUtils.htmlEncode(data.TEMPLATE.MEMORY)+'MB'))+
                 ' - '+
                 get_provision_disk_image(data) +
               '</li>'+
-              '<li class="provision-bullet-item text-left" >'+
+              '<li class="provision-bullet-item" >'+
                 '<span class="">'+
                   get_provision_ips(data) +
                 '</span>'+
               '</li>'+
-              '<li class="provision-bullet-item-last text-left" >'+
+              '<li class="provision-bullet-item-last" >'+
                 '<span class="">'+
                   '<i class="fa fa-fw fa-lg fa-user"/> '+
                   data.UNAME+
@@ -189,7 +199,7 @@ define(function(require) {
                 '</span>'+
               '</li>'+
             '</ul>'+
-          '</li>');
+          '</div>');
 
         return nRow;
       }
@@ -213,17 +223,14 @@ define(function(require) {
       }
     })
 
-    ResourceSelect.insert(
-      ".provision_list_vms_filter",
-      context,
-      "User",
-      (opts.filter_expression ? opts.filter_expression : "-2"),
-      false,
-      '<option value="-2">'+Locale.tr("ALL")+'</option>',
-      null,
-      null,
-      true,
-      true);
+    ResourceSelect.insert({
+        context: $('.provision_list_vms_filter', context),
+        resourceName: 'User',
+        initValue: (opts.filter_expression ? opts.filter_expression : "-2"),
+        extraOptions: '<option value="-2">' + Locale.tr("ALL") + '</option>',
+        triggerChange: true,
+        onlyName: true
+      });
 
     context.on("click", ".provision_vms_list_filter_button", function(){
       $(".provision_list_vms_filter", context).fadeIn();
@@ -233,7 +240,7 @@ define(function(require) {
     OpenNebula.Action.clear_cache("VM");
     update_provision_vms_datatable(provision_vms_datatable, 0);
 
-    $(document).foundation();
+    // $(document).foundation();
   }
 
   function setup_info_vm(context) {
@@ -249,91 +256,68 @@ define(function(require) {
         },
         error: Notifier.onError,
         success: function(request, response){
-          Sunstone.insertPanels(TAB_ID, response, TAB_ID, $(".provision-right-info", context));
+          Sunstone.insertPanels(TAB_ID, response, TAB_ID, $(".provision-sunstone-info", context));
 
           var data = response.VM
           var state = get_provision_vm_state(data);
 
-          switch (state.color) {
-            case "deploying":
-              $(".provision_reboot_confirm_button", context).hide();
-              $(".provision_poweroff_confirm_button", context).hide();
-              $(".provision_undeploy_confirm_button", context).hide();
-              $(".provision_poweron_button", context).hide();
-              $(".provision_delete_confirm_button", context).show();
-              $(".provision_shutdownhard_confirm_button", context).hide();
-              $(".provision_snapshot_button", context).hide();
-              $(".provision_vnc_button", context).hide();
-              $(".provision_snapshot_button_disabled", context).hide();
-              $(".provision_vnc_button_disabled", context).hide();
-              break;
-            case "running":
-              $(".provision_reboot_confirm_button", context).show();
-              $(".provision_poweroff_confirm_button", context).show();
-              $(".provision_undeploy_confirm_button", context).show();
-              $(".provision_poweron_button", context).hide();
-              $(".provision_delete_confirm_button", context).hide();
-              $(".provision_shutdownhard_confirm_button", context).show();
-              $(".provision_snapshot_button", context).hide();
-              $(".provision_vnc_button", context).show();
-              $(".provision_snapshot_button_disabled", context).show();
-              $(".provision_vnc_button_disabled", context).hide();
-              break;
-            case "off":
-              $(".provision_reboot_confirm_button", context).hide();
-              $(".provision_poweroff_confirm_button", context).hide();
-              $(".provision_undeploy_confirm_button", context).hide();
-              $(".provision_poweron_button", context).show();
-              $(".provision_delete_confirm_button", context).hide();
-              $(".provision_shutdownhard_confirm_button", context).show();
-              $(".provision_snapshot_button", context).show();
-              $(".provision_vnc_button", context).hide();
-              $(".provision_snapshot_button_disabled", context).hide();
-              $(".provision_vnc_button_disabled", context).show();
-              break;
-            case "undeployed":
-              $(".provision_reboot_confirm_button", context).hide();
-              $(".provision_poweroff_confirm_button", context).hide();
-              $(".provision_undeploy_confirm_button", context).hide();
-              $(".provision_poweron_button", context).show();
-              $(".provision_delete_confirm_button", context).hide();
-              $(".provision_shutdownhard_confirm_button", context).show();
-              $(".provision_snapshot_button", context).hide();
-              $(".provision_vnc_button", context).hide();
-              $(".provision_snapshot_button_disabled", context).hide();
-              $(".provision_vnc_button_disabled", context).show();
-              break;
-            case "powering_off":
-            case "error":
-              $(".provision_reboot_confirm_button", context).hide();
-              $(".provision_poweroff_confirm_button", context).hide();
-              $(".provision_undeploy_confirm_button", context).hide();
-              $(".provision_poweron_button", context).hide();
-              $(".provision_delete_confirm_button", context).show();
-              $(".provision_shutdownhard_confirm_button", context).hide();
-              $(".provision_snapshot_button", context).hide();
-              $(".provision_vnc_button", context).hide();
-              $(".provision_snapshot_button_disabled", context).hide();
-              $(".provision_vnc_button_disabled", context).hide();
-              break;
-            default:
-              color = 'secondary';
-              $(".provision_reboot_confirm_button", context).hide();
-              $(".provision_poweroff_confirm_button", context).hide();
-              $(".provision_undeploy_confirm_button", context).hide();
-              $(".provision_poweron_button", context).hide();
-              $(".provision_delete_confirm_button", context).show();
-              $(".provision_shutdownhard_confirm_button", context).hide();
-              $(".provision_snapshot_button", context).hide();
-              $(".provision_vnc_button", context).hide();
-              $(".provision_snapshot_button_disabled", context).hide();
-              $(".provision_vnc_button_disabled", context).hide();
-              break;
+          // helper, cleaner code
+          function enabled(action){
+            return Config.isTabActionEnabled("provision-tab", action) &&
+                   StateActions.enabledStateAction(action, data.STATE, data.LCM_STATE);
           }
 
-          if (!OpenNebula.VM.isVNCSupported(data) && !OpenNebula.VM.isSPICESupported(data)) {
-              $(".provision_vnc_button", context).hide();
-              $(".provision_vnc_button_disabled", context).hide();
+          if (enabled("VM.reboot") || enabled("VM.reboot_hard")){
+            $(".provision_reboot_confirm_button", context).show();
+          } else {
+            $(".provision_reboot_confirm_button", context).hide();
+          }
+
+          if (enabled("VM.poweroff") || enabled("VM.poweroff_hard")){
+            $(".provision_poweroff_confirm_button", context).show();
+          } else {
+            $(".provision_poweroff_confirm_button", context).hide();
+          }
+
+          if (enabled("VM.undeploy") || enabled("VM.undeploy_hard")){
+            $(".provision_undeploy_confirm_button", context).show();
+          } else {
+            $(".provision_undeploy_confirm_button", context).hide();
+          }
+
+          if (enabled("VM.resume")){
+            $(".provision_resume_button", context).show();
+          } else {
+            $(".provision_resume_button", context).hide();
+          }
+
+          if (enabled("VM.terminate") || enabled("VM.terminate_hard")){
+            $(".provision_terminate_confirm_button", context).show();
+          } else {
+            $(".provision_terminate_confirm_button", context).hide();
+          }
+
+          if(Config.isTabActionEnabled("provision-tab", "VM.save_as_template")){
+            if (enabled("VM.save_as_template")){
+              $(".provision_save_as_template_confirm_button", context).show();
+              $(".provision_save_as_template_confirm_button_disabled", context).hide();
+            } else {
+              $(".provision_save_as_template_confirm_button", context).hide();
+              $(".provision_save_as_template_confirm_button_disabled", context).show();
+            }
+          }else{
+            $(".provision_save_as_template_confirm_button", context).hide();
+            $(".provision_save_as_template_confirm_button_disabled", context).hide();
+          }
+
+          if (OpenNebula.VM.isVNCSupported(data) ||
+             OpenNebula.VM.isSPICESupported(data)) {
+
+            $(".provision_vnc_button", context).show();
+            $(".provision_vnc_button_disabled", context).hide();
+          }else{
+            $(".provision_vnc_button", context).hide();
+            $(".provision_vnc_button_disabled", context).show();
           }
 
           $(".provision_info_vm", context).attr("vm_id", data.ID);
@@ -371,49 +355,39 @@ define(function(require) {
           }
 
           $(".provision-pricing-table_vm_info", context).html(
-              '<li class="text-left provision-bullet-item">'+
-                '<span class="'+ state.color +'-color">'+
-                  '<i class="fa fa-fw fa-lg fa-square"/>&emsp;'+
+              '<li class="provision-title">'+
+                '<span class="without-link '+ state.color +'-color">'+
+                  '<span class="'+ state.color +'-color right" title="'+state.str+'">'+
+                    '<i class="fa fa-square"/>'+
+                  '</span>'+
                   state.str+
                 '</span>'+
               '</li>'+
-              '<li class="text-left provision-bullet-item">'+
-                '<hr style="margin: 0px">'+
-              '</li>'+
-              '<li class="text-left provision-bullet-item" >'+
+              '<li class="provision-bullet-item" >'+
                 '<span>'+
-                  '<i class="fa fa-fw fa-lg fa-laptop"/>&emsp;'+
-                  'x'+data.TEMPLATE.CPU+' - '+
+                  '<i class="fa fa-fw fa-lg fa-laptop"/> '+
+                  'x'+TemplateUtils.htmlEncode(data.TEMPLATE.CPU)+' - '+
                   ((data.TEMPLATE.MEMORY > 1000) ?
                     (Math.floor(data.TEMPLATE.MEMORY/1024)+'GB') :
-                    (data.TEMPLATE.MEMORY+'MB'))+
+                    (TemplateUtils.htmlEncode(data.TEMPLATE.MEMORY)+'MB'))+
                 '</span>'+
                 ' - '+
                 '<span>'+
                   get_provision_disk_image(data) +
                 '</span>'+
               '</li>'+
-              '<li class="text-left provision-bullet-item" >'+
+              '<li class="provision-bullet-item" >'+
                 '<span>'+
                   get_provision_ips(data) +
                 '</span>'+
               '</li>'+
-              //'<li  class="text-left provision-bullet-item" >'+
-              //  '<span style="color: #afafaf;" style="font-size: 16px">'+
-              //    "ID: " +
-              //    data.ID+
-              //  '</span>' +
-              //'</li>'+
-              '<li class="text-left provision-bullet-item">'+
-                '<hr style="margin: 0px">'+
-              '</li>'+
-              '<li class="text-right provision-bullet-item">'+
-                '<span class="left" style="color: #999;">'+
-                  '<i class="fa fa-fw fa-lg fa-user"/>&emsp;'+
+              '<li class="provision-bullet-item-last text-right">'+
+                '<span class="left">'+
+                  '<i class="fa fa-fw fa-lg fa-user"/> '+
                   data.UNAME+
                 '</span>'+
-                '<span class="right" style="color: #999;">'+
-                  '<i class="fa fa-fw fa-lg fa-clock-o"/>&emsp;'+
+                '<span>'+
+                  '<i class="fa fa-fw fa-lg fa-clock-o"/> '+
                   Humanize.prettyTimeAgo(data.STIME)+
                   ' - '+
                   'ID: '+
@@ -496,57 +470,13 @@ define(function(require) {
       })
     }
 
-    if (Config.isProvisionTabEnabled("provision-tab", "templates")) {
-      context.on("click", ".provision_snapshot_button", function(){
+    if (Config.isTabActionEnabled("provision-tab", "VM.save_as_template")) {
+      context.on("click", ".provision_save_as_template_confirm_button", function(){
         $(".provision_confirm_action:first", context).html(
-          '<div data-alert class="alert-box secondary radius">'+
-            '<div class="row">'+
-              '<div class="large-12 columns">'+
-                '<span style="font-size: 14px; line-height: 20px">'+
-                  Locale.tr("This Virtual Machine will be saved in a new Template.")+
-                '<br>'+
-                  Locale.tr("You can then create a new Virtual Machine using this Template.")+
-                '</span>'+
-              '</div>'+
-            '</div>'+
-            '<br>'+
-            '<div class="row">'+
-              '<div class="large-11 large-centered columns">'+
-                '<input type="text" class="provision_snapshot_name" placeholder="'+Locale.tr("Template Name")+'" style="height: 40px !important; font-size: 16px; padding: 0.5rem  !important; margin: 0px"/>'+
-              '</div>'+
-            '</div>'+
-            '<br>'+
-            '<div class="row">'+
-              '<div class="large-12 columns">'+
-                '<span style="font-size: 14px; line-height: 20px">'+
-                  Locale.tr("The new Virtual Machine's disks can be made persistent. In a persistent Virtual Machine the changes made survive after it is destroyed. On the other hand, you cannot create more than one simultaneous Virtual Machine from a Template with persistent disks.")+
-                '</span>'+
-              '</div>'+
-            '</div>'+
-            '<br>'+
-            '<div class="row">'+
-              '<div class="large-12 columns">'+
-                '<label class="left" style="margin-left: 25px">'+
-                  '<input type="radio" name="provision_snapshot_radio" value="persistent" class="provision_snapshot_persistent_radio">'+
-                  ' <i class="fa fa-fw fa-save"/> '+Locale.tr("Persistent")+
-                '</label>'+
-                '<label class="left" style="margin-left: 25px">'+
-                  '<input type="radio" name="provision_snapshot_radio" value="nonpersistent" class="provision_snapshot_nonpersisten_radio" checked>'+
-                  ' <i class="fa fa-fw fa-trash-o"/> '+Locale.tr("Non-persistent")+
-                '</label>'+
-              '</div>'+
-            '</div>'+
-            '<br>'+
-            '<div class="row">'+
-              '<div class="large-11 large-centered columns">'+
-                '<a href"#" class="provision_snapshot_create_button success button large-12 radius right">'+Locale.tr("Save Virtual Machine to Template")+'</a>'+
-              '</div>'+
-            '</div>'+
-            '<a href="#" class="close" style="top: 20px">&times;</a>'+
-          '</div>');
+          TemplateConfirmSaveAsTemplate());
       });
 
-      context.on("click", ".provision_snapshot_create_button", function(){
+      context.on("click", ".provision_save_as_template_button", function(){
         var button = $(this);
         button.attr("disabled", "disabled");
         var context = $(".provision_info_vm[vm_id]");
@@ -587,172 +517,98 @@ define(function(require) {
       });
     }
 
-    context.on("click", ".provision_delete_confirm_button", function(){
-      $(".provision_confirm_action:first", context).html(
-        '<div data-alert class="alert-box secondary radius">'+
-          '<div class="row">'+
-          '<div class="large-9 columns">'+
-            '<span style="font-size: 14px; line-height: 20px">'+
-              Locale.tr("Be careful, this action will inmediately destroy your Virtual Machine")+
-              '<br>'+
-              Locale.tr("All the information will be lost!")+
-            '</span>'+
-          '</div>'+
-          '<div class="large-3 columns">'+
-            '<a href"#" class="provision_delete_button alert button large-12 radius right" style="margin-right: 15px">'+Locale.tr("Delete")+'</a>'+
-          '</div>'+
-          '</div>'+
-          '<a href="#" class="close">&times;</a>'+
-        '</div>');
-    });
+    context.on("click", ".provision_terminate_confirm_button", function(){
+      var data = $(".provision_info_vm", context).data("vm");
 
-    context.on("click", ".provision_shutdownhard_confirm_button", function(){
-      $(".provision_confirm_action:first", context).html(
-        '<div data-alert class="alert-box secondary radius">'+
-          '<div class="row">'+
-          '<div class="large-9 columns">'+
-            '<span style="font-size: 14px; line-height: 20px">'+
-              Locale.tr("Be careful, this action will inmediately destroy your Virtual Machine")+
-              '<br>'+
-              Locale.tr("All the information will be lost!")+
-            '</span>'+
-          '</div>'+
-          '<div class="large-3 columns">'+
-            '<a href"#" class="provision_shutdownhard_button alert button large-12 radius right" style="margin-right: 15px">'+Locale.tr("Delete")+'</a>'+
-          '</div>'+
-          '</div>'+
-          '<a href="#" class="close" style="top: 20px">&times;</a>'+
-        '</div>');
+      var hard = Config.isTabActionEnabled("provision-tab", "VM.terminate_hard") &&
+             StateActions.enabledStateAction("VM.terminate_hard", data.STATE, data.LCM_STATE);
+
+      var soft = Config.isTabActionEnabled("provision-tab", "VM.terminate") &&
+             StateActions.enabledStateAction("VM.terminate", data.STATE, data.LCM_STATE);
+
+      var opts = {};
+
+      if(hard && soft){
+        opts.both = true;
+      } else if(hard){
+        opts.hard = true;
+      }
+
+      $(".provision_confirm_action:first", context).html(TemplateConfirmTerminate({opts: opts}));
     });
 
     context.on("click", ".provision_poweroff_confirm_button", function(){
-      $(".provision_confirm_action:first", context).html(
-        '<div data-alert class="alert-box secondary radius">'+
-          '<div class="row">'+
-          '<div class="large-11 columns">'+
-            '<span style="font-size: 14px; line-height: 20px">'+
-              Locale.tr("This action will power off this Virtual Machine. The Virtual Machine will remain in the poweroff state, and can be powered on later")+
-              '<br>'+
-              '<br>'+
-              Locale.tr("You can send the power off signal to the Virtual Machine (this is equivalent to execute the command from the console). If that doesn't effect your Virtual Machine, try to Power off the machine (this is equivalent to pressing the power off button in a physical computer).")+
-            '</span>'+
-          '</div>'+
-          '</div>'+
-          '<br>'+
-          '<div class="row">'+
-          '<div class="large-12 columns">'+
-            '<a href"#" class="provision_poweroff_button button radius right" style="margin-right: 15px">'+Locale.tr("Power off")+'</a>'+
-            '<label class="left" style="margin-left: 25px">'+
-              '<input type="radio" name="provision_poweroff_radio" value="poweroff_hard" class="provision_poweroff_hard_radio">'+
-              ' <i class="fa fa-fw fa-bolt"/> '+Locale.tr("Power off the machine")+
-            '</label>'+
-            '<label class="left" style="margin-left: 25px">'+
-              '<input type="radio" name="provision_poweroff_radio" value="poweroff" class="provision_poweroff_radio" checked>'+
-              ' <i class="fa fa-fw fa-power-off"/> '+Locale.tr("Send the power off signal")+
-            '</label>'+
-          '</div>'+
-          '</div>'+
-          '<a href="#" class="close" style="top: 20px">&times;</a>'+
-        '</div>');
+      var data = $(".provision_info_vm", context).data("vm");
+
+      var hard = Config.isTabActionEnabled("provision-tab", "VM.poweroff_hard") &&
+             StateActions.enabledStateAction("VM.poweroff_hard", data.STATE, data.LCM_STATE);
+
+      var soft = Config.isTabActionEnabled("provision-tab", "VM.poweroff") &&
+             StateActions.enabledStateAction("VM.poweroff", data.STATE, data.LCM_STATE);
+
+      var opts = {};
+
+      if(hard && soft){
+        opts.both = true;
+      } else if(hard){
+        opts.hard = true;
+      }
+
+      $(".provision_confirm_action:first", context).html(TemplateConfirmPoweroff({opts: opts}));
     });
 
     context.on("click", ".provision_undeploy_confirm_button", function(){
-      $(".provision_confirm_action:first", context).html(
-        '<div data-alert class="alert-box secondary radius">'+
-          '<div class="row">'+
-          '<div class="large-11 columns">'+
-            '<span style="font-size: 14px; line-height: 20px">'+
-              Locale.tr("This action will power off this Virtual Machine and will be undeployed from the physical machine")+
-              '<br>'+
-              '<br>'+
-              Locale.tr("You can send the power off signal to the Virtual Machine (this is equivalent to execute the command from the console). If that doesn't effect your Virtual Machine, try to Power off the machine (this is equivalent to pressing the power off button in a physical computer).")+
-            '</span>'+
-          '</div>'+
-          '</div>'+
-          '<br>'+
-          '<div class="row">'+
-          '<div class="large-12 columns">'+
-            '<a href"#" class="provision_undeploy_button button radius right" style="margin-right: 15px">'+Locale.tr("Power off and undeploy")+'</a>'+
-            '<label class="left" style="margin-left: 25px">'+
-              '<input type="radio" name="provision_undeploy_radio" value="undeploy_hard" class="provision_undeploy_hard_radio">'+
-              ' <i class="fa fa-fw fa-bolt"/> '+Locale.tr("Power off and undeploy the VM")+
-            '</label>'+
-            '<label class="left" style="margin-left: 25px">'+
-              '<input type="radio" name="provision_undeploy_radio" value="undeploy" class="provision_undeploy_radio" checked>'+
-              ' <i class="fa fa-fw fa-power-off"/> '+Locale.tr("Send the power off signal and undeploy the VM")+
-            '</label>'+
-          '</div>'+
-          '</div>'+
-          '<a href="#" class="close" style="top: 20px">&times;</a>'+
-        '</div>');
+      var data = $(".provision_info_vm", context).data("vm");
+
+      var hard = Config.isTabActionEnabled("provision-tab", "VM.undeploy_hard") &&
+             StateActions.enabledStateAction("VM.undeploy_hard", data.STATE, data.LCM_STATE);
+
+      var soft = Config.isTabActionEnabled("provision-tab", "VM.undeploy") &&
+             StateActions.enabledStateAction("VM.undeploy", data.STATE, data.LCM_STATE);
+
+      var opts = {};
+
+      if(hard && soft){
+        opts.both = true;
+      } else if(hard){
+        opts.hard = true;
+      }
+
+      $(".provision_confirm_action:first", context).html(TemplateConfirmUndeploy({opts: opts}));
     });
 
     context.on("click", ".provision_reboot_confirm_button", function(){
-      $(".provision_confirm_action:first", context).html(
-        '<div data-alert class="alert-box secondary radius">'+
-          '<div class="row">'+
-          '<div class="large-11 columns">'+
-            '<span style="font-size: 14px; line-height: 20px">'+
-              Locale.tr("This action will reboot this Virtual Machine.")+
-              '<br>'+
-              '<br>'+
-              Locale.tr("You can send the reboot signal to the Virtual Machine (this is equivalent to execute the reboot command form the console). If that doesn't effect your Virtual Machine, try to Reboot the machine (this is equivalent to pressing the reset button a physical computer).")+
-            '</span>'+
-          '</div>'+
-          '</div>'+
-          '<br>'+
-          '<div class="row">'+
-          '<div class="large-12 columns">'+
-            '<a href"#" class="provision_reboot_button button radius right" style="margin-right: 15px">'+Locale.tr("Reboot")+'</a>'+
-            '<label class="left" style="margin-left: 25px">'+
-              '<input type="radio" name="provision_reboot_radio" value="reset" class="provision_reboot_hard_radio">'+
-              ' <i class="fa fa-fw fa-bolt"/> '+Locale.tr("Reboot the machine")+
-            '</label>'+
-            '<label class="left" style="margin-left: 25px">'+
-              '<input type="radio" name="provision_reboot_radio" value="reboot" class="provision_reboot_radio" checked>'+
-              ' <i class="fa fa-fw fa-power-off"/> '+Locale.tr("Send the reboot signal")+
-            '</label>'+
-          '</div>'+
-          '</div>'+
-          '<a href="#" class="close" style="top: 20px">&times;</a>'+
-        '</div>');
+      var data = $(".provision_info_vm", context).data("vm");
+
+      var hard = Config.isTabActionEnabled("provision-tab", "VM.reboot_hard") &&
+             StateActions.enabledStateAction("VM.reboot_hard", data.STATE, data.LCM_STATE);
+
+      var soft = Config.isTabActionEnabled("provision-tab", "VM.reboot") &&
+             StateActions.enabledStateAction("VM.reboot", data.STATE, data.LCM_STATE);
+
+      var opts = {};
+
+      if(hard && soft){
+        opts.both = true;
+      } else if(hard){
+        opts.hard = true;
+      }
+
+      $(".provision_confirm_action:first", context).html(TemplateConfirmReboot({opts: opts}));
     });
 
-    context.on("click", ".provision_delete_button", function(){
+    context.on("click", ".provision_terminate_button", function(){
       var button = $(this);
       button.attr("disabled", "disabled");
       var vm_id = $(".provision_info_vm", context).attr("vm_id");
+      var terminate_action = $('input[name=provision_terminate_radio]:checked').val()
 
-      OpenNebula.VM.del({
+      OpenNebula.VM[terminate_action]({
         data : {
           id: vm_id
         },
         success: function(request, response){
-          $(".provision_back", context).click();
-          $(".provision_vms_list_refresh_button", context).click();
-          button.removeAttr("disabled");
-        },
-        error: function(request, response){
-          Notifier.onError(request, response);
-          button.removeAttr("disabled");
-        }
-      })
-
-      return false;
-    });
-
-    context.on("click", ".provision_shutdownhard_button", function(){
-      var button = $(this);
-      button.attr("disabled", "disabled");
-      var vm_id = $(".provision_info_vm", context).attr("vm_id");
-
-      OpenNebula.VM.shutdown_hard({
-        data : {
-          id: vm_id
-        },
-        success: function(request, response){
-          $(".provision_back", context).click();
-          $(".provision_vms_list_refresh_button", context).click();
+          update_provision_vm_info(vm_id, context);
           button.removeAttr("disabled");
         },
         error: function(request, response){
@@ -834,7 +690,7 @@ define(function(require) {
       return false;
     });
 
-    context.on("click", ".provision_poweron_button", function(){
+    context.on("click", ".provision_resume_button", function(){
       var button = $(this);
       button.attr("disabled", "disabled");
       var vm_id = $(".provision_info_vm", context).attr("vm_id");
@@ -966,7 +822,7 @@ define(function(require) {
       case OpenNebulaVM.STATES.PENDING:
       case OpenNebulaVM.STATES.HOLD:
         state_color = 'deploying';
-        state_str = Locale.tr("DEPLOYING") + " (1/3)";
+        state_str = Locale.tr("DEPLOYING") + " (2/4)";
         break;
       case OpenNebulaVM.STATES.ACTIVE:
         var lcm_state = parseInt(data.LCM_STATE);
@@ -974,13 +830,13 @@ define(function(require) {
         switch (lcm_state) {
           case OpenNebulaVM.LCM_STATES.LCM_INIT:
             state_color = 'deploying';
-            state_str = Locale.tr("DEPLOYING") + " (1/3)";
+            state_str = Locale.tr("DEPLOYING") + " (2/4)";
             break;
           case OpenNebulaVM.LCM_STATES.PROLOG:
           case OpenNebulaVM.LCM_STATES.PROLOG_RESUME:
           case OpenNebulaVM.LCM_STATES.PROLOG_UNDEPLOY:
             state_color = 'deploying';
-            state_str = Locale.tr("DEPLOYING") + " (2/3)";
+            state_str = Locale.tr("DEPLOYING") + " (3/4)";
             break;
           case OpenNebulaVM.LCM_STATES.BOOT:
           case OpenNebulaVM.LCM_STATES.BOOT_UNKNOWN:
@@ -989,7 +845,7 @@ define(function(require) {
           case OpenNebulaVM.LCM_STATES.BOOT_STOPPED:
           case OpenNebulaVM.LCM_STATES.BOOT_UNDEPLOY:
             state_color = 'deploying';
-            state_str = Locale.tr("DEPLOYING") + " (3/3)";
+            state_str = Locale.tr("DEPLOYING") + " (4/4)";
             break;
           case OpenNebulaVM.LCM_STATES.RUNNING:
           case OpenNebulaVM.LCM_STATES.HOTPLUG_SNAPSHOT:
@@ -1006,6 +862,7 @@ define(function(require) {
           case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE:
           case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_POWEROFF:
           case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_SUSPEND:
+          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_UNKNOWN:
             state_color = 'running';
             state_str = Locale.tr("RUNNING");
             break;
@@ -1033,6 +890,7 @@ define(function(require) {
           case OpenNebulaVM.LCM_STATES.BOOT_STOPPED_FAILURE:
           case OpenNebulaVM.LCM_STATES.PROLOG_RESUME_FAILURE:
           case OpenNebulaVM.LCM_STATES.PROLOG_UNDEPLOY_FAILURE:
+          case OpenNebulaVM.LCM_STATES.PROLOG_MIGRATE_UNKNOWN_FAILURE:
             state_color = 'error';
             state_str = Locale.tr("ERROR");
             break;
@@ -1075,6 +933,17 @@ define(function(require) {
         state_str = Locale.tr("UNDEPLOYED");
 
         break;
+
+      case OpenNebulaVM.STATES.CLONING:
+        state_color = 'deploying';
+        state_str = Locale.tr("DEPLOYING") + " (1/4)";
+        break;
+
+      case OpenNebulaVM.STATES.CLONING_FAILURE:
+        state_color = 'error';
+        state_str = Locale.tr("ERROR");
+        break;
+
       default:
         state_color = 'powering_off';
         state_str = Locale.tr("UNKNOWN");
