@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -21,8 +21,12 @@ define(function(require) {
 
   var TabDataTable = require('utils/tab-datatable');
   var SunstoneConfig = require('sunstone-config');
+  var Sunstone = require('sunstone');
+  var OpenNebula = require('opennebula');
   var Locale = require('utils/locale');
   var Humanize = require('utils/humanize');
+  var Notifier = require('utils/notifier');
+  var LabelsUtils = require('utils/labels/utils');
 
   /*
     CONSTANTS
@@ -31,6 +35,8 @@ define(function(require) {
   var RESOURCE = "Template";
   var XML_ROOT = "VMTEMPLATE";
   var TAB_NAME = require('./tabId');
+  var LABELS_COLUMN = 6;
+  var TEMPLATE_ATTR = 'TEMPLATE';
 
   /*
     CONSTRUCTOR
@@ -42,6 +48,7 @@ define(function(require) {
     this.dataTableId = dataTableId;
     this.resource = RESOURCE;
     this.xmlRoot = XML_ROOT;
+    this.labelsColumn = LABELS_COLUMN;
 
     this.dataTableOptions = {
       "bAutoWidth": false,
@@ -60,7 +67,8 @@ define(function(require) {
       Locale.tr("Owner") ,
       Locale.tr("Group"),
       Locale.tr("Name"),
-      Locale.tr("Registration time")
+      Locale.tr("Registration time"),
+      Locale.tr("Labels")
     ];
 
     this.selectOptions = {
@@ -69,15 +77,25 @@ define(function(require) {
       "select_resource": Locale.tr("Please select a Template from the list"),
       "you_selected": Locale.tr("You selected the following Template:"),
       "select_resource_multiple": Locale.tr("Please select one or more Templates from the list"),
-      "you_selected_multiple": Locale.tr("You selected the following Templates:")
+      "you_selected_multiple": Locale.tr("You selected the following Templates:"),
+
+      // By default the filter is set to return all VM Templates that
+      // are NOT virtual router templates
+      "filter_fn": function(tmpl){
+        return (tmpl.TEMPLATE.VROUTER == undefined || tmpl.TEMPLATE.VROUTER.toUpperCase() != "YES");
+      }
     };
 
+    this.labels = [];
+
     TabDataTable.call(this);
   };
 
   Table.prototype = Object.create(TabDataTable.prototype);
   Table.prototype.constructor = Table;
   Table.prototype.elementArray = _elementArray;
+  Table.prototype.preUpdateView = _preUpdateView;
+  Table.prototype.postUpdateView = _postUpdateView;
 
   return Table;
 
@@ -96,7 +114,14 @@ define(function(require) {
         element.UNAME,
         element.GNAME,
         element.NAME,
-        Humanize.prettyTime(element.REGTIME)
+        Humanize.prettyTime(element.REGTIME),
+        (LabelsUtils.labelsStr(element[TEMPLATE_ATTR])||'')
     ];
   }
+
+  function _preUpdateView() {
+  }
+
+  function _postUpdateView() {
+  }
 });
