@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -27,6 +27,8 @@ define(function(require) {
   var CustomTagsTable = require('utils/custom-tags-table');
   var FilesTable = require('tabs/files-tab/datatable')
   var OpenNebulaHost = require('opennebula/host');
+  var UserInputs = require('utils/user-inputs');
+  var UniqueId = require('utils/unique-id');
 
   /*
     TEMPLATES
@@ -49,12 +51,12 @@ define(function(require) {
       throw "Wizard Tab not enabled";
     }
 
-    this.wizardTabId = WIZARD_TAB_ID;
+    this.wizardTabId = WIZARD_TAB_ID + UniqueId.id();
     this.icon = 'fa-folder';
     this.title = Locale.tr("Context");
-    this.classes = "hypervisor only_kvm only_vmware only_xen only_vcenter";
+    this.classes = "hypervisor only_kvm only_vcenter";
 
-    this.contextFilesTable = new FilesTable(this.wizardTabId + 'ContextTable', {
+    this.contextFilesTable = new FilesTable('ContextTable' + UniqueId.id(), {
       'select': true,
       'selectOptions': {
         'multiple_choice': true,
@@ -78,6 +80,8 @@ define(function(require) {
 
   function _html() {
     return TemplateHTML({
+      'uniqueId': UniqueId.id(),
+      'userInputsHTML': UserInputs.html(),
       'customTagsTableHTML': CustomTagsTable.html(),
       'contextFilesTableHTML': this.contextFilesTable.dataTableHTML
     });
@@ -104,19 +108,19 @@ define(function(require) {
       }
     });
 
-    context.on("change", "select#vcenter_customizations", function(){
+    context.on("change", "select.vcenter_customizations", function(){
       var option = $("option:selected", this);
 
       if (option.attr("custom") == "true"){
-        $('input#vcenter_customizations_value', context).show();
+        $('input.vcenter_customizations_value', context).show();
       } else {
-        $('input#vcenter_customizations_value', context).hide();
+        $('input.vcenter_customizations_value', context).hide();
       }
 
-      $('input#vcenter_customizations_value', context).val( $(this).val() );
+      $('input.vcenter_customizations_value', context).val( $(this).val() );
     });
 
-    $('input#vcenter_customizations_value', context).hide();
+    $('input.vcenter_customizations_value', context).hide();
 
     OpenNebulaHost.vcenterCustomizations({
       data : {},
@@ -132,19 +136,20 @@ define(function(require) {
       }
     });
 
-    context.on("change", "input#vcenter_customizations_value", function(){
+    context.on("change", "input.vcenter_customizations_value", function(){
       var opt =
         $('option'+
-          '[value="'+$('input#vcenter_customizations_value', context).val()+'"]', context);
+          '[value="'+$('input.vcenter_customizations_value', context).val()+'"]', context);
 
       if (opt.size() == 0){
         opt = $('option[custom="true"]', context);
-        $('input#vcenter_customizations_value', context).show();
+        $('input.vcenter_customizations_value', context).show();
       }
 
       opt.attr('selected', 'selected');
     });
 
+    UserInputs.setup(context);
     CustomTagsTable.setup(context);
 
     var selectOptions = {
@@ -160,34 +165,6 @@ define(function(require) {
 
     that.contextFilesTable.initialize(selectOptions);
     that.contextFilesTable.refreshResourceTableSelect();
-
-    context.on("click", ".add_service_custom_attr", function() {
-      $(".service_custom_attrs tbody").append(
-        '<tr>' +
-          '<td>' +
-            '<input class="user_input_name" type="text" pattern="[\\w]+"/>' +
-            '<small class="error">' + Locale.tr("Only word characters are allowed") + '</small>' +
-          '</td>' +
-          '<td>' +
-            '<select class="user_input_type" >' +
-              '<option value="text">' + Locale.tr("text") + '</option>' +
-              '<option value="text64">' + Locale.tr("text (base64)") + '</option>' +
-              '<option value="password">' + Locale.tr("password") + '</option>' +
-            '</select>' +
-          '</td>' +
-          '<td>' +
-            '<textarea class="user_input_description"/>' +
-          '</td>' +
-          '<td>' +
-            '<a href="#"><i class="fa fa-times-circle remove-tab"></i></a>' +
-          '</td>' +
-        '</tr>');
-    })
-
-    context.on("click", ".service_custom_attrs i.remove-tab", function() {
-      var tr = $(this).closest('tr');
-      tr.remove();
-    });
   }
 
   function _fillCustomizations(context, customizations) {
@@ -203,14 +180,14 @@ define(function(require) {
 
     html += '</select>';
 
-    $("#vcenter_customizations", context).html(html);
+    $(".vcenter_customizations", context).html(html);
   }
 
   function _retrieve(context) {
     var templateJSON = {};
 
     if($("input[name='context_type']:checked", context).val() == "context_type_vcenter"){
-      var customization = $('input#vcenter_customizations_value', context).val();
+      var customization = WizardFields.retrieveInput($('input.vcenter_customizations_value', context));
 
       if (customization) {
         templateJSON["VCENTER_PUBLIC_CLOUD"] = {
@@ -221,38 +198,34 @@ define(function(require) {
       var contextJSON = WizardFields.retrieve(context);
       $.extend(contextJSON, CustomTagsTable.retrieve(context));
 
-      if ($("#ssh_context", context).is(":checked")) {
-        var public_key = $("#ssh_public_key", context).val();
+      if ($(".ssh_context", context).is(":checked")) {
+        var public_key = WizardFields.retrieveInput($("#ssh_public_key", context));
         if (public_key) {
-          contextJSON["SSH_PUBLIC_KEY"] = TemplateUtils.escapeDoubleQuotes(public_key);
+          contextJSON["SSH_PUBLIC_KEY"] = public_key;
         } else {
           contextJSON["SSH_PUBLIC_KEY"] = '$USER[SSH_PUBLIC_KEY]';
         }
       }
 
-      if ($("#network_context", context).is(":checked")) {
+      if ($(".network_context", context).is(":checked")) {
         contextJSON["NETWORK"] = "YES";
       }
 
-      if ($("#token_context", context).is(":checked")) {
+      if ($(".token_context", context).is(":checked")) {
         contextJSON["TOKEN"] = "YES";
       }
 
-      var userInputsJSON = {};
-      $(".service_custom_attrs tbody tr", context).each(function() {
-        if ($(".user_input_name", $(this)).val()) {
-          var attr_name = $(".user_input_name", $(this)).val();
-          var attr_type = $(".user_input_type", $(this)).val();
-          var attr_desc = $(".user_input_description", $(this)).val();
-          userInputsJSON[attr_name] = "M|" + attr_type + "|" + attr_desc;
-          contextJSON[attr_name] = "$" + attr_name.toUpperCase();
-        }
+      var userInputsJSON = UserInputs.retrieve(context);
+
+      $.each(userInputsJSON, function(key,value){
+        var name = key.toUpperCase();
+        contextJSON[name] = "$" + name;
       });
 
-      var start_script = $("#START_SCRIPT", context).val();
+      var start_script = WizardFields.retrieveInput($(".START_SCRIPT", context));
       if (start_script != "") {
-        if ($("#ENCODE_START_SCRIPT", context).is(":checked")) {
-          contextJSON["START_SCRIPT_BASE64"] = btoa(start_script);
+        if ($(".ENCODE_START_SCRIPT", context).is(":checked")) {
+          contextJSON["START_SCRIPT_BASE64"] = btoa($(".START_SCRIPT", context).val());
         } else {
           contextJSON["START_SCRIPT"] = start_script;
         }
@@ -282,7 +255,7 @@ define(function(require) {
           $("input#context_type_vcenter", context).click();
 
           if(this["CUSTOMIZATION_SPEC"]){
-            $('input#vcenter_customizations_value', context).val(this["CUSTOMIZATION_SPEC"]).change();
+            WizardFields.fillInput($('input.vcenter_customizations_value', context), this["CUSTOMIZATION_SPEC"]);
           } else if(userInputsJSON || contextJSON) {
             $("input#context_type_opennebula", context).click();
           }
@@ -292,23 +265,17 @@ define(function(require) {
       });
     }
 
-    $("#ssh_context", context).removeAttr('checked');
-    $("#network_context", context).removeAttr('checked');
+    $(".ssh_context", context).removeAttr('checked');
+    $(".network_context", context).removeAttr('checked');
 
     if (userInputsJSON) {
-      $.each(userInputsJSON, function(key, value) {
-        $(".add_service_custom_attr", context).trigger("click");
+      UserInputs.fill(context, templateJSON);
 
-        var context = $(".service_custom_attrs tbody tr", context).last();
-        var parts = value.split("|");
-        $(".user_input_name", context).val(key);
-        $(".user_input_type", context).val(parts[1]);
-        $(".user_input_description", context).val(TemplateUtils.escapeDoubleQuotes(TemplateUtils.htmlDecode(parts[2])));
-
-        if (contextJSON) {
+      if (contextJSON) {
+        $.each(userInputsJSON, function(key, value){
           delete contextJSON[key];
-        }
-      });
+        });
+      }
 
       delete templateJSON['USER_INPUTS'];
     }
@@ -326,19 +293,19 @@ define(function(require) {
       var customTagsJSON = {};
       $.each(contextJSON, function(key, value) {
         if (ssh_regexp.test(key)) {
-          $("#ssh_context", context).prop('checked', 'checked');
+          $(".ssh_context", context).prop('checked', 'checked');
 
           if (!publickey_regexp.test(value)) {
-            $("#ssh_public_key", context).val(TemplateUtils.htmlDecode(value));
+            WizardFields.fillInput($("#ssh_public_key", context), value);
           }
         } else if (token_regexp.test(key)) {
-          $("#token_context", context).prop('checked', 'checked');
+          $(".token_context", context).prop('checked', 'checked');
         } else if (net_regexp.test(key)) {
-          $("#network_context", context).prop('checked', 'checked');
+          $(".network_context", context).prop('checked', 'checked');
         } else if ("INIT_SCRIPTS" == key) {
-          $("input#INIT_SCRIPTS").val(TemplateUtils.htmlDecode(value));
+          WizardFields.fillInput($("input.INIT_SCRIPTS", context), value);
         } else if ("FILES_DS" == key) {
-          $('#FILES_DS', context).val(TemplateUtils.escapeDoubleQuotes(TemplateUtils.htmlDecode(contextJSON["FILES_DS"])))
+          WizardFields.fillInput($('.FILES_DS', context), contextJSON["FILES_DS"]);
           var files = [];
           while (match = file_ds_regexp.exec(value)) {
             files.push(match[1])
@@ -349,10 +316,10 @@ define(function(require) {
             }
           that.contextFilesTable.selectResourceTableSelect(selectedResources);
         } else if ("START_SCRIPT_BASE64" == key) {
-          $("#ENCODE_START_SCRIPT", context).prop('checked', 'checked');
-          $("#START_SCRIPT", context).val(atob(value));
+          $(".ENCODE_START_SCRIPT", context).prop('checked', 'checked');
+          $(".START_SCRIPT", context).val(atob(value));
         } else if ("START_SCRIPT" ==  key) {
-          $("#START_SCRIPT", context).val(TemplateUtils.escapeDoubleQuotes(TemplateUtils.htmlDecode(value)));
+          WizardFields.fillInput($(".START_SCRIPT", context), value);
         } else {
           customTagsJSON[key] = value;
         }
@@ -372,6 +339,6 @@ define(function(require) {
       req_string.push("$FILE[IMAGE_ID="+ fileId +"]");
     });
 
-    $('#FILES_DS', context).val(req_string.join(" "));
+    $('.FILES_DS', context).val(req_string.join(" "));
   };
 });
