@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -21,8 +21,11 @@ define(function(require) {
 
   var Locale = require('utils/locale');
   var Tips = require('utils/tips');
-  var CapacityInputs = require('./general/capacity-inputs');
+  var CapacityCreate = require('./general/capacity-create');
   var WizardFields = require('utils/wizard-fields');
+  var Config = require('sunstone-config');
+  var UserInputs = require('utils/user-inputs');
+  var UniqueId = require('utils/unique-id');
 
   /*
     TEMPLATES
@@ -35,24 +38,13 @@ define(function(require) {
    */
 
   var WIZARD_TAB_ID = require('./general/wizardTabId');
-  var LOGOS = [
-    {'path': "images/logos/arch.png",       'title': Locale.tr("Arch Linux")},
-    {'path': "images/logos/centos.png",     'title': Locale.tr("CentOS")},
-    {'path': "images/logos/debian.png",     'title': Locale.tr("Debian")},
-    {'path': "images/logos/fedora.png",     'title': Locale.tr("Fedora")},
-    {'path': "images/logos/linux.png",      'title': Locale.tr("Linux")},
-    {'path': "images/logos/redhat.png",     'title': Locale.tr("Redhat")},
-    {'path': "images/logos/ubuntu.png",     'title': Locale.tr("Ubuntu")},
-    {'path': "images/logos/windowsxp.png",  'title': Locale.tr("Windows XP/2003")},
-    {'path': "images/logos/windows8.png",   'title': Locale.tr("Windows 8")}
-  ]
 
   /*
     CONSTRUCTOR
    */
 
   function WizardTab() {
-    this.wizardTabId = WIZARD_TAB_ID;
+    this.wizardTabId = WIZARD_TAB_ID + UniqueId.id();
     this.icon = 'fa-laptop';
     this.title = Locale.tr("General");
   }
@@ -72,8 +64,8 @@ define(function(require) {
 
   function _html() {
     return TemplateHTML({
-      'capacityInputsHTML': CapacityInputs.html(),
-      'logos': LOGOS
+      'capacityCreateHTML': CapacityCreate.html(),
+      'logos': Config.vmLogos
     });
   }
 
@@ -90,12 +82,12 @@ define(function(require) {
       $('#NAME', context).removeAttr("required");
     }
 
-    context.foundation('slider', 'reflow');
+    //context.foundation('slider', 'reflow');
   }
 
   function _setup(context) {
     $(document).on('click', "[href='#" + this.wizardTabId + "']", function(){
-      context.foundation('slider', 'reflow');
+      //context.foundation('slider', 'reflow');
     });
 
     context.on("change", "#LOGO", function() {
@@ -113,7 +105,7 @@ define(function(require) {
       // There is another listener in context.js setup
     });
 
-    CapacityInputs.setup(context);
+    CapacityCreate.setup($("div.capacityCreate", context));
   }
 
   function _retrieve(context) {
@@ -122,38 +114,87 @@ define(function(require) {
     if (templateJSON["HYPERVISOR"] == 'vcenter') {
       templateJSON["VCENTER_PUBLIC_CLOUD"] = {
         'TYPE': 'vcenter',
-        'VM_TEMPLATE': $("#vcenter_template_uuid", context).val()
+        'VM_TEMPLATE': WizardFields.retrieveInput($("#vcenter_template_uuid", context))
       };
 
       templateJSON["KEEP_DISKS_ON_DONE"] = $("#KEEP_DISKS", context).is(':checked')?"YES":"NO"
     }
 
-    if ($('#sunstone_capacity_select:checked', context).length > 0) {
-      templateJSON["SUNSTONE_CAPACITY_SELECT"] = "NO"
-    }
+    var sunstone_template = {};
 
     if ($('#sunstone_network_select:checked', context).length > 0) {
-      templateJSON["SUNSTONE_NETWORK_SELECT"] = "NO"
+      sunstone_template["NETWORK_SELECT"] = "NO"
     }
 
+    if (!$.isEmptyObject(sunstone_template)) {
+      templateJSON['SUNSTONE'] = sunstone_template;
+    }
+
+    var userInputs = {};
+    
+    // Retrieve Datastore Attribute
+    var dsInput = $(".vcenter_datastore_input", context);
+    if (dsInput.length > 0) {
+      var dsModify = WizardFields.retrieveInput($('.modify_datastore', dsInput));
+      var dsInitial = WizardFields.retrieveInput($('.initial_datastore', dsInput));
+      var dsParams = WizardFields.retrieveInput($('.available_datastores', dsInput));
+
+      if (dsModify === 'fixed' && dsInitial !== '') {
+        templateJSON['VCENTER_DATASTORE'] = dsInitial;
+      } else if (dsModify === 'list' && dsParams !== '') {
+        var dsUserInputsStr = UserInputs.marshall({
+            type: 'list',
+            description: Locale.tr("Which datastore you want this VM to run on?"),
+            initial: dsInitial,
+            params: dsParams
+          });
+
+        userInputs['VCENTER_DATASTORE'] = dsUserInputsStr;
+      }
+    }
+
+    // Retrieve Resource Pool Attribute
+    var rpInput = $(".vcenter_rp_input", context);
+    if (rpInput.length > 0) {
+      var rpModify = WizardFields.retrieveInput($('.modify_rp', rpInput));
+      var rpInitial = WizardFields.retrieveInput($('.initial_rp', rpInput));
+      var rpParams = WizardFields.retrieveInput($('.available_rps', rpInput));
+
+      if (rpModify === 'fixed' && rpInitial !== '') {
+        templateJSON['RESOURCE_POOL'] = rpInitial;
+      } else if (rpModify === 'list' && rpParams !== '') {
+        var rpUserInputs = UserInputs.marshall({
+            type: 'list',
+            description: Locale.tr("Which resource pool you want this VM to run in?"),
+            initial: rpInitial,
+            params: WizardFields.retrieveInput($('.available_rps', rpInput))
+          });
+
+        userInputs['RESOURCE_POOL'] = rpUserInputs;
+      }
+    }
+
+    // Since the USER_INPUTS section is not enabled for vCenter, we can assume that there are no more user inputs defined
+    if (!$.isEmptyObject(userInputs)) {
+      templateJSON['USER_INPUTS'] = userInputs;
+    }
+
+    $.extend(true, templateJSON, CapacityCreate.retrieve($("div.capacityCreate", context)));
+
     return templateJSON;
   }
 
   function _fill(context, templateJSON) {
-    if (templateJSON["SUNSTONE_CAPACITY_SELECT"] &&
-          (templateJSON["SUNSTONE_CAPACITY_SELECT"].toUpperCase() == "NO")) {
-      $("#sunstone_capacity_select", context).attr("checked", "checked");
+    var sunstone_template = templateJSON.SUNSTONE;
+    if (sunstone_template) {
+      if (sunstone_template["NETWORK_SELECT"] &&
+          sunstone_template["NETWORK_SELECT"].toUpperCase() == "NO") {
+        $("#sunstone_network_select", context).attr("checked", "checked");
+      }
+
+      delete sunstone_template["NETWORK_SELECT"];
     }
 
-    delete templateJSON["SUNSTONE_CAPACITY_SELECT"];
-
-    if (templateJSON["SUNSTONE_NETWORK_SELECT"] &&
-          (templateJSON["SUNSTONE_NETWORK_SELECT"].toUpperCase() == "NO")) {
-      $("#sunstone_network_select", context).attr("checked", "checked");
-    }
-
-    delete templateJSON["SUNSTONE_NETWORK_SELECT"];
-
     if (templateJSON["HYPERVISOR"] == 'vcenter' &&
       templateJSON["KEEP_DISKS_ON_DONE"] &&
         templateJSON["KEEP_DISKS_ON_DONE"].toLowerCase() == "yes" ) {
@@ -172,7 +213,7 @@ define(function(require) {
 
         $.each(publicClouds, function(){
           if(this["TYPE"] == "vcenter"){
-            $("#vcenter_template_uuid", context).val(this["VM_TEMPLATE"]);
+            WizardFields.fillInput($("#vcenter_template_uuid", context), this["VM_TEMPLATE"]);
             return false;
           }
         });
@@ -184,6 +225,34 @@ define(function(require) {
       delete templateJSON["HYPERVISOR"];
     }
 
+    if (templateJSON["USER_INPUTS"]) {
+      if (templateJSON["USER_INPUTS"]["VCENTER_DATASTORE"]) {
+        var ds = UserInputs.unmarshall(templateJSON["USER_INPUTS"]["VCENTER_DATASTORE"]);
+        $('.modify_datastore', context).val('list');
+        $('.initial_datastore', context).val(ds.initial);
+        $('.available_datastores', context).val(ds.params);
+      }
+
+      if (templateJSON["USER_INPUTS"]["RESOURCE_POOL"]) {
+        var rp = UserInputs.unmarshall(templateJSON["USER_INPUTS"]["RESOURCE_POOL"]);
+        $('.modify_rp', context).val('list');
+        $('.initial_rp', context).val(rp.initial);
+        $('.available_rps', context).val(rp.params);
+      }
+    }
+
+    if (templateJSON["VCENTER_DATASTORE"]) {
+      $('.modify_datastore', context).val('fixed');
+      WizardFields.fillInput($('.initial_datastore', context), templateJSON["VCENTER_DATASTORE"]);
+    }
+
+    if (templateJSON["RESOURCE_POOL"]) {
+      $('.modify_rp', context).val('fixed');
+      WizardFields.fillInput($('.initial_rp', context), templateJSON["RESOURCE_POOL"]);
+    }
+
+    CapacityCreate.fill($("div.capacityCreate", context), templateJSON);
+
     WizardFields.fill(context, templateJSON);
   }
 });
