@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,12 +19,13 @@ define(function(require) {
     DEPENDENCIES
    */
 
-  require('foundation.tab');
+//  require('foundation.tab');
   var Config = require('sunstone-config');
   var Locale = require('utils/locale');
   var Tips = require('utils/tips');
   var WizardFields = require('utils/wizard-fields');
   var NicTab = require('./network/nic-tab');
+  var UniqueId = require('utils/unique-id');
 
   /*
     TEMPLATES
@@ -44,15 +45,19 @@ define(function(require) {
     CONSTRUCTOR
    */
 
-  function WizardTab() {
+  function WizardTab(opts) {
     if (!Config.isTemplateCreationTabEnabled('network')) {
       throw "Wizard Tab not enabled";
     }
 
-    this.wizardTabId = WIZARD_TAB_ID;
+    this.wizardTabId = WIZARD_TAB_ID + UniqueId.id();
     this.icon = 'fa-globe';
     this.title = Locale.tr("Network");
-    this.classes = "hypervisor only_kvm only_vmware only_xen only_vcenter"
+    this.classes = "hypervisor only_kvm only_vcenter"
+
+    if(opts.listener != undefined){
+      this.listener = opts.listener;
+    }
   }
 
   WizardTab.prototype.constructor = WizardTab;
@@ -78,6 +83,9 @@ define(function(require) {
   }
 
   function _onShow(context, panelForm) {
+    $.each(this.nicTabObjects, function(id, tab) {
+      tab.onShow();
+    })
   }
 
   function _setup(context) {
@@ -86,27 +94,7 @@ define(function(require) {
     that.numberOfNics = 0;
     that.nicTabObjects = {};
 
-    context.foundation('tab', 'reflow');
-
-    // close icon: removing the tab on click
-    context.on("click", "i.remove-tab", function() {
-      var target = $(this).parent().attr("href");
-      var dd = $(this).closest('dd');
-      var dl = $(this).closest('dl');
-      var content = $(target);
-
-      dd.remove();
-      content.remove();
-
-      var nicId = content.attr("nicId");
-      delete that.nicTabObjects[nicId];
-
-      if (dd.attr("class") == 'active') {
-        $('a', dl.children('dd').last()).click();
-      }
-
-      that.renameTabLinks(context);
-    });
+    Foundation.reflow(context, 'tabs');
 
     context.on("click", "#tf_btn_nics", function() {
       that.addNicTab(context);
@@ -114,6 +102,12 @@ define(function(require) {
     });
 
     that.addNicTab(context);
+
+    if(that.listener != undefined){
+      $(context).on("change", "input", function(){
+        that.listener.notify();
+      });
+    }
   }
 
   function _retrieve(context) {
@@ -127,7 +121,7 @@ define(function(require) {
 
     if (nicsJSON.length > 0) { templateJSON['NIC'] = nicsJSON; };
 
-    var nicDefault = $('#DEFAULT_MODEL', context).val();
+    var nicDefault = WizardFields.retrieveInput($('#DEFAULT_MODEL', context));
     if (nicDefault) {
       templateJSON['NIC_DEFAULT'] = {
         'MODEL': nicDefault
@@ -163,7 +157,7 @@ define(function(require) {
     var nicDefault = templateJSON.NIC_DEFAULT
     if (nicDefault != undefined) {
       if (nicDefault.MODEL) {
-        $('#DEFAULT_MODEL', context).val(nicDefault.MODEL);
+        WizardFields.fillInput($('#DEFAULT_MODEL', context), nicDefault.MODEL);
       }
 
       delete templateJSON.NIC_DEFAULT;
@@ -171,29 +165,56 @@ define(function(require) {
   }
 
   function _addNicTab(context) {
-    this.numberOfNics++;
-    var nicTab = new NicTab(this.numberOfNics);
+    var that = this;
+    that.numberOfNics++;
+    var nicTab = new NicTab(that.numberOfNics);
 
-    var content = $('<div id="' + nicTab.nicTabId + '" class="active nic wizard_internal_tab content">' +
+    var content = $('<div id="' + nicTab.nicTabId + '" class="nic wizard_internal_tab tabs-panel">' +
         nicTab.html() +
       '</div>').appendTo($("#" + CONTENTS_CONTAINER_ID, context));
 
-    var a = $("<dd class='active'>" +
+    var a = $("<li class='tabs-title'>" +
        "<a href='#" + nicTab.nicTabId + "'>" + Locale.tr("NIC") + "</a>" +
-      "</dd>").appendTo($("#" + LINKS_CONTAINER_ID, context));
+      "</li>").appendTo($("#" + LINKS_CONTAINER_ID, context));
+
+    Foundation.reInit($("#" + LINKS_CONTAINER_ID, context));
 
     $("a", a).trigger("click");
 
     nicTab.setup(content);
-    content.attr("nicId", this.numberOfNics);
+    content.attr("nicId", that.numberOfNics);
 
-    this.renameTabLinks(context);
-    this.nicTabObjects[this.numberOfNics] = nicTab;
+    that.renameTabLinks(context);
+    that.nicTabObjects[that.numberOfNics] = nicTab;
+
+    // close icon: removing the tab on click
+    a.on("click", "i.remove-tab", function() {
+      var target = $(this).parent().attr("href");
+      var li = $(this).closest('li');
+      var ul = $(this).closest('ul');
+      var content = $(target);
+
+      li.remove();
+      content.remove();
+
+      var nicId = content.attr("nicId");
+      delete that.nicTabObjects[nicId];
+
+      if (li.hasClass('is-active')) {
+        $('a', ul.children('li').last()).click();
+      }
+
+      that.renameTabLinks(context);
+    });
   }
 
   function _renameTabLinks(context) {
-    $("#" + LINKS_CONTAINER_ID + " dd", context).each(function(index) {
+    $("#" + LINKS_CONTAINER_ID + " li", context).each(function(index) {
       $("a", this).html(Locale.tr("NIC") + ' ' + index + " <i class='fa fa-times-circle remove-tab'></i>");
     })
+
+    if(this.listener != undefined){
+      this.listener.notify();
+    }
   }
 });
