@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,7 +19,7 @@ define(function(require) {
     DEPENDENCIES
    */
 
-  require('foundation.tab');
+//  require('foundation.tab');
   var Config = require('sunstone-config');
   var Locale = require('utils/locale');
   var Tips = require('utils/tips');
@@ -27,6 +27,7 @@ define(function(require) {
   var TemplateUtils = require('utils/template-utils');
   var HostsTable = require('tabs/hosts-tab/datatable');
   var ClustersTable = require('tabs/clusters-tab/datatable');
+  var UniqueId = require('utils/unique-id');
 
   /*
     TEMPLATES
@@ -49,7 +50,7 @@ define(function(require) {
       throw "Wizard Tab not enabled";
     }
 
-    this.wizardTabId = WIZARD_TAB_ID;
+    this.wizardTabId = WIZARD_TAB_ID + UniqueId.id();
     this.icon = 'fa-sitemap';
     this.title = Locale.tr("Scheduling");
 
@@ -89,24 +90,24 @@ define(function(require) {
 
   function _setup(context) {
     var  that = this;
-    context.foundation('tab', 'reflow');
+    Foundation.reflow(context, 'tabs');
 
     context.on("change", "input[name='req_select']", function() {
       if ($("input[name='req_select']:checked").val() == "host_select") {
-        $("div.host_select",    context).toggle();
+        $("div.host_select",    context).show();
         $("div.cluster_select", context).hide();
       } else {
         $("div.host_select",    context).hide();
-        $("div.cluster_select", context).toggle();
+        $("div.cluster_select", context).show();
       }
     });
 
     context.on("change", "input[name='rank_select']", function() {
-      $("#SCHED_RANK", context).val(this.value);
+      WizardFields.fillInput($("#SCHED_RANK", context), this.value);
     });
 
     context.on("change", "input[name='ds_rank_select']", function() {
-      $("#SCHED_DS_RANK", context).val(this.value);
+      WizardFields.fillInput($("#SCHED_DS_RANK", context), this.value);
     });
 
     var selectOptions = {
@@ -131,6 +132,8 @@ define(function(require) {
   }
 
   function _fill(context, templateJSON) {
+    WizardFields.fill(context, templateJSON);
+
     var reqJSON = templateJSON['SCHED_REQUIREMENTS'];
     if (reqJSON) {
       var req = TemplateUtils.escapeDoubleQuotes(reqJSON);
@@ -161,14 +164,12 @@ define(function(require) {
 
       this.clustersTable.selectResourceTableSelect(selectedResources);
 
-      $('input#SCHED_REQUIREMENTS', context).val(TemplateUtils.htmlDecode(req));
       delete templateJSON['SCHED_REQUIREMENTS'];
     }
 
     var dsReqJSON = templateJSON['SCHED_DS_REQUIREMENTS'];
     if (dsReqJSON) {
       var dsReq = TemplateUtils.escapeDoubleQuotes(dsReqJSON);
-      $('input#SCHED_DS_REQUIREMENTS', context).val(TemplateUtils.htmlDecode(dsReq));
       delete templateJSON['SCHED_DS_REQUIREMENTS'];
     }
 
@@ -189,8 +190,6 @@ define(function(require) {
             $('input[name="rank_select"]#loadawareRadio', context).click()
         }
 
-        $('input#SCHED_RANK', context).val(TemplateUtils.htmlDecode(rankJSON));
-
         delete templateJSON["SCHED_RANK"];
     }
 
@@ -206,8 +205,6 @@ define(function(require) {
             $('input[name="ds_rank_select"]#packingDSRadio', context).click()
         }
 
-        $('input#SCHED_DS_RANK', context).val(TemplateUtils.htmlDecode(dsRankJSON));
-
         delete templateJSON["SCHED_DS_RANK"];
     }
   }
@@ -218,11 +215,11 @@ define(function(require) {
       var selected_clusters = this.clustersTable.retrieveResourceTableSelect();
 
       $.each(selected_hosts, function(index, hostId) {
-        req_string.push('ID=\\"'+hostId+'\\"');
+        req_string.push('ID="'+hostId+'"');
       });
 
       $.each(selected_clusters, function(index, clusterId) {
-        req_string.push('CLUSTER_ID=\\"'+clusterId+'\\"');
+        req_string.push('CLUSTER_ID="'+clusterId+'"');
       });
 
       $('#SCHED_REQUIREMENTS', context).val(req_string.join(" | "));
