@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -70,25 +70,26 @@ define(function(require) {
 
     this.userCreation.setup(context);
 
-    context.foundation('abide', 'reflow');
-    context.off('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form');
-    context.off('valid.fndtn.abide', '#' + DIALOG_ID + 'Form');
+    Foundation.reflow(context, 'abide');
 
-    context.on('invalid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
-      Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
-    }).on('valid.fndtn.abide', '#' + DIALOG_ID + 'Form', function(e) {
-      var inputs = that.userCreation.retrieve(context);
+    $('#' + DIALOG_ID + 'Form')
+      .on('forminvalid.zf.abide', function(ev, frm) {
+        Notifier.notifyError(Locale.tr("One or more required fields are missing or malformed."));
+      })
+      .on('formvalid.zf.abide', function(ev, frm) {
+        var inputs = that.userCreation.retrieve(context);
 
-      var selElems = Sunstone.getDataTable(TAB_ID).elements();
+        var selElems = Sunstone.getDataTable(TAB_ID).elements();
 
-      Sunstone.runAction('User.chauth', selElems, inputs.auth_driver);
+        Sunstone.runAction('User.chauth', selElems, inputs.auth_driver);
 
-      Sunstone.getDialog(DIALOG_ID).hide();
-      Sunstone.getDialog(DIALOG_ID).reset();
-      Sunstone.runAction('User.refresh');
-
-      return false;
-    });
+        Sunstone.getDialog(DIALOG_ID).hide();
+        Sunstone.getDialog(DIALOG_ID).reset();
+        Sunstone.runAction('User.refresh');
+      })
+      .on("submit", function(ev) {
+        ev.preventDefault();
+      });
 
     return false;
   }
