@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -31,11 +31,13 @@ define(function(require) {
   var VNC_DIALOG_ID              = require('./dialogs/vnc/dialogId');
   var SPICE_DIALOG_ID            = require('./dialogs/spice/dialogId');
   var SAVE_AS_TEMPLATE_DIALOG_ID = require('./dialogs/saveas-template/dialogId');
+  var UPDATECONF_FORM_ID         = require('./form-panels/updateconf/formPanelId');
 
   var XML_ROOT = "VM";
   var RESOURCE = "VM";
 
-  var _commonActions = new CommonActions(OpenNebulaVM, RESOURCE, TAB_ID);
+  var _commonActions = new CommonActions(OpenNebulaVM, RESOURCE, TAB_ID,
+    XML_ROOT, Locale.tr("VM created"));
 
   var _actions = {
     "VM.list":    _commonActions.list(),
@@ -55,24 +57,21 @@ define(function(require) {
       error: Notifier.onError
     },
     "VM.refresh": _commonActions.refresh(),
-    "VM.delete":  _commonActions.del(),
     "VM.chown": _commonActions.multipleAction('chown'),
     "VM.chgrp": _commonActions.multipleAction('chgrp'),
-
     "VM.hold":    _commonActions.multipleAction('hold'),
     "VM.release": _commonActions.multipleAction('release'),
     "VM.suspend": _commonActions.multipleAction('suspend'),
     "VM.resume": _commonActions.multipleAction('resume'),
     "VM.stop": _commonActions.multipleAction('stop'),
-    "VM.reboot_hard": _commonActions.multipleAction('reset'),
-    "VM.delete_recreate": _commonActions.multipleAction('resubmit'),
     "VM.reboot": _commonActions.multipleAction('reboot'),
+    "VM.reboot_hard": _commonActions.multipleAction('reboot_hard'),
     "VM.poweroff": _commonActions.multipleAction('poweroff'),
     "VM.poweroff_hard": _commonActions.multipleAction('poweroff_hard'),
     "VM.undeploy": _commonActions.multipleAction('undeploy'),
     "VM.undeploy_hard": _commonActions.multipleAction('undeploy_hard'),
-    "VM.shutdown": _commonActions.multipleAction('shutdown'),
-    "VM.shutdown_hard": _commonActions.multipleAction('shutdown_hard'),
+    "VM.terminate": _commonActions.multipleAction('terminate'),
+    "VM.terminate_hard": _commonActions.multipleAction('terminate_hard'),
     "VM.recover": _commonActions.multipleAction('recover'),
     "VM.resched": _commonActions.multipleAction('resched'),
     "VM.unresched": _commonActions.multipleAction('unresched'),
@@ -80,6 +79,7 @@ define(function(require) {
     "VM.chmod": _commonActions.singleAction('chmod'),
     "VM.rename": _commonActions.singleAction('rename'),
     "VM.update_template": _commonActions.updateTemplate(),
+    "VM.append_template": _commonActions.appendTemplate(),
     "VM.deploy_action": _commonActions.singleAction('deploy'),
     "VM.migrate_action": _commonActions.singleAction('migrate'),
     "VM.migrate_live_action": _commonActions.singleAction('livemigrate'),
@@ -106,10 +106,9 @@ define(function(require) {
       type: "custom",
       call: function(id, name) {
         Sunstone.runAction("Template.instantiate", [id], name);
-        Sunstone.runAction("VM.refresh");
       },
       callback: function(request, response) {
-        Sunstone.getDataTable(TAB_ID).addElement(request, response);
+        Sunstone.runAction("VM.refresh");
       },
       error: Notifier.onError
     },
@@ -196,7 +195,7 @@ define(function(require) {
       },
       notify: true
     },
-    "VM.saveas_template" : {
+    "VM.save_as_template" : {
       type: "single",
       call: function() {
        var dialog = Sunstone.getDialog(SAVE_AS_TEMPLATE_DIALOG_ID);
@@ -207,6 +206,24 @@ define(function(require) {
       },
       notify: false
     },
+    "VM.updateconf": {
+      type: "single",
+      call: OpenNebulaVM.updateconf,
+      callback: function (req) {
+        Sunstone.resetFormPanel(TAB_ID, UPDATECONF_FORM_ID);
+        Sunstone.hideFormPanel(TAB_ID);
+
+        Sunstone.runAction("VM.refresh");
+      },
+      elements: function() {
+        return Sunstone.getDataTable(TAB_ID).elements();
+      },
+      error: function(request, response) {
+        Sunstone.hideFormPanelLoading(TAB_ID);
+        Notifier.onError(request, response);
+      },
+      notify: false
+    }
   };
 
   return _actions;
