@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -72,7 +72,7 @@ define(function(require) {
     Tips.setup(context);
 
     $('#' + DIALOG_ID + 'Form', context).submit(function() {
-      var templateJSON = CapacityInputs.retrieveResize(context);
+      var templateJSON = CapacityInputs.retrieveChanges(context);
 
       var enforce = $("#enforce", this).is(":checked");
 
@@ -96,7 +96,7 @@ define(function(require) {
     $('#vm_id', context).text(that.element.ID);
     CapacityInputs.fill(context, that.element);
 
-    context.foundation('slider', 'reflow');
+    // TODO context.foundation('slider', 'reflow');
     
     return false;
   }
