@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,6 +25,8 @@ define(function(require) {
   var PermissionsTable = require('utils/panel/permissions-table');
   var TemplateTable = require('utils/panel/template-table');
   var OpenNebulaVM = require('opennebula/vm');
+  var Sunstone = require('sunstone');
+  var Config = require('sunstone-config');
 
   /*
     TEMPLATES
@@ -73,9 +75,22 @@ define(function(require) {
     var lcmStateStr = OpenNebulaVM.lcmStateStr(this.element.LCM_STATE);
     var hostname = OpenNebulaVM.hostnameStr(this.element);
 
-    var deployId = (typeof(this.element.DEPLOY_ID) == "object" ? "-" : this.element.DEPLOY_ID);
+    var deployId = (typeof(this.element.DEPLOY_ID) == "object" ? "--" : this.element.DEPLOY_ID);
     var resched = (parseInt(this.element.RESCHED) ? Locale.tr("yes") : Locale.tr("no"))
-    var templateTableHTML = TemplateTable.html(this.element.USER_TEMPLATE, RESOURCE, Locale.tr("Attributes"));
+
+    // Get rid of the unwanted (for show) SCHED_* keys
+    var that = this;
+    var strippedTemplate = {};
+    var unshownValues = {};
+    $.each(that.element.USER_TEMPLATE, function(key, value) {
+      if (!key.match(/^SCHED_*/)) {
+        strippedTemplate[key] = value;
+      } else {
+        unshownValues[key] = value;
+      }
+    })
+
+    var templateTableHTML = TemplateTable.html(strippedTemplate, RESOURCE, Locale.tr("Attributes"), unshownValues);
 
     var monitoring = $.extend({}, this.element.MONITORING);
     delete monitoring.CPU;
@@ -110,6 +125,15 @@ define(function(require) {
     RenameTr.setup(TAB_ID, RESOURCE, this.element.ID, context);
     PermissionsTable.setup(TAB_ID, RESOURCE, this.element, context);
 
+    $("a.vrid", context).on("click", function(){
+      // TODO: this should be checked internally in showElement,
+      // but it won't work because of bug #4198
+
+      if (Config.isTabEnabled("vrouters-tab")){
+        Sunstone.showElement("vrouters-tab", "VirtualRouter.show", $(this).text());
+      }
+    });
+
     // Get rid of the unwanted (for show) SCHED_* keys
     var that = this;
     var strippedTemplate = {};
