@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -91,10 +91,10 @@ define(function(require) {
             that.element.LCM_STATE) &&
           OpenNebulaVM.isNICAttachSupported(that.element)) {
         html += '\
-             <button id="attach_nic" class="button tiny success right radius" >' + Locale.tr("Attach nic") + '</button>'
+             <button id="attach_nic" class="button small success right radius" >' + Locale.tr("Attach nic") + '</button>'
       } else {
         html += '\
-             <button id="attach_nic" class="button tiny success right radius" disabled="disabled">' + Locale.tr("Attach nic") + '</button>'
+             <button id="attach_nic" class="button small success right radius" disabled="disabled">' + Locale.tr("Attach nic") + '</button>'
       }
     }
 
@@ -137,11 +137,11 @@ define(function(require) {
       html += '\
           <div class="row">\
               <div class="medium-6 columns">\
-                <div class="row text-center">\
-                  <h3 class="subheader"><small>' + Locale.tr("NET RX") + '</small></h3>\
+                <div class="row">\
+                  <span>' + Locale.tr("NET RX") + '</span3>\
                 </div>\
                 <div class="row">\
-                  <div class="large-12 columns centered graph text-center" id="vm_net_rx_graph" style="height: 100px;">\
+                  <div class="large-12 columns centered graph" id="vm_net_rx_graph" style="height: 100px;">\
                     <span  id="provision_dashboard_total" style="font-size:80px">\
                       <i class="fa fa-spinner fa-spin"></i>\
                     </span>\
@@ -153,11 +153,11 @@ define(function(require) {
                 </div>\
               </div>\
               <div class="medium-6 columns">\
-                <div class="row text-center">\
-                  <h3 class="subheader"><small>' + Locale.tr("NET TX") + '</small></h3>\
+                <div class="row">\
+                  <span>' + Locale.tr("NET TX") + '</span3>\
                 </div>\
                 <div class="row">\
-                  <div class="large-12 columns centered graph text-center" id="vm_net_tx_graph" style="height: 100px;">\
+                  <div class="large-12 columns centered graph" id="vm_net_tx_graph" style="height: 100px;">\
                     <span  id="provision_dashboard_total" style="font-size:80px">\
                       <i class="fa fa-spinner fa-spin"></i>\
                     </span>\
@@ -169,11 +169,11 @@ define(function(require) {
                 </div>\
               </div>\
               <div class="medium-6 columns">\
-                <div class="row text-center">\
-                  <h3 class="subheader"><small>' + Locale.tr("NET DOWNLOAD SPEED") + '</small></h3>\
+                <div class="row">\
+                  <span>' + Locale.tr("NET DOWNLOAD SPEED") + '</span3>\
                 </div>\
                 <div class="row">\
-                  <div class="large-12 columns centered graph text-center" id="vm_net_rx_speed_graph" style="height: 100px;">\
+                  <div class="large-12 columns centered graph" id="vm_net_rx_speed_graph" style="height: 100px;">\
                     <span  id="provision_dashboard_total" style="font-size:80px">\
                       <i class="fa fa-spinner fa-spin"></i>\
                     </span>\
@@ -185,11 +185,11 @@ define(function(require) {
                 </div>\
               </div>\
               <div class="medium-6 columns">\
-                <div class="row text-center">\
-                  <h3 class="subheader"><small>' + Locale.tr("NET UPLOAD SPEED") + '</small></h3>\
+                <div class="row">\
+                  <span>' + Locale.tr("NET UPLOAD SPEED") + '</span3>\
                 </div>\
                 <div class="row">\
-                  <div class="large-12 columns centered graph text-center" id="vm_net_tx_speed_graph" style="height: 100px;">\
+                  <div class="large-12 columns centered graph" id="vm_net_tx_speed_graph" style="height: 100px;">\
                     <span  id="provision_dashboard_total" style="font-size:80px">\
                       <i class="fa fa-spinner fa-spin"></i>\
                     </span>\
@@ -254,21 +254,41 @@ define(function(require) {
           });
         }
 
-        if (that.element.TEMPLATE.SECURITY_GROUP_RULE != undefined) {
-          $.each(that.element.TEMPLATE.SECURITY_GROUP_RULE, function() {
+        var rules = that.element.TEMPLATE.SECURITY_GROUP_RULE;
+
+        if (rules != undefined) {
+          if (!$.isArray(rules)) {
+            rules = [rules];
+          }
+
+          $.each(rules, function() {
             if (nic_secgroups[this.SECURITY_GROUP_ID]) {
               secgroups.push(this);
             }
           });
         }
 
+        function ipTr(attr){
+          var v = "--";
+
+          if (nic[attr] != undefined){
+            v = nic[attr];
+
+            if (nic["VROUTER_"+attr] != undefined){
+              v += ("<br/>" + nic["VROUTER_"+attr] + Locale.tr(" (VRouter)"));
+            }
+          }
+
+          return v;
+        }
+
         nic_dt_data.push({
           NIC_ID : nic.NIC_ID,
           NETWORK : nic.NETWORK,
-          IP : (nic.IP ? nic.IP : "--"),
+          IP : ipTr("IP"),
           MAC : nic.MAC,
-          IP6_ULA : (nic.IP6_ULA ? nic.IP6_ULA : "--"),
-          IP6_GLOBAL : (nic.IP6_GLOBAL ? nic.IP6_GLOBAL : "--"),
+          IP6_ULA : ipTr("IP6_ULA"),
+          IP6_GLOBAL : ipTr("IP6_GLOBAL"),
           ACTIONS : actions,
           SECURITY_GROUP_RULES : secgroups
         });
@@ -287,10 +307,10 @@ define(function(require) {
         },
         {"data": "NIC_ID",     "defaultContent": ""},
         {"data": "NETWORK",    "defaultContent": ""},
-        {"data": "IP",         "defaultContent": ""},
+        {"data": "IP",         "defaultContent": "", "class": "nowrap"},
         {"data": "MAC",        "defaultContent": ""},
-        {"data": "IP6_ULA",    "defaultContent": ""},
-        {"data": "IP6_GLOBAL", "defaultContent": ""},
+        {"data": "IP6_ULA",    "defaultContent": "", "class": "nowrap"},
+        {"data": "IP6_GLOBAL", "defaultContent": "", "class": "nowrap"},
         {"data": "ACTIONS",    "defaultContent": "", "orderable": false},
         {"defaultContent": "", "orderable": false}
       ],
@@ -372,7 +392,7 @@ define(function(require) {
 
         Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
           //header :
-          body : Locale.tr("This will detach the nic inmediately"),
+          body : Locale.tr("This will detach the nic immediately"),
           //question :
           submit : function(){
             Sunstone.runAction('VM.detachnic', that.element.ID, nic_id);
