@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -72,7 +72,7 @@ define(function(require) {
     var diskCost = this.element.TEMPLATE.DISK_COST;
 
     if (diskCost == undefined){
-      diskCost = Config.defaultCost.diskCost;
+      diskCost = Config.onedConf.DEFAULT_COST.DISK_COST;
     }
 
     return TemplateHtml({
@@ -149,7 +149,7 @@ define(function(require) {
 
     var disks = [];
     if ($.isArray(that.element.TEMPLATE.DISK))
-      disks = that.element.TEMPLATE.DISK;
+      disks = that.element.TEMPLATE.DISK.slice(0); // clone
     else if (!$.isEmptyObject(that.element.TEMPLATE.DISK))
       disks = [that.element.TEMPLATE.DISK];
 
@@ -350,7 +350,7 @@ define(function(require) {
 
         Sunstone.getDialog(CONFIRM_DIALOG_ID).setParams({
           //header :
-          body : Locale.tr("This will detach the disk inmediately"),
+          body : Locale.tr("This will detach the disk immediately"),
           //question :
           submit : function(){
             Sunstone.runAction('VM.detachdisk', that.element.ID, disk_id);
