@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -66,13 +66,16 @@ define(function(require) {
 
   function _setup(context) {
     var that = this;
-    context.foundation('abide', 'reflow');
+    Foundation.reflow(context, 'abide');
     
     that.arTab.setup(context, "update_ar");
 
-    $('#update_ar_form',context).on('invalid.fndtn.abide', function () {
+
+    $('#update_ar_form', context)
+      .on('forminvalid.zf.abide', function(ev, frm) {
         Notifier.notifyError(Locale.tr("One or more required fields are missing."));
-    }).on('valid.fndtn.abide', function () {
+      })
+      .on('formvalid.zf.abide', function(ev, frm) {
         var data = that.arTab.retrieve();
 
         data['AR_ID'] = that.arId;
@@ -81,7 +84,10 @@ define(function(require) {
         Sunstone.runAction('Network.update_ar', that.vnetId, obj);
 
         return false;
-    });
+      })
+      .on("submit", function(ev) {
+        ev.preventDefault();
+      });
   }
 
   function _onShow(context) {
