@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -23,11 +23,12 @@ define(function(require) {
     CONSTRUCTOR
    */
 
-  function CommonActions(openNebulaResource, resourceStr, tabId, xmlRoot) {
+  function CommonActions(openNebulaResource, resourceStr, tabId, xmlRoot, createdStr) {
     this.openNebulaResource = openNebulaResource;
     this.tabId = tabId;
     this.resourceStr = resourceStr;
     this.xmlRoot = xmlRoot;
+    this.createdStr = createdStr;
   }
 
   CommonActions.prototype.list = _list;
@@ -39,9 +40,11 @@ define(function(require) {
   CommonActions.prototype.create = _create;
   CommonActions.prototype.showCreate = _showCreate;
   CommonActions.prototype.showUpdate = _showUpdate;
+  CommonActions.prototype.checkAndShow = _checkAndShow;
   CommonActions.prototype.checkAndShowUpdate = _checkAndShowUpdate;
   CommonActions.prototype.update = _update;
   CommonActions.prototype.updateTemplate = _updateTemplate;
+  CommonActions.prototype.appendTemplate = _appendTemplate;
 
   return CommonActions;
 
@@ -95,7 +98,8 @@ define(function(require) {
       type: "multiple",
       call : that.openNebulaResource.del,
       callback : function(request, response) {
-        Sunstone.getDataTable(that.tabId).deleteElement(request, response);
+        var elementId = request.request.data.toString();
+        Sunstone.getDataTable(that.tabId).deleteElement(elementId);
       },
       elements: function() {
         return Sunstone.getDataTable(that.tabId).elements();
@@ -145,13 +149,15 @@ define(function(require) {
       callback : function(request, response) {
         Sunstone.resetFormPanel(that.tabId, formPanelId);
         Sunstone.hideFormPanel(that.tabId);
-        Sunstone.getDataTable(that.tabId).addElement(request, response);
+        that.refresh();
+
+        Notifier.notifyCustom(that.createdStr, " ID: " + response[that.xmlRoot].ID, false);
       },
       error: function(request, response) {
         Sunstone.hideFormPanelLoading(that.tabId);
         Notifier.onError(request, response);
       },
-      notify: true
+      notify: false
     }
   }
 
@@ -187,16 +193,33 @@ define(function(require) {
   function _checkAndShowUpdate() {
     var that = this;
     return {
-      type: "single",
+      type: 'single',
       call: function() {
-        var selected_nodes = Sunstone.getDataTable(that.tabId).elements();
-        if (selected_nodes.length != 1) {
-          Notifier.notifyMessage("Please select one (and just one) resource to update.");
+        var selectedNodes = Sunstone.getDataTable(that.tabId).elements();
+        if (selectedNodes.length != 1) {
+          Notifier.notifyMessage('Please select one (and just one) resource to update.');
           return false;
         }
 
-        var resource_id = "" + selected_nodes[0];
-        Sunstone.runAction(that.resourceStr + ".show_to_update", resource_id);
+        var resourceId = '' + selectedNodes[0];
+        Sunstone.runAction(that.resourceStr + '.show_to_update', resourceId);
+      }
+    }
+  }
+
+  function _checkAndShow(action) {
+    var that = this;
+    return {
+      type: 'single',
+      call: function() {
+        var selectedNodes = Sunstone.getDataTable(that.tabId).elements();
+        if (selectedNodes.length != 1) {
+          Notifier.notifyMessage('Please select one (and just one) resource to update.');
+          return false;
+        }
+
+        var resourceId = '' + selectedNodes[0];
+        Sunstone.runAction(that.resourceStr + '.' + action, resourceId);
       }
     }
   }
@@ -227,4 +250,16 @@ define(function(require) {
       error: Notifier.onError
     }
   }
+
+  function _appendTemplate() {
+    var that = this;
+    return {
+      type: "single",
+      call: that.openNebulaResource.append,
+      callback: function(request) {
+        Sunstone.runAction(that.resourceStr + '.show', request.request.data[0][0]);
+      },
+      error: Notifier.onError
+    }
+  }
 });
