@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -344,11 +344,11 @@ define(function(require) {
           <td class="rvms_bar"></td>\
         </tr>');
 
-      ResourceSelect.insert(
-        'td.image_select',
-        $(".image_quota_table tbody tr", context).last(),
-        "Image",
-        null, true);
+      ResourceSelect.insert({
+          context: $('.image_select', $(".image_quota_table tbody tr", context).last()),
+          resourceName: 'Image',
+          emptyValue: true
+        });
 
       $(".image_quota_table tbody tr", context).last().off(
                                             "change", ".resource_list_select");
@@ -499,11 +499,11 @@ define(function(require) {
           <td class="size_bar"></td>\
         </tr>');
 
-      ResourceSelect.insert(
-        'td.ds_select',
-        $(".ds_quota_table tbody tr", context).last(),
-        "Datastore",
-        null, true);
+      ResourceSelect.insert({
+          context: $('.ds_select', $(".ds_quota_table tbody tr", context).last()),
+          resourceName: 'Datastore',
+          emptyValue: true
+        });
 
       $(".ds_quota_table tbody tr", context).last().off(
                                             "change", ".resource_list_select");
@@ -651,11 +651,11 @@ define(function(require) {
               <td class="leases_bar"></td>\
           </tr>');
 
-      ResourceSelect.insert(
-          'td.network_select',
-          $(".network_quota_table tbody tr", context).last(),
-          "Network",
-          null, true);
+      ResourceSelect.insert({
+          context: $('.network_select', $(".network_quota_table tbody tr", context).last()),
+          resourceName: 'Network',
+          emptyValue: true
+        });
 
       $(".network_quota_table tbody tr", context).last().off(
                                             "change", ".resource_list_select");
@@ -761,12 +761,12 @@ define(function(require) {
 
     quotas_html +=
         '<div class="row">\
-          <div class="large-6 columns">' + vms_quota + '</div>\
-          <div class="large-6 columns">' + cpu_quota + '</div>\
+          <div class="medium-6 columns">' + vms_quota + '</div>\
+          <div class="medium-6 columns">' + cpu_quota + '</div>\
         </div>\
         <div class="row">\
-          <div class="large-6 columns">' + memory_quota + '</div>\
-          <div class="large-6 columns">' + system_disk_size_quota+ '</div>\
+          <div class="medium-6 columns">' + memory_quota + '</div>\
+          <div class="medium-6 columns">' + system_disk_size_quota+ '</div>\
         </div>\
         <br><br>\
         <div class="row">\
@@ -1026,77 +1026,78 @@ define(function(require) {
 
       html +=
           '<div class="small-5 columns">\
-            <ul class="button-group">\
-              <li><a style="font-size: 1em; margin: 0px" class="button tiny secondary quotabar_edit_btn"><span class="fa fa-pencil"></span></a></li>\
-              <li><a style="font-size: 1em; margin: 0px" class="button tiny secondary quotabar_default_btn"><span class="fa fa-file-o"></span></a></li>\
-              <li><a style="font-size: 1em; margin: 0px" class="button tiny secondary quotabar_unlimited_btn"><strong>&infin;</strong></a></li>\
-            </ul>\
+            <div class="button-group">\
+              <a class="button tiny secondary quotabar_edit_btn"><span class="fa fa-pencil"></span></a>\
+              <a class="button tiny secondary quotabar_default_btn"><span class="fa fa-file-o"></span></a>\
+              <a class="button tiny secondary quotabar_unlimited_btn"><strong>&infin;</strong></a>\
+            </div>\
           </div>\
         </div>\
         <div class="row collapse non_editable_quota">\
-          <div class="large-12 columns">\
-            <div class="progress-text right" style="font-size: 12px">\
-              '+info_str+'\
-            </div>\
-            <br>\
-            <div class="progress radius" style="height: 10px; margin-bottom:0px">\
-              <span class="meter" style="width: '
-                +percentage+'%" />\
-            </div>\
-          </div>\
+          <div class="large-12 columns">'+
+            ProgressBar.html(usage, limit, info_str)+
+          '</div>\
         </div>\
       </div>';
       return html;
   }
 
-  function _quotaInfo(usage, limit, default_limit, not_html){
-      var int_usage = parseInt(usage, 10);
-      var int_limit = _quotaIntLimit(limit, default_limit);
-      return _quotaBaseInfo(int_usage, int_limit, null, not_html);
+  /**
+   * @param      {number}   usage     The usage
+   * @param      {number}   limit     The max limit
+   * @param      {string}   info_str  The info str, can be null
+   * @return     {Object}           { "percentage", "str" }
+   */
+  function _process(usage, limit, info_str) {
+    info_str = info_str || (usage + ' / ' + ((limit >= 0) ? limit : '-'));
+
+    percentage = 0;
+
+    if (limit > 0) {
+      percentage = Math.floor((usage / limit) * 100);
+
+      if (percentage > 100) {
+        percentage = 100;
+      }
+    } else if (limit == 0 && usage > 0) {
+      percentage = 100;
+    }
+
+    return {
+      "percentage": percentage,
+      "str": info_str
+    }
   }
 
-  function _quotaMBInfo(usage, limit, default_limit, not_html){
+  /**
+   *  Returns an object with {"percentage", "str"}
+   */
+  function _quotaInfo(usage, limit, default_limit){
+      var int_usage = parseInt(usage, 10);
+      var int_limit = _quotaIntLimit(limit, default_limit);
+      return _process(int_usage, int_limit, null);
+  }
+
+  /**
+   *  Returns an object with {"percentage", "str"}
+   */
+  function _quotaMBInfo(usage, limit, default_limit){
       var int_usage = parseInt(usage, 10);
       var int_limit = _quotaIntLimit(limit, default_limit);
 
       info_str = Humanize.size(int_usage * 1024)+' / '
               +((int_limit >= 0) ? Humanize.size(int_limit * 1024) : '-')
 
-      return _quotaBaseInfo(int_usage, int_limit, info_str, not_html);
+      return _process(int_usage, int_limit, info_str);
   }
 
-  function _quotaFloatInfo(usage, limit, default_limit, not_html){
+  /**
+   *  Returns an object with {"percentage", "str"}
+   */
+  function _quotaFloatInfo(usage, limit, default_limit){
       var float_usage = parseFloat(usage, 10);
       var float_limit = _quotaFloatLimit(limit, default_limit);
-      return _quotaBaseInfo(float_usage, float_limit, null, not_html);
-  }
-
-  function _quotaBaseInfo(usage, limit, info_str, not_html){
-      percentage = 0;
-
-      if (limit > 0){
-          percentage = Math.floor((usage / limit) * 100);
-
-          if (percentage > 100){
-              percentage = 100;
-          }
-      } else if (limit == 0 && usage > 0){
-          percentage = 100;
-      }
-
-      info_str = info_str || ( usage+' / '+((limit >= 0) ? limit : '-') );
-
-      if (not_html) {
-          return {
-              "percentage": percentage,
-              "str": info_str
-          }
-      } else {
-          html = '<span class="progress-text right" style="font-size: 12px">'+info_str+'</span><br><div class="progress radius" style="height: 10px; margin-bottom:0px"><span class="meter" style="width: '
-              +percentage+'%"></div>';
-
-          return html;
-      }
+      return _process(float_usage, float_limit, null);
   }
 
   function _quotaBar(usage, limit, default_limit){
@@ -1157,26 +1158,26 @@ define(function(require) {
   // Quotas dialog
   //============================================================================
 
-  function _quotas_tmpl(){
+  function _quotas_tmpl(uniqueId){
     return '<div class="row">\
         <div class="large-12 columns">\
-          <dl class="tabs right-info-tabs text-center" data-tab>\
-               <dd class="active"><a href="#vm_quota"><i class="fa fa-cloud"></i><br>'+Locale.tr("VM")+'</a></dd>\
-               <dd><a href="#datastore_quota"><i class="fa fa-folder-open"></i><br>'+Locale.tr("Datastore")+'</a></dd>\
-               <dd><a href="#image_quota"><i class="fa fa-upload"></i><br>'+Locale.tr("Image")+'</a></dd>\
-               <dd><a href="#network_quota"><i class="fa fa-globe"></i><br>'+Locale.tr("VNet")+'</a></dd>\
+          <dl id="' + uniqueId + 'Tabs" class="tabs sunstone-info-tabs text-center" data-tabs>\
+               <dd class="tabs-title is-active"><a href="#vm_quota"><i class="fa fa-cloud"></i><br>'+Locale.tr("VM")+'</a></dd>\
+               <dd class="tabs-title"><a href="#datastore_quota"><i class="fa fa-folder-open"></i><br>'+Locale.tr("Datastore")+'</a></dd>\
+               <dd class="tabs-title"><a href="#image_quota"><i class="fa fa-upload"></i><br>'+Locale.tr("Image")+'</a></dd>\
+               <dd class="tabs-title"><a href="#network_quota"><i class="fa fa-globe"></i><br>'+Locale.tr("VNet")+'</a></dd>\
           </dl>\
         </div>\
       </div>\
       <div class="row">\
-        <div class="tabs-content">\
-          <div id="vm_quota" class="content active">\
+        <div class="tabs-content" data-tabs-content="' + uniqueId + 'Tabs">\
+          <div id="vm_quota" class="tabs-panel is-active">\
           </div>\
-          <div id="datastore_quota" class="content">\
+          <div id="datastore_quota" class="tabs-panel">\
           </div>\
-          <div id="image_quota" class="content">\
+          <div id="image_quota" class="tabs-panel">\
           </div>\
-          <div id="network_quota" class="content">\
+          <div id="network_quota" class="tabs-panel">\
           </div>\
         </div>\
       </div>';
@@ -1186,6 +1187,8 @@ define(function(require) {
    * Sets up a dialog to edit and update user and group quotas
    */
   function _setupQuotasDialog(context){
+    Foundation.reflow(context, 'tabs');
+
     $('form', context).submit(function(){
       var obj = retrieveQuotasValues(context);
       var action = $('div.form_buttons button',this).val();
@@ -1207,10 +1210,10 @@ define(function(require) {
     var datastore_quota = _datastoreWidget(resource_info, default_quotas);
 
     $("#vm_quota", context).html(
-        '<div class="large-6 columns">' + vms_quota + '</div>\
-        <div class="large-6 columns">' + cpu_quota + '</div>\
-        <div class="large-6 columns">' + memory_quota + '</div>\
-        <div class="large-6 columns">' + system_disk_size_quota+ '</div>');
+        '<div class="medium-6 columns">' + vms_quota + '</div>\
+        <div class="medium-6 columns">' + cpu_quota + '</div>\
+        <div class="medium-6 columns">' + memory_quota + '</div>\
+        <div class="medium-6 columns">' + system_disk_size_quota+ '</div>');
 
     $("#datastore_quota", context).html(
         '<div class="large-12 columns">' + datastore_quota + '</div>');
@@ -1228,8 +1231,6 @@ define(function(require) {
     _setupDatastoreWidget(context, default_quotas);
 
     quotasPanelEditAction(context);
-
-    context.foundation();
   }
 
   return {
