@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -19,7 +19,8 @@ define(function(require) {
     DEPENDENCIES
    */
 
-  require('foundation-datatables');
+  require('datatables.net');
+  require('datatables.foundation');
   var TemplateEmptyTable = require('hbs!./tab-datatable/empty-table');
   var Sunstone = require('sunstone');
   var SunstoneConfig = require('sunstone-config');
@@ -28,6 +29,7 @@ define(function(require) {
   var OpenNebula = require('opennebula');
   var Notifier = require('utils/notifier');
   var OpenNebulaUser = require('opennebula/user');
+  var LabelsUtils = require('utils/labels/utils');
 
   /*
     TEMPLATES
@@ -48,13 +50,25 @@ define(function(require) {
 
   /* Set the defaults for DataTables initialisation */
   $.extend(true, $.fn.dataTable.defaults, {
-    dom: "t<'row collapse'<'small-6 columns'i><'small-6 columns'lp>>",
+    dom:
+      "t"+
+      "<'row'<'small-6 columns'li><'small-6 columns'p>>",
     renderer: 'foundation',
+    autoWidth: false,
     language: {
       "sLengthMenu": "_MENU_",
       "emptyTable": TemplateEmptyTable()
     }
-  });
+  } );
+
+  //$.extend(true, $.fn.dataTable.defaults, {
+  //  dom: "t<'row collapse'<'small-6 columns'i><'small-6 columns'lp>>",
+  //  renderer: 'foundation',
+  //  language: {
+  //    "sLengthMenu": "_MENU_",
+  //    "emptyTable": TemplateEmptyTable()
+  //  }
+  //});
 
   /*
     CONSTRUCTOR
@@ -113,6 +127,10 @@ define(function(require) {
         $.each(that.selectOptions.starred_ids, function() {
           that.selectOptions.starred_ids_map[this] = true;
         });
+
+        if (that.selectOptions.starred_icon == undefined) {
+          that.selectOptions.starred_icon = '<i class="fa fa-star fa-fw"></i>';
+        }
       }
 
       if (that.selectOptions.multiple_choice == undefined) {
@@ -137,7 +155,6 @@ define(function(require) {
     'tableCheckboxesListener': _tableCheckboxesListener,
     'onlyOneCheckboxListener': _onlyOneCheckboxListener,
     'infoListener': _infoListener,
-    'addElement': _addElement,
     'deleteElement': _deleteElement,
     'updateElement': _updateElement,
     'elements': _elements,
@@ -153,7 +170,9 @@ define(function(require) {
     'idInput': _idInput,
     'initSelectResourceTableSelect': _initSelectResourceTableSelect,
     'updateFn': _updateFn,
-    'list': _list
+    'list': _list,
+    'clearLabelsFilter': _clearLabelsFilter,
+    'setLabelsFilter': _setLabelsFilter
   }
 
   return TabDatatable;
@@ -259,15 +278,9 @@ define(function(require) {
     });
   }
 
-  //call back for actions creating a zone element
-  function _addElement(request, element_json) {
-    var element = this.elementArray(element_json);
-    this.dataTable.fnAddData(element);
-  }
-
-  //deletes an element with id 'tag' from a dataTable
-  function _deleteElement(req) {
-    var tag = '#' + this.resource.toLowerCase() + '_' + req.request.data;
+  //deletes an element with id 'elementId' from a dataTable
+  function _deleteElement(elementId) {
+    var tag = '#' + this.resource.toLowerCase() + '_' + elementId;
     var tr = $(tag, this.dataTable).parents('tr')[0];
     this.dataTable.fnDeleteRow(tr);
     this.recountCheckboxes();
@@ -306,7 +319,7 @@ define(function(require) {
       context = this.conf.customTabContext;
     } else {
       context = table.parents('.tab');
-      if ($(".right-info", context).is(':visible')) {
+      if ($(".sunstone-info", context).is(':visible')) {
         return;
       }
     }
@@ -481,31 +494,46 @@ define(function(require) {
       });
     }
 
+    if (that.labelsColumn && !SunstoneConfig.isTabEnabled("provision-tab")) {
+      if (SunstoneConfig.isTabEnabled(that.tabId)) {
+        if ($("#" + that.tabId).is(':visible')) {
+          LabelsUtils.insertLabelsMenu({'tabName': that.tabId});
+          LabelsUtils.insertLabelsDropdown(that.tabId);
+        }
+      }
+    }
+
     if (that.postUpdateView) {
       that.postUpdateView();
     }
   }
 
   //replaces an element with id 'tag' in a dataTable with a new one
-  function _updateElement(request, element_json) {
-    var id = element_json[this.xmlRoot].ID;
-    var element = this.elementArray(element_json);
-    var tag = '#' + this.resource.toLowerCase() + '_' + id;
-    // fnGetData should be used instead, otherwise it depends on the visible columns
-    var nodes = this.dataTable.fnGetNodes();
-    var tr = $(tag, nodes).parents('tr')[0];
-    if (tr) {
-      var checked_val = $('input.check_item', tr).prop('checked');
-      var position = this.dataTable.fnGetPosition(tr);
-      this.dataTable.fnUpdate(element, position, undefined, false);
-      $('input.check_item', tr).prop('checked', checked_val);
-      this.recountCheckboxes();
-    }
+  function _updateElement(request, elementJSON) {
+    var that = this;
+    var elementId = elementJSON[that.xmlRoot].ID;
+    var element = that.elementArray(elementJSON);
+
+    $.each(that.dataTable.fnGetData(), function(index, aData) {
+      if (aData[that.selectOptions.id_index] === elementId) {
+        var nodes = that.dataTable.fnGetNodes();
+        var checkId = '#' + that.resource.toLowerCase() + '_' + elementId;
+        var checkVal = $(checkId, nodes).prop('checked');
+        that.dataTable.fnUpdate(element, index, undefined, false);
+        if (checkVal) {
+          $(checkId, nodes).prop('checked', checkVal);
+        }
+        that.recountCheckboxes();
+        return false;
+      }
+    });
   }
 
   function _getElementData(id, resource_tag) {
+    // TODO If the element is not included in the visible rows of 
+    // the table, it will not be included in the fnGetNodes response
     var nodes = this.dataTable.fnGetNodes();
-    var tr = $(resource_tag + '_' + id, nodes).parents('tr')[0];
+    var tr = $('#' + resource_tag + '_' + id, nodes).closest('tr');
     return this.dataTable.fnGetData(tr);
   }
 
@@ -568,7 +596,7 @@ define(function(require) {
         var row_id = aData[that.selectOptions.id_index];
 
         var ids = $('#selected_ids_row_' + that.dataTableId, section).data("ids");
-        if (ids[row_id]) {
+        if (ids != undefined && ids[row_id]) {
           $("td", nRow).addClass('markrowchecked');
           $('input.check_item', nRow).prop('checked', true);
         } else {
@@ -611,6 +639,12 @@ define(function(require) {
 
       $('#selected_resource_multiple_' + that.dataTableId, section).hide();
       $('#select_resource_multiple_' + that.dataTableId, section).show();
+    } else {
+      $('#selected_resource_' + that.dataTableId, section).hide();
+      $('#select_resource_' + that.dataTableId, section).show();
+
+      $('#selected_resource_multiple_' + that.dataTableId, section).hide();
+      $('#select_resource_multiple_' + that.dataTableId, section).hide();
     }
 
     $('#selected_resource_name_' + that.dataTableId, section).hide();
@@ -664,14 +698,15 @@ define(function(require) {
           $('#select_resource_multiple_' + that.dataTableId, section).hide();
         }
 
-        $('.alert-box', section).hide();
-
         return true;
       };
 
       $('#' + that.dataTableId + ' tbody', section).on("click", "tr", function(e) {
         var aData = that.dataTable.fnGetData(this);
-        row_click(this, aData);
+
+        if(aData != undefined){
+          row_click(this, aData);
+        }
       });
 
       $(section).on("click", '#selected_ids_row_' + that.dataTableId + ' span.fa.fa-times', function() {
@@ -713,19 +748,22 @@ define(function(require) {
         $("td.markrow", that.dataTable).removeClass('markrow');
         $('tbody input.check_item', that.dataTable).prop('checked', false);
 
-        $('#selected_resource_' + that.dataTableId, section).show();
-        $('#select_resource_' + that.dataTableId, section).hide();
-        $('.alert-box', section).hide();
+        if (aData != undefined){
+          $("td", this).addClass('markrow');
+          $('input.check_item', this).prop('checked', true);
 
-        $("td", this).addClass('markrow');
-        $('input.check_item', this).prop('checked', true);
+          $('#selected_resource_' + that.dataTableId, section).show();
+          $('#select_resource_' + that.dataTableId, section).hide();
 
-        $('#selected_resource_id_' + that.dataTableId, section).val(aData[that.selectOptions.id_index]).change();
+          $('#selected_resource_id_' + that.dataTableId, section).val(aData[that.selectOptions.id_index]).trigger("change");
 
-        $('#selected_resource_name_' + that.dataTableId, section).text(aData[that.selectOptions.name_index]).change();
-        $('#selected_resource_name_' + that.dataTableId, section).show();
+          $('#selected_resource_name_' + that.dataTableId, section).text(aData[that.selectOptions.name_index]).trigger("change");
+          $('#selected_resource_name_' + that.dataTableId, section).show();
 
-        that.selectOptions.select_callback(aData, that.selectOptions);
+          that.selectOptions.select_callback(aData, that.selectOptions);
+        }
+
+        $('#selected_resource_id_' + that.dataTableId, section).removeData("pending_select");
 
         return true;
       });
@@ -830,15 +868,7 @@ define(function(require) {
 
         var row_name = "" + row_id;
 
-        // TODO: improve preformance, linear search. Needed to get the
-        // name of the resource in the label. If function getName() was
-        // indexed in the cache, it could be used here
-        $.each(that.dataTable.fnGetData(), function(index, row) {
-          if (row[that.selectOptions.id_index] == row_id) {
-            row_name = row[that.selectOptions.name_index];
-            return false;
-          }
-        });
+        row_name = OpenNebula[that.resource].getName(row_id);
 
         $('#selected_ids_row_' + that.dataTableId, section).append('<span row_id="' + row_id + '" class="radius label">' + row_name + ' <span class="fa fa-times blue"></span></span> ');
       });
@@ -853,8 +883,6 @@ define(function(require) {
         $('#select_resource_multiple_' + that.dataTableId, section).hide();
       }
 
-      $('.alert-box', section).hide();
-
       that.dataTable.fnDraw();
     } else {
       $("td.markrow", that.dataTable).removeClass('markrow');
@@ -862,7 +890,6 @@ define(function(require) {
 
       $('#selected_resource_' + that.dataTableId, section).show();
       $('#select_resource_' + that.dataTableId, section).hide();
-      $('.alert-box', section).hide();
 
       var row_id = undefined;
       var row_name = "";
@@ -873,15 +900,8 @@ define(function(require) {
 
         row_name = "" + row_id;
 
-        // TODO: improve preformance, linear search. Needed to get the
-        // name of the resource in the label. If function getName() was
-        // indexed in the cache, it could be used here
-        $.each(that.dataTable.fnGetData(), function(index, row) {
-          if (row[that.selectOptions.id_index] == row_id) {
-            row_name = row[that.selectOptions.name_index];
-            return false;
-          }
-        });
+        row_name = OpenNebula[that.resource].getName(row_id);
+
       } else if (selectedResources.names != undefined) {
         row_name = selectedResources.names.name;
         var row_uname = selectedResources.names.uname;
@@ -894,14 +914,20 @@ define(function(require) {
             return false;
           }
         });
+
+        //if (row_id == undefined){
+        //  $('#selected_resource_id_' + that.dataTableId, section).data("pending_select", selectedResources);
+        //}
       }
 
       //        $("td", this).addClass('markrow');
       //        $('input.check_item', this).prop('checked', true);
 
-      $('#selected_resource_id_' + that.dataTableId, section).val(row_id).change();
+      if (row_id !== undefined) {
+        $('#selected_resource_id_' + that.dataTableId, section).val(row_id).trigger("change");
+      }
 
-      $('#selected_resource_name_' + that.dataTableId, section).text(row_name).change();
+      $('#selected_resource_name_' + that.dataTableId, section).text(row_name).trigger("change");
       $('#selected_resource_name_' + that.dataTableId, section).show();
 
       that.refreshResourceTableSelect(section, that.dataTableId);
@@ -937,7 +963,7 @@ define(function(require) {
           if (that.selectOptions.starred_ids != undefined){
             if (that.selectOptions.starred_ids_map[this[that.xmlRoot].ID]){
               elementArray[that.selectOptions.name_index] =
-                  ('<i class="fa fa-star fa-fw"></i> ' +
+                  (that.selectOptions.starred_icon + ' ' +
                     elementArray[that.selectOptions.name_index]);
             } else {
               elementArray[that.selectOptions.name_index] =
@@ -967,6 +993,13 @@ define(function(require) {
       });
 
       that.updateView(null, list_array, true);
+
+      var section = $('#' + that.dataTableId + 'Container');
+      var selectedResources = $('#selected_resource_id_' + that.dataTableId, section).data("pending_select");
+      if (selectedResources != undefined){
+        $('#selected_resource_id_' + that.dataTableId, section).removeData("pending_select");
+        that.selectResourceTableSelect(selectedResources);
+      }
     }
 
     var error_func = function(request, error_json, container) {
@@ -1000,4 +1033,12 @@ define(function(require) {
       error: Notifier.onError
     });
   }
+
+  function _setLabelsFilter(regExp) {
+    LabelsUtils.setLabelsFilter(this.dataTable, this.labelsColumn, regExp);
+  }
+
+  function _clearLabelsFilter() {
+    LabelsUtils.clearLabelsFilter(this.dataTable, this.labelsColumn);
+  }
 })
