@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -38,6 +38,7 @@ SUNSTONE_LOG              = LOG_LOCATION + "/sunstone.log"
 CONFIGURATION_FILE        = ETC_LOCATION + "/sunstone-server.conf"
 
 PLUGIN_CONFIGURATION_FILE = ETC_LOCATION + "/sunstone-plugins.yaml"
+LOGOS_CONFIGURATION_FILE = ETC_LOCATION + "/sunstone-logos.yaml"
 
 SUNSTONE_ROOT_DIR = File.dirname(__FILE__)
 
@@ -48,7 +49,38 @@ $: << SUNSTONE_ROOT_DIR+'/models'
 
 SESSION_EXPIRE_TIME = 60*60
 
-DISPLAY_NAME_XPATH = 'TEMPLATE/SUNSTONE_DISPLAY_NAME'
+DISPLAY_NAME_XPATH = 'TEMPLATE/SUNSTONE/DISPLAY_NAME'
+TABLE_ORDER_XPATH = 'TEMPLATE/SUNSTONE/TABLE_ORDER'
+DEFAULT_VIEW_XPATH = 'TEMPLATE/SUNSTONE/DEFAULT_VIEW'
+GROUP_ADMIN_DEFAULT_VIEW_XPATH = 'TEMPLATE/SUNSTONE/GROUP_ADMIN_DEFAULT_VIEW'
+TABLE_DEFAULT_PAGE_LENGTH_XPATH = 'TEMPLATE/SUNSTONE/TABLE_DEFAULT_PAGE_LENGTH'
+LANG_XPATH = 'TEMPLATE/SUNSTONE/LANG'
+
+ONED_CONF_OPTS = {
+    # If no costs are defined in oned.conf these values will be used
+    'DEFAULT_COST' => {
+        'CPU_COST' => 0,
+        'MEMORY_COST' => 0,
+        'DISK_COST' => 0
+    },
+    # Only these values will be shown when retrieving oned.conf from the browser
+    'ALLOWED_KEYS' => [
+        'DEFAULT_COST',
+        'DS_MAD_CONF',
+        'MARKET_MAD_CONF',
+        'VM_MAD',
+        'IM_MAD',
+        'AUTH_MAD'
+    ],
+    # Generate an array if there is only 1 element
+    'ARRAY_KEYS' => [
+        'DS_MAD_CONF',
+        'MARKET_MAD_CONF',
+        'VM_MAD',
+        'IM_MAD',
+        'AUTH_MAD'
+    ]
+}
 
 ##############################################################################
 # Required libraries
@@ -60,6 +92,8 @@ require 'yaml'
 require 'securerandom'
 require 'tmpdir'
 require 'fileutils'
+require 'base64'
+require 'rexml/document'
 
 require 'CloudAuth'
 require 'SunstoneServer'
@@ -224,14 +258,14 @@ helpers do
             # - WSS CONECTION
             # - TABLE ORDER
 
-            if user['TEMPLATE/LANG']
-                session[:lang] = user['TEMPLATE/LANG']
+            if user[LANG_XPATH]
+                session[:lang] = user[LANG_XPATH]
             else
                 session[:lang] = $conf[:lang]
             end
 
-            if user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH']
-                session[:page_length] = user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH']
+            if user[TABLE_DEFAULT_PAGE_LENGTH_XPATH]
+                session[:page_length] = user[TABLE_DEFAULT_PAGE_LENGTH_XPATH]
             else
                 session[:page_length] = DEFAULT_PAGE_LENGTH
             end
@@ -241,18 +275,18 @@ helpers do
             session[:vnc_wss] = (wss == true || wss == "yes" || wss == "only" ?
                              "yes" : "no")
 
-            if user['TEMPLATE/TABLE_ORDER']
-                session[:table_order] = user['TEMPLATE/TABLE_ORDER']
+            if user[TABLE_ORDER_XPATH]
+                session[:table_order] = user[TABLE_ORDER_XPATH]
             else
                 session[:table_order] = $conf[:table_order] || DEFAULT_TABLE_ORDER
             end
 
-            if user['TEMPLATE/DEFAULT_VIEW']
-                session[:default_view] = user['TEMPLATE/DEFAULT_VIEW']
-            elsif group.contains_admin(user.id) && group['TEMPLATE/GROUP_ADMIN_DEFAULT_VIEW']
-                session[:default_view] = group['TEMPLATE/GROUP_ADMIN_DEFAULT_VIEW']
-            elsif group['TEMPLATE/DEFAULT_VIEW']
-                session[:default_view] = group['TEMPLATE/DEFAULT_VIEW']
+            if user[DEFAULT_VIEW_XPATH]
+                session[:default_view] = user[DEFAULT_VIEW_XPATH]
+            elsif group.contains_admin(user.id) && group[GROUP_ADMIN_DEFAULT_VIEW_XPATH]
+                session[:default_view] = group[GROUP_ADMIN_DEFAULT_VIEW_XPATH]
+            elsif group[DEFAULT_VIEW_XPATH]
+                session[:default_view] = group[DEFAULT_VIEW_XPATH]
             else
                 session[:default_view] = $views_config.available_views(session[:user], session[:user_gname]).first
             end
@@ -271,6 +305,7 @@ helpers do
             zone = OpenNebula::Zone.new_with_id(rc['FEDERATION/ZONE_ID'].to_i, client)
             zone.info
             session[:zone_name] = zone.name
+            session[:zone_id]   = zone.id
 
             session[:federation_mode] = rc['FEDERATION/MODE'].upcase
 
@@ -282,10 +317,6 @@ helpers do
         session.clear
         return [204, ""]
     end
-
-    def cloud_view_instance_types
-        $conf[:instance_types] || []
-    end
 end
 
 before do
@@ -296,7 +327,7 @@ before do
     request.body.rewind
 
     unless %w(/ /login /vnc /spice).include?(request.path)
-        halt 401 unless authorized? && valid_csrftoken?
+        halt [401, "csrftoken"] unless authorized? && valid_csrftoken?
     end
 
     if env['HTTP_ZONE_NAME']
@@ -373,9 +404,47 @@ get '/' do
         return erb :login
     end
 
+    logos_conf = nil
+
+    begin
+        logos_conf = YAML.load_file(LOGOS_CONFIGURATION_FILE)
+    rescue Exception => e
+        logger.error { "Error parsing config file #{LOGOS_CONFIGURATION_FILE}: #{e.message}" }
+        error 500, ""
+    end
+
+    serveradmin_client = $cloud_auth.client(nil, session[:active_zone_endpoint])
+
+    rc = OpenNebula::System.new(serveradmin_client).get_configuration
+
+    if OpenNebula.is_error?(rc)
+        logger.error { rc.message }
+        error 500, ""
+    end
+
+    oned_conf_template = rc.to_hash()['TEMPLATE']
+
+    oned_conf = {}
+    ONED_CONF_OPTS['ALLOWED_KEYS'].each do |key|
+        value = oned_conf_template[key]
+        if key == 'DEFAULT_COST'
+            if value
+                oned_conf[key] = value
+            else
+                oned_conf[key] = ONED_CONF_OPTS['DEFAULT_COST']
+            end
+        else
+            if ONED_CONF_OPTS['ARRAY_KEYS'].include?(key) && !value.is_a?(Array)
+                oned_conf[key] = [value]
+            else
+                oned_conf[key] = value
+            end
+        end
+    end
+
     response.set_cookie("one-user", :value=>"#{session[:user]}")
 
-    erb :index
+    erb :index, :locals => {:logos_conf => logos_conf, :oned_conf => oned_conf}
 end
 
 get '/login' do
@@ -448,11 +517,10 @@ post '/config' do
         error 500, ""
     end
 
-    session[:lang]         = user['TEMPLATE/LANG'] if user['TEMPLATE/LANG']
-    session[:vnc_wss]      = user['TEMPLATE/VNC_WSS'] if user['TEMPLATE/VNC_WSS']
-    session[:default_view] = user['TEMPLATE/DEFAULT_VIEW'] if user['TEMPLATE/DEFAULT_VIEW']
-    session[:table_order]  = user['TEMPLATE/TABLE_ORDER'] if user['TEMPLATE/TABLE_ORDER']
-    session[:page_length]  = user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH'] if user['TEMPLATE/TABLE_DEFAULT_PAGE_LENGTH']
+    session[:lang]         = user[LANG_XPATH] if user[LANG_XPATH]
+    session[:default_view] = user[DEFAULT_VIEW_XPATH] if user[DEFAULT_VIEW_XPATH]
+    session[:table_order]  = user[TABLE_ORDER_XPATH] if user[TABLE_ORDER_XPATH]
+    session[:page_length]  = user[TABLE_DEFAULT_PAGE_LENGTH_XPATH] if user[TABLE_DEFAULT_PAGE_LENGTH_XPATH]
     session[:display_name] = user[DISPLAY_NAME_XPATH] || user['NAME']
 
     [204, ""]
@@ -498,23 +566,6 @@ get '/infrastructure' do
     [200, infrastructure.to_json]
 end
 
-get '/onedconf' do
-    serveradmin_client = $cloud_auth.client(nil, session[:active_zone_endpoint])
-
-    rc = OpenNebula::System.new(serveradmin_client).get_configuration
-
-    if OpenNebula.is_error?(rc)
-        logger.error { rc.message }
-        error 500, ""
-    end
-
-    onedconf = {
-        :DEFAULT_COST => rc.to_hash()['TEMPLATE']['DEFAULT_COST']
-    }
-
-    [200, onedconf.to_json]
-end
-
 get '/vm/:id/log' do
     @SunstoneServer.get_vm_log(params[:id])
 end
@@ -561,17 +612,6 @@ get '/vm/showback' do
     @SunstoneServer.get_vm_showback(params)
 end
 
-##############################################################################
-# Marketplace
-##############################################################################
-get '/marketplace' do
-    @SunstoneServer.get_appliance_pool
-end
-
-get '/marketplace/:id' do
-    @SunstoneServer.get_appliance(params[:id])
-end
-
 ##############################################################################
 # GET Pool information
 ##############################################################################
@@ -680,6 +720,45 @@ post '/upload_chunk' do
     ""
 end
 
+##############################################################################
+# Download marketplaceapp
+##############################################################################
+get '/marketplaceapp/:id/download' do
+    dl_resource = @SunstoneServer.download_marketplaceapp(params[:id])
+
+    # If the first element of dl_resource is a number, it is the exit_code after
+    # an error happend, so return it.
+    return dl_resource if dl_resource[0].kind_of?(Fixnum)
+
+    download_cmd, filename = dl_resource
+
+    # Send headers
+    headers['Cache-Control']       = "no-transform" # Do not use Rack::Deflater
+    headers['Content-Disposition'] = "attachment; filename=\"#{filename}\""
+
+    content_type :'application/octet-stream'
+
+    # Start stream
+    stream do |out|
+        Open3.popen3(download_cmd) do |_,o,e,w|
+
+            until o.eof?
+                # Read in chunks of 16KB
+                out << o.read(16384)
+            end
+
+            if !w.value.success?
+                error_message = "downloader.sh: " << e.read
+                logger.error { error_message }
+
+                if request.user_agent == "OpenNebula CLI"
+                    out << "@^_^@ #{error_message} @^_^@"
+                end
+            end
+        end
+    end
+end
+
 ##############################################################################
 # Create a new Resource
 ##############################################################################
