@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -25,13 +25,6 @@
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-time_t VirtualMachinePool::_monitor_expiration;
-bool   VirtualMachinePool::_submit_on_hold;
-float VirtualMachinePool::_default_cpu_cost;
-float VirtualMachinePool::_default_mem_cost;
-float VirtualMachinePool::_default_disk_cost;
-
-
 const char * VirtualMachinePool::import_table = "vm_import";
 
 const char * VirtualMachinePool::import_db_names = "deploy_id, vmid";
@@ -40,24 +33,25 @@ const char * VirtualMachinePool::import_db_bootstrap =
     "CREATE TABLE IF NOT EXISTS vm_import "
     "(deploy_id VARCHAR(128), vmid INTEGER, PRIMARY KEY(deploy_id))";
 
-
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 VirtualMachinePool::VirtualMachinePool(
         SqlDB *                     db,
-        vector<const Attribute *>   hook_mads,
+        vector<const VectorAttribute *>   hook_mads,
         const string&               hook_location,
         const string&               remotes_location,
-        vector<const Attribute *>&  restricted_attrs,
+        vector<const SingleAttribute *>&  restricted_attrs,
         time_t                      expire_time,
         bool                        on_hold,
         float                       default_cpu_cost,
         float                       default_mem_cost,
         float                       default_disk_cost)
-    : PoolSQL(db, VirtualMachine::table, true, false)
+    : PoolSQL(db, VirtualMachine::table, true, false),
+    _monitor_expiration(expire_time), _submit_on_hold(on_hold),
+    _default_cpu_cost(default_cpu_cost), _default_mem_cost(default_mem_cost),
+    _default_disk_cost(default_disk_cost)
 {
-    const VectorAttribute * vattr;
 
     string name;
     string on;
@@ -65,12 +59,6 @@ VirtualMachinePool::VirtualMachinePool(
     string arg;
     bool   remote;
 
-    _monitor_expiration = expire_time;
-    _submit_on_hold = on_hold;
-    _default_cpu_cost = default_cpu_cost;
-    _default_mem_cost = default_mem_cost;
-    _default_disk_cost= default_disk_cost;
-
     if ( _monitor_expiration == 0 )
     {
         clean_all_monitoring();
@@ -78,15 +66,15 @@ VirtualMachinePool::VirtualMachinePool(
 
     for (unsigned int i = 0 ; i < hook_mads.size() ; i++ )
     {
-        vattr = static_cast<const VectorAttribute *>(hook_mads[i]);
+        const VectorAttribute * vattr = hook_mads[i];
 
-        name = vattr->vector_value("NAME");
-        on   = vattr->vector_value("ON");
-        cmd  = vattr->vector_value("COMMAND");
-        arg  = vattr->vector_value("ARGUMENTS");
-        vattr->vector_value("REMOTE", remote);
+        name = hook_mads[i]->vector_value("NAME");
+        on   = hook_mads[i]->vector_value("ON");
+        cmd  = hook_mads[i]->vector_value("COMMAND");
+        arg  = hook_mads[i]->vector_value("ARGUMENTS");
+        hook_mads[i]->vector_value("REMOTE", remote);
 
-        transform (on.begin(),on.end(),on.begin(),(int(*)(int))toupper);
+        one_util::toupper(on);
 
         if ( on.empty() || cmd.empty() )
         {
@@ -296,6 +284,8 @@ int VirtualMachinePool::allocate (
     if ( _submit_on_hold == true || on_hold )
     {
         vm->state = VirtualMachine::HOLD;
+
+        vm->user_obj_template->replace("SUBMIT_ON_HOLD", true);
     }
     else
     {
@@ -800,10 +790,10 @@ int VirtualMachinePool::calculate_showback(
         history.xpath(h_stime,  "/HISTORY/STIME", 0);
         history.xpath(h_etime,  "/HISTORY/ETIME", 0);
 
-        history.xpath(cpu,  "/HISTORY/VM/TEMPLATE/CPU", 0);
+        history.xpath<float>(cpu,  "/HISTORY/VM/TEMPLATE/CPU", 0.0);
         history.xpath(mem,  "/HISTORY/VM/TEMPLATE/MEMORY", 0);
-        history.xpath(disk, "sum(/HISTORY/VM/TEMPLATE/DISK/SIZE | "
-            "/HISTORY/VM/TEMPLATE/DISK/DISK_SNAPSHOT_TOTAL_SIZE)", 0);
+        history.xpath<float>(disk, "sum(/HISTORY/VM/TEMPLATE/DISK/SIZE | "
+            "/HISTORY/VM/TEMPLATE/DISK/DISK_SNAPSHOT_TOTAL_SIZE)", 0.0);
 
         history.xpath(cpu_cost, "/HISTORY/VM/TEMPLATE/CPU_COST", _default_cpu_cost);
         history.xpath(mem_cost, "/HISTORY/VM/TEMPLATE/MEMORY_COST", _default_mem_cost);
@@ -1111,7 +1101,7 @@ void VirtualMachinePool::delete_attach_disk(int vid)
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void VirtualMachinePool::delete_attach_nic(int vid)
+void VirtualMachinePool::delete_hotplug_nic(int vid, bool attach)
 {
     VirtualMachine *  vm;
     VectorAttribute * nic;
@@ -1127,10 +1117,18 @@ void VirtualMachinePool::delete_attach_nic(int vid)
         return;
     }
 
-    nic = vm->delete_attach_nic();
-    uid = vm->get_uid();
-    gid = vm->get_gid();
-    oid = vm->get_oid();
+    if (attach)
+    {
+        nic  = vm->attach_nic_failure();
+    }
+    else
+    {
+        nic = vm->detach_nic_success();
+    }
+
+    uid  = vm->get_uid();
+    gid  = vm->get_gid();
+    oid  = vm->get_oid();
 
     update(vm);
 
@@ -1147,3 +1145,6 @@ void VirtualMachinePool::delete_attach_nic(int vid)
         VirtualMachine::release_network_leases(nic, oid);
     }
 }
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
