@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -43,13 +43,6 @@ require 'rexml/document'
 require 'pp'
 
 class VmmAction
-    # List of xpaths required by the VNM driver actions
-    XPATH_LIST = %w(
-        ID DEPLOY_ID
-        TEMPLATE/NIC
-        TEMPLATE/SECURITY_GROUP_RULE
-        HISTORY_RECORDS/HISTORY/HOSTNAME
-    )
 
     attr_reader :data
 
@@ -69,7 +62,6 @@ class VmmAction
         @data = Hash.new
 
         get_data(:host)
-        get_data(:net_drv)
         get_data(:deploy_id)
         get_data(:checkpoint_file)
 
@@ -78,7 +70,6 @@ class VmmAction
 
         # For migration
         get_data(:dest_host, :MIGR_HOST)
-        get_data(:dest_driver, :MIGR_NET_DRV)
 
         # For disk hotplugging
         get_data(:disk_target_path)
@@ -89,33 +80,29 @@ class VmmAction
         @data[:vm]  = Base64.encode64(vm_template).delete("\n")
 
         # VM data for VNM
-        vm_template_xml = REXML::Document.new(vm_template).root
-        vm_vnm_xml = REXML::Document.new('<VM></VM>').root
-
-        XPATH_LIST.each do |xpath|
-            elements = vm_template_xml.elements.each(xpath) do |element|
-                add_element_to_path(vm_vnm_xml, element, xpath)
-            end
-        end
+        src_drvs, src_xml = get_vnm_drivers(vm_template)
 
         # Initialize streams and vnm
         @ssh_src = @vmm.get_ssh_stream(action, @data[:host], @id)
-        @vnm_src = VirtualNetworkDriver.new(@data[:net_drv],
+        @vnm_src = VirtualNetworkDriver.new(src_drvs,
                             :local_actions  => @vmm.options[:local_actions],
-                            :message        => vm_vnm_xml.to_s,
+                            :message        => src_xml.to_s,
                             :ssh_stream     => @ssh_src)
 
         if @data[:dest_host] and !@data[:dest_host].empty?
+            dst_drvs, dst_xml = get_vnm_drivers(vm_template)
+
             @ssh_dst = @vmm.get_ssh_stream(action, @data[:dest_host], @id)
-            @vnm_dst = VirtualNetworkDriver.new(@data[:dest_driver],
+            @vnm_dst = VirtualNetworkDriver.new(dst_drvs,
                             :local_actions  => @vmm.options[:local_actions],
-                            :message        => vm_vnm_xml.to_s,
+                            :message        => dst_xml.to_s,
                             :ssh_stream     => @ssh_dst)
         end
 
         @tm = TransferManagerDriver.new(nil)
     end
 
+
     #Execute a set of steps defined with
     #  - :driver :vmm or :vnm to execute the step
     #  - :action for the step
@@ -124,17 +111,24 @@ class VmmAction
     #  - :fail_action steps to be executed if steps fail
     #  - :stdin for the action
     #  @param [Array] of steps
-    def run(steps, info_on_success = nil)
+    #  @param [String] additional informartion to prepend to action information
+    def run(steps, extra_info = nil)
         result = execute_steps(steps)
 
         @ssh_src.close if @ssh_src
         @ssh_dst.close if @ssh_dst
 
         #Prepare the info for the OpenNebula core
-        if DriverExecHelper.failed?(result)
-            info = @data[:failed_info]
+        if !extra_info.nil?
+            info = extra_info + " "
         else
-            info = @data["#{@main_action.to_s}_info".to_sym]
+            info = ""
+        end
+
+        if DriverExecHelper.failed?(result)
+            info << @data[:failed_info]
+        elsif !@data["#{@main_action.to_s}_info".to_sym].nil?
+            info << @data["#{@main_action.to_s}_info".to_sym]
         end
 
         @vmm.send_message(VirtualMachineDriver::ACTION[@main_action],
@@ -142,13 +136,52 @@ class VmmAction
     end
 
     private
+    # List of xpaths required by the VNM driver actions. TEMPLATE/NIC is
+    # also required but added separately to the driver xml
+    XPATH_LIST = %w(
+        ID DEPLOY_ID
+        TEMPLATE/SECURITY_GROUP_RULE
+        HISTORY_RECORDS/HISTORY/HOSTNAME
+        HISTORY_RECORDS/HISTORY/VM_MAD
+    )
 
     DRIVER_NAMES = {
         :vmm => "virtualization driver",
         :vnm => "network driver",
-        :tm => "transfer manager driver"
+        :tm  => "transfer manager driver"
     }
 
+    # Prepares the list of drivers executed on the host and the xml that will
+    # be sent to the network drivers. Builds VM template with required attributes
+    # for the VirtualNetworkDriver.
+    #  @param[String] The template of the VM.
+    #  @return[Array] Returns a list of network drivers and the associated xml
+    #  template.
+    def get_vnm_drivers(vm_template)
+        vm_template_xml = REXML::Document.new(vm_template).root
+        vm_vnm_xml      = REXML::Document.new('<VM></VM>').root
+
+        vnm_drivers = []
+
+        XPATH_LIST.each do |xpath|
+            vm_template_xml.elements.each(xpath) do |element|
+                add_element_to_path(vm_vnm_xml, element, xpath)
+            end
+        end
+
+        vm_template_xml.elements.each("TEMPLATE/NIC") do |element|
+            vn_mad = element.get_text("VN_MAD").to_s
+
+            next if vn_mad.empty?
+
+            vnm_drivers << vn_mad unless vnm_drivers.include?(vn_mad)
+
+            add_element_to_path(vm_vnm_xml, element, "TEMPLATE/NIC")
+        end
+
+        return [vnm_drivers, vm_vnm_xml]
+    end
+
     # Executes a set of steps. If one step fails any recover action is performed
     # and the step execution breaks.
     # @param [Array] array of steps to be executed
@@ -170,13 +203,15 @@ class VmmAction
                     ssh  = @ssh_src
                 end
 
+                stdin = step[:stdin] || @xml_data.to_s
+
                 result, info = @vmm.do_action(get_parameters(step[:parameters]),
                                               @id,
                                               host,
                                               step[:action],
                                               :ssh_stream => ssh,
                                               :respond => false,
-                                              :stdin => step[:stdin])
+                                              :stdin => stdin)
             when :vnm
                 if step[:destination]
                     vnm = @vnm_dst
@@ -333,8 +368,22 @@ class ExecDriver < VirtualMachineDriver
         # ----------------------------------------------------------------------
         #  Deployment Steps
         # ----------------------------------------------------------------------
+        xml_data = decode(drv_message)
 
-        steps=[
+        tm_command = xml_data.elements['TM_COMMAND']
+        tm_command = tm_command.text if tm_command
+
+        steps = []
+
+        if tm_command && !tm_command.empty?
+            steps << {
+                :driver     => :tm,
+                :action     => :tm_context,
+                :parameters => tm_command.strip.split(' ')
+            }
+        end
+
+        steps.concat([
             # Execute pre-boot networking setup
             {
                 :driver   => :vnm,
@@ -360,7 +409,7 @@ class ExecDriver < VirtualMachineDriver
                     }
                 ]
             }
-        ]
+        ])
 
         action.run(steps)
     end
@@ -439,9 +488,24 @@ class ExecDriver < VirtualMachineDriver
     # RESTORE action, restore a VM from a previous state, and restores network
     #
     def restore(id, drv_message)
+        xml_data = decode(drv_message)
+
+        tm_command = xml_data.elements['TM_COMMAND']
+        tm_command = tm_command.text if tm_command
+
+        steps = []
+
+        if tm_command && !tm_command.empty?
+            steps << {
+                :driver     => :tm,
+                :action     => :tm_context,
+                :parameters => tm_command.strip.split(' ')
+            }
+        end
+
         action=VmmAction.new(self, id, :restore, drv_message)
 
-        steps=[
+        steps.concat([
             # Execute pre-boot networking setup
             {
                 :driver     => :vnm,
@@ -466,7 +530,7 @@ class ExecDriver < VirtualMachineDriver
                     }
                 ],
             }
-        ]
+        ])
 
         action.run(steps)
     end
@@ -589,12 +653,6 @@ class ExecDriver < VirtualMachineDriver
 
         action = VmmAction.new(self, id, :attach_disk, drv_message)
 
-        # Bug #1355, argument character limitation in ESX
-        # Message not used in vmware anyway
-        if @hypervisor == "vmware"
-            drv_message = "drv_message"
-        end
-
         steps = [
             # Perform a PROLOG on the disk
             {
@@ -791,20 +849,31 @@ class ExecDriver < VirtualMachineDriver
     #  ATTACHNIC action to attach a new nic interface
     #
     def attach_nic(id, drv_message)
+        action = ACTION[:attach_nic]
         xml_data = decode(drv_message)
 
+        tm_command = xml_data.elements['TM_COMMAND']
+        tm_command = tm_command.text if tm_command
+
+        target_path = xml_data.elements['DISK_TARGET_PATH']
+        target_path = target_path.text if target_path
+
+        target_device = xml_data.elements['VM/TEMPLATE/CONTEXT/TARGET']
+        target_device = target_device.text if target_device
+
         begin
             source = xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/BRIDGE"]
-            source_ovs =
-                xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/BRIDGE_OVS"]
             mac    = xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/MAC"]
+            target = xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/TARGET"]
+            vn_mad = xml_data.elements["VM/TEMPLATE/NIC[ATTACH='YES']/VN_MAD"]
 
             source = source.text.strip
-            source_ovs = source_ovs.text.strip if source_ovs
             mac    = mac.text.strip
+            target = target.text.strip
+            vn_mad = vn_mad.text.strip
         rescue
             send_message(action, RESULT[:failure], id,
-                "Error in #{ACTION[:attach_nic]}, BRIDGE and MAC needed in NIC")
+                "Missing VN_MAD, BRIDGE, TARGET or MAC in VM NIC")
             return
         end
 
@@ -814,15 +883,6 @@ class ExecDriver < VirtualMachineDriver
         model = model.strip if !model.nil?
         model = "-" if model.nil?
 
-
-        net_drv = xml_data.elements["NET_DRV"]
-
-        net_drv = net_drv.text if !net_drv.nil?
-        net_drv = net_drv.strip if !net_drv.nil?
-        net_drv = "-" if net_drv.nil?
-
-        source = source_ovs if net_drv == 'ovswitch' && source_ovs
-
         action = VmmAction.new(self, id, :attach_nic, drv_message)
 
         steps=[
@@ -835,7 +895,7 @@ class ExecDriver < VirtualMachineDriver
             {
                 :driver     => :vmm,
                 :action     => :attach_nic,
-                :parameters => [:deploy_id, mac, source, model, net_drv]
+                :parameters => [:deploy_id, mac, source, model, vn_mad, target]
             },
             # Execute post-boot networking setup
             {
@@ -849,9 +909,31 @@ class ExecDriver < VirtualMachineDriver
                         :parameters => [:deploy_id, mac]
                     }
                 ]
+            },
+            {
+                :driver     => :vmm,
+                :action     => :prereconfigure,
+                :parameters => [:deploy_id, target_device]
             }
         ]
 
+        if tm_command && !tm_command.empty?
+            steps << {
+                :driver     => :tm,
+                :action     => :tm_context,
+                :parameters => tm_command.strip.split(' ')
+            }
+        end
+
+        steps << {
+            :driver     => :vmm,
+            :action     => :reconfigure,
+            :parameters => [:deploy_id, target_device, target_path]
+        }
+
+        STDERR.puts "TM_COMMAND"
+        STDERR.puts tm_command.inspect
+
         action.run(steps)
     end
 
@@ -890,201 +972,63 @@ class ExecDriver < VirtualMachineDriver
     end
 
     #
-    # DISKSNAPSHOTCREATE action, takes a snapshot of a disk
+    # DISKSNAPSHOTCREATE action, takes a snapshot of a dis
     #
     def disk_snapshot_create(id, drv_message)
-        snap_action  = prepare_snap_action(id, drv_message,
-                                           :disk_snapshot_create)
-        action       = snap_action[:action]
-        strategy     = snap_action[:strategy]
-        drv_message  = snap_action[:drv_message]
-        target       = snap_action[:target]
-        target_index = snap_action[:target_index]
-        xml_data     = snap_action[:xml_data]
+        xml_data = decode(drv_message)
+        action   = ACTION[:disk_snapshot_create]
+
+        # Check that live snapshot is supported
+        vmm_driver_path = 'VM/HISTORY_RECORDS/HISTORY/VM_MAD'
+        tm_driver_path  = "VM/TEMPLATE/DISK[DISK_SNAPSHOT_ACTIVE='YES']/TM_MAD"
+
+        vmm_driver = ensure_xpath(xml_data, id, action, vmm_driver_path)||return
+        tm_driver  = ensure_xpath(xml_data, id, action, tm_driver_path) ||return
+
+        if !LIVE_DISK_SNAPSHOTS.include?("#{vmm_driver}-#{tm_driver}")
+            send_message(action, RESULT[:failure], id,
+                "Cannot perform a live #{action} operation")
+            return
+        end
 
         # Get TM command
         tm_command = ensure_xpath(xml_data, id, action, 'TM_COMMAND') || return
 
-        # Build the process
-        case strategy
-        when :live
-            tm_command_split = tm_command.split
-            tm_command_split[0] += "_LIVE"
+        tm_command_split     = tm_command.split
+        tm_command_split[0] += "_LIVE"
 
-            steps = [
-                {
-                    :driver     => :tm,
-                    :action     => :tm_snap_create_live,
-                    :parameters => tm_command_split
-                }
-            ]
+        action = VmmAction.new(self, id, :disk_snapshot_create, drv_message)
 
-        when :detach
-            steps = [
-                # detach_disk or save
-                {
-                    :driver     => :vmm,
-                    :action     => :detach_disk,
-                    :parameters => [:deploy_id, :disk_target_path, target,
-                                    target_index]
-                },
-                # run TM
-                {
-                    :driver     => :tm,
-                    :action     => :tm_snap_create,
-                    :parameters => tm_command.split,
-                    :no_fail    => true
-                },
-                # attach_disk or restore
-                {
-                    :driver     => :vmm,
-                    :action     => :attach_disk,
-                    :parameters => [:deploy_id, :disk_target_path, target,
-                                    target_index, drv_message]
-                }
-            ]
-        when :suspend
-               steps = [
-                # detach_disk or save
-                {
-                    :driver     => :vmm,
-                    :action     => :save,
-                    :parameters => [:deploy_id, :checkpoint_file, :host]
-                },
-                # network drivers (clean)
-                {
-                    :driver   => :vnm,
-                    :action   => :clean
-                },
-                # run TM
-                {
-                    :driver     => :tm,
-                    :action     => :tm_snap_create,
-                    :parameters => tm_command.split,
-                    :no_fail    => true
-                },
-                # network drivers (pre)
-                {
-                    :driver   => :vnm,
-                    :action   => :pre
-                },
-                # attach_disk or restore
-                {
-                    :driver     => :vmm,
-                    :action     => :restore,
-                    :parameters => [:checkpoint_file, :host, :deploy_id]
-                },
-                # network drivers (post)
-                {
-                    :driver       => :vnm,
-                    :action       => :post,
-                    :parameters   => [:deploy_id],
-                    :fail_actions => [
-                        {
-                            :driver     => :vmm,
-                            :action     => :cancel,
-                            :parameters => [:deploy_id, :host]
-                        }
-                    ]
-                }
-            ]
-        else
-            return
-        end
+        steps = [
+            {
+                :driver     => :tm,
+                :action     => :tm_snap_create_live,
+                :parameters => tm_command_split
+            }
+        ]
 
         action.run(steps)
     end
 
     #
-    # DISKSNAPSHOTREVERT action, takes a snapshot of a disk
+    # UPDATESG action, deletes iptables rules and regenerate them
     #
-    def disk_snapshot_revert(id, drv_message)
-        snap_action  = prepare_snap_action(id, drv_message,
-                                           :disk_snapshot_revert)
-        action       = snap_action[:action]
-        strategy     = @options[:snapshots_strategy]
-        drv_message  = snap_action[:drv_message]
-        target       = snap_action[:target]
-        target_index = snap_action[:target_index]
-        xml_data     = snap_action[:xml_data]
+    def update_sg(id, drv_message)
+        xml_data = decode(drv_message)
+        sg_id    = xml_data.elements['SECURITY_GROUP_ID'].text
 
-        # Get TM command
-        tm_command = ensure_xpath(xml_data, id, action, 'TM_COMMAND') || return
+        action = VmmAction.new(self, id, :update_sg, drv_message)
 
-        case strategy
-        when :detach
-            steps = [
-                # Save VM state / detach the disk
-                {
-                    :driver     => :vmm,
-                    :action     => :detach_disk,
-                    :parameters => [:deploy_id, :disk_target_path, target, target_index]
-                },
-                # Do the snapshot
-                {
-                    :driver     => :tm,
-                    :action     => :tm_snap_revert,
-                    :parameters => tm_command.split,
-                    :no_fail    => true,
-                },
-                # Restore VM / attach the disk
-                {
-                    :driver     => :vmm,
-                    :action     => :attach_disk,
-                    :parameters => [:deploy_id, :disk_target_path, target, target_index,
-                           drv_message]
-                }
-            ]
-        when :suspend
-            steps = [
-                # Save VM state / detach the disk
-                {
-                    :driver     => :vmm,
-                    :action     => :save,
-                    :parameters => [:deploy_id, :checkpoint_file, :host]
-                },
-                # network drivers (clean)
-                {
-                    :driver   => :vnm,
-                    :action   => :clean
-                },
-                # Do the snapshot
-                {
-                    :driver     => :tm,
-                    :action     => :tm_snap_revert,
-                    :parameters => tm_command.split,
-                    :no_fail    => true,
-                },
-                # network drivers (pre)
-                {
-                    :driver   => :vnm,
-                    :action   => :pre
-                },
-                # Restore VM / attach the disk
-                {
-                    :driver     => :vmm,
-                    :action     => :restore,
-                    :parameters => [:checkpoint_file, :host, :deploy_id]
-                },
-                # network drivers (post)
-                {
-                    :driver       => :vnm,
-                    :action       => :post,
-                    :parameters   => [:deploy_id],
-                    :fail_actions => [
-                        {
-                            :driver     => :vmm,
-                            :action     => :cancel,
-                            :parameters => [:deploy_id, :host]
-                        }
-                    ]
-                }
-            ]
-        else
-            return
-        end
+        steps=[
+            # Execute update networking action
+            {
+                :driver       => :vnm,
+                :action       => :update_sg,
+                :parameters   => [:deploy_id]
+            }
+        ]
 
-        action.run(steps)
+        action.run(steps, sg_id)
     end
 
 private
@@ -1100,48 +1044,6 @@ private
             nil
         end
     end
-
-    def prepare_snap_action(id, drv_message, action)
-        xml_data = decode(drv_message)
-
-        # Make sure disk target has been defined
-        target_xpath = "VM/TEMPLATE/DISK[DISK_SNAPSHOT_ACTIVE='YES']/TARGET"
-        target       = ensure_xpath(xml_data, id, ACTION[action],
-                                    target_xpath) || return
-        target_index = target.downcase[-1..-1].unpack('c').first - 97
-
-        # Always send ATTACH='YES' for the selected target in case it will end
-        # up being a 'detach' strategy
-        disk   = xml_data.elements[target_xpath].parent
-        attach = REXML::Element.new('ATTACH')
-
-        attach.add_text('YES')
-        disk.add(attach)
-
-        drv_message = Base64.encode64(xml_data.to_s)
-        action = VmmAction.new(self, id, action, drv_message)
-
-        # Determine the strategy
-        vmm_driver_path = 'VM/HISTORY_RECORDS/HISTORY/VMMMAD'
-        tm_driver_path  = "VM/TEMPLATE/DISK[DISK_SNAPSHOT_ACTIVE='YES']/TM_MAD"
-
-        vmm_driver = ensure_xpath(xml_data, id, action, vmm_driver_path) || return
-        tm_driver  = ensure_xpath(xml_data, id, action, tm_driver_path)  || return
-
-        strategy = @options[:snapshots_strategy]
-        if @options[:live_snapshots] && LIVE_DISK_SNAPSHOTS.include?("#{vmm_driver}-#{tm_driver}")
-            strategy = :live
-        end
-
-        {
-            :action       => action,
-            :strategy     => strategy,
-            :drv_message  => drv_message,
-            :target       => target,
-            :target_index => target_index,
-            :xml_data     => xml_data
-        }
-    end
 end
 
 ################################################################################
@@ -1161,9 +1063,7 @@ opts = GetoptLong.new(
     [ '--threads',           '-t', GetoptLong::OPTIONAL_ARGUMENT ],
     [ '--local',             '-l', GetoptLong::REQUIRED_ARGUMENT ],
     [ '--shell',             '-s', GetoptLong::REQUIRED_ARGUMENT ],
-    [ '--parallel',          '-p', GetoptLong::NO_ARGUMENT ],
-    [ '--live-snapshots',    '-i', GetoptLong::NO_ARGUMENT ],
-    [ '--default-snapshots', '-d', GetoptLong::REQUIRED_ARGUMENT ]
+    [ '--parallel',          '-p', GetoptLong::NO_ARGUMENT ]
 )
 
 hypervisor         = ''
@@ -1172,8 +1072,6 @@ threads            = 15
 shell              = 'bash'
 local_actions      = {}
 single_host        = true
-live_snapshots     = false
-snapshots_strategy = :suspend # Either :detach or :suspend
 
 begin
     opts.each do |opt, arg|
@@ -1188,10 +1086,6 @@ begin
                 shell = arg
             when '--parallel'
                 single_host = false
-            when '--default-snapshots'
-                snapshots_strategy = arg.to_sym
-            when '--live-snapshots'
-                live_snapshots = true
         end
     end
 rescue Exception => e
@@ -1209,8 +1103,6 @@ exec_driver = ExecDriver.new(hypervisor,
                 :retries            => retries,
                 :local_actions      => local_actions,
                 :shell              => shell,
-                :single_host        => single_host,
-                :snapshots_strategy => snapshots_strategy,
-                :live_snapshots     => live_snapshots)
+                :single_host        => single_host)
 
 exec_driver.start_driver
