@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -133,7 +133,8 @@ class EC2Driver
                     :opt => 'placement/availability_zone'
                 },
                 "EBS_OPTIMIZED" => {
-                    :opt => 'ebs_optimized'
+                    :opt => 'ebs_optimized',
+                    :proc => lambda {|str| str.downcase.eql? "true"}
                 }
             }
         },
@@ -214,6 +215,8 @@ class EC2Driver
 
         public_cloud_ec2_conf  = YAML::load(File.read(EC2_DRIVER_CONF))
 
+        @state_change_timeout = public_cloud_ec2_conf['state_wait_timeout_seconds'].to_i
+
         @instance_types = public_cloud_ec2_conf['instance_types']
 
         regions = public_cloud_ec2_conf['regions']
@@ -296,6 +299,11 @@ class EC2Driver
 
         if ec2_value(ec2_info, 'ELASTICIP')
             begin
+                start_time = Time.now
+                while instance.status == :pending
+                    break if Time.now - start_time > @state_change_timeout
+                    sleep 5
+                end
                 instance.associate_elastic_ip(ec2_value(ec2_info, 'ELASTICIP'))
             rescue => e
                 STDERR.puts(e.message)
@@ -378,7 +386,8 @@ class EC2Driver
         usedmemory = 0
 
         # Build an array of VMs and last_polls for monitoring
-        vpool      = OpenNebula::VirtualMachinePool.new OpenNebula::Client.new
+        vpool      = OpenNebula::VirtualMachinePool.new(OpenNebula::Client.new,
+                                    OpenNebula::VirtualMachinePool::INFO_ALL_VM)
         vpool.info
         onevm_info = {}
 
@@ -462,7 +471,7 @@ private
         ec2 = nil
         ec2_deprecated = nil
 
-        all_ec2_elements = xml.root.get_elements("//USER_TEMPLATE/EC2")
+        all_ec2_elements = xml.root.get_elements("//USER_TEMPLATE/PUBLIC_CLOUD")
 
         # First, let's see if we have an EC2 site that matches
         # our desired host name
@@ -486,8 +495,9 @@ private
             if all_ec2_elements.size == 1
                 ec2 = all_ec2_elements[0]
             else
-                STDERR.puts("Cannot find EC2 element in deployment file or no" \
-                    "EC2 site matching in the template.")
+                STDERR.puts("Cannot find PUBLIC_CLOUD element in deployment "\
+                    " file or no HOST site matching the requested in the "\
+                    "template.")
                 exit(-1)
             end
         end
