@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -40,13 +40,14 @@ VirtualNetwork::VirtualNetwork(int                      _uid,
                                const string&            _gname,
                                int                      _umask,
                                int                      _pvid,
-                               int                      _cluster_id,
-                               const string&            _cluster_name,
+                               const set<int>           &_cluster_ids,
                                VirtualNetworkTemplate * _vn_template):
             PoolObjectSQL(-1,NET,"",_uid,_gid,_uname,_gname,table),
-            Clusterable(_cluster_id, _cluster_name),
+            Clusterable(_cluster_ids),
             bridge(""),
-            parent_vid(_pvid)
+            vlan_id_automatic(false),
+            parent_vid(_pvid),
+            vrouters("VROUTERS")
 {
     if (_vn_template != 0)
     {
@@ -89,25 +90,23 @@ void VirtualNetwork::get_permissions(PoolObjectAuth& auths)
 const char * VirtualNetwork::table    = "network_pool";
 
 const char * VirtualNetwork::db_names =
-        "oid, name, body, uid, gid, owner_u, group_u, other_u, cid, pid";
+        "oid, name, body, uid, gid, owner_u, group_u, other_u, pid";
 
 const char * VirtualNetwork::db_bootstrap = "CREATE TABLE IF NOT EXISTS"
     " network_pool (oid INTEGER PRIMARY KEY, name VARCHAR(128),"
     " body MEDIUMTEXT, uid INTEGER, gid INTEGER,"
     " owner_u INTEGER, group_u INTEGER, other_u INTEGER,"
-    " cid INTEGER, pid INTEGER, UNIQUE(name,uid))";
+    " pid INTEGER, UNIQUE(name,uid))";
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 int VirtualNetwork::insert(SqlDB * db, string& error_str)
 {
-    vector<Attribute *> ars;
+    vector<VectorAttribute *> ars;
     ostringstream       ose;
 
-    string sg_str;
-
-    bool b_vlan;
+    string sg_str, vis;
 
     int rc, num_ars;
 
@@ -124,31 +123,32 @@ int VirtualNetwork::insert(SqlDB * db, string& error_str)
         goto error_name;
     }
 
+    // ------------ VN_MAD --------------------
+
+    erase_template_attribute("VN_MAD", vn_mad);
+
+    if (vn_mad.empty())
+    {
+        goto error_vn_mad;
+    }
+
+    add_template_attribute("VN_MAD", vn_mad);
+
     // ------------ PHYDEV --------------------
 
     erase_template_attribute("PHYDEV", phydev);
 
     add_template_attribute("PHYDEV", phydev);
 
-    // ------------ VLAN_ID -------------------
+    // ---- VLAN_ID if not set allocated in VirtualNetworkPool, if needed -----
 
-    erase_template_attribute("VLAN_ID", vlan_id);
-
-    add_template_attribute("VLAN_ID", vlan_id);
-
-    // ------------ VLAN ----------------------
-
-    erase_template_attribute("VLAN", b_vlan);
-
-    if (b_vlan || !phydev.empty())
+    if (PoolObjectSQL::get_template_attribute("VLAN_ID", vis) && !vis.empty())
     {
-        vlan = 1;
-        add_template_attribute("VLAN", "YES");
-    }
-    else
-    {
-        vlan = 0;
-        add_template_attribute("VLAN", "NO");
+        erase_template_attribute("VLAN_ID", vlan_id);
+
+        add_template_attribute("VLAN_ID", vlan_id);
+
+        vlan_id_automatic = false;
     }
 
     // ------------ BRIDGE --------------------
@@ -237,6 +237,10 @@ error_name:
     ose << "No NAME in template for Virtual Network.";
     goto error_common;
 
+error_vn_mad:
+    ose << "No VN_MAD in template for Virtual Network.";
+    goto error_common;
+
 error_bridge:
     ose << "No BRIDGE in template for Virtual Network.";
     goto error_common;
@@ -257,36 +261,28 @@ error_common:
 int VirtualNetwork::post_update_template(string& error)
 {
     string new_bridge;
-    bool   b_vlan;
     string sg_str;
 
     /* ---------------------------------------------------------------------- */
     /* Update Configuration Attributes (class & template)                     */
+    /*  - VN_MAD                                                              */
     /*  - PHYDEV                                                              */
     /*  - VLAN_ID                                                             */
-    /*  - VLAN                                                                */
     /*  - BRIDGE                                                              */
     /*  - SECURITY_GROUPS                                                     */
     /* ---------------------------------------------------------------------- */
+    erase_template_attribute("VN_MAD", vn_mad);
+
+    add_template_attribute("VN_MAD", vn_mad);
+
     erase_template_attribute("PHYDEV", phydev);
 
     add_template_attribute("PHYDEV", phydev);
 
-    erase_template_attribute("VLAN_ID", vlan_id);
-
-    add_template_attribute("VLAN_ID", vlan_id);
-
-    erase_template_attribute("VLAN", b_vlan);
-
-    if (b_vlan || !phydev.empty())
+    if (!vlan_id_automatic)
     {
-        vlan = 1;
-        add_template_attribute("VLAN", "YES");
-    }
-    else
-    {
-        vlan = 0;
-        add_template_attribute("VLAN", "NO");
+        erase_template_attribute("VLAN_ID", vlan_id);
+        add_template_attribute("VLAN_ID", vlan_id);
     }
 
     erase_template_attribute("BRIDGE",new_bridge);
@@ -359,7 +355,6 @@ int VirtualNetwork::insert_replace(SqlDB *db, bool replace, string& error_str)
         <<          owner_u     << ","
         <<          group_u     << ","
         <<          other_u     << ","
-        <<          cluster_id  << ","
         <<          parent_vid  << ")";
 
     rc = db->exec(oss);
@@ -398,43 +393,46 @@ string& VirtualNetwork::to_xml(string& xml) const
 {
     const vector<int> empty;
 
-    return to_xml_extended(xml,false, empty, empty);
+    return to_xml_extended(xml,false, empty, empty, empty);
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 string& VirtualNetwork::to_xml_extended(string& xml, const vector<int>& vms,
-        const vector<int>& vnets) const
+        const vector<int>& vnets, const vector<int>& vrs) const
 {
-    return to_xml_extended(xml,true, vms, vnets);
+    return to_xml_extended(xml,true, vms, vnets, vrs);
 }
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
 string& VirtualNetwork::to_xml_extended(string& xml, bool extended,
-    const vector<int>& vms, const vector<int>& vnets) const
+    const vector<int>& vms, const vector<int>& vnets,
+    const vector<int>& vrs) const
 {
     ostringstream   os;
 
+    string clusters_xml;
+    string vrouters_xml;
     string template_xml;
     string leases_xml;
     string perm_str;
 
+    int int_vlan_id_automatic = vlan_id_automatic ? 1 : 0;
+
     os <<
         "<VNET>" <<
-            "<ID>"        << oid       << "</ID>"        <<
-            "<UID>"       << uid       << "</UID>"       <<
-            "<GID>"       << gid       << "</GID>"       <<
-            "<UNAME>"     << uname     << "</UNAME>"     <<
-            "<GNAME>"     << gname     << "</GNAME>"     <<
-            "<NAME>"      << name      << "</NAME>"      <<
-            perms_to_xml(perm_str)     <<
-            "<CLUSTER_ID>"<< cluster_id<< "</CLUSTER_ID>"<<
-            "<CLUSTER>"   << cluster   << "</CLUSTER>"   <<
-            "<BRIDGE>"    << bridge    << "</BRIDGE>"    <<
-            "<VLAN>"      << vlan      << "</VLAN>";
+            "<ID>"     << oid      << "</ID>"    <<
+            "<UID>"    << uid      << "</UID>"   <<
+            "<GID>"    << gid      << "</GID>"   <<
+            "<UNAME>"  << uname    << "</UNAME>" <<
+            "<GNAME>"  << gname    << "</GNAME>" <<
+            "<NAME>"   << name     << "</NAME>"  <<
+            perms_to_xml(perm_str) <<
+            Clusterable::to_xml(clusters_xml)    <<
+            "<BRIDGE>" << one_util::escape_xml(bridge) << "</BRIDGE>";
 
     if (parent_vid != -1)
     {
@@ -445,9 +443,18 @@ string& VirtualNetwork::to_xml_extended(string& xml, bool extended,
         os << "<PARENT_NETWORK_ID/>";
     }
 
+    if (!vn_mad.empty())
+    {
+        os << "<VN_MAD>" << one_util::escape_xml(vn_mad) << "</VN_MAD>";
+    }
+    else
+    {
+        os << "<VN_MAD/>";
+    }
+
     if (!phydev.empty())
     {
-        os << "<PHYDEV><![CDATA[" << phydev << "]]></PHYDEV>";
+        os << "<PHYDEV>" << one_util::escape_xml(phydev) << "</PHYDEV>";
     }
     else
     {
@@ -456,17 +463,23 @@ string& VirtualNetwork::to_xml_extended(string& xml, bool extended,
 
     if (!vlan_id.empty())
     {
-        os << "<VLAN_ID><![CDATA[" << vlan_id << "]]></VLAN_ID>";
+        os << "<VLAN_ID>" << one_util::escape_xml(vlan_id) << "</VLAN_ID>";
+        os << "<VLAN_ID_AUTOMATIC>" << int_vlan_id_automatic
+           <<"</VLAN_ID_AUTOMATIC>";
     }
     else
     {
         os << "<VLAN_ID/>";
+        os << "<VLAN_ID_AUTOMATIC/>";
     }
-    os  << "<USED_LEASES>"<< ar_pool.get_used_addr() << "</USED_LEASES>";
 
-    os  << obj_template->to_xml(template_xml);
+    os << "<USED_LEASES>"<< ar_pool.get_used_addr() << "</USED_LEASES>";
 
-    os  << ar_pool.to_xml(leases_xml, extended, vms, vnets);
+    os << vrouters.to_xml(vrouters_xml);
+
+    os << obj_template->to_xml(template_xml);
+
+    os << ar_pool.to_xml(leases_xml, extended, vms, vnets, vrs);
 
     os << "</VNET>";
 
@@ -483,28 +496,36 @@ int VirtualNetwork::from_xml(const string &xml_str)
     vector<xmlNodePtr> content;
 
     int rc = 0;
+    int int_vlan_id_automatic;
 
     // Initialize the internal XML object
     update_from_str(xml_str);
 
     // Get class base attributes
-    rc += xpath(oid,       "/VNET/ID", -1);
-    rc += xpath(uid,       "/VNET/UID", -1);
-    rc += xpath(gid,       "/VNET/GID", -1);
-    rc += xpath(uname,     "/VNET/UNAME", "not_found");
-    rc += xpath(gname,     "/VNET/GNAME", "not_found");
-    rc += xpath(name,      "/VNET/NAME", "not_found");
-    rc += xpath(bridge,    "/VNET/BRIDGE", "not_found");
-    rc += xpath(vlan,      "/VNET/VLAN", 0);
-    rc += xpath(cluster_id,"/VNET/CLUSTER_ID", -1);
-    rc += xpath(cluster,   "/VNET/CLUSTER", "not_found");
+    rc += xpath(oid,    "/VNET/ID",  -1);
+    rc += xpath(uid,    "/VNET/UID", -1);
+    rc += xpath(gid,    "/VNET/GID", -1);
+    rc += xpath(uname,  "/VNET/UNAME", "not_found");
+    rc += xpath(gname,  "/VNET/GNAME", "not_found");
+    rc += xpath(name,   "/VNET/NAME",  "not_found");
+    rc += xpath(bridge, "/VNET/BRIDGE","not_found");
 
     // Permissions
     rc += perms_from_xml();
 
+    xpath(vn_mad, "/VNET/VN_MAD", "");
     xpath(phydev, "/VNET/PHYDEV", "");
     xpath(vlan_id,"/VNET/VLAN_ID","");
     xpath(parent_vid,"/VNET/PARENT_NETWORK_ID",-1);
+    xpath(int_vlan_id_automatic,"/VNET/VLAN_ID_AUTOMATIC",0);
+
+    vlan_id_automatic = int_vlan_id_automatic;
+
+    // Set of cluster IDs
+    rc += Clusterable::from_xml(this, "/VNET/");
+
+    // VRouter IDs
+    rc += vrouters.from_xml(this, "/VNET/");
 
     // Virtual Network template
     ObjectXML::get_nodes("/VNET/TEMPLATE", content);
@@ -556,6 +577,10 @@ int VirtualNetwork::nic_attribute(
         const vector<string>&   inherit_attrs)
 {
     string inherit_val;
+    string target;
+
+    ostringstream oss;
+
     vector<string>::const_iterator it;
 
     set<int> nic_sgs;
@@ -569,13 +594,9 @@ int VirtualNetwork::nic_attribute(
     nic->replace("NETWORK_ID", oid);
     nic->replace("BRIDGE", bridge);
 
-    if ( vlan == 1 )
+    if (!vn_mad.empty())
     {
-        nic->replace("VLAN", "YES");
-    }
-    else
-    {
-        nic->replace("VLAN", "NO");
+        nic->replace("VN_MAD", vn_mad);
     }
 
     if (!phydev.empty())
@@ -588,11 +609,23 @@ int VirtualNetwork::nic_attribute(
         nic->replace("VLAN_ID", vlan_id);
     }
 
-    if ( get_cluster_id() != ClusterPool::NONE_CLUSTER_ID )
+    if (parent_vid != -1)
     {
-        nic->replace("CLUSTER_ID", get_cluster_id());
+        nic->replace("PARENT_NETWORK_ID", parent_vid);
     }
 
+    target = nic->vector_value("TARGET");
+
+    if (target.empty())
+    {
+        oss << "one-" << vid << "-" << nic->vector_value("NIC_ID");
+        nic->replace("TARGET", oss.str());
+    }
+
+    set<int> cluster_ids = get_cluster_ids();
+
+    nic->replace("CLUSTER_ID", one_util::join(cluster_ids, ','));
+
     for (it = inherit_attrs.begin(); it != inherit_attrs.end(); it++)
     {
         PoolObjectSQL::get_template_attribute((*it).c_str(), inherit_val);
@@ -613,15 +646,15 @@ int VirtualNetwork::nic_attribute(
 
     if (!ip.empty())
     {
-        rc = allocate_by_ip(vid, ip, nic, inherit_attrs);
+        rc = allocate_by_ip(PoolObjectSQL::VM, vid, ip, nic, inherit_attrs);
     }
     else if (!mac.empty())
     {
-        rc = allocate_by_mac(vid, mac, nic, inherit_attrs);
+        rc = allocate_by_mac(PoolObjectSQL::VM, vid, mac, nic, inherit_attrs);
     }
     else
     {
-        rc = allocate_addr(vid, nic, inherit_attrs);
+        rc = allocate_addr(PoolObjectSQL::VM, vid, nic, inherit_attrs);
     }
 
     //--------------------------------------------------------------------------
@@ -648,6 +681,57 @@ int VirtualNetwork::nic_attribute(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
+int VirtualNetwork::vrouter_nic_attribute(
+        VectorAttribute *       nic,
+        int                     vrid,
+        const vector<string>&   inherit_attrs)
+{
+    int     rc = 0;
+    bool    floating;
+    vector<string>::const_iterator it;
+
+    //--------------------------------------------------------------------------
+    //  Set default values from the Virtual Network
+    //--------------------------------------------------------------------------
+
+    nic->replace("NETWORK", name);
+    nic->replace("NETWORK_ID", oid);
+
+    //--------------------------------------------------------------------------
+    //  Get the lease from the Virtual Network
+    //--------------------------------------------------------------------------
+    nic->vector_value("FLOATING_IP", floating);
+
+    if (floating)
+    {
+        string ip  = nic->vector_value("IP");
+        string mac = nic->vector_value("MAC");
+
+        if (!ip.empty())
+        {
+            rc = allocate_by_ip(PoolObjectSQL::VROUTER, vrid, ip, nic, inherit_attrs);
+        }
+        else if (!mac.empty())
+        {
+            rc = allocate_by_mac(PoolObjectSQL::VROUTER, vrid, mac, nic, inherit_attrs);
+        }
+        else
+        {
+            rc = allocate_addr(PoolObjectSQL::VROUTER, vrid, nic, inherit_attrs);
+        }
+    }
+
+    if (rc == 0)
+    {
+        vrouters.add(vrid);
+    }
+
+    return rc;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
 void VirtualNetwork::process_security_rule(
         VectorAttribute *        rule,
         vector<VectorAttribute*> &new_rules)
@@ -658,19 +742,11 @@ void VirtualNetwork::process_security_rule(
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-int VirtualNetwork::add_var(vector<Attribute *> &var, string& error_msg)
+int VirtualNetwork::add_var(vector<VectorAttribute *> &var, string& error_msg)
 {
-    for (vector<Attribute *>::iterator it=var.begin(); it!=var.end(); it++)
+    for (vector<VectorAttribute *>::iterator it=var.begin(); it!=var.end(); it++)
     {
-        VectorAttribute * oar = dynamic_cast<VectorAttribute *>(*it);
-
-        if (oar == 0)
-        {
-            error_msg = "Invalid format for address range";
-            return -1;
-        }
-
-        VectorAttribute * ar = oar->clone();
+        VectorAttribute * ar = (*it)->clone();
 
         if (ar_pool.from_vattr(ar, error_msg) != 0)
         {
@@ -686,18 +762,11 @@ int VirtualNetwork::add_var(vector<Attribute *> &var, string& error_msg)
 
 int VirtualNetwork::add_ar(VirtualNetworkTemplate * ars_tmpl, string& error_msg)
 {
-    vector<Attribute *> var;
+    const VectorAttribute * ar = ars_tmpl->get("AR");
 
-    if (ars_tmpl->get("AR", var) <= 0)
+    if ( ar == 0 )
     {
-        return 0;
-    }
-
-    const VectorAttribute * ar = dynamic_cast<const VectorAttribute *>(var[0]);
-
-    if (ar == 0)
-    {
-        error_msg = "Wrong AR definition";
+        error_msg = "Wrong AR definition. AR vector attribute is missing.";
         return -1;
     }
 
@@ -720,12 +789,11 @@ int VirtualNetwork::update_ar(
         bool                    keep_restricted,
         string&                 error_msg)
 {
-    vector<Attribute *> tmp_ars;
+    vector<VectorAttribute *> tmp_ars;
 
     if(ars_tmpl->get("AR", tmp_ars) == 0)
     {
         error_msg = "Wrong AR definition. AR vector attribute is missing.";
-
         return -1;
     }
 
@@ -748,16 +816,7 @@ int VirtualNetwork::rm_ar(unsigned int ar_id, string& error_msg)
 int VirtualNetwork::hold_leases(VirtualNetworkTemplate * leases_template,
                                 string&                  error_msg)
 {
-    vector<const Attribute *> vleases;
-    const VectorAttribute *   lease = 0;
-
-    if (leases_template->get("LEASES", vleases) <= 0)
-    {
-        error_msg = "Empty lease description.";
-        return -1;
-    }
-
-   lease = dynamic_cast<const VectorAttribute *>(vleases[0]);
+    const VectorAttribute * lease = leases_template->get("LEASES");
 
     if ( lease == 0 )
     {
@@ -815,17 +874,7 @@ int VirtualNetwork::hold_leases(VirtualNetworkTemplate * leases_template,
 int VirtualNetwork::free_leases(VirtualNetworkTemplate * leases_template,
                                 string&                  error_msg)
 {
-
-    vector<const Attribute *> vleases;
-    const VectorAttribute *   lease = 0;
-
-    if (leases_template->get("LEASES", vleases) <= 0)
-    {
-        error_msg = "Empty lease description.";
-        return -1;
-    }
-
-   lease = dynamic_cast<const VectorAttribute *>(vleases[0]);
+    const VectorAttribute * lease = leases_template->get("LEASES");
 
     if ( lease == 0 )
     {
@@ -838,7 +887,6 @@ int VirtualNetwork::free_leases(VirtualNetworkTemplate * leases_template,
     string  ip  = lease->vector_value("IP");
     string  mac = lease->vector_value("MAC");
 
-
     if (ip.empty() && mac.empty())
     {
         error_msg = "Missing MAC or IP.";
