@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                */
+/* Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -21,6 +21,7 @@
 #include "PoolObjectAuth.h"
 #include "AuthManager.h"
 #include "AddressRange.h"
+
 #include <sstream>
 #include <ctype.h>
 
@@ -28,32 +29,46 @@
 /* -------------------------------------------------------------------------- */
 
 unsigned int VirtualNetworkPool::_mac_prefix;
+
 unsigned int VirtualNetworkPool::_default_size;
 
+const char * VirtualNetworkPool::vlan_table = "network_vlan_bitmap";
+
+const int VirtualNetworkPool::VLAN_BITMAP_ID = 0;
+
 /* -------------------------------------------------------------------------- */
 
 VirtualNetworkPool::VirtualNetworkPool(
     SqlDB *                             db,
     const string&                       prefix,
     int                                 __default_size,
-    vector<const Attribute *>&          restricted_attrs,
-    vector<const Attribute *>           hook_mads,
+    vector<const SingleAttribute *>&    restricted_attrs,
+    vector<const VectorAttribute *>&    hook_mads,
     const string&                       remotes_location,
-    const vector<const Attribute *>&    _inherit_attrs):
-    PoolSQL(db, VirtualNetwork::table, true, true)
+    const vector<const SingleAttribute *>& _inherit_attrs,
+    const VectorAttribute *             _vlan_conf,
+    const VectorAttribute *             _vxlan_conf):
+        PoolSQL(db, VirtualNetwork::table, true, true), vlan_conf(_vlan_conf),
+        vlan_id_bitmap(vlan_conf, VLAN_BITMAP_ID, vlan_table),
+        vxlan_conf(_vxlan_conf)
 {
     istringstream iss;
     size_t        pos   = 0;
     int           count = 0;
     unsigned int  tmp;
 
-    vector<const Attribute *>::const_iterator it;
+    vector<const SingleAttribute *>::const_iterator it;
 
     string mac = prefix;
 
     _mac_prefix   = 0;
     _default_size = __default_size;
 
+    if ( vlan_id_bitmap.select(VLAN_BITMAP_ID, db) != 0 )
+    {
+        vlan_id_bitmap.insert(VLAN_BITMAP_ID, db);
+    }
+
     while ( (pos = mac.find(':')) !=  string::npos )
     {
         mac.replace(pos,1," ");
@@ -82,9 +97,7 @@ VirtualNetworkPool::VirtualNetworkPool(
 
     for (it = _inherit_attrs.begin(); it != _inherit_attrs.end(); it++)
     {
-        const SingleAttribute* sattr = static_cast<const SingleAttribute *>(*it);
-
-        inherit_attrs.push_back(sattr->value());
+        inherit_attrs.push_back((*it)->value());
     }
 }
 
@@ -92,17 +105,16 @@ VirtualNetworkPool::VirtualNetworkPool(
 /* -------------------------------------------------------------------------- */
 
 int VirtualNetworkPool::allocate (
-    int            uid,
-    int            gid,
-    const string&  uname,
-    const string&  gname,
-    int            umask,
-    int            pvid,
-    VirtualNetworkTemplate * vn_template,
-    int *          oid,
-    int            cluster_id,
-    const string&  cluster_name,
-    string&        error_str)
+        int                         uid,
+        int                         gid,
+        const string&               uname,
+        const string&               gname,
+        int                         umask,
+        int                         pvid,
+        VirtualNetworkTemplate *    vn_template,
+        int *                       oid,
+        const set<int>              &cluster_ids,
+        string&                     error_str)
 {
     VirtualNetwork * vn;
     VirtualNetwork * vn_aux = 0;
@@ -112,7 +124,7 @@ int VirtualNetworkPool::allocate (
     ostringstream oss;
 
     vn = new VirtualNetwork(uid, gid, uname, gname, umask, pvid,
-                            cluster_id, cluster_name, vn_template);
+                            cluster_ids, vn_template);
 
     // Check name
     vn->PoolObjectSQL::get_template_attribute("NAME", name);
@@ -130,8 +142,25 @@ int VirtualNetworkPool::allocate (
         goto error_duplicated;
     }
 
+    // Insert the VN in the DB
     *oid = PoolSQL::allocate(vn, error_str);
 
+    // Get a free VLAN_ID from the pool if needed
+    if ( *oid != -1 )
+    {
+        vn = get(*oid, true);
+
+        if ( set_vlan_id(vn) != 0 )
+        {
+            error_str = "Cannot automatically assign VLAN_ID to network.";
+            drop(vn, error_str);
+
+            *oid = -1;
+        };
+
+        vn->unlock();
+    }
+
     return *oid;
 
 
@@ -239,25 +268,28 @@ VirtualNetwork * VirtualNetworkPool::get_nic_by_id(const string& id_s,
     return vnet;
 }
 
-int VirtualNetworkPool::nic_attribute(VectorAttribute * nic,
-                                      int     nic_id,
-                                      int     uid,
-                                      int     vid,
-                                      string& error)
+int VirtualNetworkPool::nic_attribute(
+        PoolObjectSQL::ObjectType   ot,
+        VectorAttribute*            nic,
+        int                         nic_id,
+        int                         uid,
+        int                         vid,
+        string&                     error)
 {
+    int              rc;
     string           network;
     VirtualNetwork * vnet = 0;
 
     nic->replace("NIC_ID", nic_id);
 
-    if (!(network = nic->vector_value("NETWORK")).empty())
-    {
-        vnet = get_nic_by_name (nic, network, uid, error);
-    }
-    else if (!(network = nic->vector_value("NETWORK_ID")).empty())
+    if (!(network = nic->vector_value("NETWORK_ID")).empty())
     {
         vnet = get_nic_by_id(network, error);
     }
+    else if (!(network = nic->vector_value("NETWORK")).empty())
+    {
+        vnet = get_nic_by_name (nic, network, uid, error);
+    }
     else //Not using a pre-defined network
     {
         return -2;
@@ -268,7 +300,14 @@ int VirtualNetworkPool::nic_attribute(VectorAttribute * nic,
         return -1;
     }
 
-    int rc = vnet->nic_attribute(nic, vid, inherit_attrs);
+    if (ot == PoolObjectSQL::VM)
+    {
+        rc = vnet->nic_attribute(nic, vid, inherit_attrs);
+    }
+    else // (ot == PoolObjectSQL::VROUTER)
+    {
+        rc = vnet->vrouter_nic_attribute(nic, vid, inherit_attrs);
+    }
 
     if ( rc == 0 )
     {
@@ -277,7 +316,8 @@ int VirtualNetworkPool::nic_attribute(VectorAttribute * nic,
     else
     {
         ostringstream oss;
-        oss << "Cannot get IP/MAC lease from virtual network " << vnet->get_oid() << ".";
+        oss << "Cannot get IP/MAC lease from virtual network "
+            << vnet->get_oid() << ".";
 
         error = oss.str();
     }
@@ -291,16 +331,22 @@ int VirtualNetworkPool::nic_attribute(VectorAttribute * nic,
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
 
-void VirtualNetworkPool::authorize_nic(VectorAttribute * nic,
-                                       int uid,
-                                       AuthRequest * ar)
+void VirtualNetworkPool::authorize_nic(
+        PoolObjectSQL::ObjectType   ot,
+        VectorAttribute *           nic,
+        int                         uid,
+        AuthRequest *               ar)
 {
     string           network;
     VirtualNetwork * vnet = 0;
     PoolObjectAuth   perm;
     string           error;
 
-    if (!(network = nic->vector_value("NETWORK")).empty())
+    if (!(network = nic->vector_value("NETWORK_ID")).empty())
+    {
+        vnet = get_nic_by_id(network, error);
+    }
+    else if (!(network = nic->vector_value("NETWORK")).empty())
     {
         vnet = get_nic_by_name (nic, network, uid, error);
 
@@ -309,10 +355,6 @@ void VirtualNetworkPool::authorize_nic(VectorAttribute * nic,
             nic->replace("NETWORK_ID", vnet->get_oid());
         }
     }
-    else if (!(network = nic->vector_value("NETWORK_ID")).empty())
-    {
-        vnet = get_nic_by_id(network, error);
-    }
     else //Not using a pre-defined network
     {
         return;
@@ -329,3 +371,112 @@ void VirtualNetworkPool::authorize_nic(VectorAttribute * nic,
 
     ar->add_auth(AuthRequest::USE, perm);
 }
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+int VirtualNetworkPool::set_vlan_id(VirtualNetwork * vn)
+{
+    string vn_mad;
+
+    unsigned int start_vlan, hint_vlan;
+    unsigned int vlan_id;
+
+    ostringstream oss;
+
+    if ( !vn->vlan_id.empty() )
+    {
+        return 0;
+    }
+
+    if ( vn->vn_mad.empty() )
+    {
+        return 0;
+    }
+
+    switch (VirtualNetwork::str_to_driver(vn->vn_mad))
+    {
+        case VirtualNetwork::VXLAN:
+            if (vxlan_conf.vector_value("START", start_vlan) != 0)
+            {
+                start_vlan = 2; //default in oned.conf
+            }
+
+            vlan_id = start_vlan + vn->get_oid();
+            oss << vlan_id;
+
+            vn->vlan_id = oss.str();
+
+            vn->vlan_id_automatic = true;
+
+            update(vn);
+
+            break;
+
+        case VirtualNetwork::VLAN:
+        case VirtualNetwork::OVSWITCH:
+            start_vlan = vlan_id_bitmap.get_start_bit();
+            hint_vlan  = start_vlan + (vn->get_oid() % (4095 - start_vlan ));
+
+            if ( vlan_id_bitmap.get(hint_vlan, vlan_id) != 0 )
+            {
+                return -1;
+            }
+
+            oss << vlan_id;
+
+            vn->vlan_id = oss.str();
+            vn->vlan_id_automatic = true;
+
+            vlan_id_bitmap.update(db);
+
+            update(vn);
+
+            break;
+
+        default:
+            break;
+    }
+
+    return 0;
+}
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+void VirtualNetworkPool::release_vlan_id(VirtualNetwork *vn)
+{
+    istringstream  is;
+
+    unsigned int vlan_id;
+
+    if ( !vn->vlan_id_automatic )
+    {
+        return;
+    }
+
+    if ( vn->vlan_id.empty() )
+    {
+        return;
+    }
+
+    if ( vn->vn_mad.empty() )
+    {
+        return;
+    }
+
+    is.str(vn->vlan_id);
+    is >> vlan_id;
+
+    switch (VirtualNetwork::str_to_driver(vn->vn_mad))
+    {
+        case VirtualNetwork::VLAN:
+        case VirtualNetwork::OVSWITCH:
+            vlan_id_bitmap.reset(vlan_id);
+            vlan_id_bitmap.update(db);
+            break;
+
+        default:
+            break;
+    }
+}
