@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2015, OpenNebula Project, OpenNebula Systems                #
+# Copyright 2002-2016, OpenNebula Project, OpenNebula Systems                #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -27,15 +27,16 @@ class VXLANDriver < VNMMAD::VLANDriver
 
     # DRIVER name and XPATH for relevant NICs
     DRIVER       = "vxlan"
-    XPATH_FILTER = "TEMPLATE/NIC[VLAN='YES']"
+    XPATH_FILTER = "TEMPLATE/NIC[VN_MAD='vxlan']"
 
     ############################################################################
     # Create driver device operations are locked
     ############################################################################
-    def initialize(vm, deploy_id = nil, hypervisor = nil)
+    def initialize(vm, xpath_filter = nil, deploy_id = nil)
         @locking = true
 
-        super(vm, XPATH_FILTER, deploy_id, hypervisor)
+        xpath_filter ||= XPATH_FILTER
+        super(vm, xpath_filter, deploy_id)
     end
 
     ############################################################################
